/* LELUYA - GOROSTAS program block */
/* calculates the radiative equilibrium equation: 
         Function1(T) - Function2 = 0
   where:
    Function1(T) = T^4 Int(q_abs Planck(T) dlambda)
    Function2 = T_sub[1]^4 Int(q_abs U(Y) dlambda) / Y^2 / Psi
*/

#include "GOROSTAS.h"

double _GOROSTAS_Equilibrium(double T, double x, double z, int dust_component,
                           double Function2){
  double *f;
  double Function1;
  int counter,error;

  f=dvector(0,lambda_Max-1);
  for (counter=0; counter<lambda_Max; counter++) {
    f[counter]=q_abs(x,z,counter,dust_component,&error)*
               _MATH_LIB_Planck(T,Lambda_Grid[counter]);
  }
  Function1 = _MATH_LIB_simpson(lambda_Max,0,lambda_Max-1,Lambda_Grid,f);
  Function1 *= (T*T*T*T);
  
  free_dvector(f,0,lambda_Max-1);

  return (Function1 - Function2);
}
