/* LELUYA - GOROSTAS program block */


/* This function creates a list of radii used in selecting a subset 
   of vertices for the radiative transfer */

#include "GOROSTAS.h"

int _GOROSTAS_Find_Radial_Bins (void){

  int lambda_k,lambda_max,i,j,dust_i;
  double *List_of_radii;
  struct _vertex_structure   *loop_vertex; /* a grid point */
  double dummy;


  /* first, find the grid with the most of vertices */
  lambda_max = 0;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++)
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES > 
	grid_per_lambda[lambda_max].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES) 
      lambda_max = lambda_k;

  /* set the global grid */
  TOTAL_NUMBER_OF_VERTICES = 
    grid_per_lambda[lambda_max].TOTAL_NUMBER_OF_VERTICES;
  TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES =
    grid_per_lambda[lambda_max].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;
  start_vertex_list = grid_per_lambda[lambda_max].start_vertex;
  TOTAL_NUMBER_OF_TRIANGLES = 
    grid_per_lambda[lambda_max].TOTAL_NUMBER_OF_TRIANGLES;
  start_triangle_list = grid_per_lambda[lambda_max].start_triangle;

  /* create the list of radial bins */
  /* (it is destroyed in LELUYA.c) */  
  Radial_Bins = dvector(0,TOTAL_NUMBER_OF_GRID_BINS);

  /* create a temporary list of all radii*/
  List_of_radii = dvector(0,TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES-1);
  loop_vertex = start_vertex_list;
  i = 0;
  do {
    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
          loop_vertex->T_dust[dust_i]>0.0) {
	List_of_radii[i]=sqrt(loop_vertex->x*loop_vertex->x +
                               loop_vertex->z*loop_vertex->z);
	i++;
        break;
      }
    loop_vertex = loop_vertex->next;
  } while (loop_vertex != start_vertex_list);
  
  /* sort the radii (bubble sort) */
  for (i=0;i<TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES-1;i++)
    for (j=i+1;j<TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;j++) 
      if (List_of_radii[j]<List_of_radii[i]){
	dummy = List_of_radii[j];
	List_of_radii[j]=List_of_radii[i];
	List_of_radii[i]=dummy;
      }

  /* set the bin radii */
  Radial_Bins[0]=0.0;
  for (i=1;i<TOTAL_NUMBER_OF_GRID_BINS;i++){
    j = (int)(i*TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES/
                TOTAL_NUMBER_OF_GRID_BINS);
    Radial_Bins[i]=List_of_radii[j];
  }
  Radial_Bins[TOTAL_NUMBER_OF_GRID_BINS]=2*Rout; /* just to be sure */

  /* print the radial bins */
  if (my_MPI_rank==0) {
    printf("#######\n");
    for (i=0;i<=TOTAL_NUMBER_OF_GRID_BINS;i++)
      printf("Radius[bin=%d]=%lf\n",i,Radial_Bins[i]);
    printf("#######\n");
  }
  free_dvector(List_of_radii,0,TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES-1);
  return 1;
}
