/* LELUYA - GOROSTAS program block */


/* This function creates a list of temperatures used in selecting a subset 
   of vertices for the radiative transfer */

#include "GOROSTAS.h"

int _GOROSTAS_Find_Temperature_Bins (void){

  int lambda_k,lambda_max,i,j,dust_i;
  double *List_of_temperatures;
  struct _vertex_structure   *loop_vertex; /* a grid point */
  double dummy;


  /* first, find the grid with the most of vertices */
  lambda_max = 0;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++)
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES > 
	grid_per_lambda[lambda_max].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES) 
      lambda_max = lambda_k;

  /* set the global grid */
  TOTAL_NUMBER_OF_VERTICES = 
    grid_per_lambda[lambda_max].TOTAL_NUMBER_OF_VERTICES;
  TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES =
    grid_per_lambda[lambda_max].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;
  start_vertex_list = grid_per_lambda[lambda_max].start_vertex;
  TOTAL_NUMBER_OF_TRIANGLES = 
    grid_per_lambda[lambda_max].TOTAL_NUMBER_OF_TRIANGLES;
  start_triangle_list = grid_per_lambda[lambda_max].start_triangle;

  /* create the list of temperature bins */
  /* (it is destroyed in LELUYA.c) */  
  Temperature_Bins = dvector(0,TOTAL_NUMBER_OF_GRID_BINS);

  /* create a temporary list of temperatures*/
  List_of_temperatures = dvector(0,TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES-1);
  loop_vertex = start_vertex_list;
  i = 0;
  do {
    dummy = -1.0;    
    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
          loop_vertex->T_dust[dust_i]>0.0)
        if(loop_vertex->T_dust[dust_i]>dummy)
	  dummy = loop_vertex->T_dust[dust_i];
    if (dummy>0.0) {List_of_temperatures[i]=dummy;i++;}
    loop_vertex = loop_vertex->next;
  } while (loop_vertex != start_vertex_list);
  
  /* sort the temperatures (bubble sort) */
  for (i=0;i<TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES-1;i++)
    for (j=i+1;j<TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;j++) 
      if (List_of_temperatures[j]>List_of_temperatures[i]){
	dummy = List_of_temperatures[j];
	List_of_temperatures[j]=List_of_temperatures[i];
	List_of_temperatures[i]=dummy;
      }

  /* set the bin temperatures */
  Temperature_Bins[0]=-1.0;
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
      if (Dust_Sublim_Temp[dust_i]>Temperature_Bins[0])
	Temperature_Bins[0]=Dust_Sublim_Temp[dust_i];
  Temperature_Bins[0] *= 10.0; /* just to be sure */
  for (i=1;i<TOTAL_NUMBER_OF_GRID_BINS;i++){
    j = (int)(i*TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES/
                TOTAL_NUMBER_OF_GRID_BINS);
    Temperature_Bins[i]=List_of_temperatures[j];
  }
  Temperature_Bins[TOTAL_NUMBER_OF_GRID_BINS]=0.0;

  /* print the temperature bins */
  printf("#######\n");
  for (i=0;i<=TOTAL_NUMBER_OF_GRID_BINS;i++)
    printf("Temperature[bin=%d]=%lf\n",i,Temperature_Bins[i]);
  printf("#######\n");

  free_dvector(List_of_temperatures,0,TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES-1);
  return 1;
}
