/* LELUYA - GOROSTAS program block */

/* This function is creating the angular integration directions at a given 
   vertex */

/* HOW TO USE IT:

  - before calling _GOROSTAS_create_integral_directions you have to call
    _GOROSTAS_create_3D_cloud 
  - this _GOROSTAS_create_3D_cloud creates points in 3D space used for 
    calculating the distribution of integral rays
  - when we are finished with one LELUYA grid, and we do not need this
    3D cloud any more, destroy it with _GOROSTAS_destroy_3D_cloud
*/

#include "GOROSTAS.h"

  /*For this we need to divide a unit sphere into triangle zones (spherical
    triangles):
          integral_directions[MAX_NUMBER_OF_SPHERICAL_TRIANGLES+4]
    Their centroids are going to be the integration rays:
          integral_directions[].direction
    The integral weight is the area of these triangles:
          integral_directions[].weight
    The angular size of triangles will be controlled by the number of 
    other grid vertices visible through a spherical triangle from 
    the 'central_vertex': 
          integral_directions[].ray_counter

    All together, there are total_number_of_spherical_triangles rays
  */

  /* The grid of spherical triangles starts with an icosahedron. But I think 
     that it should start with smaller triangles. Thus the initial icosahedron
     is going to be divided into smaller triangles several times. 
     This variable says how many times: */
  int initial_triangle_division = 2; /* 3 would create more than 700 rays!!*/

/* arrays  which will help us to split spherical triangles faster.
     The files will have the ID numbers of spherical triangles to which 
     all possible vertex directions belong. */
int array_counter_total;
int *Array_list_of_triangle_ID;
float *Array_list_of_vertex_x,*Array_list_of_vertex_y,*Array_list_of_vertex_z;
float *Cloud_3D_point_x,*Cloud_3D_point_y,*Cloud_3D_point_z;
int *Cloud_3D_lookup_table;

/*********************************************************************************/
/*********************************************************************************/
/* create 3D cloud of points */
/* int _GOROSTAS_create_3D_cloud (); <--- declared in GOROSTAS.h */
/* destroy 3D cloud of points */
/* int _GOROSTAS_destroy_3D_cloud (); -- declared in GOROSTAS.h */

/* The spherical triangles are created with this function
   (see the end of this file): */
int _GOROSTAS_create_spherical_triangle ();

/* ... and split by this function: */
void _GOROSTAS_split_spherical_triangle (int split_triangle, int *available_ID, 
                                       int *triangle1,     int *triangle2,
		                       int *triangle3,     int *triangle4);
/* Calculate the average direction from three given directions */
struct _3DVector _GOROSTAS_find_medium_direction(struct _3DVector v1, struct _3DVector v2,
                                               struct _3DVector v3);
/* If we want to know if a direction line goes through a given spherical
   triangle, use this function: */
int _GOROSTAS_spherical_direction_locator (int check_triangle,
		     struct _3DVector visible_vertex_direction);

/*********************************************************************************/
/*********************************************************************************/

#define Almost_Zero_Dude 1E-13 /* to avoid errors with zero floating point */

/*********************************************************************************/
/*********************************************************************************/

void _GOROSTAS_create_integral_directions (struct _vertex_structure *central_vertex){

  /* We are going to work with a list of spherical triangles. Here is its counter: */ 
  int counter_triangle; 

  /* Each spherical triangle will have a specific ID number. 
     This number is the next available: */
  int available_ID;

  /* Each set of divisions is called "level" */
  int level;

  /* The spherical triangles will be divided up to this level: */
  int final_level;
  /* This final level is calculated form the limiting angular grid resolution based
     on the minimal length Dmin resolved from the distance of Rout:

      final_level=(int)(3+log(PI^2 Rout^2 /6/sqrt(3)/Dmin^2)/log(4))

      ... and we need this minimal resolved length Dmin (for example, 10% of
      the source radius): */
  double Dmin; 

  /* We already have the 2D computational grid in the X-Z plane, but
     now we need the third dimension, too. Thus we introduce an equidistant
     angular phi-grid, where phi is an angle in the X-Y plane. Due to the
     rotational symmetry around the Z-axis, we need only the Y>=0 half-space.
     The phi-grid has '_GOROSTAS_N_phi_grid'+1 angles with the grid step of 
     'Delta_phi_grid':
            Delta_phi_grid = Pi / _GOROSTAS_N_phi_grid
     The grid starts at phi=0 (+X axis) and ends at phi=Pi (-X axis). */
  int phi_grid_counter;

  /* There is one practical problem: the grid in X-Z plane is defined only 
     for Z>=0. Thus we need to go twice thorough it because during the second 
     time we need to use the negative z-coordinates (multiply the z-coordinate
     of each grid vertex by -1 ) */
  int Z_sign;  /* +1 or -1 */

  /****************************************************************************/
  /* The spherical triangles will be divided as much as necessary.
     The criterium is simple: divide a spherical triangle if more
     than 'limit_rays_per_spherical_triangle' grid vertices is visible 
     through it from 'central_vertex'. This variable is controled by 
     an optimal total number of the integration rays 'optimal_number_of_rays':

                                     TOTAL_NUMBER_OF_VERTICES*(N_phi_grid+1)
 limit_rays_per_spherical_triangle = -----------------------------------------
                                             optimal_number_of_rays

  */
  int optimal_number_of_rays = 600;
  double limit_rays_per_spherical_triangle = 4.33*TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES*
                          (_GOROSTAS_N_phi_grid+1)/optimal_number_of_rays;
  /****************************************************************************/

  /* A variable which tells us if we have to continue with the division
     process is this: */
  int divide_spherical_triangles;
  int parent_spherical_ID; /* ID of the divided triangle */

  /* a flag which shows with a grid point is the empty space or not */
  int This_is_empty_space;
  int dust_i; /* dust type counter */

  /* We will need to go through the list of grid vertices. The following 
     variable will be used for this purpose: */
  struct _vertex_structure *grid_vertex;

  /* A vertex form the list of grid vertices is visible from 'central_vertex'
     in the following direction: */
  struct _3DVector visible_vertex_direction;
  /* The distance to 'central_vertex' is: */
  double visible_vertex_distance;
 
  /* a spherical triangle is divided into these 4 new triangles */
  int triangle1,triangle2,triangle3,triangle4;

  /* to calculate the area of a spherical triangle: */
  struct _3DVector vector1, vector2;
  double angle1, angle2, angle3;


  FILE *out; char filename[50]; /* debugg file for somtin'*/
  double dummy, local_phi_angle;
  struct _3DVector V0;

  int array_counter,N_3D_lookup,N_3D_lookup_old; 

  /* The search for optimal spherical triangles starts with the icosahedron
    structure: */
  /* THIS IS NOW A GLOBAL VARIABLE (see LELUYA.c)
  struct _3DVector basic_icosahedron[12]={
    { 0.303530998, 0.525731112, 0.794654472},
    { 0.303530998,-0.525731112, 0.794654472},
    {-0.303530998, 0.525731112,-0.794654472},
    {-0.303530998,-0.525731112,-0.794654472},
    { 0.982246946, 0.0,         0.187592475},
    { 0.607061996, 0.0,        -0.794654471},
    {-0.607061996, 0.0,         0.794654471},
    {-0.982246946, 0.0,        -0.187592475},
    { 0.491123473, 0.850650808,-0.187592474},
    { 0.491123473,-0.850650808,-0.187592474},
    {-0.491123473, 0.850650808, 0.187592474},
    {-0.491123473,-0.850650808, 0.187592474}};
  */
  struct _3DVector icosahedron[12];

  /* rotate the basic icosahedron in such a way that its local z-axis goes through
     the origin of the global coordinate system */
  struct _3DVector central_vertex_direction;
  int counter;
  double cos_central_vertex, sin_central_vertex; /* cos and sin of the theta angle
                                                    toward the central vertex */
  double central_vertex_distance; /* distance from (0,0,0) to centarl_vertex */

  /* search for optical depth gradient */
  int err;
  double *temperature, Eta0, Eta1, Step3D;
  double tau1,tau2,tau3;
  struct _3DVector tau_point;
  struct _triangle_structure *central_triangle,*tau_triangle;

  temperature = dvector(0,Max_Dust_Type-1);  

  if (central_vertex->triangles!=NULL)
    central_triangle = central_vertex->triangles->triangle;
  else
    _ZELENJAK_locate_point_in_grid(central_vertex->x,central_vertex->z,
                                  &central_triangle);

  /* find initial temperature and eta */ 
  if (central_vertex->T_dust==NULL) {
    if (central_triangle==NULL)
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) temperature[dust_i]=-1;
    else for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
        if (    central_triangle->v1->T_dust[dust_i]>0.0 && 
                central_triangle->v2->T_dust[dust_i]>0.0 &&
	        central_triangle->v3->T_dust[dust_i]>0.0    ){
	  temperature[dust_i]= _MATH_LIB_plane_interpolation 
                            (central_triangle->v1->x,central_triangle->v1->z,
                             central_triangle->v1->T_dust[dust_i],
		             central_triangle->v2->x,central_triangle->v2->z,
                             central_triangle->v2->T_dust[dust_i],
			     central_triangle->v3->x,central_triangle->v3->z,
                             central_triangle->v3->T_dust[dust_i],
			     central_vertex->x,central_vertex->z);
	}
        else temperature[dust_i]=-1;
      }
      else temperature[dust_i]=-1;
    }
  }
  else
   for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
                         temperature[dust_i]=central_vertex->T_dust[dust_i];

  Eta0 = Eta(central_vertex->x,central_vertex->z,Lambda_Total,temperature,&err);
  /*if (central_triangle!=NULL) */
  /*  Step3D = central_triangle->integral_step;*/
  /*else */
    Step3D = _ZELENJAK_size_limit(central_vertex->x,central_vertex->z);

  cos_central_vertex = sqrt(central_vertex->x*central_vertex->x+
                            central_vertex->z*central_vertex->z);
  sin_central_vertex = central_vertex->x/cos_central_vertex;
  cos_central_vertex = central_vertex->z/cos_central_vertex;

  /**********************************************************************/
  /* central vertex direction */
  central_vertex_direction.x = - central_vertex->x;
  central_vertex_direction.y = 0.0;
  central_vertex_direction.z = -central_vertex->z;
  /* normalize this vector */
  central_vertex_distance = sqrt(
                     central_vertex_direction.x*central_vertex_direction.x +
                     central_vertex_direction.z*central_vertex_direction.z);
  central_vertex_direction.x /= central_vertex_distance;
  central_vertex_direction.z /= central_vertex_distance;
  /**********************************************************************/

  for (counter=0; counter<12; counter++) {
    /* rotation by PI+theta */ 
    icosahedron[counter].x = - basic_icosahedron[counter].x*cos_central_vertex 
                             - basic_icosahedron[counter].z*sin_central_vertex;
    icosahedron[counter].y = basic_icosahedron[counter].y;
    icosahedron[counter].z =   basic_icosahedron[counter].x*sin_central_vertex 
                             - basic_icosahedron[counter].z*cos_central_vertex;
  }

  /* create the initial icosahedron */
  /* allowed icosahedron's sides: (0,4,8),(0,8,10),(0,10,6),(0,6,1),(0,1,4),
                                  (4,5,8),(8,5,2),(8,2,10),(10,2,7),(10,7,6),
                                  (2,5,3),(2,3,7)
     (notice the right-hand orientation of the triangles!!) */

  available_ID=1;

  /* set the number of triangles to zero */ 
  total_number_of_spherical_triangles=0;

  /* triangle (0,4,8) (the first one in the list) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[0];
  integral_directions[counter_triangle].v2 = icosahedron[4];
  integral_directions[counter_triangle].v3 = icosahedron[8];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;
  integral_directions[counter_triangle].weight = 4*PI/20;

  /* triangle (0,8,10) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[0];
  integral_directions[counter_triangle].v2 = icosahedron[8];
  integral_directions[counter_triangle].v3 = icosahedron[10];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (0,10,6) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[0];
  integral_directions[counter_triangle].v2 = icosahedron[10];
  integral_directions[counter_triangle].v3 = icosahedron[6];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (0,6,1) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[0];
  integral_directions[counter_triangle].v2 = icosahedron[6];
  integral_directions[counter_triangle].v3 = icosahedron[1];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (0,1,4) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[0];
  integral_directions[counter_triangle].v2 = icosahedron[1];
  integral_directions[counter_triangle].v3 = icosahedron[4];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (4,5,8) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[4];
  integral_directions[counter_triangle].v2 = icosahedron[5];
  integral_directions[counter_triangle].v3 = icosahedron[8];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (8,5,2) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[8];
  integral_directions[counter_triangle].v2 = icosahedron[5];
  integral_directions[counter_triangle].v3 = icosahedron[2];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (8,2,10) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[8];
  integral_directions[counter_triangle].v2 = icosahedron[2];
  integral_directions[counter_triangle].v3 = icosahedron[10];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (10,2,7) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[10];
  integral_directions[counter_triangle].v2 = icosahedron[2];
  integral_directions[counter_triangle].v3 = icosahedron[7];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (10,7,6) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[10];
  integral_directions[counter_triangle].v2 = icosahedron[7];
  integral_directions[counter_triangle].v3 = icosahedron[6];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (2,5,3) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[2];
  integral_directions[counter_triangle].v2 = icosahedron[5];
  integral_directions[counter_triangle].v3 = icosahedron[3];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2, 
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;
  /* triangle (2,3,7) */
  counter_triangle = _GOROSTAS_create_spherical_triangle();
  integral_directions[counter_triangle].v1 = icosahedron[2];
  integral_directions[counter_triangle].v2 = icosahedron[3];
  integral_directions[counter_triangle].v3 = icosahedron[7];
  integral_directions[counter_triangle].direction = _GOROSTAS_find_medium_direction
    (integral_directions[counter_triangle].v1,integral_directions[counter_triangle].v2,
     integral_directions[counter_triangle].v3);
  integral_directions[counter_triangle].ID_number = available_ID;
  available_ID++;  
  integral_directions[counter_triangle].weight =  integral_directions[0].weight;

  /* set variuos triangle minimal size related variables */ 
  if (Source_Radius==0.0) {Dmin=0.01; final_level = 20;}
  else {
    Dmin = 0.1*Source_Radius;
    final_level = (int)(3.+log(PI*PI*Rout*Rout/(6.*SQRT3*Dmin*Dmin))/log(4.));
  }

  /* continue dividing only if we need more rays */
  level = 1;
  if (total_number_of_spherical_triangles<optimal_number_of_rays && level<final_level) {

  /* memorize the ID numbers of triangles to which all vertices in
     the computational space belong. */

    for(array_counter=0;array_counter<array_counter_total;array_counter++) {

      visible_vertex_direction.x = Cloud_3D_point_x[array_counter]  -
                                       central_vertex->x;
      visible_vertex_direction.y = Cloud_3D_point_y[array_counter];
      visible_vertex_direction.z = Cloud_3D_point_z[array_counter]  -
                                       central_vertex->z;
      if (visible_vertex_direction.x==0.0 && visible_vertex_direction.y == 0.0 &&
          visible_vertex_direction.z==0.0){
	Array_list_of_triangle_ID[array_counter] = 0;
	Array_list_of_vertex_x[array_counter] = 0.0;
	Array_list_of_vertex_y[array_counter] = 0.0;
	Array_list_of_vertex_z[array_counter] = 0.0;
	continue;
      }
      /* normalize this vector */
      visible_vertex_distance = sqrt(
                     visible_vertex_direction.x*visible_vertex_direction.x +
                     visible_vertex_direction.y*visible_vertex_direction.y +
                     visible_vertex_direction.z*visible_vertex_direction.z);
      visible_vertex_direction.x /= visible_vertex_distance;
      visible_vertex_direction.y /= visible_vertex_distance;
      visible_vertex_direction.z /= visible_vertex_distance;
      /* Let's find now through which spherical triangle we see the vertex
         from 'central_vertex'. For that we need to go through the list of
         spherical triangles. */
      for (counter_triangle=0; counter_triangle<total_number_of_spherical_triangles;
           counter_triangle++) {
        if (_GOROSTAS_spherical_direction_locator(counter_triangle,
                                                visible_vertex_direction)) {
          /* 'visible_vertex_direction' belongs to that spherical triangle.*/
	  Array_list_of_triangle_ID[array_counter] =
                                         integral_directions[counter_triangle].ID_number;
	  Array_list_of_vertex_x[array_counter] =
                                              visible_vertex_direction.x;
	  Array_list_of_vertex_y[array_counter] =
                                              visible_vertex_direction.y;
	  Array_list_of_vertex_z[array_counter] =
                                              visible_vertex_direction.z;
          integral_directions[counter_triangle].ray_counter++; 
          break; 
	}
      } /* end of the loop over spherical triangles */
      if (counter_triangle==total_number_of_spherical_triangles){
	/* to avoid crashing of a program run, we will write zero in
           this case, and avoid this vertex in the analysis below */
	Array_list_of_triangle_ID[array_counter] = 0;
	Array_list_of_vertex_x[array_counter] = 0.0;
	Array_list_of_vertex_y[array_counter] = 0.0;
	Array_list_of_vertex_z[array_counter] = 0.0;
      }
    }

  /* The spherical triangles will be divided into smaller ones as long as 
     it is necessary. When we don't have any of them to divide any more, 
     we can stop the loop over the division process. 
     A variable which keeps information about the necessity for division is
     'divide_spherical_triangles' */

  /* set the lookup table to the initial values */
  for (array_counter=0;array_counter<array_counter_total;array_counter++)
    Cloud_3D_lookup_table[array_counter] = array_counter;
  N_3D_lookup = array_counter_total;

  do {
    divide_spherical_triangles = FALSE;
    for (counter_triangle=total_number_of_spherical_triangles-1;counter_triangle>=0;
	 counter_triangle--) {

      /* check if we have to divide that triangle */ 
      if (integral_directions[counter_triangle].ray_counter>limit_rays_per_spherical_triangle
          || level <= initial_triangle_division){

	divide_spherical_triangles = TRUE; 
	parent_spherical_ID = integral_directions[counter_triangle].ID_number;

	/* split the triangle */
	_GOROSTAS_split_spherical_triangle (counter_triangle,&available_ID,
		   &triangle1,&triangle2,&triangle3,&triangle4);

	/* list over 3D vertices */
	for(array_counter=0;array_counter<N_3D_lookup;array_counter++)
          if (Cloud_3D_lookup_table[array_counter]>=0)
	  if (Array_list_of_triangle_ID[Cloud_3D_lookup_table[array_counter]] 
                                                                 == parent_spherical_ID){

            counter = Cloud_3D_lookup_table[array_counter];
	    /*find the direction of the current vertex from 'central_vertex' */
            visible_vertex_direction.x=Array_list_of_vertex_x[counter];
            visible_vertex_direction.y=Array_list_of_vertex_y[counter];
            visible_vertex_direction.z=Array_list_of_vertex_z[counter];

	    /* find the triangle for this vertex */
	    if (_GOROSTAS_spherical_direction_locator(triangle1,
						    visible_vertex_direction)) {
	      integral_directions[triangle1].ray_counter++;
	      Array_list_of_triangle_ID[counter]=
                 integral_directions[triangle1].ID_number;
	    }
	    else if (_GOROSTAS_spherical_direction_locator(triangle2,
						    visible_vertex_direction)) {
	      integral_directions[triangle2].ray_counter++;
	      Array_list_of_triangle_ID[counter]=
                 integral_directions[triangle2].ID_number;
	    }
	    else if (_GOROSTAS_spherical_direction_locator(triangle3,
						    visible_vertex_direction)) {
	      integral_directions[triangle3].ray_counter++;
	      Array_list_of_triangle_ID[counter]=
                  integral_directions[triangle3].ID_number;
	    }
	    else if (_GOROSTAS_spherical_direction_locator(triangle4,
						    visible_vertex_direction)) {
	      integral_directions[triangle4].ray_counter++;
	      Array_list_of_triangle_ID[counter]=
                  integral_directions[triangle4].ID_number;
	    }

	    /* mark this 3D point to use it in the next round again */
            Cloud_3D_lookup_table[array_counter] *= -1;
            Cloud_3D_lookup_table[array_counter] -= 1;

	  } /* end of loop over 3D vertices  */

	/* destroy the old spherical triangle */
	integral_directions[counter_triangle].parent_triangle_ID_number =
 integral_directions[total_number_of_spherical_triangles-1].parent_triangle_ID_number;
	integral_directions[counter_triangle].ID_number =
 integral_directions[total_number_of_spherical_triangles-1].ID_number;
	integral_directions[counter_triangle].ray_counter =
 integral_directions[total_number_of_spherical_triangles-1].ray_counter;
	integral_directions[counter_triangle].weight =
 integral_directions[total_number_of_spherical_triangles-1].weight;
	integral_directions[counter_triangle].direction =
 integral_directions[total_number_of_spherical_triangles-1].direction;
	integral_directions[counter_triangle].v1 =
 integral_directions[total_number_of_spherical_triangles-1].v1;
	integral_directions[counter_triangle].v2 =
 integral_directions[total_number_of_spherical_triangles-1].v2;
	integral_directions[counter_triangle].v3 =
 integral_directions[total_number_of_spherical_triangles-1].v3;
	total_number_of_spherical_triangles--;

      } /* end of if (integral_directions[counter_triangle].ray_counter > limit) */ 
      else if (integral_directions[counter_triangle].ray_counter >= 0 ) {
	/*** check for tau gradient ****/

        /* find tau step close to vertex 1 */
        tau_point.x = (2./3.)*integral_directions[counter_triangle].v1.x +
	              (1./6.)*integral_directions[counter_triangle].v2.x +
                      (1./6.)*integral_directions[counter_triangle].v3.x;
        tau_point.y = (2./3.)*integral_directions[counter_triangle].v1.y +
	              (1./6.)*integral_directions[counter_triangle].v2.y +
                      (1./6.)*integral_directions[counter_triangle].v3.y;
        tau_point.z = (2./3.)*integral_directions[counter_triangle].v1.z +
	              (1./6.)*integral_directions[counter_triangle].v2.z +
                      (1./6.)*integral_directions[counter_triangle].v3.z;
        tau_point.x *= Step3D; tau_point.y *= Step3D; tau_point.z *= Step3D; 
        tau_point.x += central_vertex->x; 
        tau_point.z += central_vertex->z; 
        /* locate tau_triangle */
	Eta1=_ZELENJAK_3Dstep(tau_point,central_triangle,tau_point,-1.0,&V0,&tau_triangle);
        /* find temperature and eta */ 
        tau_point.x = sqrt(tau_point.x*tau_point.x + tau_point.y*tau_point.y);
        if (tau_point.z<0.0) tau_point.z *= 1.0;
        if (tau_triangle==NULL)
          for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) temperature[dust_i]=-1;
        else for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
          if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
            if (    tau_triangle->v1->T_dust[dust_i]>0.0 && 
                    tau_triangle->v2->T_dust[dust_i]>0.0 &&
	            tau_triangle->v3->T_dust[dust_i]>0.0    ){
	      temperature[dust_i]= _MATH_LIB_plane_interpolation 
                            (tau_triangle->v1->x,tau_triangle->v1->z,
                             tau_triangle->v1->T_dust[dust_i],
		             tau_triangle->v2->x,tau_triangle->v2->z,
                             tau_triangle->v2->T_dust[dust_i],
			     tau_triangle->v3->x,tau_triangle->v3->z,
                             tau_triangle->v3->T_dust[dust_i],
			     tau_point.x,tau_point.z);
	    }
            else temperature[dust_i]=-1;
          }
          else temperature[dust_i]=-1;
        }
        Eta1 = Eta(tau_point.x,tau_point.z,Lambda_Total,temperature,&err);
        /* optical depth (we care only about the gradient) */
        tau1 = Eta0 + Eta1;

        /* find tau step close to vertex 2 */
        tau_point.x = (1./6.)*integral_directions[counter_triangle].v1.x +
	              (2./3.)*integral_directions[counter_triangle].v2.x +
                      (1./6.)*integral_directions[counter_triangle].v3.x;
        tau_point.y = (1./6.)*integral_directions[counter_triangle].v1.y +
	              (2./3.)*integral_directions[counter_triangle].v2.y +
                      (1./6.)*integral_directions[counter_triangle].v3.y;
        tau_point.z = (1./6.)*integral_directions[counter_triangle].v1.z +
	              (2./3.)*integral_directions[counter_triangle].v2.z +
                      (1./6.)*integral_directions[counter_triangle].v3.z;
        tau_point.x *= Step3D; tau_point.y *= Step3D; tau_point.z *= Step3D; 
        tau_point.x += central_vertex->x; 
        tau_point.z += central_vertex->z; 
        /* locate tau_triangle */
	Eta1=_ZELENJAK_3Dstep(tau_point,central_triangle,tau_point,-1.0,&V0,&tau_triangle);
        /* find temperature and eta */ 
        tau_point.x = sqrt(tau_point.x*tau_point.x + tau_point.y*tau_point.y);
        if (tau_point.z<0.0) tau_point.z *= 1.0;
        if (tau_triangle==NULL)
          for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) temperature[dust_i]=-1;
        else for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
          if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
            if (    tau_triangle->v1->T_dust[dust_i]>0.0 && 
                    tau_triangle->v2->T_dust[dust_i]>0.0 &&
	            tau_triangle->v3->T_dust[dust_i]>0.0    ){
	      temperature[dust_i]= _MATH_LIB_plane_interpolation 
                            (tau_triangle->v1->x,tau_triangle->v1->z,
                             tau_triangle->v1->T_dust[dust_i],
		             tau_triangle->v2->x,tau_triangle->v2->z,
                             tau_triangle->v2->T_dust[dust_i],
			     tau_triangle->v3->x,tau_triangle->v3->z,
                             tau_triangle->v3->T_dust[dust_i],
			     tau_point.x,tau_point.z);
	    }
            else temperature[dust_i]=-1;
          }
          else temperature[dust_i]=-1;
        }
        Eta1 = Eta(tau_point.x,tau_point.z,Lambda_Total,temperature,&err);
        /* optical depth (we care only about the gradient) */
        tau2 = Eta0 + Eta1;

        /* find tau step close to vertex 3 */
        tau_point.x = (1./6.)*integral_directions[counter_triangle].v1.x +
	              (1./6.)*integral_directions[counter_triangle].v2.x +
                      (2./3.)*integral_directions[counter_triangle].v3.x;
        tau_point.y = (1./6.)*integral_directions[counter_triangle].v1.y +
	              (1./6.)*integral_directions[counter_triangle].v2.y +
                      (2./3.)*integral_directions[counter_triangle].v3.y;
        tau_point.z = (1./6.)*integral_directions[counter_triangle].v1.z +
	              (1./6.)*integral_directions[counter_triangle].v2.z +
                      (2./3.)*integral_directions[counter_triangle].v3.z;
        tau_point.x *= Step3D; tau_point.y *= Step3D; tau_point.z *= Step3D; 
        tau_point.x += central_vertex->x; 
        tau_point.z += central_vertex->z; 
        /* locate tau_triangle */
	Eta1=_ZELENJAK_3Dstep(tau_point,central_triangle,tau_point,-1.0,&V0,&tau_triangle);
        /* find temperature and eta */ 
        tau_point.x = sqrt(tau_point.x*tau_point.x + tau_point.y*tau_point.y);
        if (tau_point.z<0.0) tau_point.z *= 1.0;
        if (tau_triangle==NULL)
          for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) temperature[dust_i]=-1;
        else for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
          if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
            if (    tau_triangle->v1->T_dust[dust_i]>0.0 && 
                    tau_triangle->v2->T_dust[dust_i]>0.0 &&
	            tau_triangle->v3->T_dust[dust_i]>0.0    ){
	      temperature[dust_i]= _MATH_LIB_plane_interpolation 
                            (tau_triangle->v1->x,tau_triangle->v1->z,
                             tau_triangle->v1->T_dust[dust_i],
		             tau_triangle->v2->x,tau_triangle->v2->z,
                             tau_triangle->v2->T_dust[dust_i],
			     tau_triangle->v3->x,tau_triangle->v3->z,
                             tau_triangle->v3->T_dust[dust_i],
			     tau_point.x,tau_point.z);
	    }
            else temperature[dust_i]=-1;
          }
          else temperature[dust_i]=-1;
        }
        Eta1 = Eta(tau_point.x,tau_point.z,Lambda_Total,temperature,&err);
        /* optical depth (we care only about the gradient) */
        tau3 = Eta0 + Eta1;

	/* sort: tau1=min tau3=max */
        if (tau2>tau3) {Eta1=tau3;tau3=tau2;tau2=Eta1;}
        if (tau1>tau2) {
          Eta1=tau1;tau1=tau2;tau2=Eta1;
          if (tau2>tau3) {Eta1=tau3;tau3=tau2;tau2=Eta1;}
	}

	/* split the triangle if the gradient is too big */
        if (tau2>2*tau1 || tau3>2*tau1) {
	  /* split the triangle */
          _GOROSTAS_split_spherical_triangle (counter_triangle,&available_ID,
		                            &triangle1,&triangle2,&triangle3,&triangle4);

          /* destroy the old spherical triangle */
	  integral_directions[counter_triangle].parent_triangle_ID_number =
 integral_directions[total_number_of_spherical_triangles-1].parent_triangle_ID_number;
	  integral_directions[counter_triangle].ID_number =
 integral_directions[total_number_of_spherical_triangles-1].ID_number;
	  integral_directions[counter_triangle].ray_counter =
 integral_directions[total_number_of_spherical_triangles-1].ray_counter;
	  integral_directions[counter_triangle].weight =
 integral_directions[total_number_of_spherical_triangles-1].weight;
	  integral_directions[counter_triangle].direction =
 integral_directions[total_number_of_spherical_triangles-1].direction;
	  integral_directions[counter_triangle].v1 =
 integral_directions[total_number_of_spherical_triangles-1].v1;
	  integral_directions[counter_triangle].v2 =
 integral_directions[total_number_of_spherical_triangles-1].v2;
	  integral_directions[counter_triangle].v3 =
 integral_directions[total_number_of_spherical_triangles-1].v3;
	  total_number_of_spherical_triangles--;

        }
        else {
          /* no splitting, just mark it as OK */
	  if (integral_directions[counter_triangle].ray_counter==0)
            integral_directions[counter_triangle].ray_counter = -1;
          else integral_directions[counter_triangle].ray_counter *= -1;
	}
      }

      /* check if we hit the memory limit */ 
      if (total_number_of_spherical_triangles>=MAX_NUMBER_OF_SPHERICAL_TRIANGLES) {
	divide_spherical_triangles=FALSE; 
        printf("\n\nWARNING : the angular grid generator hit the memory limit of %d rays!!\n\n",
               MAX_NUMBER_OF_SPHERICAL_TRIANGLES);
        break; 
      }

    } /* end of the loop over spherical triangles */

    /* update the lookup table */
    N_3D_lookup_old = N_3D_lookup;
    N_3D_lookup = 0;
    for(array_counter=0;array_counter<N_3D_lookup_old;array_counter++)
      if (Cloud_3D_lookup_table[array_counter]<0) {
        /* use this point for the next round */
        if (Cloud_3D_lookup_table[N_3D_lookup]>=0) {
          Cloud_3D_lookup_table[N_3D_lookup]=Cloud_3D_lookup_table[array_counter];
          Cloud_3D_lookup_table[array_counter] = 0;
	}
        Cloud_3D_lookup_table[N_3D_lookup] *= -1;
        Cloud_3D_lookup_table[N_3D_lookup] -= 1;
        N_3D_lookup++;
      }

    /* set the level of refinement */
    level++;

    /*printf("%d && %d<%d && %d<%d (%lf)\n",divide_spherical_triangles,
            total_number_of_spherical_triangles,optimal_number_of_rays,level,final_level,
	    limit_rays_per_spherical_triangle);*/
  } while (divide_spherical_triangles 
           && total_number_of_spherical_triangles<optimal_number_of_rays
           && level<final_level);

  } /* end of spherical triangle division */

  /* check for triangles that are half visible */

  for (counter_triangle=0;counter_triangle<total_number_of_spherical_triangles;
	 counter_triangle++) {

    /* weight of this integral (surface of the spherical tirangle on 
       a unit sphere) */
    /* spherical angle at integral_directions[counter_triangle].v1 */ 
    vector1 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v2,
					  integral_directions[counter_triangle].v1);
    vector1 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v1,
					  vector1);
    vector2 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v3,
					  integral_directions[counter_triangle].v1);
    vector2 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v1,
					  vector2);
    angle1 = vector1.x*vector2.x + vector1.y*vector2.y + vector1.z*vector2.z;
    angle1 = angle1 / sqrt(vector1.x*vector1.x+vector1.y*vector1.y+
                           vector1.z*vector1.z);
    angle1 = angle1 / sqrt(vector2.x*vector2.x+vector2.y*vector2.y+
                           vector2.z*vector2.z);
    angle1 = acos(angle1);

    /* spherical angle at integral_directions[counter_triangle].v2 */ 
    vector1 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v3,
					  integral_directions[counter_triangle].v2);
    vector1 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v2,
					  vector1);
    vector2 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v1,
					  integral_directions[counter_triangle].v2);
    vector2 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v2,
					  vector2);
    angle2 = vector1.x*vector2.x + vector1.y*vector2.y + vector1.z*vector2.z;
    angle2 = angle2 / sqrt(vector1.x*vector1.x+vector1.y*vector1.y+
                           vector1.z*vector1.z);
    angle2 = angle2 / sqrt(vector2.x*vector2.x+vector2.y*vector2.y+
                           vector2.z*vector2.z);
    angle2 = acos(angle2);

    /* spherical angle at integral_directions[counter_triangle].v3 */ 
    vector1 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v1,
					  integral_directions[counter_triangle].v3);
    vector1 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v3,
					  vector1);
    vector2 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v2,
					  integral_directions[counter_triangle].v3);
    vector2 = _MATH_LIB_3D_vector_product(integral_directions[counter_triangle].v3,
					  vector2);
    angle3 = vector1.x*vector2.x + vector1.y*vector2.y + vector1.z*vector2.z;
    angle3 = angle3 / sqrt(vector1.x*vector1.x+vector1.y*vector1.y+
                           vector1.z*vector1.z);
    angle3 = angle3 / sqrt(vector2.x*vector2.x+vector2.y*vector2.y+
                           vector2.z*vector2.z);
    angle3 = acos(angle3); 

    /* the triangle's surface */
    integral_directions[counter_triangle].weight = angle1+angle2+angle3-PI;

    if (integral_directions[counter_triangle].direction.y < Almost_Zero_Dude) 
      integral_directions[counter_triangle].weight /= 2.0;
  }

    /*----------------------------------------------------------------
    sprintf(filename,"angular_grid_triangles_%.5d.wrl",central_vertex->identification_number);
    out=fopen(filename,"wt");
    fprintf(out,"#VRML V2.0 utf8\n\n");
    for (counter_triangle=0;counter_triangle<total_number_of_spherical_triangles;
	 counter_triangle++) {
      fprintf(out,"Group{children Shape{\n");
      fprintf(out,"geometry IndexedLineSet { coord Coordinate { point [");
      fprintf(out,"%E %E %E,\n",integral_directions[counter_triangle].v1.x,
                            integral_directions[counter_triangle].v1.y,integral_directions[counter_triangle].v1.z);
      fprintf(out,"%E %E %E,\n",integral_directions[counter_triangle].v2.x,
                            integral_directions[counter_triangle].v2.y,integral_directions[counter_triangle].v2.z);
      fprintf(out,"%E %E %E,\n",integral_directions[counter_triangle].v3.x,
                            integral_directions[counter_triangle].v3.y,integral_directions[counter_triangle].v3.z);
      fprintf(out,"]}\n");
      fprintf(out,"coordIndex [0,1,2,0,-1, ]}\n");
      fprintf(out,"}}\n");
    }
    fclose(out);
    ----------------------------------------------------------------*/

  /*----------------------------------------------------------------*/
  /* list the directions and their number of verrtices visible through their triangle *
  sprintf(filename,"3Dvertex_distribution_around_V=%.4d.dat",
	  central_vertex->identification_number);
  out=fopen(filename,"wt");
  fprintf(out,"theta(deg)        phi(deg)         N\n");
  for (counter_triangle=0;counter_triangle<total_number_of_spherical_triangles;
	 counter_triangle++) {
    V0 = integral_directions[counter_triangle].direction;
    dummy = V0.x*central_vertex_direction.x + V0.z*central_vertex_direction.z;
    fprintf(out,"%E    ",acos(dummy)*180/PI);
    if (1.0-dummy*dummy>0.0)
      dummy = V0.y/sqrt(1.0-dummy*dummy);
    else 
      dummy = 0.0;
    if (dummy>=1.000) dummy=PI/2.0;
    else {
      if (central_vertex_direction.z*V0.x-central_vertex_direction.x*V0.z>0.0)
	dummy=asin(dummy);
      else dummy = PI - asin(dummy);
     }
    fprintf(out,"%E   ",dummy*180/PI);
    fprintf(out,"%d\n",integral_directions[counter_triangle].ray_counter);
  }
  fclose(out);
  *----------------------------------------------------------------*/

  /* REMOVE THIS !!! tihis is just for debugging ******
  phi_grid_counter=_VODENJAK_plot_angular_integral_rays(central_vertex,
					    "integral_rays_VRML",_GOROSTAS_N_phi_grid);
  **********/

  /*---------------------------------------------------------------- *
  sprintf(filename,"angular_grid_distribution_V=%.4d.dat",
	  central_vertex->identification_number);
  out=fopen(filename,"wt");
  fprintf(out,"cos(center*ray)  cos(y*ray)\n");
    visible_vertex_direction.x = - central_vertex->x;
    visible_vertex_direction.y = 0.0;
    visible_vertex_direction.z = -central_vertex->z;
    visible_vertex_distance = sqrt(
                     visible_vertex_direction.x*visible_vertex_direction.x +
                     visible_vertex_direction.z*visible_vertex_direction.z);
    visible_vertex_direction.x /= visible_vertex_distance;
    visible_vertex_direction.y /= visible_vertex_distance;
    visible_vertex_direction.z /= visible_vertex_distance;

    for (counter_triangle=0;counter_triangle<total_number_of_spherical_triangles;
	 counter_triangle++) {
      angle1 = visible_vertex_direction.x*integral_directions[counter_triangle].direction.x + 
	       visible_vertex_direction.z*integral_directions[counter_triangle].direction.z;
      fprintf(out,"%15.9E   ",angle1);
      if (1.0-angle1*angle1>0.0)
        angle1 = integral_directions[counter_triangle].direction.y/sqrt(1.0-angle1*angle1);
      else 
	angle1 = 0.0;
      if (angle1>=1.000) angle1=PI/2.0;
      else {
        if (visible_vertex_direction.x*integral_directions[counter_triangle].direction.z- 
            visible_vertex_direction.z*integral_directions[counter_triangle].direction.x  > 0.0)
          angle1=asin(angle1);
        else angle1 = PI - asin(angle1);
     }
      fprintf(out,"%E\n",angle1*180/PI);

    }
  fclose(out);
  *----------------------------------------------------------------*/

  free_dvector(temperature,0,Max_Dust_Type-1);

  return;
}

/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

/* Calculate the average direction from three given directions */
struct _3DVector _GOROSTAS_find_medium_direction(struct _3DVector v1, struct _3DVector v2,
                                               struct _3DVector v3){

  double temp;
  struct _3DVector v0;

  v0.x = v1.x+v2.x+v3.x;
  v0.y = v1.y+v2.y+v3.y;
  v0.z = v1.z+v2.z+v3.z;
  temp = sqrt(v0.x*v0.x+v0.y*v0.y+v0.z*v0.z);
  v0.x /= temp;
  v0.y /= temp;
  v0.z /= temp;

  return v0;
}

/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

int _GOROSTAS_create_spherical_triangle (){

  total_number_of_spherical_triangles++;

  integral_directions[total_number_of_spherical_triangles-1].direction.x=0.0; 
  integral_directions[total_number_of_spherical_triangles-1].direction.y=0.0; 
  integral_directions[total_number_of_spherical_triangles-1].direction.z=0.0;
  integral_directions[total_number_of_spherical_triangles-1].v1.x=0.0; 
  integral_directions[total_number_of_spherical_triangles-1].v1.y=0.0; 
  integral_directions[total_number_of_spherical_triangles-1].v1.z=0.0;
  integral_directions[total_number_of_spherical_triangles-1].v2.x=0.0; 
  integral_directions[total_number_of_spherical_triangles-1].v2.y=0.0; 
  integral_directions[total_number_of_spherical_triangles-1].v2.z=0.0;
  integral_directions[total_number_of_spherical_triangles-1].v3.x=0.0; 
  integral_directions[total_number_of_spherical_triangles-1].v3.y=0.0; 
  integral_directions[total_number_of_spherical_triangles-1].v3.z=0.0; 
  integral_directions[total_number_of_spherical_triangles-1].ray_counter=0;
  integral_directions[total_number_of_spherical_triangles-1].weight=0;

  return (total_number_of_spherical_triangles-1);
}

/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

void _GOROSTAS_split_spherical_triangle (int split_triangle, int *available_ID, 
                                       int *triangle1,     int *triangle2,
		                       int *triangle3,     int *triangle4){
  struct _3DVector visible_vertex_direction;
  double visible_vertex_distance;
  int new_spherical_triangle_A;
  int new_spherical_triangle_B;
  int new_spherical_triangle_C;
  int new_spherical_triangle_D;
  struct _3DVector Av0, Av1, Av2, Av3;
  struct _3DVector Bv0, Bv1, Bv2, Bv3;
  struct _3DVector Cv0, Cv1, Cv2, Cv3;

        /* the new triangles will be the following ones:
                          v2
                           O
                          / \
                         / B \
                    new O-----O new
                       / \ D / \
                      / C \ / A \
                     O-----O-----O 
                    v3    new    v1
        */
        /* triangle 'A' : */

  new_spherical_triangle_A = _GOROSTAS_create_spherical_triangle();

  integral_directions[new_spherical_triangle_A].ID_number = *available_ID;
  (*available_ID)++;

  Av1 = integral_directions[split_triangle].v1;

  Av2.x=integral_directions[split_triangle].v1.x + integral_directions[split_triangle].v2.x;
  Av2.y=integral_directions[split_triangle].v1.y + integral_directions[split_triangle].v2.y;
  Av2.z=integral_directions[split_triangle].v1.z + integral_directions[split_triangle].v2.z;
  visible_vertex_direction = Av2;
  visible_vertex_distance = sqrt(
               visible_vertex_direction.x*visible_vertex_direction.x +
               visible_vertex_direction.y*visible_vertex_direction.y +
               visible_vertex_direction.z*visible_vertex_direction.z);
  Av2.x /= visible_vertex_distance;
  Av2.y /= visible_vertex_distance;
  Av2.z /= visible_vertex_distance;

  Av3.x=integral_directions[split_triangle].v1.x + integral_directions[split_triangle].v3.x;
  Av3.y=integral_directions[split_triangle].v1.y + integral_directions[split_triangle].v3.y;
  Av3.z=integral_directions[split_triangle].v1.z + integral_directions[split_triangle].v3.z;
  visible_vertex_direction = Av3;
  visible_vertex_distance = sqrt(
               visible_vertex_direction.x*visible_vertex_direction.x +
               visible_vertex_direction.y*visible_vertex_direction.y +
               visible_vertex_direction.z*visible_vertex_direction.z);
  Av3.x /= visible_vertex_distance;
  Av3.y /= visible_vertex_distance;
  Av3.z /= visible_vertex_distance;

  if (Av1.y <= Almost_Zero_Dude && Av2.y <= Almost_Zero_Dude && Av3.y <= Almost_Zero_Dude)
    {
    /* remove this triangle */
    total_number_of_spherical_triangles--;
    new_spherical_triangle_A=-1;
    }
  else  
    Av0 = _GOROSTAS_find_medium_direction(Av1,Av2,Av3);

  /* triangle 'B' : */
  new_spherical_triangle_B = _GOROSTAS_create_spherical_triangle();

  integral_directions[new_spherical_triangle_B].ID_number = *available_ID;
  (*available_ID)++;

  Bv1 = Av2;

  Bv2 = integral_directions[split_triangle].v2;

  Bv3.x=integral_directions[split_triangle].v2.x + integral_directions[split_triangle].v3.x;
  Bv3.y=integral_directions[split_triangle].v2.y + integral_directions[split_triangle].v3.y;
  Bv3.z=integral_directions[split_triangle].v2.z + integral_directions[split_triangle].v3.z;
  visible_vertex_direction = Bv3;
  visible_vertex_distance = sqrt(
               visible_vertex_direction.x*visible_vertex_direction.x +
               visible_vertex_direction.y*visible_vertex_direction.y +
               visible_vertex_direction.z*visible_vertex_direction.z);
  Bv3.x /= visible_vertex_distance;
  Bv3.y /= visible_vertex_distance;
  Bv3.z /= visible_vertex_distance;

  if (Bv1.y <= Almost_Zero_Dude && Bv2.y <= Almost_Zero_Dude && Bv3.y <= Almost_Zero_Dude)
    {
    /* remove this triangle */
    total_number_of_spherical_triangles--;
    new_spherical_triangle_B=-1;
    }
  else   
    Bv0 = _GOROSTAS_find_medium_direction(Bv1,Bv2,Bv3);

  /* triangle 'C' : */
  new_spherical_triangle_C = _GOROSTAS_create_spherical_triangle();

  integral_directions[new_spherical_triangle_C].ID_number = *available_ID;
  (*available_ID)++;

  Cv1 = Av3;
  Cv2 = Bv3;
  Cv3 = integral_directions[split_triangle].v3;

  if (Cv1.y <= Almost_Zero_Dude && Cv2.y <= Almost_Zero_Dude && Cv3.y <= Almost_Zero_Dude)
    {
    /* remove this triangle */
    total_number_of_spherical_triangles--;
    new_spherical_triangle_C=-1;
    }
  else   Cv0 = _GOROSTAS_find_medium_direction(Cv1,Cv2,Cv3);

  /* triangle 'D' : */
  new_spherical_triangle_D = _GOROSTAS_create_spherical_triangle();

  integral_directions[new_spherical_triangle_D].ID_number = *available_ID;
  (*available_ID)++;

  integral_directions[new_spherical_triangle_D].v1 = Av3;
  integral_directions[new_spherical_triangle_D].v2 = Av2;
  integral_directions[new_spherical_triangle_D].v3 = Bv3;
  integral_directions[new_spherical_triangle_D].direction = 
                                             integral_directions[split_triangle].direction;
  integral_directions[new_spherical_triangle_D].weight=
                                           0.25*integral_directions[split_triangle].weight;

  /* destroy triangles in the Y<0 half-space */
  if (new_spherical_triangle_A >= 0) {
    integral_directions[new_spherical_triangle_A].direction=Av0;
    integral_directions[new_spherical_triangle_A].v1=Av1;
    integral_directions[new_spherical_triangle_A].v2=Av2;
    integral_directions[new_spherical_triangle_A].v3=Av3;
    integral_directions[new_spherical_triangle_A].weight=
                                           0.25*integral_directions[split_triangle].weight;
  }
  if (new_spherical_triangle_B>=0) {
    integral_directions[new_spherical_triangle_B].direction=Bv0;
    integral_directions[new_spherical_triangle_B].v1=Bv1;
    integral_directions[new_spherical_triangle_B].v2=Bv2;
    integral_directions[new_spherical_triangle_B].v3=Bv3;
    integral_directions[new_spherical_triangle_B].weight=
                                           0.25*integral_directions[split_triangle].weight;
  }
  if (new_spherical_triangle_C>=0) {
    integral_directions[new_spherical_triangle_C].direction=Cv0;
    integral_directions[new_spherical_triangle_C].v1=Cv1;
    integral_directions[new_spherical_triangle_C].v2=Cv2;
    integral_directions[new_spherical_triangle_C].v3=Cv3;
    integral_directions[new_spherical_triangle_C].weight=
                                           0.25*integral_directions[split_triangle].weight;
  }

  *triangle1 = new_spherical_triangle_A;
  *triangle2 = new_spherical_triangle_B;
  *triangle3 = new_spherical_triangle_C;
  *triangle4 = new_spherical_triangle_D;
}

/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

int _GOROSTAS_spherical_direction_locator (int check_triangle,
		     struct _3DVector visible_vertex_direction){

  struct _3DVector vector1, vector2;
  int inside;

  /* We have to find out if this direction 
     'visible_vertex_direction' goes through the spherical 
     triangle 'spherical_triangle'. There is an easy way how to
     do this: follow the orientation of triangles made from 
     'visible_vertex_direction' and two vertices of the spherical
     triangle! 
     That is, the three "test" triangles shown below: */

  if (check_triangle<0 || check_triangle>=total_number_of_spherical_triangles) 
    return FALSE;
  /* check if the triangle and vertex are in different hemispheres */
  if (integral_directions[check_triangle].v1.x>0.0 && 
      integral_directions[check_triangle].v2.x>0.0 &&
      integral_directions[check_triangle].v3.x>0.0 && visible_vertex_direction.x<0.0)
    return FALSE;
  else if (integral_directions[check_triangle].v1.x<0.0 && 
           integral_directions[check_triangle].v2.x<0.0 &&
           integral_directions[check_triangle].v3.x<0.0 && visible_vertex_direction.x>0.0)
    return FALSE;
  else if (integral_directions[check_triangle].v1.z>0.0 && 
           integral_directions[check_triangle].v2.z>0.0 &&
           integral_directions[check_triangle].v3.z>0.0 && visible_vertex_direction.z<0.0)
    return FALSE;
  else if (integral_directions[check_triangle].v1.z<0.0 && 
           integral_directions[check_triangle].v2.z<0.0 &&
           integral_directions[check_triangle].v3.z<0.0 && visible_vertex_direction.z>0.0)
    return FALSE;

  /* The first test triangle (marked by 'T'): 

      integral_directions[check_triangle].v2
                                          O-----O visible_vertex_direction
                                         / \ T /
                                        / N \ /
integral_directions[check_triangle].v3 O-----O integral_directions[check_triangle].v1
	       
    The test triangle 'T' should have the same orientation as the
    triangle 'N'. BE CAREFUL, WE WORK WITH SPHERICAL TRIANGLES,
    NOT WITH ORDINARY PLANE TRIANGLES!!! 
    To check the orientation, we need vectors at 'v1' which will 
    be tangent to the sphere and pointing toward 
    'visible_vertex_direction' (vector 'vector1') and 
    'integral_directions[check_triangle].v2' (vector 'vector2')
    The 'visible_vertex_direction' is inside if (vector2 x vector1)*v1 > 0    
  */

  inside = FALSE;
  vector1 = _MATH_LIB_3D_vector_product(visible_vertex_direction,
                                        integral_directions[check_triangle].v1);
  if (vector1.x==0.0 && vector1.y==0.0 && vector1.z==0.0) {
    vector1 = integral_directions[check_triangle].v1;
    vector2 = visible_vertex_direction;
    if (vector1.x*vector2.x + vector1.y*vector2.y +
        vector1.z*vector2.z > 0.0) inside=1;
  }
  else {
    vector1 = _MATH_LIB_3D_vector_product(integral_directions[check_triangle].v1,vector1);
    vector2 = _MATH_LIB_3D_vector_product(integral_directions[check_triangle].v2,
				      integral_directions[check_triangle].v1);
    vector2 = _MATH_LIB_3D_vector_product(integral_directions[check_triangle].v1,vector2);
    vector1 = _MATH_LIB_3D_vector_product(vector2,vector1);
    if (vector1.x*integral_directions[check_triangle].v1.x +
        vector1.y*integral_directions[check_triangle].v1.y +
        vector1.z*integral_directions[check_triangle].v1.z >= 0.0) inside=1;
  }
  /* continue if the first 'test' triangle is OK */
  if (inside) {

    /* The second test triangle (marked by 'P'): 

          visible_vertex_direction  O-----O integral_directions[check_triangle].v2 
                                     \ P / \
                                      \ / N \ 
integral_directions[check_triangle].v3 O-----O integral_directions[check_triangle].v1
	       
    The test triangle 'P' should have the same orientation as the
    triangle 'N'. BE CAREFUL, WE WORK WITH SPHERICAL TRIANGLES,
    NOT WITH ORDINARY PLANE TRIANGLES!!! */

    inside = FALSE;
    vector1 = _MATH_LIB_3D_vector_product(visible_vertex_direction,
                                          integral_directions[check_triangle].v2);
    if (vector1.x==0.0 && vector1.y==0.0 && vector1.z==0.0) {
      vector1 = integral_directions[check_triangle].v2;
      vector2 = visible_vertex_direction;
      if (vector1.x*vector2.x + vector1.y*vector2.y +
          vector1.z*vector2.z > 0.0) inside=1;
    }
    else {
      vector1 = _MATH_LIB_3D_vector_product(integral_directions[check_triangle].v2,vector1);
      vector2 = _MATH_LIB_3D_vector_product(integral_directions[check_triangle].v3,
				      integral_directions[check_triangle].v2);
      vector2 = _MATH_LIB_3D_vector_product(integral_directions[check_triangle].v2,vector2);
      vector1 = _MATH_LIB_3D_vector_product(vector2,vector1);
      if (vector1.x*integral_directions[check_triangle].v2.x +
          vector1.y*integral_directions[check_triangle].v2.y +
          vector1.z*integral_directions[check_triangle].v2.z >= 0.0) inside=1;
    }
    /* continue if the second 'test' triangle is OK */
    if (inside) {

      /* The third test triangle (marked by 'K'): 

      integral_directions[check_triangle].v2
                                           O 
                                          / \
                                         / N \ 
 integral_directions[check_triangle].v3 O-----O integral_directions[check_triangle].v1
                                         \ K /
                                          \ /
                                           O
                            visible_vertex_direction
	       
      The test triangle 'K' should have the same orientation as the
      triangle 'N'. BE CAREFUL, WE WORK WITH SPHERICAL TRIANGLES,
      NOT WITH ORDINARY PLANE TRIANGLES!!! */

      inside = FALSE;
      vector1 = _MATH_LIB_3D_vector_product(visible_vertex_direction,
                                        integral_directions[check_triangle].v3);
      if (vector1.x==0.0 && vector1.y==0.0 && vector1.z==0.0) {
        vector1 = integral_directions[check_triangle].v3;
        vector2 = visible_vertex_direction;
        if (vector1.x*vector2.x + vector1.y*vector2.y +
          vector1.z*vector2.z > 0.0) inside=1;
      }
      else {
        vector1 = _MATH_LIB_3D_vector_product(integral_directions[check_triangle].v3,vector1);
        vector2 = _MATH_LIB_3D_vector_product(integral_directions[check_triangle].v1,
                                              integral_directions[check_triangle].v3);
        vector2 = _MATH_LIB_3D_vector_product(integral_directions[check_triangle].v3,vector2);
        vector1 = _MATH_LIB_3D_vector_product(vector2,vector1);
      if (vector1.x*integral_directions[check_triangle].v2.x +
          vector1.y*integral_directions[check_triangle].v2.y +
          vector1.z*integral_directions[check_triangle].v2.z >= 0.0) inside=1;
      }
      /* continue if the third 'test' triangle is OK */
      if (inside){
        /* OK, that's it! 'visible_vertex_direction' belongs to 
	   that spherical triangle.*/
	return TRUE;

      } /* end of checking the third 'test' triangle */
      else return FALSE;
    } /* end of checking the second 'test' triangle */
    else return FALSE;
  } /* end of checking the first 'test' triangle */
  else return FALSE;

}

/*********************************************************************************/
/*********************************************************************************/

/* create 3D cloud of points */
int _GOROSTAS_create_3D_cloud (){

  struct _vertex_structure *grid_vertex;
  double local_phi_angle;
  int This_is_empty_space, dust_i,Z_sign,phi_grid_counter;

  /* first find array_counter_total */
  array_counter_total = 0;
  Z_sign = 1; /* Z_sign=+1 ==> +Z half-space
                 Z_sign=-1 ==> -Z half_space */
  do {  
    /* go through the list of vertices */ 
    grid_vertex = start_vertex_list;
    do {
      /* check if this is empty space */
      This_is_empty_space = TRUE;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) 
	  if (grid_vertex->T_dust[dust_i]>0.0)
	    {This_is_empty_space = FALSE; break;}

      /* continue only if this is not the empty space */
      if (!This_is_empty_space) {

      /* loop over the phi angle (in the X-Y plane) */
      for (phi_grid_counter=0; phi_grid_counter<=_GOROSTAS_N_phi_grid; 
           phi_grid_counter++) {
        if (grid_vertex->x == 0.0) phi_grid_counter=_GOROSTAS_N_phi_grid;

        /* find the local angle for making the 3D cloud of points */
        if (grid_vertex->x == 0.0) local_phi_angle = 0.0;
	else {
            local_phi_angle = sqrt(grid_vertex->x*grid_vertex->x +
                                   grid_vertex->z*grid_vertex->z);
            local_phi_angle = 
               phi_grid_counter*(local_phi_angle/grid_vertex->x)*PI/_GOROSTAS_N_phi_grid;
            if (local_phi_angle>PI) break;
	}
	/* finally, this point is part of the 3D cloud */
	array_counter_total++;

      } /* end of the phi-grid loop */
      } /* end of: if (!This_is_empty_space) */
      grid_vertex=grid_vertex->next;
    } while (grid_vertex!=start_vertex_list);
    Z_sign *= -1; /* switch the Z-half-space */ 
  } while (Z_sign<0);

  /* allocate the memory */
  Array_list_of_triangle_ID = ivector(0,array_counter_total-1);
  Array_list_of_vertex_x = vector(0,array_counter_total-1);
  Array_list_of_vertex_y = vector(0,array_counter_total-1);
  Array_list_of_vertex_z = vector(0,array_counter_total-1);
  Cloud_3D_point_x = vector(0,array_counter_total-1);
  Cloud_3D_point_y = vector(0,array_counter_total-1);
  Cloud_3D_point_z = vector(0,array_counter_total-1);
  Cloud_3D_lookup_table = ivector(0,array_counter_total-1);
  /* now, repeat the whole thing, but this time calculate
     the coordinates of each point in the 3D cloud */

  array_counter_total = 0;
  Z_sign = 1; /* Z_sign=+1 ==> +Z half-space
                 Z_sign=-1 ==> -Z half_space */
  do {  
    /* go through the list of vertices */ 
    grid_vertex = start_vertex_list;
    do {
      /* check if this is empty space */
      This_is_empty_space = TRUE;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) 
	  if (grid_vertex->T_dust[dust_i]>0.0)
	    {This_is_empty_space = FALSE; break;}

      /* continue only if this is not the empty space */
      if (!This_is_empty_space) {

      /* loop over the phi angle (in the X-Y plane) */
      for (phi_grid_counter=0; phi_grid_counter<=_GOROSTAS_N_phi_grid; 
           phi_grid_counter++) {
        if (grid_vertex->x == 0.0) phi_grid_counter=_GOROSTAS_N_phi_grid;
        /* find the local angle for making the 3D cloud of points */
	if (grid_vertex->x == 0.0) local_phi_angle = 0.0;
	else {
            local_phi_angle = sqrt(grid_vertex->x*grid_vertex->x +
                                   grid_vertex->z*grid_vertex->z);
            local_phi_angle = 
               phi_grid_counter*(local_phi_angle/grid_vertex->x)*PI/_GOROSTAS_N_phi_grid;
            if (local_phi_angle>PI) break;
	}
	/* finally, this point is part of the 3D cloud */
        Cloud_3D_point_x[array_counter_total] = grid_vertex->x*cos(local_phi_angle);
        Cloud_3D_point_y[array_counter_total] = grid_vertex->x*sin(local_phi_angle);
	if (phi_grid_counter==_GOROSTAS_N_phi_grid) 
                                            Cloud_3D_point_y[array_counter_total]=0.0;
        Cloud_3D_point_z[array_counter_total] = grid_vertex->z*Z_sign;

	array_counter_total++;

      } /* end of the phi-grid loop */
      } /* end of: if (!This_is_empty_space) */
      grid_vertex=grid_vertex->next;
    } while (grid_vertex!=start_vertex_list);
    Z_sign *= -1; /* switch the Z-half-space */ 
  } while (Z_sign<0);


  if (my_MPI_rank==0)
   if (DEBUG.SCREEN_MESSAGE >= 2 )
     printf("total # of 3D points = %d\n",array_counter_total);

  return array_counter_total;
}


/* destroy 3D cloud of points */
int _GOROSTAS_destroy_3D_cloud (){
  free_ivector(Array_list_of_triangle_ID,0,array_counter_total-1);
  free_vector(Array_list_of_vertex_x,0,array_counter_total-1);
  free_vector(Array_list_of_vertex_y,0,array_counter_total-1);
  free_vector(Array_list_of_vertex_z,0,array_counter_total-1);
  free_vector(Cloud_3D_point_x,0,array_counter_total-1);
  free_vector(Cloud_3D_point_y,0,array_counter_total-1);
  free_vector(Cloud_3D_point_z,0,array_counter_total-1);
  free_ivector(Cloud_3D_lookup_table,0,array_counter_total-1);
  array_counter_total = 0;

  return 0;
}
