/* LELUYA - GOROSTAS program block */

/* This function puts log(density) into the density grid */

#include "GOROSTAS.h"

void _GOROSTAS_set_logarithmic_density_at_pixels (int grid_flag) {

  /* grid_flag = 0  create the density grid
     grid_flag = 1  create a new density grid and overwrite the old one
     grid_flag = -1 destroy the density grid 
  */

  int err;                /* error flags */
  int dust_i;             /* counter over the dust components */
  int lambda_k,lambda_kk; /* counter over the lambda grid steps */
  int vertexID;
  struct _density_grid_structure Density_2D_Grid_temp;

  struct _vertex_structure *v;

  if (DEBUG.DENSITY_UPDATE<0) {
    printf("Error: the flag for density grid is off\n");
    return;
  }

  printf("%d: Density grid creation\n",my_MPI_rank);
  
if(grid_flag==-1) {
  if (DEBUG.DENSITY_UPDATE>=0)
  if (Density_2D_Grid.triangle!=NULL) {
    free_ivector(Density_2D_Grid.dust_types,0,Density_2D_Grid.dust-1);
    free_dmatrix(Density_2D_Grid.density,0,Density_2D_Grid.dust-1,
                 0,Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES-1);
    Density_2D_Grid.dust_types=NULL;
    Density_2D_Grid.density=NULL;
    Density_2D_Grid.triangle=NULL;
    Density_2D_Grid.dust=-1;
    Density_2D_Grid.lambda=-1;
    Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES=-1;
  }
}
else if(grid_flag==0) {

  /* find max tau grid */
  lambda_kk=0;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++)
    if (grid_per_lambda[lambda_k].tau > grid_per_lambda[lambda_kk].tau) {
      lambda_kk=lambda_k;
    }

  /* create the density grid */
  Density_2D_Grid.lambda = lambda_kk;
  /* find how many dust types is involved */
  Density_2D_Grid.dust = 0;
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
      if (INPUT.DUST_DENSITY_TYPE[dust_i]==8) /* Flared Disk */
	{Density_2D_Grid.dust++;}

  if (Density_2D_Grid.dust==0) {
    printf("%d: Error: can't create the density grid\n",my_MPI_rank);
    printf("%d: ... because there is no flared disks\n",my_MPI_rank);
    Density_2D_Grid.lambda = -1;
    Density_2D_Grid.triangle =NULL;
    Density_2D_Grid.dust=-1;
    DEBUG.DENSITY_UPDATE = -1;
    return;
  }
    
  /* allocate memory for the density grid */
  Density_2D_Grid.dust_types=ivector(0,Density_2D_Grid.dust-1);
  Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES =
                 grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES;
  Density_2D_Grid.density=dmatrix(0,Density_2D_Grid.dust-1,
                 0,Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES-1);
  /* fill the grid */
  Density_2D_Grid.dust = 0;
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
      if (INPUT.DUST_DENSITY_TYPE[dust_i]==8) { /* Flared Disk */
	Density_2D_Grid.dust_types[Density_2D_Grid.dust] = dust_i;
        v = grid_per_lambda[lambda_kk].start_vertex;
        vertexID = 0;
	do {
           Density_2D_Grid.density[Density_2D_Grid.dust][vertexID] = 
	     Dust_Density (v->x,v->z,dust_i,NULL,&err);
           if (Density_2D_Grid.density[Density_2D_Grid.dust][vertexID]<=0.0) {
            Density_2D_Grid.density[Density_2D_Grid.dust][vertexID] = -1.E30;
            if (v->T_dust[dust_i]>0.0) {
               v->T_dust[dust_i] = -1.0;
               v->T_error[dust_i] = 1.0E30;
	    }
	   }
	  else 
            Density_2D_Grid.density[Density_2D_Grid.dust][vertexID] =
	      log(Density_2D_Grid.density[Density_2D_Grid.dust][vertexID]);
	  v = v->next;
          vertexID++;
	} while (v!=grid_per_lambda[lambda_kk].start_vertex);

        Density_2D_Grid.dust++;
      }

  /* this will activate this grid in _ZELENJAK_Dust_Density :  */
  Density_2D_Grid.triangle = grid_per_lambda[lambda_kk].start_triangle;
}
else if(grid_flag==1) {

  /* create the density grid from the old grid */

  /* find max tau grid */
  lambda_kk=0;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++)
    if (grid_per_lambda[lambda_k].tau > grid_per_lambda[lambda_kk].tau) {
      lambda_kk=lambda_k;
    }

  /* create the density grid */
  Density_2D_Grid_temp.lambda = lambda_kk;
  /* find how many dust types is involved */
  Density_2D_Grid_temp.dust = 0;
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
      if (INPUT.DUST_DENSITY_TYPE[dust_i]==8) /* Flared Disk */
	{Density_2D_Grid_temp.dust++;}

  if (Density_2D_Grid_temp.dust==0) {
    printf("%d: Error: can't create the density grid\n",my_MPI_rank);
    printf("%d: ... because there is no flared disks\n",my_MPI_rank);
    Density_2D_Grid_temp.lambda = -1;
    Density_2D_Grid_temp.triangle =NULL;
    Density_2D_Grid_temp.dust=-1;
    DEBUG.DENSITY_UPDATE = -1;
    return;
  }
    
  /* allocate memory for the density grid */
  Density_2D_Grid_temp.dust_types=ivector(0,Density_2D_Grid_temp.dust-1);
  Density_2D_Grid_temp.TOTAL_NUMBER_OF_VERTICES =
                 grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES;
  Density_2D_Grid_temp.density=dmatrix(0,Density_2D_Grid_temp.dust-1,
                 0,Density_2D_Grid_temp.TOTAL_NUMBER_OF_VERTICES-1);
  /* fill the grid */
  /* (density is caculated from the old grid in Dust_Density() )*/
  Density_2D_Grid_temp.dust = 0;
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
      if (INPUT.DUST_DENSITY_TYPE[dust_i]==8) { /* Flared Disk */
	Density_2D_Grid_temp.dust_types[Density_2D_Grid_temp.dust] = dust_i;
        v = grid_per_lambda[lambda_kk].start_vertex;
        vertexID = 0;
	do {
          if (v->T_dust[dust_i]<=0.0)
            Density_2D_Grid_temp.density[Density_2D_Grid_temp.dust][vertexID] = -1.E30;
	  else 
            Density_2D_Grid_temp.density[Density_2D_Grid_temp.dust][vertexID] =
	      log(Dust_Density (v->x,v->z,dust_i,NULL,&err));
	  v = v->next;
          vertexID++; 
	} while (v!=grid_per_lambda[lambda_kk].start_vertex);

        Density_2D_Grid_temp.dust++;
      }

  /* this will activate this grid in _ZELENJAK_Dust_Density :  */
  Density_2D_Grid_temp.triangle = grid_per_lambda[lambda_kk].start_triangle;
  
  /* destroy the old density grid */
  free_ivector(Density_2D_Grid.dust_types,0,Density_2D_Grid.dust-1);
  free_dmatrix(Density_2D_Grid.density,0,Density_2D_Grid.dust-1,
               0,Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES-1);
  Density_2D_Grid.dust_types=NULL;
  Density_2D_Grid.density=NULL;
  Density_2D_Grid.triangle=NULL;

  /* activate the new grid */
  Density_2D_Grid.dust = Density_2D_Grid_temp.dust;
  Density_2D_Grid.lambda = Density_2D_Grid_temp.lambda;
  Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES =
       Density_2D_Grid_temp.TOTAL_NUMBER_OF_VERTICES;
  Density_2D_Grid.dust_types = Density_2D_Grid_temp.dust_types;
  Density_2D_Grid.density = Density_2D_Grid_temp.density;
  Density_2D_Grid.triangle = Density_2D_Grid_temp.triangle;

}

}
