/* LELUYA - GOROSTAS program block */

/* This function updates the density grid */

#include "GOROSTAS.h"

void _GOROSTAS_update_density (void) {

  int dust_i, dust_ii;             /* counter over the dust components */
  int lambda_k,lambda_kk; /* counter over the lambda grid steps */
  int flag;
  double T0, T1, T2, r, theta, dtheta;
  struct _triangle_structure *t,*nextt;
  struct _vertex_structure *v;
  struct _3DVector point,direction,next_point;
  double integral;
  double param1,param2;

  if (DEBUG.DENSITY_UPDATE!=1) {
    printf("Error: the flag for density update is off\n");
    return;
  }

  /* go over the dust types to see which one to update */

  lambda_kk=Density_2D_Grid.lambda;
  start_vertex_list = grid_per_lambda[lambda_kk].start_vertex;
  TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES;
  TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES = grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;
  start_triangle_list = grid_per_lambda[lambda_kk].start_triangle;
  TOTAL_NUMBER_OF_TRIANGLES = grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_TRIANGLES;

  for (dust_ii=0; dust_ii<Density_2D_Grid.dust; dust_ii++) {
    dust_i = Density_2D_Grid.dust_types[dust_ii];


    if (INPUT.DUST_DENSITY_TYPE[dust_i]==8) { /* Flared Disk */  
      /* loop over vertices */
      v = grid_per_lambda[lambda_kk].start_vertex;
      do {
	if (Density_2D_Grid.density[dust_ii][v->identification_number]> -1.0E29) {

	  /* update this density */
	  r=sqrt(v->x*v->x+v->z*v->z);
          if (r>0.995*Rout) r=0.995*Rout;
          t = NULL;
	  T0 = 0.000001;
	  flag=_ZELENJAK_locate_point_in_grid(r,T0,&t);
          if (t!=NULL) {
            if (t->v1->T_dust[dust_i]>0.0 && t->v2->T_dust[dust_i]>0.0 
                && t->v3->T_dust[dust_i]>0.0)
              T1=_MATH_LIB_plane_interpolation_logZ(t->v1->x,t->v1->z,t->v1->T_dust[dust_i],
                                                  t->v2->x,t->v2->z,t->v2->T_dust[dust_i],
                                                  t->v3->x,t->v3->z,t->v3->T_dust[dust_i],
                                                  r,0.0);
	    else T1 = 0.0;
	  }
	  else T1 = 0.0;
          T0 = T1;
          printf("T0=%lf ", T0);
          theta = 0.0; integral = 0.0;
          point.x = r*cos(theta); point.y = 0.0; point.z = r*sin(theta);
          while (point.z<v->z && T0>0.0) {
            direction.x = -sin(theta); direction.y = 0.0; direction.z = cos(theta);
            dtheta = _ZELENJAK_3Dstep(point,t,direction,10.0,&next_point,&nextt)/r;
            theta += dtheta;
	    /* next point */
            point.x = r*cos(theta); point.y = 0.0; point.z = r*sin(theta);
            if (point.z>v->z) {
              if (r<0.995*Rout) {
                point.x=v->x;  
                point.z=v->z; 
	      }
	      else {
                point.z=v->z;
                point.x=sqrt(r*r - point.z*point.z);
	      }
              dtheta = asin(point.z/r) - (theta-dtheta);
              theta = asin(point.z/r);
	    }
            t = nextt;
            if (! _ZELENJAK_point_in_triangle(point.x,point.z,t))
              flag=_ZELENJAK_locate_point_in_grid(point.x,point.z,&t);
	    if (t!=NULL) {
              if (t->v1->T_dust[dust_i]>0.0 && t->v2->T_dust[dust_i]>0.0 
                  && t->v3->T_dust[dust_i]>0.0)
                T2=_MATH_LIB_plane_interpolation_logZ(t->v1->x,t->v1->z,t->v1->T_dust[dust_i],
                                                      t->v2->x,t->v2->z,t->v2->T_dust[dust_i],
                                                      t->v3->x,t->v3->z,t->v3->T_dust[dust_i],
                                                      point.x,point.z);
	      else T2 = 0.0;
	    }
	    else T2 = 0.0;
            if (T1>0.0 && T2>0.0) integral += (0.5*((theta-dtheta)/T1 + theta/T2)*dtheta); 
	  }

          r=sqrt(v->x*v->x+v->z*v->z);
	  /* calculate the density */
          integral *= (5.323E7 * DEBUG.DENSITY_UPDATE_M * Source_Radius);
          integral /= (DEBUG.DENSITY_UPDATE_R * r);
          if (T0>0.0) integral -= (log(v->T_dust[dust_i]/T0));
          else integral = 1E30;
          /*      parameters:
              param1= p  = Dust_Density_Param[dust_counter].param
              param2=Rho0= Dust_Density_Param[dust_counter].next->next->next->param
          */
          param1 = - Dust_Density_Param[dust_i].param;
          param2 = Dust_Density_Param[dust_i].next->next->next->param;
          
          /* density */
	  printf("(%E,%E) oldD=%E newD=%E\n",v->x,v->z,Density_2D_Grid.density[dust_ii][v->identification_number], log(param2*pow(r,param1)) - integral);
          Density_2D_Grid.density[dust_ii][v->identification_number] = 
               log(param2*pow(r,param1)) - integral;

	}
        v = v->next;
      } while (v!=grid_per_lambda[lambda_kk].start_vertex);
    } /* end of INPUT.DUST_DENSITY_TYPE[dust_i]==8 (Flared disk) */

  } /* end of the loop over dust types */
}
