/* LELUYA - LEPAMARA program block */
/* read the data dumped by _PESJANEK__Dump_Data_STOP2 () */
/* This function is activated by the flag '-c n' (n>=2)*/

/* the list of the dumped variables (in exactly that order!) :

   int     MAX_Dust_Type
   int     INPUT.LAMBDA_TYPE
   int     INPUT.SOURCE_TYPE
   int     INPUT.DUST_TYPE[Max_Dust_Type]
   int     INPUT.DUST_DENSITY_TYPE[Max_Dust_Type]
   char   *INPUT.DUST_DENSITY_TYPE_NAME[Max_Dust_Type]
   int     lambda_Max
   double  Lambda_Grid[lambda_Max]
   double  Tau_Total
   double  Theta_Total_Value
   int     Theta_Total
   double  Lambda_Total_Value
   int     Lambda_Total
   double  Source_Flux[lambda_Max]
   double  Rout
   double  Source_Radius
   double  Source_Teff
   char   *Dust_Component_Name[Max_Dust_Type]
   double  Dust_Sublim_Temp[Max_Dust_Type]
   double  Sigma_Dust_Abs[Max_Dust_Type][lambda_Max]
   double  Sigma_Dust_Sca[Max_Dust_Type][lambda_Max]
   int     Dust_Density_Param.next   --> repeat Dust_Density_Param 
   double  Dust_Density_Param.param  --> while (Dust_Density_Param.next!= NULL)
   double  Eta_Denominator
   double  Psi
*/

#include "LEPAMARA.h"

void _LEPAMARA_Read_Data_CONTINUE2_error ( void );

int _LEPAMARA_Read_Data_CONTINUE2 ( FILE *DUMP_FILE ) {

  int  dust_i, lambda_k, number,counter;
  struct density_parameters *density_param_dump;
  int err_location, err;
  char current_character;

  if (DEBUG.SCREEN_MESSAGE >= 2) printf("READ THE DUMPED DATA!!\n");

  if (DUMP_FILE == NULL ) {
    err_location = 999;
    err = 3;
    _PESJANEK__Error_Message (err_location,err);
    return 1;
  }
  
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  if (!err) err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  if (!err) err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  if (err) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}

  err = fscanf(DUMP_FILE,"%d",&Max_Dust_Type);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) 
       printf("Max_Dust_Type=%d\n",Max_Dust_Type);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  
  err = fscanf(DUMP_FILE,"%d",&INPUT.LAMBDA_TYPE);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) 
       printf("INPUT.LAMBDA_TYPE=%d\n",INPUT.LAMBDA_TYPE);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%d",&INPUT.SOURCE_TYPE);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) 
       printf("INPUT.SOURCE_TYPE=%d\n",INPUT.SOURCE_TYPE);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  /* there is not "empty space" dust-type in the model for now:*/
  INPUT.EMPTY_SPACE = FALSE;

  INPUT.DUST_TYPE=ivector(0,Max_Dust_Type-1);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    err = fscanf(DUMP_FILE,"%d %d",&number,&INPUT.DUST_TYPE[dust_i]);
    if (err!=2) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
    if (INPUT.DUST_TYPE[dust_i]==99999) INPUT.EMPTY_SPACE = TRUE;
    if (DEBUG.SCREEN_MESSAGE >= 3) 
       printf("INPUT.DUST_TYPE[%.3d]=%d\n",dust_i,INPUT.DUST_TYPE[dust_i]);
  }  
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  INPUT.DUST_DENSITY_TYPE=ivector(0,Max_Dust_Type-1);
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    err = fscanf(DUMP_FILE,"%d %d",&number,&INPUT.DUST_DENSITY_TYPE[dust_i]);
    if (err!=2) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
    if (DEBUG.SCREEN_MESSAGE >= 3) 
       printf("INPUT.DUST_DENSITY_TYPE[%.3d]=%d\n",
               dust_i,INPUT.DUST_DENSITY_TYPE[dust_i]);
  }  
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  INPUT.DUST_DENSITY_TYPE_NAME=cmatrix(0,Max_Dust_Type-1,
                                       0,_DUST_DENSITY_TYPE_NAME_SIZE_-1);
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    number = 0;
    current_character=fgetc(DUMP_FILE);;
    while (number < _DUST_DENSITY_TYPE_NAME_SIZE_ && current_character!='\n'){
      if (feof(DUMP_FILE)) 
                        {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
      INPUT.DUST_DENSITY_TYPE_NAME[dust_i][number]=current_character;
      current_character = fgetc(DUMP_FILE);
      number++;
    }
    INPUT.DUST_DENSITY_TYPE_NAME[dust_i][number]='\0';
    if (DEBUG.SCREEN_MESSAGE >= 3) 
      printf("INPUT.DUST_DENSITY_TYPE_NAME[%.3d]=%s\n",
               dust_i,INPUT.DUST_DENSITY_TYPE_NAME[dust_i]);
  }  

  err = fscanf(DUMP_FILE,"%d",&lambda_Max);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) printf("lambda_Max=%d\n",lambda_Max);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  Lambda_Grid=dvector(0,lambda_Max-1);
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++) {
     err = fscanf(DUMP_FILE,"%d %lf",&number,&Lambda_Grid[lambda_k]);
     if (err!=2) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
     if (DEBUG.SCREEN_MESSAGE >= 3) printf("Lambda_Grid[%.3d]=%lf\n",
               lambda_k,Lambda_Grid[lambda_k]);
  }
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%lf",&Tau_Total);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) printf("Tau_Total=%E\n",Tau_Total);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%lf",&Theta_Total_Value);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) 
    printf("Theta_Total_Value=%E\n",Theta_Total_Value);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%d",&Theta_Total);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) printf("Theta_Total=%d\n",Theta_Total);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%lf",&Lambda_Total_Value);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) 
    printf("Lambda_Total_Value=%E\n",Lambda_Total_Value);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%d",&Lambda_Total);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) printf("lambda_Total=%d\n",Lambda_Total);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  Source_Flux=dvector (0,lambda_Max-1);
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++) {
     err = fscanf(DUMP_FILE,"%d %lf",&number,&Source_Flux[lambda_k]);
     if (err!=2) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
     if (DEBUG.SCREEN_MESSAGE >= 3) printf("Source_Flux[%.3d]=%E\n",
               lambda_k,Source_Flux[lambda_k]);
  }
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%lf",&Rout);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) printf("Rout=%E\n",Rout);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%lf",&Source_Radius);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) printf("Source_Radius=%E\n",Source_Radius);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%lf",&Source_Teff);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) printf("Source_Teff=%E\n",Source_Teff);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  Dust_Component_Name=cmatrix(0,Max_Dust_Type-1,
                              0,_DUST_COMPONENT_NAME_SIZE_-1);
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    err = _PATULJAK__Search_for_char (DUMP_FILE,"=");
    if (err) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
    number = 0;
    current_character=fgetc(DUMP_FILE);;
    while (number < _DUST_COMPONENT_NAME_SIZE_ && current_character!='\n'){
      if (feof(DUMP_FILE)) 
                        {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
      Dust_Component_Name[dust_i][number]=current_character;
      current_character = fgetc(DUMP_FILE);
      number++;
    }
    Dust_Component_Name[dust_i][number]='\0';
    if (DEBUG.SCREEN_MESSAGE >= 3) printf("Dust_Component_Name[%.3d]=%s\n",
                                          dust_i,Dust_Component_Name[dust_i]);
  }  

  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  Dust_Sublim_Temp=dvector(0,Max_Dust_Type-1);
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    err = fscanf(DUMP_FILE,"%d %lf",&number,&Dust_Sublim_Temp[dust_i]);
    if (err!=2) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
    if (DEBUG.SCREEN_MESSAGE >= 3) printf("Dust_Sublim_Temp[%.3d]=%E\n",
                                           dust_i,Dust_Sublim_Temp[dust_i]);
    if (Max_Dust_Temp>0.0) if (Max_Dust_Temp<Dust_Sublim_Temp[dust_i])
      Max_Dust_Temp = Dust_Sublim_Temp[dust_i];
  }  
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  Sigma_Dust_Abs=dmatrix(0,Max_Dust_Type-1,0,lambda_Max-1);
  Sigma_Dust_Sca=dmatrix(0,Max_Dust_Type-1,0,lambda_Max-1);
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
    if (err) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}  
    for (lambda_k=0; lambda_k<lambda_Max; lambda_k++) {
      err = fscanf(DUMP_FILE,"%d %lf %lf",&number,
                   &Sigma_Dust_Abs[dust_i][lambda_k],
                   &Sigma_Dust_Sca[dust_i][lambda_k]);
      if (err!=3) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
      if (DEBUG.SCREEN_MESSAGE >= 3) 
         printf("Sigma_Dust[%.3d][%d]: Abs=%E  Sca=%E\n",dust_i,lambda_k,
                Sigma_Dust_Abs[dust_i][lambda_k],
                Sigma_Dust_Sca[dust_i][lambda_k]);
    }
    err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  }

  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  Dust_Density_Param=density_parameters_vector(0,Max_Dust_Type-1);
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    density_param_dump = &Dust_Density_Param[dust_i];
    counter = 0;
    do {
      err = fscanf(DUMP_FILE,"%d",&number);
      if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
      err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
      err = fscanf(DUMP_FILE,"%lf",&(density_param_dump->param));
      if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
      err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
      if (DEBUG.SCREEN_MESSAGE >= 3) {
        printf("Dust_Density_Param[%.3d][_%.4d_]",dust_i,counter);
        printf(".param=%E\n",density_param_dump->param);
      }
      if (number>0) {
        density_param_dump->next = (struct density_parameters *) 
                                  malloc (sizeof(struct density_parameters));
        density_param_dump=density_param_dump->next;
      }
      else density_param_dump->next = NULL;
      counter++;
    } while (number!=0);
  }
  
  err = fscanf(DUMP_FILE,"%lf",&Eta_Denominator);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE>=3) printf("Eta_Denominator=%E\n",Eta_Denominator);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%lf",&Psi);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) printf("Psi=%E\n",Psi);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  err = fscanf(DUMP_FILE,"%lf",&CELL_SIZE_LIMIT);
  if (err!=1) {_LEPAMARA_Read_Data_CONTINUE2_error();return 1;}
  if (DEBUG.SCREEN_MESSAGE >= 3) 
    printf("CELL_SIZE_LIMIT=%E\n",CELL_SIZE_LIMIT);
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");

  if (my_MPI_rank==0) 
      printf("LEPAMARA: CELL SIZE LIMIT = %lf\n",CELL_SIZE_LIMIT);

  if (DEBUG.SCREEN_MESSAGE >= 3)
    printf("-------------------------------------------------\n");
  return 0;

}


void _LEPAMARA_Read_Data_CONTINUE2_error ( void ) {

    int err_location = 999;
    int err = 4;
    _PESJANEK__Error_Message (err_location,err);
}

