/* LELUYA - LEPAMARA program block */
/* read the grids dumped by _PESJANEK__Dump_Data_STOP202 () */
/* This function is activated by the flag '-c 202' */

#include "LEPAMARA.h"

int _LEPAMARA_Read_Data_CONTINUE202 (FILE *DUMP_FILE) {

  int lambda_k,dust_i,err; 
  int total_number_of_lambdas;
  struct _vertex_structure   *loop_vertex;
  struct _vertex_triangles   *new_loop_vertex_triangle,*loop_vertex_triangle;
  struct _triangle_structure *loop_triangle;
  int vertex_ID, NUMBER_OF_VERTICES;
  int triangle_ID, NUMBER_OF_TRIANGLES;
  int v1,v2,v3,t;


  if (DEBUG.SCREEN_MESSAGE >= 2) printf("READ THE DUMPED GRIDS!!\n");

  /* create the list of X-Z grids (one X-Z grid per wavelength) */
  grid_per_lambda = grid_per_lambda_vector(0,lambda_Max-1);
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++) {
    grid_per_lambda[lambda_k].lambda = Lambda_Grid[lambda_k];
    grid_per_lambda[lambda_k].tau=0.0;
    grid_per_lambda[lambda_k].Ustar = NULL; /* stellar energy density */
    grid_per_lambda[lambda_k].Usca  = NULL; /* dust scattering energy */
    grid_per_lambda[lambda_k].Uem   = NULL; /* dust emission energy */
    grid_per_lambda[lambda_k].Utot_new = NULL;/* updated total energy */
    grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES = 0;
    grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES = 0;
    grid_per_lambda[lambda_k].start_vertex=NULL;
    grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_TRIANGLES=0;
    grid_per_lambda[lambda_k].start_triangle=NULL;
  }

  /* skip the first three lines of comments */
  err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  if (!err) err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  if (!err) err = _PATULJAK__Search_for_char (DUMP_FILE,"\n");
  if (err) return 1;

  /* in the beginning there is no grids */ 
  total_number_of_lambdas = 0;

  /* read until all lambdas have their grid */
  while (total_number_of_lambdas < lambda_Max) {

    /* read the vertex list */

    /* get the total number of vertex points */
    err = fscanf(DUMP_FILE,"%d",&NUMBER_OF_VERTICES);
    if (err!=1) return 1;

    TOTAL_NUMBER_OF_VERTICES = 0; /* this variable is used in PERUNIKA block */
    /* in the end, it should be TOTAL_NUMBER_OF_VERTICES=NUMBER_OF_VERTICES */

    start_vertex_list = NULL;
    for(vertex_ID=0; vertex_ID<NUMBER_OF_VERTICES; vertex_ID++){

     /* create a new vertex */
      loop_vertex = _PERUNIKA_create_vertex (&err); 
      loop_vertex->identification_number = vertex_ID;

      err = fscanf(DUMP_FILE,"%d %d %lf %lf",&(loop_vertex->level),
		   &(loop_vertex->type),&(loop_vertex->x),&(loop_vertex->z));
      if (err!=4) return 1;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++){
	err = fscanf(DUMP_FILE,"%lf %lf",&(loop_vertex->T_dust[dust_i]),
		     &(loop_vertex->T_error[dust_i]));
        if (err!=2) return 1;
         /* if (dust_i==1 && loop_vertex->T_dust[dust_i]<0.0 && loop_vertex->T_dust[0]>0.0 )
            loop_vertex->T_dust[dust_i]=loop_vertex->T_dust[0]; */
	/*//////////////////////////////////////////////////////
        ////////////// transforming empty space into dust //////
	if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
	    loop_vertex->T_dust[dust_i]<0.0  &&
            loop_vertex->z/loop_vertex->x>0.1188)
	  loop_vertex->T_dust[dust_i] *= -1;
	/////////////////////////////////////////////////////*/
      } 
      err = fscanf(DUMP_FILE,"%lf",&(loop_vertex->Eta));
      if (err!=1) return 1;
    }

    /* read the list of triangles */

    /* get the total number of vertex points */
    err = fscanf(DUMP_FILE,"%d",&NUMBER_OF_TRIANGLES);
    if (err!=1) return 1;

    TOTAL_NUMBER_OF_TRIANGLES = 0; /* this variable is used in PERUNIKA block */
    /* in the end,it should be TOTAL_NUMBER_OF_TRIANGLES=NUMBER_OF_TRIANGLES */

    start_triangle_list = NULL;
    for(triangle_ID=0; triangle_ID<NUMBER_OF_TRIANGLES; triangle_ID++){

      /* create a new triangle */
      loop_triangle = _PERUNIKA_create_triangle (&err); 
      loop_triangle->ID_number = triangle_ID;

      /* read the ID numbers of the triangle's vertices and find them*/
      err = fscanf(DUMP_FILE,"%d %d %d",&v1,&v2,&v3);
      if (err!=3) return 1;
      loop_vertex = start_vertex_list;
      do {
	if (v1==loop_vertex->identification_number) 
	  loop_triangle->v1 = loop_vertex;
	if (v2==loop_vertex->identification_number) 
	  loop_triangle->v2 = loop_vertex;
	if (v3==loop_vertex->identification_number) 
	  loop_triangle->v3 = loop_vertex;
	loop_vertex = loop_vertex->next;
      } while (loop_vertex != start_vertex_list && (loop_triangle->v1==NULL 
                      || loop_triangle->v2==NULL || loop_triangle->v3==NULL));
      if (loop_triangle->v1==NULL) return 1;
      if (loop_triangle->v2==NULL) return 1;
      if (loop_triangle->v3==NULL) return 1;

      err = fscanf(DUMP_FILE,"%d %lf %lf %lf %lf %lf",
		   &(loop_triangle->level),
		   &(loop_triangle->bound_box_down_left_x),
		   &(loop_triangle->bound_box_down_left_z),
		   &(loop_triangle->bound_box_up_right_x),
		   &(loop_triangle->bound_box_up_right_z),
		   &(loop_triangle->integral_step));
      if (err!=6) return 1;

    } /* end of reading triangles */

    /* read the list of triangles at each vertex */
    loop_vertex = start_vertex_list;
    do {

      loop_vertex_triangle=NULL;
      do {
	err = fscanf(DUMP_FILE,"%d",&t);
	if (err!=1) return 1;
	if (t>=0 && t<NUMBER_OF_TRIANGLES) {
	 new_loop_vertex_triangle = (struct _vertex_triangles *)malloc
	                            (sizeof(struct _vertex_triangles));
	 new_loop_vertex_triangle->next=NULL;
	 if (loop_vertex_triangle==NULL)
	   loop_vertex->triangles = new_loop_vertex_triangle;
	 else loop_vertex_triangle->next = new_loop_vertex_triangle;
	 loop_vertex_triangle = new_loop_vertex_triangle;
	 /* find that triangle in the triangle grid list */ 
	 loop_triangle = start_triangle_list;
	 while (t!=loop_triangle->ID_number) loop_triangle=loop_triangle->next;
	 /* create a link to the triangle */
	 new_loop_vertex_triangle->triangle = loop_triangle;

	 err = fscanf(DUMP_FILE,"%lf",&(new_loop_vertex_triangle->nu));
	 if (err!=1) return 1;
	}

      } while (t >= 0 && t<NUMBER_OF_TRIANGLES);

      loop_vertex = loop_vertex->next;
    } while (loop_vertex != start_vertex_list);

    /* find how many vertices are not empty space */
    TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES = 0;
    loop_vertex=start_vertex_list;
    do {
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	  if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
              loop_vertex->T_dust[dust_i]>0.0)
	    {TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES++;break;}
	loop_vertex = loop_vertex->next;
    } while (loop_vertex!=start_vertex_list); 


    /* read the list of wavelengths and energy densities */
    do {
      err = fscanf(DUMP_FILE,"%d",&lambda_k);
      if (err!=1) return 1;
      if (lambda_k >=0 && lambda_k < lambda_Max) {
	total_number_of_lambdas++;/* this shows how many of them we have read*/
	err = fscanf(DUMP_FILE,"%lf",&(grid_per_lambda[lambda_k].tau));
	if (err!=1) return 1;

	grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES =
                                                     TOTAL_NUMBER_OF_VERTICES;
	grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES =
                                           TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;
	grid_per_lambda[lambda_k].start_vertex=start_vertex_list;
	grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_TRIANGLES =
	                                             TOTAL_NUMBER_OF_TRIANGLES;
	grid_per_lambda[lambda_k].start_triangle=start_triangle_list;
	/* create the energy density arrays */ 
	grid_per_lambda[lambda_k].Ustar=dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
        grid_per_lambda[lambda_k].Usca =dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
        grid_per_lambda[lambda_k].Uem  =dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
        grid_per_lambda[lambda_k].Utot_new
                                       =dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
	/* read the energy densities */
	for(vertex_ID=0; vertex_ID<NUMBER_OF_VERTICES; vertex_ID++){
	  err = fscanf(DUMP_FILE,"%lf %lf %lf %lf",
		       &(grid_per_lambda[lambda_k].Ustar[vertex_ID]),
		       &(grid_per_lambda[lambda_k].Usca[vertex_ID]),
		       &(grid_per_lambda[lambda_k].Uem[vertex_ID]),
		       &(grid_per_lambda[lambda_k].Utot_new[vertex_ID]));
	  if (err!=4) return 1;
	}
      }
    } while (lambda_k >=0 && lambda_k < lambda_Max);

  } /* end of reading while (total_number_of_lambdas < lambda_Max) */

  /* read the type of grid */
  err = fscanf(DUMP_FILE,"%d",&DEBUG.AZIMUTHAL_GRID);
  if (err!=1) {printf("Error in reading the grid type\n"); return 1;}
  if (DEBUG.AZIMUTHAL_GRID==1 || DEBUG.AZIMUTHAL_GRID==2 || DEBUG.AZIMUTHAL_GRID==3) {
    err = fscanf(DUMP_FILE,"%lf",&DEBUG.AZIMUTHAL_GRID1);
    if (err!=1) {printf("Error in reading the grid type (min. angle)\n"); return 1;}
  }

  /* read the vertical hydrostatic equilibrium flag */
  err = fscanf(DUMP_FILE,"%d",&DEBUG.INPUT_DENSITY_UPDATE);
  if (err!=1) {printf("Error in reading the density update flag\n"); return 1;}
  err = fscanf(DUMP_FILE,"%lf",&DEBUG.INPUT_DENSITY_UPDATE_M);
  if (err!=1) {printf("Error in reading the density update flag\n"); return 1;}
  err = fscanf(DUMP_FILE,"%lf",&DEBUG.INPUT_DENSITY_UPDATE_R);
  if (err!=1) {printf("Error in reading the density update flag\n"); return 1;}
  
  if (DEBUG.INPUT_DENSITY_UPDATE==1 && DEBUG.DENSITY_UPDATE<1) {
    DEBUG.DENSITY_UPDATE = DEBUG.INPUT_DENSITY_UPDATE;
    DEBUG.DENSITY_UPDATE_M = DEBUG.INPUT_DENSITY_UPDATE_M;
    DEBUG.DENSITY_UPDATE_R = DEBUG.INPUT_DENSITY_UPDATE_R;
    if (my_MPI_rank==0) {
      printf("WARNING: Density update flag set to 1\n");
      printf("         M=%lf, R=%lf\n",DEBUG.DENSITY_UPDATE_M,DEBUG.DENSITY_UPDATE_R);
    }
  }
  else if (DEBUG.INPUT_DENSITY_UPDATE==0 && DEBUG.DENSITY_UPDATE<0) {
    DEBUG.DENSITY_UPDATE = DEBUG.INPUT_DENSITY_UPDATE;
    if (my_MPI_rank==0)
      printf("WARNING: the grid is using ln(density) interpolation\n");
  }

  if (DEBUG.DENSITY_UPDATE>=0 && DEBUG.INPUT_DENSITY_UPDATE>=0) {
     err = fscanf(DUMP_FILE,"%d %d %d",&Density_2D_Grid.lambda,
                                       &Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES,
                                       &Density_2D_Grid.dust);
     if (err!=3) {printf("Error in reading the density grid\n"); return 1;}

     Density_2D_Grid.dust_types=ivector(0,Density_2D_Grid.dust-1);
     for (dust_i=0; dust_i<Density_2D_Grid.dust; dust_i++)
       err = fscanf(DUMP_FILE,"%d",&Density_2D_Grid.dust_types[dust_i]);

     Density_2D_Grid.density=dmatrix(0,Density_2D_Grid.dust-1,
                 0,Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES-1);

     for (dust_i=0; dust_i<Density_2D_Grid.dust; dust_i++)
       for (lambda_k=0; lambda_k<Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES; lambda_k++){
	 err = fscanf(DUMP_FILE,"%lf",&Density_2D_Grid.density[dust_i][lambda_k]);
         if (err!=1) {printf("Error in reading the density grid\n"); return 1;}
       }
  
     Density_2D_Grid.triangle = grid_per_lambda[Density_2D_Grid.lambda].start_triangle;

  }

  /* check if the density update is possible */
  if (DEBUG.DENSITY_UPDATE == 1) {
    if (DEBUG.AZIMUTHAL_GRID<0) {
      printf("##############\n");
      printf("ERROR!!! -v flag activated without -Z (azimuthal grid) !!\n");
      printf("##############\n");
      return 1;
    }
  }

  return 0;

}



