/* LELUYA - MATH_LIB program block */
/* normalized Planck function:

    Planck(T,lambda)= PI*Planck/(sigma*T^4) = A/(T^4*lambda^5*(e^x-1))
  
  where
         x = hc/(lambda*k*T) = 14404/lambda/T
         A = 2*PI*h*c^2/sigma *(10^6)^4 / (1000K^4) = 6597.
         [lambda] = microns
         [T] = Kelvins
  so that:
          Int(Planck dlambda) = 1

*/

#include "MATH_LIB.h"

double _MATH_LIB_Planck (double T, double lambda){

  double b;

  T /= 1000.;
  b = 14.40435/(lambda*T);
  if (b>100.0) {return 0.0;}
  if (b<0.00001) {
    b = 6597. /(b * T*T*T*T * lambda*lambda*lambda*lambda*lambda);
    return b;
  }
  b = 1.0 / (exp(b) - 1.0);
  b = b* 6597. /(T*T*T*T * lambda*lambda*lambda*lambda*lambda);
  return b;
}
