/* LELUYA - MATH_LIB program block */
/* this function finds a root of the _GOROSTAS_Equilibrium function 
   (radiative equilibrium) by using the Ridders method (see Num.Rec.)*/
/* It is called from the _GOROSTAS_Find_Temperature function ! */

#include "MATH_LIB.h"

double _MATH_LIB_Ridder_Root(
                   double (*equilibrium)(double, double, double, int, double),
                   double x, double z, int dust_component, double Function2,
                   double T1, double T2, double T_acc, int *err){
  /* equilibrium   = function which calculates deviation from the radiative
                     equilibrium
     x,z           = coordinates of the point
     dust_component = the dust component used in this calculation
     Function2     = the second part of the radiative equilibrium function
                     used in 'equilibrium'
     T1,T2         = temperatures between which the solution should exist
     T_acc         = accuracy of the solution
     err           = error flag
   */

#define MAXIT 60
#define UNUSED (-1.11E30)

   int j;
   double ans,fh,fl,fm,fnew,s,Th,Tl,Tm,Tnew;

   *err = 0;
   fl = (*equilibrium)(T1,x,z,dust_component,Function2);
   fh = (*equilibrium)(T2,x,z,dust_component,Function2);
   if ((fl>0.0 && fh<0.0) || (fl<0.0 && fh>0.0)) {
     Tl=T1;
     Th=T2;
     ans = UNUSED;
     for (j=1;j<=MAXIT;j++) {
       Tm=0.5*(Tl+Th);
       fm=(*equilibrium)(Tm,x,z,dust_component,Function2);
       s=sqrt(fm*fm-fl*fh);
       if (s == 0.0) return ans;

       Tnew=Tm+(Tm-Tl)*((fl>=fh ? 1.0 : -1.0)*fm/s);
       if (fabs(Tnew-ans)<=T_acc) return ans;
       ans=Tnew;
       fnew=(*equilibrium)(ans,x,z,dust_component,Function2);
       if (fnew==0.0) return ans;
       if (SIGN(fm,fnew) != fm) {
         Tl=Tm;
         fl=fm;
         Th=ans;
         fh=fnew;
       } else if (SIGN(fl,fnew) != fl) {
         Th=ans;
         fh=fnew;
       } else if (SIGN(fh,fnew) != fh) {
         Tl=ans;
         fl=fnew;
       } else {*err=1; return 0.0;}
       if (fabs(Th-Tl) <= T_acc) return ans;
     }
     *err=1;
     return 0.0;
   }
   else {
     if (fl == 0.0) return T1;
     if (fh == 0.0) return T2;
     *err=1;
     return 0.0;
   }
   /* never get here **********
   *err=1;
   return 0.0;
   ***************************/
} /* and of the _MATH_LIB_Ridder_Root function */
