/* LELUYA - MATH_LIB program block */

/* do the lines intersect? This function gives the answer */

#include "MATH_LIB.h"

int _MATH_LIB_intersecting_lines(double Ax,double Ay,double Bx,double By,
                             double Cx,double Cy,double Dx,double Dy){
  /* here is the geometry:

              LineCD_left   LineAB_right        a bounding box
                 (Cx,Cy)   (Bx,By)             ----------------UR
                      \     /                  |               |
                       \   /                   |               |
                        \ /                    |               |
                         X                     |               |
                        / \                    |               |
                       /   \                   |               |
                      /     \                  LL---------------
                 (Ax,Ay)   (Dx,Dy)
              LineAB_left   LineCD_right
  */
  /* The lines AB and CD are defined as: */
  struct __vector__ {double x,y;} LineAB_left, LineAB_right,
                                      LineCD_left, LineCD_right; 
  /* and the lines look like this :   0=ax+by+c  */
  double lineABa, lineABb, lineABc, lineCDa, lineCDb, lineCDc;
  /* the lines AB and CD have their bouding box: */
  struct __bounding_box__ {struct __vector__ LL,UR;} bboxAB, bboxCD;
  /* a denominator in the intersection matrices */
  double denum;
  /* coordinates of the intersection point */
  double Inter_x, Inter_y; 

  /* find the left point on the LineAB */
  if (Ax < Bx) {
    LineAB_left.x = Ax; LineAB_left.y = Ay;
    LineAB_right.x= Bx; LineAB_right.y= By;
  }
  else{
    LineAB_left.x = Bx; LineAB_left.y = By;
    LineAB_right.x= Ax; LineAB_right.y= Ay;
  }
  /* find the left point on the LineCD */
  if (Cx < Dx) {
    LineCD_left.x = Cx; LineCD_left.y = Cy;
    LineCD_right.x= Dx; LineCD_right.y= Dy;
  }
  else{
    LineCD_left.x = Dx; LineCD_left.y = Dy;
    LineCD_right.x= Cx; LineCD_right.y= Cy;
  }

  /* find the bounding box for the LineAB */
  bboxAB.LL.x = LineAB_left.x;
  bboxAB.UR.x = LineAB_right.x;
  if (LineAB_left.y < LineAB_right.y) {
    bboxAB.LL.y = LineAB_left.y;
    bboxAB.UR.y = LineAB_right.y;
  }
  else {
    bboxAB.LL.y = LineAB_right.y;
    bboxAB.UR.y = LineAB_left.y;
  }
  /* find the bounding box for the LineCD */
  bboxCD.LL.x = LineCD_left.x;
  bboxCD.UR.x = LineCD_right.x;
  if (LineCD_left.y < LineCD_right.y) {
    bboxCD.LL.y = LineCD_left.y;
    bboxCD.UR.y = LineCD_right.y;
  }
  else {
    bboxCD.LL.y = LineCD_right.y;
    bboxCD.UR.y = LineCD_left.y;
  }

  /* check if the bounding boxes are overlaying */
  if (bboxAB.LL.x > bboxCD.UR.x || bboxAB.LL.y > bboxCD.UR.y ||
      bboxCD.LL.x > bboxAB.UR.x || bboxCD.LL.y > bboxAB.UR.y) return 0;

  /* find the equations of the lines: */
  /* line AB : */ 
  if (LineAB_left.x == LineAB_right.x) {
    lineABa=1.0; lineABb=0.0; lineABc = - LineAB_left.x; 
  }
  else if (LineAB_left.y == LineAB_right.y) {
    lineABa=0.0; lineABb=1.0; lineABc = - LineAB_left.y; 
  }
  else {
    lineABa=1.0; 
    lineABb=-(LineAB_left.x-LineAB_right.x)/(LineAB_left.y-LineAB_right.y);
    lineABc= - LineAB_left.x - lineABb*LineAB_left.y; 
  }
  /* line CD : */ 
  if (LineCD_left.x == LineCD_right.x) {
    lineCDa=1.0; lineCDb=0.0; lineCDc = - LineCD_left.x; 
  }
  else if (LineCD_left.y == LineCD_right.y) {
    lineCDa=0.0; lineCDb=1.0; lineCDc = - LineCD_left.y; 
  }
  else {
    lineCDa=1.0; 
    lineCDb=-(LineCD_left.x-LineCD_right.x)/(LineCD_left.y-LineCD_right.y);
    lineCDc= - LineCD_left.x - lineCDb*LineCD_left.y; 
  }

  /* check if the lines are parallel */
  denum = lineABa*lineCDb-lineCDa*lineABb;
  if (denum == 0.0) return 0;
  else {
    /* find the intersection point */
    Inter_x = (lineABb*lineCDc - lineCDb*lineABc) / denum;
    Inter_y = (lineABc*lineCDa - lineCDc*lineABa) / denum;
    /* check if theintersection point is on the lines */
    if (Inter_x >= bboxAB.LL.x && Inter_x <= bboxAB.UR.x &&
        Inter_y >= bboxAB.LL.y && Inter_y <= bboxAB.UR.y &&
        Inter_x >= bboxCD.LL.x && Inter_x <= bboxCD.UR.x &&
        Inter_y >= bboxCD.LL.y && Inter_y <= bboxCD.UR.y) 
      return 1;
    else 
      return 0;
  }

}
