/* LELUYA - MATH_LIB program block */
/* this function returns the linear plane interpolation from three given
   points in a 3D space */

#include "MATH_LIB.h"

double _MATH_LIB_plane_interpolation (double x1, double y1, double z1,
				      double x2, double y2, double z2,
				      double x3, double y3, double z3,
				      double x , double y){
  /* given points: (x1,y1,z1), (x2,y2,z2), (x3,y3,z3)
     given x and y of an unknown point

     The plane is given by:   |x-x1  y-y1  z-z1 |
                              |x2-x1 y2-y1 z2-z1|=0
                              |x3-x1 y3-y1 z3-z1|

     The unkown z-cooridnate is:

                   |z2-z1 y2-y1|         |x2-x1 z2-z1|
             (x-x1)|z3-z1 y3-y1| + (y-y1)|x3-x1 z3-z1|
    z = z1 + ------------------------------------------
                           |x2-x1 y2-y1|
                           |x3-x1 y3-y1|
  */

  double z, denominator;

  denominator = (x2-x1)*(y3-y1)-(y2-y1)*(x3-x1);
  if (denominator==0.0) z=0.0;
  else {
    z  = (x-x1)*((z2-z1)*(y3-y1)-(y2-y1)*(z3-z1));
    z += (y-y1)*((x2-x1)*(z3-z1)-(z2-z1)*(x3-x1));
    z /= denominator;
    z += z1;
  }

  return z;

}

