/* ran2 */
#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

/* bessj */
#define ACC 40.0
#define BIGNO 1.0e10
#define BIGNI 1.0e-10

/* indexx */
#define SWAP(a,b) itemp=(a);(a)=(b);(b)=itemp;
#define M 7
#define NSTACK 50

/* ludcmp */
#define TINY 1.0E-20

#include "nrutil.h"

#include <math.h>
#include <stdio.h>
#include <time.h> 
#include <string.h>

void indexx(long n,double arr[],long indx[])
{
   long i,indxt,ir = n, itemp, j,k,l=1;
   int jstack=0,*istack;
   double a;

   istack=ivector(1,NSTACK);
   for (j=1;j<=n;j++) indx[j]=j;
   for (;;) {
      if (ir-l<M) {
	 for (j=l+1;j<=ir;j++) {
	    indxt=indx[j];
	    a=arr[indxt];
	    for (i=j-1;i>=l;i--) {
		if (arr[indx[i]] <= a) break;
		indx[i+1]=indx[i];
	    }
	    indx[i+1]=indxt;
	 }
	 if (jstack == 0) break;
	 ir = istack[jstack--];
	 l=istack[jstack--];
      } else {
	 k=(l+ir) >> 1;
	 SWAP(indx[k],indx[l+1]);
	 if (arr[indx[l]]>arr[indx[ir]]) {
	    SWAP(indx[l],indx[ir])
	 }
	 if (arr[indx[l+1]]>arr[indx[ir]]) {
	    SWAP (indx[l+1],indx[ir])
	 }if (arr[indx[l]]>arr[indx[l+1]]) {
	    SWAP(indx[l],indx[l+1])
	 }
	 i=l+1;
	 j=ir;
	 indxt=indx[l+1];
	 a=arr[indxt];
	 for (;;) {
	    do i++; while (arr[indx[i]]<a);
	    do j--; while (arr[indx[j]]>a);
	    if (j<i) break;
	    SWAP (indx[i],indx[j]);
	 }
	 indx[l+1]=indx[j];
	 indx[j]=indxt;
	 jstack += 2;
	 if (jstack>NSTACK) nrerror("NSTACK too small in indexx.");
	 if (ir-i+1 >= j-l) {
	    istack[jstack]=ir;
	    istack[jstack-1]=i;
	    ir=j-1;
	 } else {
	    istack[jstack]=j-1;
	    istack[jstack-1]=l;
	    l=i;
	 }
      }
   }
   free_ivector(istack,1,NSTACK);
}



double ran2(long *idum)
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0;
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum)<1) *idum=1;
      else *idum = -(*idum);
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
	 k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum<0) *idum += IM1;
	 if (j<NTAB) iv[j] = *idum;
      }
      iy=iv[0];
   }
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum<0) *idum += IM1;
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2<0) idum2 += IM2;
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy<1) iy += IMM1;
   if ((temp=AM*iy) > RNMX) return RNMX;
   else return temp;
}



void spline(float x[], float y[], int n, float yp1, float ypn, float y2[])

{
   int i,k;
   float p,qn,sig,un,*u;

   u=vector(1,n-1);
   if (yp1 > 0.99e30)
      y2[1]=u[1]=0.0;
   else {
      y2[1] = -0.5;
      u[1] = (3.0/(x[2]-x[1]))*((y[2]-y[1])/(x[2]-x[1])-yp1);
   }
   for (i=2;i<=n-1;i++) {
      sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
      p=sig*y2[i-1]+2.0;
      y2[i]=(sig-1.0)/p;
      u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
      u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
   }
   if (ypn > 0.99e30)
      qn=un=0.0;
   else {
      qn=0.5;
      un=(3.0/(x[n]-x[n-1]))*(ypn-(y[n]-y[n-1])/(x[n]-x[n-1]));
   }
   y2[n]=(un-qn*u[n-1])/(qn*y2[n-1]+1.0);
   for (k=n-1;k>=1;k--)
      y2[k]=y2[k]*y2[k+1]+u[k];
   free_vector(u,1,n-1);
}


void splint(float xa[], float ya[], float y2a[], int n, float x, float *y)
{
   void nrerror(char error_text[]);
   int klo,khi,k;
   float h,b,a;

   klo=1;
   khi=n;
   while (khi-klo > 1) {
      k=(khi+klo) >> 1;
      if (xa[k] > x) khi=k;
      else klo=k;
   }
   h=xa[khi]-xa[klo];
   if (h == 0.0) nrerror("Bad xa input to routine SPLINT");
   a=(xa[khi]-x)/h;
   b=(x-xa[klo])/h;
   *y=a*ya[klo]+b*ya[khi]+((a*a*a-a)*y2a[klo]+(b*b*b-b)*y2a[khi])*(h*h)/6.0;
}


void splie2(float x1a[], float x2a[], float **ya, int m, int n, float **y2a)
{
   void spline(float x[], float y[], int n, float yp1, float ypn, float y2[]);
   int j;
   
   for (j=1;j<=m;j++)
     spline (x2a,ya[j],n,1.0e30,1.0e30,y2a[j]);
}


void splin2(float x1a[], float x2a[], float **ya, float **y2a, int m, int n, float x1, float x2, float *y)
{
  void spline(float x[], float y[], int n, float yp1, float ypn, float y2[]);
  void splint(float xa[], float ya[], float y2a[], int n, float x, float *y);
  int j;
  float *ytmp,*yytmp;
  
  ytmp=vector(1,m);
  yytmp=vector(1,m);
  for (j=1;j<=m;j++)
    splint(x2a, ya[j], y2a[j], n, x2, &yytmp[j]);
  spline(x1a,yytmp,m,1.0e30,1.0e30,ytmp);
  splint(x1a,yytmp,ytmp,m,x1,y);
  free_vector(yytmp,1,m);
  free_vector(ytmp,1,m);
}


float plgndr(int l, int m, float x)
{
   void nrerror(char error_text[]);
   float fact,pll,pmm,pmmp1,somx2;
   int i,ll;

   if (m<0 || m>l || fabs(x) > 1.0) nrerror("Bad arguments in routine plgndr");
   pmm=1.0;
   if (m > 0) {
      somx2=sqrt((1.0-x)*(1.0+x));
      fact=1.0;
      for (i=1;i<=m;i++) {
	 pmm *= -fact*somx2;
	 fact += 2.0;
      }
   }
   if (l==m) return pmm;
   else {
      pmmp1=x*(2*m+1)*pmm;
      if (l == (m+1)) return pmmp1;
      else {
	 for (ll=m+2;ll<=l;ll++) {
	    pll=(x*(2*ll-1)*pmmp1-(ll+m-1)*pmm)/(ll-m);
	    pmm=pmmp1;
	    pmmp1=pll;
	 }
	 return pll;
      }
   }
}

/* the Bessel functions */

/* Returns the Bessel function J0(x) for any real x */
double bessj0(double x)
{
   double ax,z;
   double xx,y,ans,ans1,ans2;

   if ((ax=fabs(x))<8.0) {
      y=x*x;
      ans1=57568490574.0+y*(-13362590354.0+y*(651619640.7+
	   y*(-11214424.18+y*(77392.33017+y*(-184.9052456)))));
	   ans2=57568490411.0+y*(1029532985.0+y*(9494680.718+
	   y*(59272.64853+y*(267.8532712+y*1.0))));
	   ans=ans1/ans2;
   } else {
      z=8.0/ax;
      y=z*z;
      xx=ax-0.785398164;
      ans1=1.0+y*(-0.1098628627e-2+y*(.2734510407e-4+
	   y*(-0.2073370639e-5+y*0.2093887211e-6)));
      ans2=-0.1562499995e-1+y*(0.1430488765e-3+
	   y*(-0.6911147651e-5+y*(0.7621095161e-6
	   -y*0.934935152e-7)));
      ans=sqrt(0.636619772/ax)*(cos(xx)*ans1-z*sin(xx)*ans2);
   }
   return ans;
}

/* Returns the Bessel function J1(x) for any real x */
double bessj1(double x)
{
   double ax,z;
   double xx,y,ans,ans1,ans2;
   
   if ((ax=fabs(x))<8.0) {
      y=x*x;
      ans1=x*(72362614232.0+y*(-7895059235.0+y*(242396853.1
	   +y*(-2972611.439+y*(15704.48260+y*(-30.16036606))))));
      ans2=144725228442.0+y*(2300535178.0+y*(18583304.74
	   +y*(99447.43394+y*(376.9991397+y*1.0))));
      ans=ans1/ans2;
   } else {
      z=8.0/ax;
      y=z*z;
      xx=ax-2.356194491;
      ans1=1.0+y*(0.183105e-2+y*(-0.3516396496e-4
	   +y*(0.2457520174e-5+y*(-0.240337019e-6))));
      ans2=0.04687499995+y*(-0.2002690873e-3
	   +y*(0.8449199096e-5+y*(-0.88228987e-6
	   +y*0.105787412e-6)));
      ans=sqrt(0.636619772/ax)*(cos(xx)*ans1-z*sin(xx)*ans2);
      if (x<0.0) ans=-ans;
   }
   return ans;
}

/* Returns the Bessel function JN(x) for any real x and n>=2*/
double bessj(int n, double x)
{
   int j,jsum,m;
   double ax,bj,bjm,bjp,sum,tox,ans;
   
   if (n<2) nrerror("Index n less then 2 in 'bessj'");
   ax=fabs(x);
   if (ax==0.0) return 0.0;
   else if (ax> (double) n) {
      tox=2.0/ax;
      bjm=bessj0(ax);
      bj=bessj1(ax);
      for (j=1;j<n;j++) {
	 bjp=j*tox*bj-bjm;
	 bjm=bj;
	 bj=bjp;
	 }
      ans=bj;
   } else {
      tox=2.0/ax;
      m=2*((n+(int)sqrt(ACC*n))/2);
      jsum=0;
      bjp=ans=sum=0.0;
      bj=1.0;
      for (j=m;j>0;j--) {
	 bjm=j*tox*bj-bjp;
	 bjp=bj;
	 bj=bjm;
	 if (fabs(bj)>BIGNO) {
	    bj*=BIGNI;
	    bjp*=BIGNI;
	    ans*=BIGNI;
	    sum*=BIGNI;
	    }
	 if (jsum) sum+=bj;
	 jsum=!jsum;
	 if (j==n) ans=bjp;
      }
      sum=2.0*sum-bj;
      ans/=sum;
   }
   return x < 0.0 && ( n & 1 ) ? -ans : ans;
}




/* decomposition of a dmatrix into LU form. Used in combination with 'lubksb' 
   to solve linear equations or invert a dmatrix */
void ludcmp(double **a, int n, int *indx, double *d)
{
        int i,imax,j,k;
        double big,dum,sum,temp;
        double *vv;

time_t sek;
struct tm *t;
char str[30];
time (&sek);
t = localtime(&sek);
strcpy(str,ctime(&sek));
printf("Local time in LUDCMP: %s\n",str);


        vv=dvector(1,n);
        *d=1.0;
        for (i=1;i<=n;i++) {
                big=0.0;
                for (j=1;j<=n;j++)
                        if ((temp=fabs(a[i][j])) > big) big=temp;
                if (big == 0.0) {
                   printf("ERROR: matrix line %d\n",i);
                   nrerror("Singular matrix in routine ludcmp");
                }
                vv[i]=1.0/big;
        }
        for (j=1;j<=n;j++) {
                for (i=1;i<j;i++) {
                        sum=a[i][j];
                        for (k=1;k<i;k++) sum -= a[i][k]*a[k][j];
                        a[i][j]=sum;
                }
                big=0.0;
                for (i=j;i<=n;i++) {
                        sum=a[i][j];
                        for (k=1;k<j;k++)
                                sum -= a[i][k]*a[k][j];
                        a[i][j]=sum;
                        if ( (dum=vv[i]*fabs(sum)) >= big) {
                                big=dum;
                                imax=i;
                        }
                }
                if (j != imax) {
                        for (k=1;k<=n;k++) {
                                dum=a[imax][k];
                                a[imax][k]=a[j][k];
                                a[j][k]=dum;
                        }
                        *d = -(*d);
                        vv[imax]=vv[j];
                }
                indx[j]=imax;
                if (a[j][j] == 0.0) a[j][j]=TINY;
                if (j != n) {
                        dum=1.0/(a[j][j]);
                        for (i=j+1;i<=n;i++) a[i][j] *= dum;
                }
        }
        free_dvector(vv,1,n);
}

/* Solves the set of n linear equations AX=B. It needs LU decomposed matrix
   from 'ludcmp' */
void lubksb(double **a,int n,int *indx,double b[])
{
        int i,ii=0,ip,j;
        double sum;

time_t sek;
struct tm *t;
char str[30];
time (&sek);
t = localtime(&sek);
strcpy(str,ctime(&sek));
printf("Local time in LUBKSB: %s\n",str);

        for (i=1;i<=n;i++) {
                ip=indx[i];
                sum=b[ip];
                b[ip]=b[i];
                if (ii)
                        for (j=ii;j<=i-1;j++) sum -= a[i][j]*b[j];
                else if (sum) ii=i;
                b[i]=sum;
        }
        for (i=n;i>=1;i--) {
                sum=b[i];
                for (j=i+1;j<=n;j++) sum -= a[i][j]*b[j];
                b[i]=sum/a[i][i];
        }
}


/* improves a solution dvector X of the linear set of equations AX=B */
void mprove(double **a,double **alud,int n,int indx[],double b[],double x[])
{
        int j,i;
        double sdp;
        double *r;

time_t sek;
struct tm *t;
char str[30];
time (&sek);
t = localtime(&sek);
strcpy(str,ctime(&sek));
printf("Local time in MPROVE: %s\n",str);


        r=dvector(1,n);
        for (i=1;i<=n;i++) {
                sdp = -b[i];
                for (j=1;j<=n;j++) sdp += a[i][j]*x[j];
                r[i]=sdp;
        }
        lubksb(alud,n,indx,r);
        for (i=1;i<=n;i++) x[i] -= r[i];
        free_dvector(r,1,n);
}
