/* LELUYA - PATULJAK program block */
/* this function reads the dust properties from a given file. These properties
   can be cross sections, n and k indices, .... (???) */

#include "PATULJAK.h"

int _PATULJAK__Read_Abs_Sca (FILE *INPUT_FILE,double *Absorption,
			    double *Scattering){
  /* 'Absorption' and 'Scattering' are arrays [0,...,lambda_max-1]. 
     Their values are written in the file INPUT_FILE. */
  int err;                   /* error flag used during the reading of data */
  double lambda1,lambda2;    /* wavelengths from the file INPUT_FILE and ...*/
  double abs1,abs2,sca1,sca2;/* ... and values from the file INPUT_FILE used 
                                in the interpolation of 'Absorption' and 
                                'Scattering' variables */
  int up_down;               /* flag which shows if the input file is sorted 
                                ascending or descending way */
  int lambda_counter;        /* counter used for 'Absorption' and 
                                'Scattering' variables */
  
  /* read the first two entries in the file, and ... */
  err=fscanf(INPUT_FILE,"%lf %lf %lf",&lambda1,&abs1,&sca1);
  if (err!=3) return 1;
  if (DEBUG.SCREEN_MESSAGE >= 3 )
    printf("  ---- raw data: lambda=%E abs=%E sca=%E\n",lambda1,abs1,sca1); 
  err=fscanf(INPUT_FILE,"%lf %lf %lf",&lambda2,&abs2,&sca2);
  if (err!=3) return 1;
  if (DEBUG.SCREEN_MESSAGE >= 3 )
    printf("  ---- raw data: lambda=%E abs=%E sca=%E\n",lambda2,abs2,sca2); 

  /* ... and use them to find out how the input values are sorted */
  if (lambda2 > lambda1) {up_down=1; lambda_counter=0;}
  if (lambda2 < lambda1) {up_down=-1; lambda_counter=lambda_Max-1;}

  /* check if these points are both within the lambda grid range */
  if (up_down>0 && Lambda_Grid[lambda_counter]<=lambda1) {
    while (Lambda_Grid[lambda_counter]<=lambda1){
      Absorption[lambda_counter]=abs1;
      Scattering[lambda_counter]=sca1;
      lambda_counter++;
      if (lambda_counter>=lambda_Max) return 3;
    }
  }
  if (up_down<0 && Lambda_Grid[lambda_counter]>=lambda1) {
    while (Lambda_Grid[lambda_counter]>=lambda1){
      Absorption[lambda_counter]=abs1;
      Scattering[lambda_counter]=sca1;
      lambda_counter--;
      if (lambda_counter<0) return 3;
    }
  }


  do {
    /* check if both of the first two points are out of the lambda grid step*/
    if (up_down>0 && lambda2<Lambda_Grid[lambda_counter]){
      while (lambda2<=Lambda_Grid[lambda_counter] && up_down!=0) {
        if (feof(INPUT_FILE)) up_down=0;
        else {
          lambda1=lambda2; abs1=abs2; sca1=sca2;
          err=fscanf(INPUT_FILE,"%lf %lf %lf",&lambda2,&abs2,&sca2);
          if (err!=3) return 1;
          if (lambda1>lambda2) return 2;
          if (DEBUG.SCREEN_MESSAGE >= 3 )
              printf("  ---- raw data: lambda=%E abs=%E sca=%E\n",
                     lambda2,abs2,sca2); 
	}
      }
    }
    if (up_down<0 && lambda2>=Lambda_Grid[lambda_counter]){
      while (lambda2>=Lambda_Grid[lambda_counter] && up_down!=0) {
        if (feof(INPUT_FILE)) up_down=0;
        else {
          lambda1=lambda2; abs1=abs2; sca1=sca2;
          err=fscanf(INPUT_FILE,"%lf %lf %lf",&lambda2,&abs2,&sca2);
          if (err!=3) return 1;
          if (lambda1<lambda2) return 2;
          if (DEBUG.SCREEN_MESSAGE >= 3 )
              printf("  ---- raw data: lambda=%E abs=%E sca=%E\n",
                     lambda2,abs2,sca2); 
	}
      }
    }

    /* the points have to be interpolated */
    if (up_down>0) {
      while (Lambda_Grid[lambda_counter]<=lambda2 && up_down!=0) {
        err=_MATH_LIB_log_interpolation(lambda1,abs1,lambda2,abs2,
                                      Lambda_Grid[lambda_counter],
                                      &Absorption[lambda_counter]);
        if (err) return 3;
        err=_MATH_LIB_log_interpolation(lambda1,sca1,lambda2,sca2,
                                      Lambda_Grid[lambda_counter],
                                      &Scattering[lambda_counter]);
        if (err) return 3;
        lambda_counter++;
        /* get out if the job is finished */
        if (lambda_counter==lambda_Max) up_down=0;
      }
    }
    if (up_down<0) {
      while (Lambda_Grid[lambda_counter]>=lambda2 && up_down!=0) {
        err=_MATH_LIB_log_interpolation(lambda1,abs1,lambda2,abs2,
                                      Lambda_Grid[lambda_counter],
                                      &Absorption[lambda_counter]);
        if (err) return 3;
        err=_MATH_LIB_log_interpolation(lambda1,sca1,lambda2,sca2,
                                      Lambda_Grid[lambda_counter],
                                      &Scattering[lambda_counter]);
        if (err) return 3;
        lambda_counter--;
        /* get out if the job is finished */
        if (lambda_counter<0) up_down=0;
      }
    }

    /* read the next values */
    if (feof(INPUT_FILE)) up_down=0;
    if (up_down!=0) {
      /* prepare the variables for the next set of values */
      lambda1=lambda2; abs1=abs2; sca1=sca2;
      err=fscanf(INPUT_FILE,"%lf %lf %lf",&lambda2,&abs2,&sca2);
      if (err!=3) return 1;
      if (lambda1>lambda2 && up_down>0) return 2;
      if (lambda1<lambda2 && up_down<0) return 2;
      if (DEBUG.SCREEN_MESSAGE >= 3 )
         printf("  ---- raw data: lambda=%E abs=%E sca=%E\n",
                lambda2,abs2,sca2); 
    }

  } while (up_down!=0);

  /* fill the rest of cross sections with abs2 and sca2 */
  if (lambda1<lambda2) {
    while (lambda_counter<lambda_Max){
      Absorption[lambda_counter]=abs2;
      Scattering[lambda_counter]=sca2;
      lambda_counter++;
    }
  }
  if (lambda1>lambda2) {
    while (lambda_counter>=0){
      Absorption[lambda_counter]=abs2;
      Scattering[lambda_counter]=sca2;
      lambda_counter--;
    }
  }

  /* everything is OK */
  return 0;

} /* end of function _PATULJAK__Read_Abs_Sca */

