/* LELUYA - PATULJAK program block */
/* this function reads the chemical and physical properties of 
   the 'dust_counter' dust component in the multigrain mixture */

#include "PATULJAK.h"

int _PATULJAK__Read_Dust_Sigmas(FILE *GLOBAL_FILE,int dust_counter) {
  FILE *INPUT_FILE;   /* any data file needed in this function */
  char filename[256]; /* name of the file INPUT_FILE */
  char inputfilename[256]; /* filename from the input file */
  int err;            /* error flag used for detection of a reading problem */
  int line_number;    /* line counter during the reading of INPUT_FILE */
  int lambda_counter; /* dummy variable used as a counter in the loops over
                         wavelengths (lambda grid) */

  /**** the first information about the dust is its sublimation temperature */
  /* find the '=' sign */
  err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
  if (err) return 3;
  /* read the temperature */
  err = fscanf(GLOBAL_FILE,"%lf",&(Dust_Sublim_Temp[dust_counter]));
  if (err!=1 || Dust_Sublim_Temp[dust_counter]==0.0) return 3;
  /* make correction to the maximum temperature */
  if (Max_Dust_Temp>0.0) if (Max_Dust_Temp<Dust_Sublim_Temp[dust_counter])
      Max_Dust_Temp = Dust_Sublim_Temp[dust_counter];

  /* activate the "empty space" dus type */
  if (Dust_Sublim_Temp[dust_counter]<0.0) {
    Dust_Sublim_Temp[dust_counter] = _T99999_;
    INPUT.DUST_TYPE[dust_counter]=-1;
    lambda_counter = 0;
    while (lambda_counter < lambda_Max) {
      Sigma_Dust_Abs[dust_counter][lambda_counter]=_Sigma99999_;
      Sigma_Dust_Sca[dust_counter][lambda_counter]=_Sigma99999_;
      lambda_counter++;
    }
    printf("EMPTY SPACE: dust %d is empty space\n",dust_counter);
    return 0;
  }

  /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    printf("  DUST COMPONENT #%d: sublimation temperature = %lf\n",
           dust_counter+1, Dust_Sublim_Temp[dust_counter]);
  }

  /**** the next step is to find how the input data are specified */
  /**** for this we need to read the INPUT.DUST_TYPE flag */
  /* find the '=' sign */
  err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
  if (err) return 4;
  /* read the flag */
  err = fscanf(GLOBAL_FILE,"%d",&(INPUT.DUST_TYPE[dust_counter]));
  if (err!=1 || INPUT.DUST_TYPE[dust_counter]<0) return 4;

  /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    printf("  DUST COMPONENT #%d INPUT OPTION: %d\n",dust_counter+1,
           INPUT.DUST_TYPE[dust_counter]);
  }

  /* now we can have different options */
  /*************************************/
  /* OPTION 0: read given files with the cross sections */
  if (INPUT.DUST_TYPE[dust_counter] == 0 ) {

    /* we need the name of a file with the cross sections */
    /* find the '=' sign */
    err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
    if (err) return 5;
    /* read the file name */
    err = fscanf(GLOBAL_FILE,"%s",inputfilename);
    if (err!=1) return 5;
#ifdef _2DD_MPI_
    sprintf(filename,"%s/%s",_LELUYA_MPIRUN_PATH,inputfilename);
#else
    sprintf(filename,"%s/%s",_LELUYA_RUN_PATH,inputfilename);
#endif
    /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
    if (DEBUG.SCREEN_MESSAGE >= 1 ) 
         printf("    cross section file name: %s\n",filename);
    /* open the file with cross sections*/
    INPUT_FILE=fopen(filename,"r");
    if (INPUT_FILE==NULL) return 5;

    /* read the first three lines of comments */
    err= _PATULJAK__Search_for_char (INPUT_FILE,"\n");
    if (err) return 6;
    err= _PATULJAK__Search_for_char (INPUT_FILE,"\n");
    if (err) return 6;
    err= _PATULJAK__Search_for_char (INPUT_FILE,"\n");
    if (err) return 6;

    /* read the cross sections and interpolate them on lambda grid */
    err = _PATULJAK__Read_Abs_Sca (INPUT_FILE,Sigma_Dust_Abs[dust_counter],
                                             Sigma_Dust_Sca[dust_counter]);
    if (err==1) return 6;
    if (err==2) return 7;
    if (err==3) return 8;

    /* close the file with cross sections*/
    fclose(INPUT_FILE);

    /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
    if (DEBUG.SCREEN_MESSAGE >= 2 ) {
      printf("  ----- cross sections ------\n");
      printf("  lambda    absorption   scattering\n");
      lambda_counter = 0;
      while (lambda_counter < lambda_Max) {
        printf("%E %E %E\n",
               Lambda_Grid[lambda_counter],
               Sigma_Dust_Abs[dust_counter][lambda_counter],
               Sigma_Dust_Sca[dust_counter][lambda_counter]);

        lambda_counter++;
      }
      printf("  ----- end of cross sections -----\n");
    }

    /* everything is OK*/
    return 0;
  } /* end of the OPTION 0: read given files with the cross sections */

  return 4; /* this line should not be reached !!*/
} /* end of the _PATULJAK__Read_Dust_Sigmas function */
