/* LELUYA - PATULJAK program block */
/* input from the global input file; input from the files specified in the
   global input file; input of all data necessary for the work of LELUYA; */
/* NOTE: there are two major approximations here:
          Source_Radius = 0,  Source_Teff = Black Body Temperature */

#include "PATULJAK.h"

int _PATULJAK__Read_Input_Data (FILE *GLOBAL_FILE, char Global_File_Name[]){
  /* GLOBAL_FILE is a pointer to the global input file */
  /* Global_File_Name is the name of GLOBAL_FILE */
  int err_location; /* error flag used for location of the error */
  int err;          /* error flag used for detection of a reading problem */
  int message = 0;  /* debugger flag used for checking of the code */
  int message_location;/* debugger flag used for location of the message */
  int dust_counter; /* counter in the loop over different dust types */
  struct density_parameters *dust_parameter; /* used for printing of
                       the dust parameters in a loop over dust parameters */
  int dust_parameter_counter; /* counter in a loop over dust parameters */

  /************************************************************************/
  /** variables used as help in the process of sorting the dust components*/
  double x_1,z_1,density_1,density_2;
  int counter1, counter2, counter3;
  double help1d,help5d;
  int    help2i;
  char   help3c;
  struct density_parameters help4dp, *help6dp;
  /************************************************************************/

  /************************************************************************/  
  /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    message_location = 2;
    message = 1;
    _PESJANEK__Debugger_Message (message_location,message);
  }
  /* first we have to deal with the wavelength grid */
  /* this function will take care of that */
  err = _PATULJAK__Read_Lambda_Grid (GLOBAL_FILE);
  if (err) {
    err_location = 3;
    _PESJANEK__Error_Message (err_location,err);
    if (err == 1) printf("%s\n",Global_File_Name);
    return 1;
  }
  /************************************************************************/  

  /************************************************************************/  
  /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    message_location = 2;
    message = 2;
    _PESJANEK__Debugger_Message (message_location,message);
  }
  /* read the total optical depth properties */
  /* and include its wavelength in the lambda grid */
  err = _PATULJAK__Read_Tau_Total (GLOBAL_FILE);
  if (err) {
    err_location = 4;
    _PESJANEK__Error_Message (err_location,err);
    if (err == 1 || err == 2 || err == 3 || err == 4 || 
        err == 6 || err == 7 || err == 9) 
       printf("%s\n",Global_File_Name);
    if (err == 8) printf("(%lf-%lf microns)\n",Lambda_Grid[0],
                                               Lambda_Grid[lambda_Max-1]);
    return 1;
  }
  /* transform the Theta_Total_Value angle into radians */
  Theta_Total_Value *= (PI/180.0);
  /************************************************************************/  

  /************************************************************************/  
  /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    message_location = 2;
    message = 3;
    _PESJANEK__Debugger_Message (message_location,message);
  }
  /* create an array of source fluxes at different wavelengths */
  Source_Flux=dvector (0,lambda_Max-1);
  /* Read the central energy source flux */
  err= _PATULJAK__Read_Source_Flux(GLOBAL_FILE);
  if (err) {
    err_location = 5;
    _PESJANEK__Error_Message (err_location,err);
    if (err==1 || err==2) printf("%s\n",Global_File_Name);
    return 1;
  }

/* NOTE: Source_Teff = Black Body Temperature, Source_Radius = 0.0 */
  Source_Radius = 0.0;

  /************************************************************************/  

  /************************************************************************/  
  /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    message_location = 2;
    message = 4;
    _PESJANEK__Debugger_Message (message_location,message);
  }

  /* Read the dust chemical and physical properties */
  /* First find out how many different dust types exist */
  err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
  if (err) {
    err_location = 6;
    err = 1;
    _PESJANEK__Error_Message (err_location,err);
    printf("%s\n",Global_File_Name);
    return 1;
  }
  err=fscanf(GLOBAL_FILE,"%d",&Max_Dust_Type);
  if (err!=1 || (Max_Dust_Type<1)) {
    err_location = 6;
    err = 2;
    _PESJANEK__Error_Message (err_location,err);
    printf("%s\n",Global_File_Name);
    return 1;
  }

  /* now we know how big are the cross section variables in the memory,
     and we can reserve the memory */
  Sigma_Dust_Abs=dmatrix(0,Max_Dust_Type-1,0,lambda_Max-1);
  Sigma_Dust_Sca=dmatrix(0,Max_Dust_Type-1,0,lambda_Max-1);

  /* also, we can create an array of the input flags */
  INPUT.DUST_TYPE=ivector(0,Max_Dust_Type-1);

  /* and array of sublimation temperatures */
  Dust_Sublim_Temp=dvector(0,Max_Dust_Type-1);

  /* and a list of the names of the dust components */
  Dust_Component_Name=cmatrix(0,Max_Dust_Type-1,
                              0,_DUST_COMPONENT_NAME_SIZE_-1);

  /* create a loop over the dust types, and read their chemical
     and physical properties */
  dust_counter = 0;
  while (dust_counter < Max_Dust_Type) {
    if (DEBUG.SCREEN_MESSAGE >= 1 ) 
      printf("  INPUT of the dust type #%d of %d\n",dust_counter+1,
                                                    Max_Dust_Type);
    /* for now there is no option for a user to specify the dust
       name. A default names are used: */
    sprintf(Dust_Component_Name[dust_counter],
            "#%.3d",dust_counter);
    if (DEBUG.SCREEN_MESSAGE >= 1 )
       printf(" DUST COMPONENT: %s\n",Dust_Component_Name[dust_counter]);
    err = _PATULJAK__Read_Dust_Sigmas(GLOBAL_FILE,dust_counter);
    if (err) {
      err_location = 6;
      _PESJANEK__Error_Message (err_location,err);
      if (err==3 || err==4 || err==5) 
          printf("#%d in %s\n",dust_counter+1,Global_File_Name);
      if (err==6 || err==7 || err==8) printf("#%d\n",dust_counter+1);
    }
    dust_counter++;
  } /* end of the loop over different dust types */
  /************************************************************************/  

  /************************************************************************/  
  /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    message_location = 2;
    message = 5;
    _PESJANEK__Debugger_Message (message_location,message);
  }

  /* set "empty space" */
  INPUT.EMPTY_SPACE = FALSE;

  /* create an array of dust density flags */
  INPUT.DUST_DENSITY_TYPE=ivector(0,Max_Dust_Type-1);
  INPUT.DUST_DENSITY_TYPE_NAME=cmatrix(0,Max_Dust_Type-1,
                                       0,_DUST_DENSITY_TYPE_NAME_SIZE_-1);
  /* and array of the parameters used for these dust density models */
  Dust_Density_Param=density_parameters_vector(0,Max_Dust_Type-1);

  /* read the dust density profile for each component */
  dust_counter=0;
  while (dust_counter<Max_Dust_Type) {

    if (Dust_Sublim_Temp[dust_counter] != _T99999_ ) {
      /* read the density type flag for the 'dust_component' component */
      err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
      if (err) {
        err_location = 7;
        err = 1;
        _PESJANEK__Error_Message (err_location,err);
        printf("#%d in %s\n",dust_counter,Global_File_Name);
        return 1;
      }
      err=fscanf(GLOBAL_FILE,"%d",&INPUT.DUST_DENSITY_TYPE[dust_counter]);
      if (err!=1 || (INPUT.DUST_DENSITY_TYPE[dust_counter]<0)) {
        err_location = 7;
        err = 1;
        _PESJANEK__Error_Message (err_location,err);
        printf("#%d in %s\n",dust_counter,Global_File_Name);
        return 1;
      }
    }
    else {
      INPUT.DUST_DENSITY_TYPE[dust_counter]=99999;
      INPUT.EMPTY_SPACE = TRUE;
    }
    /* if the flag is zero, then the dust density profile is the same
       for all dust components */
    if (dust_counter==0 && INPUT.DUST_DENSITY_TYPE[dust_counter]==0) {
      /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
      if (DEBUG.SCREEN_MESSAGE >= 1 ) {
        printf("  DUST DENSITY PROFILE = 0\n");
        printf("  THE SAME PROFILE FOR ALL DUST COMPONENTS\n");
      }
      /* read and memorize the profile for the first component */
      err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
      if (err) {
        err_location = 7;
        err = 2;
        _PESJANEK__Error_Message (err_location,err);
        printf("%s\n",Global_File_Name);
        return 1;
      }
      err=fscanf(GLOBAL_FILE,"%d",&INPUT.DUST_DENSITY_TYPE[dust_counter]);
      if (err!=1 || (INPUT.DUST_DENSITY_TYPE[dust_counter]<0)) {
        err_location = 7;
        err = 2;
        _PESJANEK__Error_Message (err_location,err);
        printf("%s\n",Global_File_Name);
        return 1;
      }

      /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
      if (DEBUG.SCREEN_MESSAGE >= 3 ) {
        printf("  reading of the dust density parameters for ");
        printf("the dust component #1\n");
      }
      /* read the parameters of this dust density type */
      err = _PATULJAK__Read_Density_Type (GLOBAL_FILE,dust_counter);
      if (err) {
        err_location = 7;
        err = 3;
        _PESJANEK__Error_Message (err_location,err);
        printf("%s\n",Global_File_Name);
        return 1;
      }

      /* copy these parameters to all dust components*/
      dust_counter++;
      while (dust_counter<Max_Dust_Type) {
        INPUT.DUST_DENSITY_TYPE[dust_counter] = INPUT.DUST_DENSITY_TYPE[0];
        sprintf(INPUT.DUST_DENSITY_TYPE_NAME[dust_counter],"%s",
                                             INPUT.DUST_DENSITY_TYPE_NAME[0]);
        Dust_Density_Param[dust_counter] = Dust_Density_Param[0];
        dust_counter++;
      }

    } /* end of 'if (INPUT.DUST_DENSITY_TYPE[0]==0)' */
    else { /* in this case read the dust parameters for each dust component
              separately */
      
      /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
      if (DEBUG.SCREEN_MESSAGE >= 1 ) {
        printf("  DUST COMPONENT #%d HAS THE DENSITY PROFILE = %d\n",
               dust_counter, INPUT.DUST_DENSITY_TYPE[dust_counter]);
      }

      /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
      if (DEBUG.SCREEN_MESSAGE >= 3 ) {
        printf("  reading of the dust density parameters for ");
        printf("the dust component #%d\n",dust_counter);
      }
      /* read the parameters of this dust density type */
      err = _PATULJAK__Read_Density_Type (GLOBAL_FILE,dust_counter);
      if (err) {
        err_location = 7;
        err = 4;
        _PESJANEK__Error_Message (err_location,err);
        printf("#%d in the file %s\n",dust_counter,Global_File_Name);
        return 1;
      }

      /* go to the next dust component */
      dust_counter++;
    }  /* end of 'if (INPUT.DUST_DENSITY_TYPE[0]!=0)' */

  } /* end of the loop over different dust types */

  /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >=2  ) {
    /* print all parameters */
    printf("  ------- DUST MODELS -------\n");
    /* loop over dust types */
    dust_counter=0;
    while (dust_counter<Max_Dust_Type) {
      /* print the name of dust model */
      printf("  dust component #%d has: %s\n",dust_counter, 
             INPUT.DUST_DENSITY_TYPE_NAME[dust_counter]);
      /* print parameters */
      printf("      parameter #1= %E\n",Dust_Density_Param[dust_counter].param);
      dust_parameter = Dust_Density_Param[dust_counter].next;
      dust_parameter_counter = 2;
      while (dust_parameter!=NULL) {
        printf("      parameter #%d = %E\n",dust_parameter_counter,
                                            dust_parameter->param);
        dust_parameter = dust_parameter->next;
	dust_parameter_counter++; 
      }
      dust_counter++;
    } /* end of the loop over different dust types */
  }
  /************************************************************************/  

  /************************************************************************/  
  /* sort the dust components according to their sublimation temperature */   
  /* bubble sort method */
  x_1 = 1.0 * sin(Theta_Total_Value);  /* X-cooridnate */
  z_1 = 1.0 * cos(Theta_Total_Value);  /* Z-cooridnate */
  counter1 = 0;
  while (counter1 < Max_Dust_Type) {
    counter2 = counter1 + 1;
    while (counter2 < Max_Dust_Type) {
      /* compare the temperatures, and switch the dust components if
         necessary */
      density_1 = Dust_Density(x_1,z_1,counter1,NULL,&err);
      density_2 = Dust_Density(x_1,z_1,counter2,NULL,&err);
      if (Dust_Sublim_Temp[counter1] < Dust_Sublim_Temp[counter2] || 
          (density_1<=0.0 && density_2>0.0))
        err = _ZELENJAK_exchange_components(counter1,counter2); 
      counter2++;
    } /* end of the loop over counter2 */
    counter1++;
  } /* end of the loop over counter1 */
  /* debugger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    printf("\n    SORTING OF THE DUST ACCORDING TO Tsub HAS BEEN PERFORMED \n\n");
  }
  /************************************************************************/  
  /* print all data involved in the previous sorting */
  if (DEBUG.SCREEN_MESSAGE >= 3 ) {
    printf("COMPLETE LIST OF THE DUST DATA AFTER SORTING \n");
    printf("-------------------------------------------- \n");
    for (counter1=0; counter1<Max_Dust_Type; counter1++){
      printf("DUST COMPONENT #%d (NAME=%s): Temperature = %lf\n",
             counter1,Dust_Component_Name[counter1],Dust_Sublim_Temp[counter1]);
      printf("DUST COMPONENT #%d (NAME=%s): absorption and scattering cross sections\n",
             counter1,Dust_Component_Name[counter1]);
      printf("  lambda   Absorption  Scattering\n");
      for (counter2=0; counter2<lambda_Max; counter2++)
        printf("%lf  %E  %E\n",Lambda_Grid[counter2],
               Sigma_Dust_Abs[counter1][counter2],
               Sigma_Dust_Sca[counter1][counter2]);
      printf("DUST COMPONENT #%d (NAME=%s): INPUT DUST TYPE: %d\n",
              counter1,Dust_Component_Name[counter1],INPUT.DUST_TYPE[counter1]);
      printf("DUST COMPONENT #%d (NAME=%s): INPUT DUST DENSITY TYPE: %d\n",
              counter1,Dust_Component_Name[counter1],INPUT.DUST_DENSITY_TYPE[counter1]);
      printf("DUST COMPONENT #%d (NAME=%s): NAME OF THE INPUT DUST DENSITY TYPE: %s\n",
              counter1,Dust_Component_Name[counter1],INPUT.DUST_DENSITY_TYPE_NAME[counter1]);
      printf("DUST COMPONENT #%d (NAME=%s): PARAMETERS:\n",
              counter1,Dust_Component_Name[counter1]);
      counter2 = 1;
      help6dp = &Dust_Density_Param[counter1];
      while (help6dp!=NULL){
        printf("  parameter #%d = %lf\n", counter2, help6dp->param);
        help6dp = help6dp->next;
        counter2++;
      }
    printf("-------------------------------------------- \n");
    } /* end of the 'for-loop' over counter1 */
  }
  /************************************************************************/  

  /************************************************************************/
  /* scale the cross sections by the absorption extinction coefficient
     of the dust component 0 at the point 1 and wavelength Lambda_Total */
  err = _ZELENJAK_Scale_Cross_Sections ();
  if (err>0) return 1;
  /************************************************************************/

  return 0;
} /* end of _PATULJAK__Read_Input_Data */
