/* LELUYA - PATULJAK program block */
/* this function reads wavelengths for the lambda grid */
/* or reads the default lambda grid */

#include "PATULJAK.h"

int _PATULJAK__Read_Lambda_Grid (FILE *GLOBAL_FILE){
  FILE *LAMBDA_GRID_FILE;  /* file with the wavelengths which will 
                              be the lambda grid points */
  char filename[256];      /* name of the LAMBDA_GRID_FILE file */
  char filename1[256];
  int  err;                /* error flag used for detection of a reading 
                              problem*/
  int line_number;         /* line counter during the reading of 
                              wavelengths from a file */

  /* first locate the appropriate flag in the global input file */
  err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
  if (err) return 1;
  
  /* read the flag from the global input file which will tell us where to
     to find the lambda grid */
  err = fscanf(GLOBAL_FILE,"%d",&(INPUT.LAMBDA_TYPE));
  if (err!=1) return 1;

  /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    printf("  LAMBDA GRID OPTION: %d\n",INPUT.LAMBDA_TYPE);
  }

  /* now we can have different options */
  if (INPUT.LAMBDA_TYPE == 0 ) {
    /*************************************/
    /* OPTION 0: use the default grid */
    sprintf(filename,"%s/MALIK/lambda_grid.dat",_LELUYA_SOURCE_PATH);
  }
  else if (INPUT.LAMBDA_TYPE == 1 ) {
    /*******************************************/
    /* OPTION 1: use the grid from a given file*/
    err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
    if (err) {fclose(GLOBAL_FILE);return 3;}
    err = fscanf(GLOBAL_FILE,"%s",filename1);
    if (err!=1) {fclose(GLOBAL_FILE);return 3;}
#ifdef _2DD_MPI_
    sprintf(filename,"%s/%s",_LELUYA_MPIRUN_PATH,filename1);
#else
    sprintf(filename,"%s/%s",_LELUYA_RUN_PATH,filename1);
#endif
    if (DEBUG.SCREEN_MESSAGE >= 1 ) {
      printf("read the lambda grid from:%s\n",filename);
    }
  }

  /*************************************/
  /* read the lambda-grid file         */
  if (INPUT.LAMBDA_TYPE == 0 || INPUT.LAMBDA_TYPE == 1) {
    /* open the default file */
    LAMBDA_GRID_FILE=fopen(filename,"rt");
    if (LAMBDA_GRID_FILE==NULL) return 2;

    /* read the data */
    err = _PATULJAK__Search_for_char (LAMBDA_GRID_FILE,"=");
    if (err) {
      fclose(LAMBDA_GRID_FILE);
      return 3;
    }

    /* read how many input points are in the file */
    err = fscanf(LAMBDA_GRID_FILE,"%d",&lambda_Max);
    if (err!=1) {
      fclose(LAMBDA_GRID_FILE);
      return 3;
    }

    /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
    if (DEBUG.SCREEN_MESSAGE >= 1 ) {
      printf("  total number of the lambda grid points: %d\n",lambda_Max);
    }

    /* create the vector of wavelengths which will be the lambda grid */
    Lambda_Grid=dvector(0,lambda_Max-1);

    /* read the wavelengths */
    line_number = 0;
    while (line_number < lambda_Max) {
      err = fscanf(LAMBDA_GRID_FILE,"%lf",&Lambda_Grid[line_number]);
      if (err!=1) {
        fclose(LAMBDA_GRID_FILE);
        return 3;
      }
      /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
      if (DEBUG.SCREEN_MESSAGE >= 2 ) {
        printf("  wavelength #%d = %lf\n",line_number,
                                         Lambda_Grid[line_number]);
      }
      line_number++;
    }

    /* everything has gone smoothly, close the file */
    fclose (LAMBDA_GRID_FILE);
    return 0;
  }
  else if (INPUT.LAMBDA_TYPE >= 2 ) {
    return 4;
  }
  return 1; /* if this line has been reached, then something is wrong!! */
} /* end of  _PATULJAK__Read_Lambda_Grid */
