/* LELUYA - PATULJAK program block */
/* this function reads or evaluates the source flux */

/* IMPORTANT NOTE:
   if you work with a finite size source then you need to calculate the
   point source approximation of the central flux (variable Source_Flux[])
   and scale all source fluxes by this flux, and then normalize the point
   source flux.
   I still haven't written a function which would calculate the point source
   flux from a given finite size source. */

#include "PATULJAK.h"

int _PATULJAK__Read_Source_Flux (FILE *GLOBAL_FILE){
  int err;             /* error flag */
  double Teff;         /* temperature of the central source (input OPTION 0) */
  int lambda_counter;  /* counter in the loops over lambda grid */
  double lambda;       /* wavelength in the loop above */
  char Source_File_Name[355]; /* stellar spectrum file name */
  char Source_Name[255];
  FILE *Source_File;   /* stellar spectrum file */
  char line_long[300];
  double lambda1,lambda2,F1,F2; 

  /* first locate the appropriate flag in the global input file */
  err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
  if (err) return 1;
  
  /* read the flag from the global input file which will tell us how 
     the source flux is specified */
  err = fscanf(GLOBAL_FILE,"%d",&(INPUT.SOURCE_TYPE));
  if (err!=1) return 1;

  /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    printf("  SOURCE FLUX OPTION: %d\n",INPUT.SOURCE_TYPE);
  }

  /* now we can have different options */
  /*************************************/
  /* OPTION 0: use simple black body  */
  if (INPUT.SOURCE_TYPE == 0) {
    
    /* Read the stellar temperature Teff */
    err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
    if (err) return 2;
    err = fscanf(GLOBAL_FILE,"%lf",&Teff);
    if (err!=1) return 2;
    if (Teff<=0) return 2;
    /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
    if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    printf("  SOURCE TEMPERATURE: %lf K\n",Teff);
    }

    /* in a loop over lambda calculate scaled stellar flux */
    /* create a loop */
    lambda_counter=0;
    while (lambda_counter<lambda_Max) {
      lambda = Lambda_Grid[lambda_counter];
      /* calculate the planck function scaled with temperature */
      Source_Flux[lambda_counter] = _MATH_LIB_Planck(Teff, lambda); 
      lambda_counter++;
    } /* end of the loop over lambda grid */

    /* the source bolometric flux follows from the Teff */
    Source_Teff = Teff;

    /* debuger messages if required by DEBUG.SCREEN_MESSAGE */
    if (DEBUG.SCREEN_MESSAGE >= 2 ) {
      printf("  ----- central energy source flux ------\n");
      printf("  lambda   flux/T^4 \n");
      for (lambda_counter=0;lambda_counter < lambda_Max;lambda_counter++) { 
        printf("%E %E\n",
               Lambda_Grid[lambda_counter],Source_Flux[lambda_counter]);
      }
      printf("  ----- end of the central energy source flux -----\n");
    }

    return 0;
  } /* end of (INPUT.SOURCE_TYPE == 0) */

  /*************************************/
  /* OPTION 1: read from a file (lambda, F_lambda)  */
  if (INPUT.SOURCE_TYPE == 1) {

    /* Read the stellar temperature Source_Teff */
    err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
    if (err) return 2;
    err = fscanf(GLOBAL_FILE,"%lf",&Source_Teff);
    if (err!=1) return 2;
    if (Source_Teff<=0) return 2;

    /* Read the file name */
    err = _PATULJAK__Search_for_char (GLOBAL_FILE,"=");
    if (err) return 2;
    err = fscanf(GLOBAL_FILE,"%s",&Source_Name);

#ifdef _2DD_MPI_
    sprintf(Source_File_Name,"%s/%s",_LELUYA_MPIRUN_PATH,Source_Name);
#else
    sprintf(Source_File_Name,"%s/%s",_LELUYA_RUN_PATH,Source_Name);
#endif

    Source_File = fopen(Source_File_Name,"rt");
    if (Source_File==NULL) {
      printf("SPECTRUM FILE=%s\n",Source_File_Name);
      printf("STELLAR SPECTRUM FILE MISSING!!\n"); 
      return 2;
    }

    /* read the first three lines */
    fgets(line_long,300,Source_File);
    fgets(line_long,300,Source_File);
    fgets(line_long,300,Source_File);

    /* read lambda and F_lambda */
    lambda_counter=0;
    fscanf(Source_File,"%lf %lf",&lambda1,&F1);
    while(lambda1>Lambda_Grid[lambda_counter]) {
      Source_Flux[lambda_counter] = F1;
      lambda_counter++;
    }

    while ((fscanf(Source_File,"%lf %lf",&lambda2,&F2))!=EOF) {
      while (lambda2 > Lambda_Grid[lambda_counter] && lambda_counter<lambda_Max-1) {
        _MATH_LIB_log_interpolation(lambda1,F1,lambda2,F2,Lambda_Grid[lambda_counter],
                                    &(Source_Flux[lambda_counter]));
        lambda_counter++;
      }
      lambda1 = lambda2; 
      F1 = F2;
    }
    fclose(Source_File);

    while (lambda_counter<lambda_Max) {
      F1 = Lambda_Grid[lambda_counter]/lambda2;
      Source_Flux[lambda_counter] = F2/(F1*F1*F1*F1);
      lambda_counter++;
    }

    /* normalization */
    F1 = _MATH_LIB_simpson(lambda_Max,0,lambda_Max-1,Lambda_Grid,Source_Flux);
    for (lambda_counter=0;lambda_counter < lambda_Max;lambda_counter++) { 
      Source_Flux[lambda_counter] /= F1;
    }

    return 0;
  } /* end of (INPUT.SOURCE_TYPE == 1) */

  return 1; /* this line should not be reached */

} /* end of _PATULJAK__Read_Source_flux */
