/* LELUYA - PATULJAK program block */
/* This function reads temperature from a file.

   It is called from _PERUNIKA_create_initial_array.c and it has very 
   simple structure. It reads a file with temperatures and creates 
   an array of temperatures Temp_Temp[i][j] where "i" represents 
   the radial grid and "j" angular. These two grids have their values
   (radius and angle) in Temp_r[i] and Temp_theta[j]. The number of grid 
   points is i=[0,...,N_Temp_r-1] and j=[0,...,N_Temp_theta]
*/

#include "PATULJAK.h"

int _PATULJAK__Read_Temperature (double **Temp_r,     int *N_Temp_r, 
                                double **Temp_theta, int *N_Temp_theta,
                                double ***Temp_Temp){
  FILE *Temp_FILE;  /* file with the temperatures */
  char filename[256];
  int  err;  /* error flag used for detection of a reading problem*/
  double *Temp_r_,*Temp_theta_, **Temp_Temp_; /* radial grid, angular grid,
					         and temperature grid */
  int    Nr_, Ntheta_; /* number of radial and angular grid points */
  int counter1, counter2; /* loop counters */
  char line_long[300]; /* unimportant line in the input file */
  double R1,R2,T1,T2; 

  /* default temperature file format and name                        */
  if (DEBUG.CONTINUE==100 || DEBUG.CONTINUE==200) {

#ifdef _2DD_MPI_
    sprintf(filename,"%s/LELUYA.Initial_Temperature.2dd",
            _LELUYA_MPIRUN_PATH);
#else
    sprintf(filename,"%s/LELUYA.Initial_Temperature.2dd",
            _LELUYA_RUN_PATH);
#endif
    Temp_FILE = fopen (filename,"r");
    if (Temp_FILE==NULL) {
        printf("%d:---------------------------------------\n",my_MPI_rank);
        printf("%d:---- ERROR!! Can not opet input file:\n",my_MPI_rank);
        printf("%d:               %s\n",my_MPI_rank,filename);
        return 1;
    }
    
    /* read the type of input file */
    fscanf(Temp_FILE,"%d",&err);fgets(line_long,300,Temp_FILE);

    switch (err) {

    case 1: 
   /*******************************************************************/
   /* input file of type 1 */
   /* FILE FORMAT STRUCTURE:
       
   1  <--- type of the data structure
   R* <--- scaled source radius
   N  <--- total number of radial grid points
   Nt <--- total number of angular grid points
   theta_1  <--- angle #1
   r_1_1 T_1_1 <--- radius and temperature
   r_1_2 T_1_2 <--- radius and temperature
   ... .... ...
   r_1_N T_1_N <--- radius and temperature

   ... .... ....

   theta_Nt <--- angle #Nt
   r_Nt_1 T_Nt_1 <--- radius and temperature
   r_Nt_2 T_Nt_2 <--- radius and temperature
   ... .... ...
   r_Nt_N T_Nt_N <--- radius and temperature

   */

      /* read the scaled source radius */ 
      fscanf(Temp_FILE,"%lf",&Source_Radius); fgets(line_long,300,Temp_FILE);
      if (Source_Radius != 0.0) {
        Psi = Dust_Sublim_Temp[0]/Source_Teff;
        Psi = Psi*Psi*Psi*Psi;
        Psi = Psi *4.0 / (Source_Radius*Source_Radius);
        printf("PSI from the given (Source_Radius=%lf) =%lf\n",
	   Source_Radius,Psi);
      }

      /* read the total number of radial grid points */ 
      fscanf(Temp_FILE,"%d",&Nr_); fgets(line_long,300,Temp_FILE);
      Nr_ = Nr_ + 2;

      /* read the total number of angular grid points */ 
      fscanf(Temp_FILE,"%d",&Ntheta_); fgets(line_long,300,Temp_FILE);
      Ntheta_ = Ntheta_ + 2;

      /* allocate the memory */
      Temp_r_ = dvector(0,Nr_-1);
      Temp_theta_ = dvector(0,Ntheta_-1);
      Temp_Temp_ = dmatrix(0,Nr_-1,0,Ntheta_-1);

      /* read the data */
      for (counter1=0; counter1<Ntheta_-2; counter1++) { /* loop over theta */
	fscanf(Temp_FILE,"%lf",&Temp_theta_[counter1+1]);
	if (_INITIAL_GRID_ARRAY_theta_min > Temp_theta_[counter1+1])
	  Temp_theta_[counter1+1] = _INITIAL_GRID_ARRAY_theta_min;
	if (_INITIAL_GRID_ARRAY_theta_max < Temp_theta_[counter1+1])
	  Temp_theta_[counter1+1] = _INITIAL_GRID_ARRAY_theta_max;
	if (counter1 == 0) 
          Temp_theta_[0] = _INITIAL_GRID_ARRAY_theta_min;
	if (counter1 == Ntheta_-3) 
          Temp_theta_[Ntheta_-1] = _INITIAL_GRID_ARRAY_theta_max;

        for (counter2=0; counter2<Nr_-2; counter2++) { /* loop over radius */
	  fscanf(Temp_FILE,"%lf",&Temp_r_[counter2+1]);
          if (1.0 > Temp_r_[counter2+1]) Temp_r_[counter2+1] = 1.0;
	  if (Rout < Temp_r_[counter2+1]) Temp_r_[counter2+1] = Rout;
	  if (counter2 == 0) Temp_r_[0] = 1.0;
          if (counter2 == Nr_-3) Temp_r_[Nr_-1] = Rout;

	  /* read the temperature */
	  fscanf(Temp_FILE,"%lf",&Temp_Temp_[counter2+1][counter1+1]);
	  if (counter2 == 0) 
	    Temp_Temp_[0][counter1+1] = Temp_Temp_[1][counter1+1];
          if (counter2 == Nr_-3) 
	    Temp_Temp_[Nr_-1][counter1+1] = Temp_Temp_[counter2+1][counter1+1];
	  if (counter1 == 0) 
             Temp_Temp_[counter2+1][0] = Temp_Temp_[counter2+1][1];
	  if (counter1 == Ntheta_-3) 
             Temp_Temp_[counter2+1][Ntheta_-1] = 
                                            Temp_Temp_[counter2+1][counter1+1];
	  if (counter2 == 0 && counter1 == 0) 
	    Temp_Temp_[0][0] = Temp_Temp_[1][1];
	  if (counter2 == Nr_-3 && counter1 == 0) 
	    Temp_Temp_[Nr_-1][0] = Temp_Temp_[counter2+1][1];
	  if (counter2 == 0 && counter1 == Ntheta_-3) 
             Temp_Temp_[0][Ntheta_-1] = Temp_Temp_[1][counter1+1];
	  if (counter2 == Nr_-3 && counter1 == Ntheta_-3) 
             Temp_Temp_[Nr_-1][Ntheta_-1] = Temp_Temp_[counter2+1][counter1+1];
	} /* end of the loop  over radius */
      } /* end of the loop over theta */

      break;
    case 2: 
   /*******************************************************************/
   /* input file of type 1 */
   /* FILE FORMAT STRUCTURE:
       
   2  <--- type of the data structure
   R* <--- scaled source radius
   N  <--- total number of radial grid points
   r1,r2,...,rN <---- list of radial grid points (r1=1.0, rN=Rout)
   Nt <--- total number of angular grid points
   theta_1  <--- angle #1
   r_1 T_1 <--- radius and temperature (r_1 has to be 1.0)
   r_2 T_2 <--- radius and temperature
   ... .... ...
   r_n T_n <--- radius and temperature (r_n has to be Rout)

   ... .... ....

   theta_Nt <--- angle #Nt
   r_1 T_1 <--- radius and temperature (r_1 has to be 1.0)
   r_2 T_2 <--- radius and temperature
   ... .... ...
   r_n T_n <--- radius and temperature (r_n has to be Rout)
   */

      /* read the scaled source radius */ 
      fscanf(Temp_FILE,"%lf",&Source_Radius); fgets(line_long,300,Temp_FILE);
      if (Source_Radius != 0.0) {
        Psi = Dust_Sublim_Temp[0]/Source_Teff;
        Psi = Psi*Psi*Psi*Psi;
        Psi = Psi *4.0 / (Source_Radius*Source_Radius);
        printf("PSI from the given (Source_Radius=%lf) =%lf\n",
	   Source_Radius,Psi);
      }

      /* read the total number of radial grid points */ 
      fscanf(Temp_FILE,"%d",&Nr_); fgets(line_long,300,Temp_FILE);
      Nr_ = Nr_;
      Temp_r_ = dvector(0,Nr_-1);
      /* read the radial grdi points */
      for (counter2=0; counter2<Nr_; counter2++)
	fscanf(Temp_FILE,"%lf",&Temp_r_[counter2]);
      Temp_r_[0]=1.0;
      Temp_r_[Nr_-1]=Rout;

      /* read the total number of angular grid points */ 
      fscanf(Temp_FILE,"%d",&Ntheta_); fgets(line_long,300,Temp_FILE);
      Ntheta_ = Ntheta_ + 2;
      Temp_theta_ = dvector(0,Ntheta_-1);

      /* allocate memory for the temperatures*/
      Temp_Temp_ = dmatrix(0,Nr_-1,0,Ntheta_-1);

      /* read the data */
      for (counter1=0; counter1<Ntheta_-2; counter1++) { /* loop over theta */
	fscanf(Temp_FILE,"%lf",&Temp_theta_[counter1+1]);
	if (_INITIAL_GRID_ARRAY_theta_min > Temp_theta_[counter1+1])
	  Temp_theta_[counter1+1] = _INITIAL_GRID_ARRAY_theta_min;
	if (_INITIAL_GRID_ARRAY_theta_max < Temp_theta_[counter1+1])
	  Temp_theta_[counter1+1] = _INITIAL_GRID_ARRAY_theta_max;
	if (counter1 == 0) 
          Temp_theta_[0] = _INITIAL_GRID_ARRAY_theta_min;
	if (counter1 == Ntheta_-3) 
          Temp_theta_[Ntheta_-1] = _INITIAL_GRID_ARRAY_theta_max;

	counter2=0;
	R1=1.0;
	do { /*loop over the radial list of temepratures */ 
	  fscanf(Temp_FILE,"%lf",&R2);
	  fscanf(Temp_FILE,"%lf",&T2);
	  if (counter2==0) {
	    Temp_Temp_[counter2][counter1+1]=T2;
	    counter2++;
	    T1=T2; R1=R2;
	  }
	  else {
	    while(Temp_r_[counter2]<=R2 && counter2<Nr_){
	      err = _MATH_LIB_log_interpolation(R1,T1,R2,T2,Temp_r_[counter2],
					 &(Temp_Temp_[counter2][counter1+1]));
	      counter2++;
	    }
	    T1=T2; R1=R2;
	  }
	} while (counter2<Nr_); /* end of the loop  over radius */
      } /* end of the loop over theta */

      /* copy the angular grid to the edge */ 
      for (counter2=0; counter2<Nr_; counter2++) {
	Temp_Temp_[counter2][0]=Temp_Temp_[counter2][1];
	Temp_Temp_[counter2][Ntheta_-1]=Temp_Temp_[counter2][Ntheta_-2];
      }
      break;

    default :
      printf("%d:---------------------------------------\n",my_MPI_rank);
      printf("%d:---- ERROR!! Wrong type of input file:\n",my_MPI_rank);
      printf("%d:               %s\n",my_MPI_rank,filename);
      fclose(Temp_FILE);
      return 1; 
      break;
    }
    fclose(Temp_FILE);

  }
  /*******************************************************************/
  else {
    printf("%d:---------------------------------------\n",my_MPI_rank);
    printf("%d:---- ERROR!! This temperature input type\n",my_MPI_rank);
    printf("%d:              does not exist\n",my_MPI_rank);
    return 1;
  }

  /* pass the data */
  *Temp_r = Temp_r_;
  *N_Temp_r = Nr_;
  *Temp_theta = Temp_theta_;
  *N_Temp_theta = Ntheta_;
  *Temp_Temp = Temp_Temp_;

  return 0;
}

