/* LELUYA - PERUNIKA program block */
/* This part of the LELUYA code is generating the computational grid */

#ifndef _PERUNIKA_prototypes_
#define _PERUNIKA_prototypes_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../_global_extern_variables.h"
#include "../ZELENJAK/ZELENJAK.h"
#include "../GOROSTAS/GOROSTAS.h"
#include "../VODENJAK/VODENJAK.h"
#include "../MATH_LIB/MATH_LIB.h"
#include "../MATH_LIB/nrutil.h"
#include "../PESJANEK/PESJANEK.h"
#include "../PATULJAK/PATULJAK.h"
#include "../VILOZMAJ/VILOZMAJ.h"
#include "../LEPAMARA/LEPAMARA.h"

int _PERUNIKA_main (int status);

int  _PERUNIKA_initialize_initial_arrays(int status);

struct _vertex_structure *_PERUNIKA_create_vertex (int *error);

int _PERUNIKA_destroy_vertex (struct _vertex_structure *v);

struct _triangle_structure *_PERUNIKA_create_triangle (int *error);

int _PERUNIKA_destroy_triangle (struct _triangle_structure *t);

int _PERUNIKA_create_initial_array (void);

double _PERUNIKA_initial_temperature (double x, double z, int dust_i);

double _PERUNIKA_initial_energy_density(double x,double z, int lambda_k);

double _PERUNIKA_temperature(double x,double z, int dust_i,int status,
                         double *temp_error);

double _PERUNIKA_optical_depth_distance (double Ax,double Az,
                                     double Bx,double Bz,
                                     int lambda_computational_grid,
                                     int status,short int *split_flag);

int _PERUNIKA_analyze_triangle (struct _triangle_structure *parent_triangle,
                            int lambda_computational_grid,int status);

double _PERUNIKA_distance_to_edge(struct _vertex_structure *point,
                              double edge_point_x,double edge_point_z,
                              int lambda_computational_grid, int curvature,
                              double *edge_point_distance,int status);

int _PERUNIKA_edge_triangles_removal (int lambda_computational_grid,int status);

struct _polygon_structure *_PERUNIKA_create_polygon_point 
                              (struct _polygon_structure *polygon,int *error);

int _PERUNIKA_split_polygon (struct _polygon_structure *polygon);

struct _vertex_structure *_PERUNIKA_adjacent_vertex_in_triangle(
		   struct _vertex_structure *v, struct _triangle_structure *t);

struct _vertex_structure *_PERUNIKA_adjacent_antivertex_in_triangle(
                   struct _vertex_structure *v, struct _triangle_structure *t);

int _PERUNIKA_clean_the_vertex_list (void);

struct _triangle_structure *_PERUNIKA_adjacent_triangle(
      struct _vertex_structure *vertex, struct _triangle_structure *triangle);

int _PERUNIKA_sublimation_extension (void);

int _PERUNIKA_locate_point_in_old_grid(double x,double z,int dust_i,
                                 struct _triangle_structure **old_triangle,
				 struct _quadrangle_structure **old_quadrangle);

struct _vertex_triangles   *_PERUNIKA_create_new_vertex_triangle (
	                                 struct _triangle_structure *triangle, 
                                         struct _vertex_structure *vertex);

void _PERUNIKA_destroy_vertex_triangle_list (struct _vertex_structure   *v, 
                                 struct _triangle_structure *t );

int _PERUNIKA_surface_grid_refinement (struct _vertex_structure   *vertex_edge1,
                                   struct _vertex_structure   *vertex_edge2,
                                   struct _vertex_structure   *vertex_in1,
                                   struct _vertex_structure   *vertex_in2,
                                   int  direction_1, int direction_2,
                                   int  direction_3,
                                   struct _vertex_structure   *vertex_tip,
                                   int lambda_computational_grid, int status);

int  _PERUNIKA_mark_optically_deep_vertices(void);

int _PERUNIKA_add_point_to_grids(double x, double z);

int _PERUNIKA_add_points_along_radial_line (double theta, double limit);

int _PERUNIKA_split_triangle_at_given_point(double x, double z);

int _PERUNIKA_triangle_integral_step (struct _triangle_structure *t);

void _PERUNIKA_sort_triangles_at_vertex(struct _vertex_structure *vertex);

int _PERUNIKA_smooth_grid_temperature (void);

int _PERUNIKA_smooth_grid_cells_shape (void);

int _PERUNIKA_create_azimuthal_grid (int lambda_computational_grid,int status);

void _PERUNIKA_remove_hot_dust (int status);

#endif
