/* LELUYA - PERUNIKA program block */

/* This function adds a given point to the computational grids */

#include "PERUNIKA.h"

/*************************************************************************/
/* subroutine for calculation of the 3D liner interpolation coefficients */
void _PERUNIKA_add_point_to_grids_Interpolation_Coefficient 
       (double x1, double y1, double x2, double y2, double x3, double y3, 
        double x, double y, double *c1, double *c2, double *c3);
/*************************************************************************/

/*************************************************************************/
int _PERUNIKA_add_point_to_grids(double x, double z) {

  int err;/* error flags */
  int dust_i, lambda_k, lambda_kk; /* various counters */
  int lambda_N_max; /* wavelength with the most grid points */
  int N_max;        /* number of vertices at lambda_N_max */
  double r; /* sqrt(x^2 + z^2) */
  struct _triangle_structure *interpolation_triangle;
  double c1,c2,c3;    /* interpolation coefficients */
  struct _vertex_structure *v; /* temporary vertex  (x,z) */
  struct _vertex_structure *v1,*v2,*v3; /* vertexes of interpolation_triangle */
  struct _vertex_structure *new_vertex; /* vertex  (x,z) */
  double D1,D2,D3; /* distance of (x,z) to the parent triangle sides */
  double *U; /* new array of energies */
  struct _triangle_structure *trianA,*trianB,*trianC; /* new triangles */
  struct _vertex_triangles   *vertex_triangle; /* vertex link to a triangle */
  struct _triangle_structure *side_triangle; /* adjacent triangle */
  struct _vertex_structure *P12, *P23, *P31;/* side-vertices from adjacent triangles */
  double k; /* slope of a triangle's edge */
  double newx, newz; /* new (x.z) if it is moved to the edge */
  int point_on_edge; /* flag which shows where the point is */
  int type;

  printf("    ADD A NEW POINT AT (%lf,%lf) :\n",x,z);

  r = sqrt (x*x+z*z);
  if (r>Rout || r<=Source_Radius) {
printf("ERROR: the required point (%lf,%lf) is out of the computational domain\n",x,z);
    return 1;
  }

  /* find a grid with the largest number of points (let's call it grid_N_max)*/
  lambda_N_max = 0;
  N_max = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++) {
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > N_max) {
      N_max = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES;
      lambda_N_max = lambda_k;
    }
  }

  /* find which triangle it belongs to */
  start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
  start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
  err = _ZELENJAK_locate_point_in_grid(x,z,&interpolation_triangle);

  /* is this out of te grid ? */
  if (interpolation_triangle==NULL) {
    printf("ERROR: the required point (%lf,%lf) is out of the grid\n",x,z);
    return 1;
  }
  /* is this in empty space ? */
  err = TRUE;
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
	 if ( interpolation_triangle->v1->T_dust[dust_i]>0.0 &&
	      interpolation_triangle->v2->T_dust[dust_i]>0.0 &&
              interpolation_triangle->v3->T_dust[dust_i]>0.0)
        {err=FALSE;break;}
  if (err) {
    printf("ERROR: the required point (%lf,%lf) is in empty space\n",x,z);
    return 1;
  }

  /* find the interpolation coefficients */
  _PERUNIKA_add_point_to_grids_Interpolation_Coefficient
         (interpolation_triangle->v1->x, interpolation_triangle->v1->z,
          interpolation_triangle->v2->x, interpolation_triangle->v2->z,
          interpolation_triangle->v3->x, interpolation_triangle->v3->z,
          x,z,&c1,&c2,&c3);

  /* create temporary vertex */
  v = (struct _vertex_structure *) malloc (sizeof(struct _vertex_structure));
  v->T_dust = dvector(0, Max_Dust_Type-1);
  v->T_error = dvector(0, Max_Dust_Type-1);
  v->x = x; v->z = z;
  
  /* find temperature */
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++){
    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
      v->T_dust[dust_i] =c1*interpolation_triangle->v1->T_dust[dust_i]+
                         c2*interpolation_triangle->v2->T_dust[dust_i]+
                         c3*interpolation_triangle->v3->T_dust[dust_i];
      v->T_error[dust_i]=c1*interpolation_triangle->v1->T_error[dust_i]+
                         c2*interpolation_triangle->v2->T_error[dust_i]+
                         c3*interpolation_triangle->v3->T_error[dust_i];
    }
    else {v->T_dust[dust_i]=_T99999_;v->T_error[dust_i]=1.0E+30;}
  }
  
  /*************************************************************************/
  /* go over wavelength grids */
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
    if(grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > 0) {
     
      v->x = x; v->z = z;
      type = -1;
      /* printf("--- lambda index = %d\n",lambda_k); */
      start_triangle_list = grid_per_lambda[lambda_k].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_k].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_TRIANGLES;
      /* find which triangle it belongs to */
      err = _ZELENJAK_locate_point_in_grid(x,z,&interpolation_triangle);
      /* set some vertex variables */
      P12 = P23 = P31 = NULL;
      side_triangle = NULL;
      v1 = interpolation_triangle->v1;
      v2 = interpolation_triangle->v2;
      v3 = interpolation_triangle->v3;
      /* printf("Parent triangle [%E,%E][%E,%E][%E,%E]\n",
             v1->x,v1->z,v2->x,v2->z,v3->x,v3->z);*/

      /* find the distance to the triangle sides */
      D1 = _MATH_LIB_distance_point_to_line (v,v1,v2);
      D2 = _MATH_LIB_distance_point_to_line (v,v2,v3);
      D3 = _MATH_LIB_distance_point_to_line (v,v3,v1);
      /*if (D1<0.05*interpolation_triangle->integral_step) 
        printf(" warning: distance to V1V2 side is only %E\n",D1);
      if (D2<0.05*interpolation_triangle->integral_step) 
        printf(" warning: distance to V2V3 side is only %E\n",D2);
      if (D3<0.05*interpolation_triangle->integral_step) 
        printf(" warning: distance to V3V1 side is only %E\n",D3);*/

      point_on_edge = 0;

      if (D1<0.01*interpolation_triangle->integral_step ||
          D2<0.01*interpolation_triangle->integral_step ||
          D3<0.01*interpolation_triangle->integral_step) {
        /*printf(" WARNING: moving point to the triangle's edge\n");*/
	if (D1<D2 && D1<D3) {
	  /* moving to the v1-v2 edge */
          point_on_edge = 12;
          if (v1->x==v2->x) {newx=v1->x;newz=z;} /* this is a simple case */
          else if (v1->z==v2->z) {newx=x;newz=v1->z;} /* another simple case */
	  else {
            k = (v2->z - v1->z)/(v2->x - v1->x); /* slope of the v1-v2 edge */
            newx = (k*(z-v1->z)+k*k*v1->x+x)/(k*k+1); /* new point */
            newz = -(newx-x)/k + z;
	  }
          /* check if the new point is too close to the triangle vertices */
          k = _MATH_LIB_distance(newx,newz,v2->x,v2->z)/
              _MATH_LIB_distance(v1->x,v1->z,v2->x,v2->z);
          if (k<0.2) {
	    if (DEBUG.CONTINUE == 210)
	      {newx = 0.2*v1->x+0.8*v2->x; newz = 0.2*v1->z+0.8*v2->z;}
	    else 
              {newx = 0.5*v1->x+0.5*v2->x; newz = 0.5*v1->z+0.5*v2->z;}
            /*newx = 0.2*v1->x+0.8*v2->x; newz = 0.2*v1->z+0.8*v2->z;*/
          }
          if (k>0.8) {
            if (DEBUG.CONTINUE == 210)
              {newx = 0.8*v1->x+0.2*v2->x; newz = 0.8*v1->z+0.2*v2->z;}
	    else
              {newx = 0.5*v1->x+0.5*v2->x; newz = 0.5*v1->z+0.5*v2->z;}
            /*newx = 0.8*v1->x+0.2*v2->x; newz = 0.8*v1->z+0.2*v2->z;*/
	  }
          /* set the new coordinates */ 
          v->x = newx; v->z = newz;
          /* find the side triangle */ 
          side_triangle = _PERUNIKA_adjacent_triangle(v2,interpolation_triangle);
	  /* find the side vertex */
          if (side_triangle!=NULL) {
            if (v1==side_triangle->v1) P12 = side_triangle->v2;
            else if (v1==side_triangle->v2) P12 = side_triangle->v3;
            else if (v1==side_triangle->v3) P12 = side_triangle->v1;
          }
          /* check if the new vertex is of type=2 */
	  if (v1->type==2 && v2->type==2) type =2 ;
	} /* end of moving to the v1-v2 edge */

	else if (D2<D1 && D2<D3) {
	  /* moving to the v2-v3 edge */
          point_on_edge = 23;
          if (v2->x==v3->x) {newx=v2->x;newz=z;} /* this is a simple case */
          else if (v2->z==v3->z) {newx=x;newz=v2->z;} /* another simple case */
	  else {
            k = (v3->z - v2->z)/(v3->x - v2->x); /* slope of the v2-v3 edge */
            newx = (k*(z-v2->z)+k*k*v2->x+x)/(k*k+1); /* new point */
            newz = -(newx-x)/k + z;
	  }
          /* check if the new point is too close to the triangle vertices */
          k = _MATH_LIB_distance(newx,newz,v3->x,v3->z)/
              _MATH_LIB_distance(v2->x,v2->z,v3->x,v3->z);
          if (k<0.2) {
	    if (DEBUG.CONTINUE == 210)
	      {newx = 0.2*v2->x+0.8*v3->x; newz = 0.2*v2->z+0.8*v3->z;}
	    else
              {newx = 0.5*v2->x+0.5*v3->x; newz = 0.5*v2->z+0.5*v3->z;}
            /*newx = 0.2*v2->x+0.8*v3->x; newz = 0.2*v2->z+0.8*v3->z;*/
          }
          if (k>0.8) {
	    if (DEBUG.CONTINUE == 210)
	      {newx = 0.8*v2->x+0.2*v3->x; newz = 0.8*v2->z+0.2*v3->z;}
	    else
              {newx = 0.5*v2->x+0.5*v3->x; newz = 0.5*v2->z+0.5*v3->z;}
            /*newx = 0.8*v2->x+0.2*v3->x; newz = 0.8*v2->z+0.2*v3->z;*/
          }
          /* set the new coordinates */ 
          v->x = newx; v->z = newz;
          /* find the side triangle */ 
          side_triangle = _PERUNIKA_adjacent_triangle(v3,interpolation_triangle);
          /* find the side vertex */
          if (side_triangle!=NULL) {
            if (v2==side_triangle->v1) P23 = side_triangle->v2;
            else if (v2==side_triangle->v2) P23 = side_triangle->v3;
            else if (v2==side_triangle->v3) P23 = side_triangle->v1;
          }
          /* check if the new vertex is of type=2 */
	  if (v2->type==2 && v3->type==2) type =2 ;
	} /* end of moving to the v2-v3 edge */

	else if (D3<D1 && D3<D2) {
	  /* moving to the v3-v1 edge */
          point_on_edge = 31;
          if (v3->x==v1->x){newx=v3->x;newz=z;} /* this is a simple case */
          else if (v3->z==v1->z) {newx=x;newz=v3->z;} /* another simple case */
	  else {
            k = (v1->z - v3->z)/(v1->x - v3->x); /* slope of the v3-v1 edge */
            newx = (k*(z-v3->z)+k*k*v3->x+x)/(k*k+1); /* new point */
            newz = -(newx-x)/k + z;
	  }
          /* check if the new point is too close to the triangle vertices */
          k = _MATH_LIB_distance(newx,newz,v1->x,v1->z)/
              _MATH_LIB_distance(v3->x,v3->z,v1->x,v1->z);
          if (k<0.2) {
            if (DEBUG.CONTINUE == 210)
	      {newx = 0.2*v3->x+0.8*v1->x; newz = 0.2*v3->z+0.8*v1->z;}
	    else
              {newx = 0.5*v3->x+0.5*v1->x; newz = 0.5*v3->z+0.5*v1->z;}
            /*newx = 0.2*v3->x+0.8*v1->x; newz = 0.2*v3->z+0.8*v1->z;*/
          }
          if (k>0.8) {
            if (DEBUG.CONTINUE == 210)
	      {newx = 0.8*v3->x+0.2*v1->x; newz = 0.8*v3->z+0.2*v1->z;}
	    else 
              {newx = 0.5*v3->x+0.5*v1->x; newz = 0.5*v3->z+0.5*v1->z;}
            /*newx = 0.8*v3->x+0.2*v1->x; newz = 0.8*v3->z+0.2*v1->z;*/
          }
	  /* set the new coordinates */ 
          v->x = newx; v->z = newz;
          /* find the side triangle */ 
          side_triangle = _PERUNIKA_adjacent_triangle(v1,interpolation_triangle);
	  /* find the side vertex */
          if (side_triangle!=NULL) {
            if (v3==side_triangle->v1) P31 = side_triangle->v2;
            else if (v3==side_triangle->v2) P31 = side_triangle->v3;
            else if (v3==side_triangle->v3) P31 = side_triangle->v1;
          }
          /* check if the new vertex is of type=2 */
	  if (v3->type==2 && v1->type==2) type =2 ;
	} /* end of moving to the v3-v1 edge */
      }

      /* find the interpolation coefficients */
      _PERUNIKA_add_point_to_grids_Interpolation_Coefficient
            (v1->x, v1->z,v2->x, v2->z,v3->x, v3->z, v->x, v->z, &c1, &c2, &c3);

      /* create vertex */
      new_vertex = _PERUNIKA_create_vertex (&err);
      new_vertex->v1=new_vertex->v2=new_vertex->v3=new_vertex->v4=
	new_vertex->v5=new_vertex->v6=NULL;
      new_vertex->identification_number = TOTAL_NUMBER_OF_VERTICES-1;
      new_vertex->level = 667;
      if (type==2) new_vertex->type =2;
      else new_vertex->type = 1;
      new_vertex->x = v->x;  new_vertex->z = v->z;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
        if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
          new_vertex->T_dust[dust_i] =c1*v1->T_dust[dust_i]+
                                      c2*v2->T_dust[dust_i]+
                                      c3*v3->T_dust[dust_i];
          new_vertex->T_error[dust_i]=1E30;
        }
	else {
          new_vertex->T_dust[dust_i]=_T99999_;
          new_vertex->T_error[dust_i]=1.0E+30;
	}
      }
      new_vertex->Eta =  Eta(v->x,v->z,lambda_k,new_vertex->T_dust,&err);

 
      /* update the energy with this new vertex included */
      for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
	if (grid_per_lambda[lambda_kk].start_vertex==start_vertex_list) {
          grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES = TOTAL_NUMBER_OF_VERTICES;
          /* stellar */
	  U = dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
          for (err=0;err<TOTAL_NUMBER_OF_VERTICES-1;err++)
	    U[err]=grid_per_lambda[lambda_kk].Ustar[err];
          U[TOTAL_NUMBER_OF_VERTICES-1]=0.0;/* this is the new one */
          free_dvector(grid_per_lambda[lambda_kk].Ustar,
                       0,TOTAL_NUMBER_OF_VERTICES-2);
          grid_per_lambda[lambda_kk].Ustar = U;
          /* emission */
	  U = dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
          for (err=0;err<TOTAL_NUMBER_OF_VERTICES-1;err++)
	    U[err]=grid_per_lambda[lambda_kk].Uem[err];
          U[TOTAL_NUMBER_OF_VERTICES-1]=0.0;/* this is the new one */
          free_dvector(grid_per_lambda[lambda_kk].Uem,
                       0,TOTAL_NUMBER_OF_VERTICES-2);
          grid_per_lambda[lambda_kk].Uem = U;
          /* scattering */
	  U = dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
          for (err=0;err<TOTAL_NUMBER_OF_VERTICES-1;err++)
	    U[err]=grid_per_lambda[lambda_kk].Usca[err];
          U[TOTAL_NUMBER_OF_VERTICES-1]=0.0;/* this is the new one */
          free_dvector(grid_per_lambda[lambda_kk].Usca,
                       0,TOTAL_NUMBER_OF_VERTICES-2);
          grid_per_lambda[lambda_kk].Usca = U;
          /* total */
	  U = dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
          for (err=0;err<TOTAL_NUMBER_OF_VERTICES-1;err++)
	    U[err]=grid_per_lambda[lambda_kk].Utot_new[err];
          U[TOTAL_NUMBER_OF_VERTICES-1]= 
                 c1*grid_per_lambda[lambda_kk].Utot_new[
                            v1->identification_number]
	       + c2*grid_per_lambda[lambda_kk].Utot_new[
                            v2->identification_number]
	       + c3*grid_per_lambda[lambda_kk].Utot_new[
			    v3->identification_number];
          free_dvector(grid_per_lambda[lambda_kk].Utot_new,
                       0,TOTAL_NUMBER_OF_VERTICES-2);
          grid_per_lambda[lambda_kk].Utot_new = U;
	}

      /* create new triangles: trianA, trianB, trianC 
      
                       V3-----------V2
                        \\ trianB  //
                         \ \     / /
                          \  \ /  /
                    trianC \  V  / trianA
                            \ | /
                             \|/
                              V1  
      */
      if (P12==NULL && point_on_edge!=12) {
        trianA = _PERUNIKA_create_triangle(&err);
        trianA->level = interpolation_triangle->level+1;
        trianA->v1 = new_vertex;
        trianA->v2 = v1;
        trianA->v3 = v2;
        trianA->ID_number = interpolation_triangle->ID_number;
      }

      if (P23==NULL && point_on_edge!=23) {
        trianB = _PERUNIKA_create_triangle(&err);
        trianB->level = interpolation_triangle->level+1;
        trianB->v1 = new_vertex;
        trianB->v2 = v2;
        trianB->v3 = v3;
        trianB->ID_number = TOTAL_NUMBER_OF_TRIANGLES-2;
      }

      if (P31==NULL && point_on_edge!=31) {
        trianC = _PERUNIKA_create_triangle(&err);
        trianC->level = interpolation_triangle->level+1;
        trianC->v1 = new_vertex;
        trianC->v2 = v3;
        trianC->v3 = v1;
        trianC->ID_number = TOTAL_NUMBER_OF_TRIANGLES-2;
      }

      if (P12==NULL && point_on_edge!=12) {
        /* bounding box: trianA*/
        trianA->bound_box_down_left_x=trianA->v1->x;
        trianA->bound_box_down_left_z=trianA->v1->z;
        trianA->bound_box_up_right_x=trianA->v1->x;
        trianA->bound_box_up_right_z=trianA->v1->z;
        if (trianA->bound_box_down_left_x>trianA->v2->x)
          trianA->bound_box_down_left_x=trianA->v2->x;
        if (trianA->bound_box_down_left_z>trianA->v2->z)
          trianA->bound_box_down_left_z=trianA->v2->z;
        if (trianA->bound_box_up_right_x<trianA->v2->x)
          trianA->bound_box_up_right_x=trianA->v2->x;
        if (trianA->bound_box_up_right_z<trianA->v2->z)
          trianA->bound_box_up_right_z=trianA->v2->z;
        if (trianA->bound_box_down_left_x>trianA->v3->x)
          trianA->bound_box_down_left_x=trianA->v3->x;
        if (trianA->bound_box_down_left_z>trianA->v3->z)
          trianA->bound_box_down_left_z=trianA->v3->z;
        if (trianA->bound_box_up_right_x<trianA->v3->x)
          trianA->bound_box_up_right_x=trianA->v3->x;
        if (trianA->bound_box_up_right_z<trianA->v3->z)
          trianA->bound_box_up_right_z=trianA->v3->z;
      }
      if (P23==NULL && point_on_edge!=23) {
        /* bounding box: trianB*/
        trianB->bound_box_down_left_x=trianB->v1->x;
        trianB->bound_box_down_left_z=trianB->v1->z;
        trianB->bound_box_up_right_x=trianB->v1->x;
        trianB->bound_box_up_right_z=trianB->v1->z;
        if (trianB->bound_box_down_left_x>trianB->v2->x)
          trianB->bound_box_down_left_x=trianB->v2->x;
        if (trianB->bound_box_down_left_z>trianB->v2->z)
          trianB->bound_box_down_left_z=trianB->v2->z;
        if (trianB->bound_box_up_right_x<trianB->v2->x)
          trianB->bound_box_up_right_x=trianB->v2->x;
        if (trianB->bound_box_up_right_z<trianB->v2->z)
          trianB->bound_box_up_right_z=trianB->v2->z;
        if (trianB->bound_box_down_left_x>trianB->v3->x)
          trianB->bound_box_down_left_x=trianB->v3->x;
        if (trianB->bound_box_down_left_z>trianB->v3->z)
          trianB->bound_box_down_left_z=trianB->v3->z;
        if (trianB->bound_box_up_right_x<trianB->v3->x)
          trianB->bound_box_up_right_x=trianB->v3->x;
        if (trianB->bound_box_up_right_z<trianB->v3->z)
          trianB->bound_box_up_right_z=trianB->v3->z;
      }

      if (P31==NULL && point_on_edge!=31) {
        /* bounding box: trianC*/
        trianC->bound_box_down_left_x=trianC->v1->x;
        trianC->bound_box_down_left_z=trianC->v1->z;
        trianC->bound_box_up_right_x=trianC->v1->x;
        trianC->bound_box_up_right_z=trianC->v1->z;
        if (trianC->bound_box_down_left_x>trianC->v2->x)
          trianC->bound_box_down_left_x=trianC->v2->x;
        if (trianC->bound_box_down_left_z>trianC->v2->z)
          trianC->bound_box_down_left_z=trianC->v2->z;
        if (trianC->bound_box_up_right_x<trianC->v2->x)
          trianC->bound_box_up_right_x=trianC->v2->x;
        if (trianC->bound_box_up_right_z<trianC->v2->z)
          trianC->bound_box_up_right_z=trianC->v2->z;
        if (trianC->bound_box_down_left_x>trianC->v3->x)
          trianC->bound_box_down_left_x=trianC->v3->x;
        if (trianC->bound_box_down_left_z>trianC->v3->z)
          trianC->bound_box_down_left_z=trianC->v3->z;
        if (trianC->bound_box_up_right_x<trianC->v3->x)
          trianC->bound_box_up_right_x=trianC->v3->x;
        if (trianC->bound_box_up_right_z<trianC->v3->z)
          trianC->bound_box_up_right_z=trianC->v3->z;
      }

      /* destroy the parent triangle */
      err=_PERUNIKA_destroy_triangle(interpolation_triangle);
      
      /* update the triangle list */
      for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
	if (grid_per_lambda[lambda_kk].start_vertex==start_vertex_list) {
          grid_per_lambda[lambda_kk].start_triangle = start_triangle_list; 
          grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_TRIANGLES = TOTAL_NUMBER_OF_TRIANGLES;
	}

      /* re-number the triangle list */
      lambda_kk=0;
      interpolation_triangle = start_triangle_list;
      do {
        interpolation_triangle->ID_number=lambda_kk;
        lambda_kk++;
	interpolation_triangle=interpolation_triangle->next;
      } while (interpolation_triangle != start_triangle_list);

      /* add these new triangles to the lists at each vertex */
      if (P12==NULL && point_on_edge!=12) {
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA,trianA->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA,trianA->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA,trianA->v3);
      }
      if (P23==NULL && point_on_edge!=23) {
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB,trianB->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB,trianB->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB,trianB->v3);
      }
      if (P31==NULL && point_on_edge!=31) {
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC,trianC->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC,trianC->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC,trianC->v3);
      }     
      /* sort the list of triangles at each vertex */
      _PERUNIKA_sort_triangles_at_vertex(new_vertex);
      _PERUNIKA_sort_triangles_at_vertex(v1);
      _PERUNIKA_sort_triangles_at_vertex(v2);
      _PERUNIKA_sort_triangles_at_vertex(v3);

      /* find the triangle step for each new traingle */
      if (P12==NULL && point_on_edge!=12) err = _PERUNIKA_triangle_integral_step(trianA);
      if (P23==NULL && point_on_edge!=23) err = _PERUNIKA_triangle_integral_step(trianB);
      if (P31==NULL && point_on_edge!=31) err = _PERUNIKA_triangle_integral_step(trianC);

      /* split the adjacent triangle if the new vertex is on the edge */
      if (P12!=NULL || P23!=NULL || P31!=NULL) {
	/* create new triangles */
        trianA = _PERUNIKA_create_triangle(&err);
        trianA->level = side_triangle->level+1;
        trianB = _PERUNIKA_create_triangle(&err);
        trianB->level = side_triangle->level+1;
	if (P12!=NULL) {
          trianA->v1 = P12;
          trianA->v2 = v2;
          trianA->v3 = new_vertex;
          trianB->v1 = new_vertex;
          trianB->v2 = v1;
          trianB->v3 = P12;
	}
	if (P23!=NULL) {
          trianA->v1 = P23;
          trianA->v2 = v3;
          trianA->v3 = new_vertex;
          trianB->v1 = new_vertex;
          trianB->v2 = v2;
          trianB->v3 = P23;
	}
	if (P31!=NULL) {
          trianA->v1 = P31;
          trianA->v2 = v1;
          trianA->v3 = new_vertex;
          trianB->v1 = new_vertex;
          trianB->v2 = v3;
          trianB->v3 = P31;
	}

	/*printf("New Side triangle A [%E,%E][%E,%E][%E,%E]\n",
              trianA->v1->x,trianA->v1->z,trianA->v2->x,trianA->v2->z,
              trianA->v3->x,trianA->v3->z);
	  printf("New Side triangle B [%E,%E][%E,%E][%E,%E]\n",
              trianB->v1->x,trianB->v1->z,trianB->v2->x,trianB->v2->z,
              trianB->v3->x,trianB->v3->z);*/

        trianA->ID_number = side_triangle->ID_number;
        trianB->ID_number = TOTAL_NUMBER_OF_TRIANGLES-2;
        /* bounding box: trianA*/
        trianA->bound_box_down_left_x=trianA->v1->x;
        trianA->bound_box_down_left_z=trianA->v1->z;
        trianA->bound_box_up_right_x=trianA->v1->x;
        trianA->bound_box_up_right_z=trianA->v1->z;
        if (trianA->bound_box_down_left_x>trianA->v2->x)
          trianA->bound_box_down_left_x=trianA->v2->x;
        if (trianA->bound_box_down_left_z>trianA->v2->z)
          trianA->bound_box_down_left_z=trianA->v2->z;
        if (trianA->bound_box_up_right_x<trianA->v2->x)
          trianA->bound_box_up_right_x=trianA->v2->x;
        if (trianA->bound_box_up_right_z<trianA->v2->z)
          trianA->bound_box_up_right_z=trianA->v2->z;
        if (trianA->bound_box_down_left_x>trianA->v3->x)
          trianA->bound_box_down_left_x=trianA->v3->x;
        if (trianA->bound_box_down_left_z>trianA->v3->z)
          trianA->bound_box_down_left_z=trianA->v3->z;
        if (trianA->bound_box_up_right_x<trianA->v3->x)
          trianA->bound_box_up_right_x=trianA->v3->x;
        if (trianA->bound_box_up_right_z<trianA->v3->z)
          trianA->bound_box_up_right_z=trianA->v3->z;
        /* bounding box: trianB*/
        trianB->bound_box_down_left_x=trianB->v1->x;
        trianB->bound_box_down_left_z=trianB->v1->z;
        trianB->bound_box_up_right_x=trianB->v1->x;
        trianB->bound_box_up_right_z=trianB->v1->z;
        if (trianB->bound_box_down_left_x>trianB->v2->x)
          trianB->bound_box_down_left_x=trianB->v2->x;
        if (trianB->bound_box_down_left_z>trianB->v2->z)
          trianB->bound_box_down_left_z=trianB->v2->z;
        if (trianB->bound_box_up_right_x<trianB->v2->x)
          trianB->bound_box_up_right_x=trianB->v2->x;
        if (trianB->bound_box_up_right_z<trianB->v2->z)
          trianB->bound_box_up_right_z=trianB->v2->z;
        if (trianB->bound_box_down_left_x>trianB->v3->x)
          trianB->bound_box_down_left_x=trianB->v3->x;
        if (trianB->bound_box_down_left_z>trianB->v3->z)
          trianB->bound_box_down_left_z=trianB->v3->z;
        if (trianB->bound_box_up_right_x<trianB->v3->x)
          trianB->bound_box_up_right_x=trianB->v3->x;
        if (trianB->bound_box_up_right_z<trianB->v3->z)
          trianB->bound_box_up_right_z=trianB->v3->z;
        /* destroy the parent triangle */
        err=_PERUNIKA_destroy_triangle(side_triangle);
        /* update the triangle list */
        for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
	  if (grid_per_lambda[lambda_kk].start_vertex==start_vertex_list) {
        grid_per_lambda[lambda_kk].start_triangle = start_triangle_list; 
        grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_TRIANGLES = TOTAL_NUMBER_OF_TRIANGLES;
	  }
        /* re-number the triangle list */
        lambda_kk=0;
        side_triangle = start_triangle_list;
        do {
          side_triangle->ID_number=lambda_kk;
          lambda_kk++;
	  side_triangle=side_triangle->next;
        } while (side_triangle != start_triangle_list);
        /* add these new triangles to the lists at each vertex */
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA,trianA->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA,trianA->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA,trianA->v3);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB,trianB->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB,trianB->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB,trianB->v3);
        /* sort the list of triangles at each vertex */
        _PERUNIKA_sort_triangles_at_vertex(new_vertex);
        _PERUNIKA_sort_triangles_at_vertex(trianA->v1);
        _PERUNIKA_sort_triangles_at_vertex(trianA->v2);
        _PERUNIKA_sort_triangles_at_vertex(trianB->v2);
        /* find the triangle step for each new traingle */
        err = _PERUNIKA_triangle_integral_step(trianA);
        err = _PERUNIKA_triangle_integral_step(trianB);

      } /* end of splitting the adjacent triangle */

      /* mark this grid as "checked" */
      for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
	if (grid_per_lambda[lambda_kk].start_vertex==start_vertex_list)
          grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES *= -1;
    }

  /* remove the mark from grids */
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
    if(grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES < 0)
      grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES *= -1;

  /* free memory */
  free_dvector(v->T_dust, 0, Max_Dust_Type-1);
  free_dvector(v->T_error, 0, Max_Dust_Type-1);
  free (v);

  return 0;
}

/*************************************************************************/
/*************************************************************************/
/* subroutine for calculation of the 3D liner interpolation coefficients */
void _PERUNIKA_add_point_to_grids_Interpolation_Coefficient 
       (double x1, double y1, double x2, double y2, double x3, double y3, 
        double x, double y, double *c1, double *c2, double *c3){

  double denominator;
  denominator = (x2-x1)*(y3-y1) - (x3-x1)*(y2-y1);
  if (denominator == 0.0) {
    *c1=0.0; *c2=0.0; *c3=0.0; return;
  }
  *c1 = 1 + ( (x-x1)*(y2-y3)+(y-y1)*(x3-x2) )/denominator;
  *c2 = ( (x-x1)*(y3-y1)-(y-y1)*(x3-x1) )/denominator;
  *c3 = ( (y-y1)*(x2-x1)-(x-x1)*(y2-y1) )/denominator;
  return;
}

/*************************************************************************/
