/* LELUYA - PERUNIKA program block */
/* This function splits triangles along a given radial line. */
/* (see -c 206 flag) */

/* The loop goes ("travels") step by step along the line given by the
   azimuthal angle theta. Each encountered triangle is split if at least one
   of its sides is bigger than limit*_ZELENJAK_size_limit */

#include "PERUNIKA.h"

/*********************************************************************/
/*** intersection point between two lines */
int _PERUNIKA_add_points_along_radial_line_intersection(double x1, double y1,
             double x2, double y2,double x3, double y3,double x4, double y4,
						    double *x0, double *y0);
/*********************************************************************/
int _PERUNIKA_add_points_along_radial_line (double theta, double limit) {

  struct _3DVector startpoint; /* the first point of traveling */
  struct _3DVector endpoint; /* the end pint of traveling */
  struct _3DVector min_point, max_point; /* extremes of the traveling line  */
  struct _3DVector direction; /* direction of traveling */
  struct _3DVector current_point; /* a point in 3D space during traveling */
  struct _3DVector next_point; /* point after current_point */
  struct _3DVector point1,point2; /* intersection between the traveling line
                                     and current_vertex */ 
  struct _3DVector mid_point; /* point between two intersections */
  struct _triangle_structure *current_triangle; /* location of current_point */
  struct _triangle_structure *next_triangle; /* location of next_point */
  double distance_total; /* distance between startpoint and endpoint */
  double current_distance; /* distance between startpoint and current_point */
  double distance_step; /* spatial step of traveling */
  double scale_limit; /* scale limit at the point of traveling */ 
  double side1,side2,side3; /* length of the triangle's sides */ 
  int lambda_N_max; /* wavelength with the most grid points */
  int N_max;        /* number of vertices at lambda_N_max */
  int lambda_k;     /* wavelength counter */
  int err;          /* error flag */
  int next_point_flag;

  /* find a grid with the largest number of points (let's call it grid_N_max)*/
  lambda_N_max = 0;
  N_max = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++) {
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > N_max) {
      N_max = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES;
      lambda_N_max = lambda_k;
    }
  }

  direction.x = sin(theta);
  direction.y = 0.0;
  direction.z = cos(theta);

  if (DEBUG.CONTINUE == 210) {
    startpoint.x = direction.x*DEBUG.ADD_POINTS_R1;
    startpoint.y = 0.0;
    startpoint.z = direction.z*DEBUG.ADD_POINTS_R1;
    endpoint.x = direction.x*DEBUG.ADD_POINTS_R2;
    endpoint.y = 0.0;
    endpoint.z = direction.z*DEBUG.ADD_POINTS_R2;
  }
  else {
    startpoint.x = direction.x*1.01; /* 1.01 is arbitrarily chosen */
    startpoint.y = 0.0;
    startpoint.z = direction.z*1.01;/* 1.01 is arbitrarily chosen */
    endpoint.x = direction.x*Rout;
    endpoint.y = 0.0;
    endpoint.z = direction.z*Rout;
  }

  distance_total=_MATH_LIB_3Ddistance(startpoint,endpoint);

  current_point = startpoint;
  min_point.x = direction.x*0.1;
  min_point.y = 0.0;
  min_point.z = direction.z*0.1;

  max_point.x = direction.x*1.5*Rout;
  max_point.y = 0.0;
  max_point.z = direction.z*1.5*Rout;
 
  start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
  start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
  err = _ZELENJAK_locate_point_in_grid(current_point.x,current_point.z,
				      &current_triangle);
  current_distance = 0.0;

  /************************************************************************/
  /************************ 206 and 207 ***********************************/
  if (DEBUG.CONTINUE == 206 || DEBUG.CONTINUE == 207) {

  /* now we can start traveling along the radial line */
  while (current_distance<distance_total){
    start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
    start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;

    if (current_triangle!=NULL) {
      /* decide if the triangle needs to be divided */
      scale_limit = limit*_ZELENJAK_size_limit(current_point.x,current_point.z);
      side1 = _MATH_LIB_distance(current_triangle->v1->x,current_triangle->v1->z,
                                 current_triangle->v2->x,current_triangle->v2->z);
      side2 = _MATH_LIB_distance(current_triangle->v2->x,current_triangle->v2->z,
                                 current_triangle->v3->x,current_triangle->v3->z);
      side3 = _MATH_LIB_distance(current_triangle->v3->x,current_triangle->v3->z,
                                 current_triangle->v1->x,current_triangle->v1->z);
      if (side1 > scale_limit || side2 > scale_limit || side3 > scale_limit)  {
	/* SPLIT THE TRIANGLE */

/*************************************************
 *************************************************
 ************** this was a part that put the mid point between the two
 ************** intersections with the triangle

        // find intersection between the traveling line and the triangle's sides 
        if (_MATH_LIB_intersecting_lines(min_point.x,min_point.z,
                             max_point.x,max_point.z,
                             current_triangle->v1->x,current_triangle->v1->z,
                             current_triangle->v2->x,current_triangle->v2->z)) {
	  // side V1-V2 has intersection 
          err=_PERUNIKA_add_points_along_radial_line_intersection(
                       min_point.x,min_point.z,
                       max_point.x,max_point.z,
                       current_triangle->v1->x,current_triangle->v1->z,
                       current_triangle->v2->x,current_triangle->v2->z,
                       &(point1.x),&(point1.z));
          // find the second intersection 
          if (_MATH_LIB_intersecting_lines(min_point.x,min_point.z,
                             max_point.x,max_point.z,
                             current_triangle->v2->x,current_triangle->v2->z,
                             current_triangle->v3->x,current_triangle->v3->z))
	    // side V2-V3 has intersection 
              err=_PERUNIKA_add_points_along_radial_line_intersection(
                       min_point.x,min_point.z,
                       max_point.x,max_point.z,
                       current_triangle->v2->x,current_triangle->v2->z,
                       current_triangle->v3->x,current_triangle->v3->z,
                       &(point2.x),&(point2.z));
	  else // side V3-V1 has intersection 
              err=_PERUNIKA_add_points_along_radial_line_intersection(
                       min_point.x,min_point.z,
                       max_point.x,max_point.z,
                       current_triangle->v3->x,current_triangle->v3->z,
                       current_triangle->v1->x,current_triangle->v1->z,
                       &(point2.x),&(point2.z));
	}
	else {
	  // side V2-V3 has intersection 
          err=_PERUNIKA_add_points_along_radial_line_intersection(
                       min_point.x,min_point.z,
                       max_point.x,max_point.z,
                       current_triangle->v2->x,current_triangle->v2->z,
                       current_triangle->v3->x,current_triangle->v3->z,
                       &(point1.x),&(point1.z));
          // side V3-V1 has intersection 
          err=_PERUNIKA_add_points_along_radial_line_intersection(
                       min_point.x,min_point.z,
                       max_point.x,max_point.z,
                       current_triangle->v3->x,current_triangle->v3->z,
                       current_triangle->v1->x,current_triangle->v1->z,
                       &(point2.x),&(point2.z)); 
	}

	// now find the mid_point 
        mid_point.x = 0.5*(point1.x+point2.x);
        mid_point.z = 0.5*(point1.z+point2.z);
********************************************
*******************************************/

	mid_point.x = (current_triangle->v1->x+current_triangle->v2->x+
                       current_triangle->v3->x)/3.0;
        mid_point.z = (current_triangle->v1->z+current_triangle->v2->z+
                       current_triangle->v3->z)/3.0;
 
        /* the next step has to use the updated triangle list */ 
        current_triangle = NULL;

	/* create a new vertex at mid_point */ 
        if (DEBUG.CONTINUE == 206)
           err=_PERUNIKA_add_point_to_grids(mid_point.x,mid_point.z);
	/* split triangles at mid_point */
        if (DEBUG.CONTINUE == 207) {
           err=_PERUNIKA_split_triangle_at_given_point(mid_point.x,mid_point.z);
           
	}
      }
    }

    start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
    start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
    do {
      /* find the new point */
      distance_step = _ZELENJAK_3Dstep (current_point,current_triangle,direction,
                                       1.0,&next_point,&next_triangle);

      /* check if we reached the endpoint */
      if (current_distance + distance_step > distance_total) {
        distance_step = distance_total - current_distance;
        current_distance = distance_total;
       current_point = endpoint;
       start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
       start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
       err=_ZELENJAK_locate_point_in_grid(current_point.x,current_point.z,
					&current_triangle);
      }
      else { /* ...or we have not reached the endpoint */ 
        current_distance += distance_step;
        current_point = next_point;
        current_triangle = next_triangle;
      }

      /* make sure that the next step is within the old triangles */
      next_point_flag = FALSE;
      if (current_triangle!=NULL) {
        if (DEBUG.CONTINUE == 206 && current_triangle->ID_number
	    >=  grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES - 3)
	  next_point_flag = TRUE;
        if (DEBUG.CONTINUE == 207 && current_triangle->ID_number
	    >=  grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES - 10)
	  next_point_flag = TRUE;
      }

    } while (current_distance<distance_total && next_point_flag) ;

  } /* end of the traveling loop */ 

  }
  /************************************************************************/
  /************************ 210 *******************************************/
  else if (DEBUG.CONTINUE == 210) {

  /* now we can start traveling along the radial line */
  while (current_distance<distance_total){

    start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
    start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
  
    current_triangle=NULL;
    /* find where the current point is in the grid */
    err = _ZELENJAK_locate_point_in_grid(current_point.x,current_point.z,
				        &current_triangle);

    /* set the next point to "unknown" */
    next_point.x = next_point.z = -1.0;next_point.y=0.0;

    if (current_triangle!=NULL) {
      /* search for the next point: intersection of the radial line with 
         the triangle's side */
      if (_MATH_LIB_intersecting_lines(current_point.x,current_point.z,
                                       max_point.x,max_point.z,
                             current_triangle->v1->x,current_triangle->v1->z,
                             current_triangle->v2->x,current_triangle->v2->z))
        /* side V1-V2 has intersection */
        err=_PERUNIKA_add_points_along_radial_line_intersection(
                       current_point.x,current_point.z,
                       max_point.x,max_point.z,
                       current_triangle->v1->x,current_triangle->v1->z,
                       current_triangle->v2->x,current_triangle->v2->z,
                       &(next_point.x),&(next_point.z));
      else if (_MATH_LIB_intersecting_lines(current_point.x,current_point.z,
                                            max_point.x,max_point.z,
                             current_triangle->v2->x,current_triangle->v2->z,
                             current_triangle->v3->x,current_triangle->v3->z))
        /* side V2-V3 has intersection  */
        err=_PERUNIKA_add_points_along_radial_line_intersection(
                       current_point.x,current_point.z,
                       max_point.x,max_point.z,
                       current_triangle->v2->x,current_triangle->v2->z,
                       current_triangle->v3->x,current_triangle->v3->z,
                       &(next_point.x),&(next_point.z));
      else if (_MATH_LIB_intersecting_lines(current_point.x,current_point.z,
                                            max_point.x,max_point.z,
                             current_triangle->v3->x,current_triangle->v3->z,
                             current_triangle->v1->x,current_triangle->v1->z))
        /* side V3-V1 has intersection  */
        err=_PERUNIKA_add_points_along_radial_line_intersection(
                       current_point.x,current_point.z,
                       max_point.x,max_point.z,
                       current_triangle->v3->x,current_triangle->v3->z,
                       current_triangle->v1->x,current_triangle->v1->z,
                       &(next_point.x),&(next_point.z));
    }

    if (next_point.x < 0.0 || next_point.z<0.0) {
      /* something is terribly wrong and there is no next point. 
         But we have to move, so lets use _ZELENJAK_3Dstep */ 
      distance_step = _ZELENJAK_3Dstep (current_point,current_triangle,direction,
                                       1.0,&next_point,&next_triangle);
    }
    else {
      /* move the next point just a little bit further to make it belong to 
         the next triangle for sure, and them calculate the step we make */
      next_point.x += (direction.x*0.00001);
      next_point.z += (direction.z*0.00001);
      distance_step = _MATH_LIB_3Ddistance(current_point,next_point);
    }

    /* add the current point to the grid */
    err = _PERUNIKA_add_point_to_grids(current_point.x,current_point.z);

    /* increase the distance counter */
    current_distance += distance_step;

    /* move to the next point */
    current_point = next_point;

  } /* end of the traveling loop */ 

  }
  return 0;
}

/*********************************************************************/
/*** intersection point between two lines */
int _PERUNIKA_add_points_along_radial_line_intersection(double x1, double y1,
             double x2, double y2,double x3, double y3,double x4, double y4,
	     double *x0, double *y0){


  *x0 = ((y2-y1)*(x4-x3)*x1-(y4-y3)*(x2-x1)*x3+(y3-y1)*(x2-x1)*(x4-x3))/
    ((y2-y1)*(x4-x3)-(y4-y3)*(x2-x1));
  if (x2!=x1) 
    *y0 = y1+(y2-y1)*((*x0)-x1)/(x2-x1);
  else 
    *y0 = y3+(y4-y3)*((*x0)-x3)/(x4-x3);

  return 0;
  } 

