/* LELUYA - PERUNIKA program block */
/* This part of the LELUYA code is generating the computational grid */

/* This function gives the adjacent triangle to a given triangle and
   one of its vertices (see the sketch below)  */

#include "PERUNIKA.h"

struct _triangle_structure *_PERUNIKA_adjacent_triangle(
      struct _vertex_structure *vertex, struct _triangle_structure *triangle) {

  /* The orientation of the triangles discussed in this function is always
     the same and it follows this sketch:

                  vertex
                      *-------------*
                     / \  adjacent /
                    /   \ triangle/
                   /     \       /
                  /       \     /
                 / triangle\   / 
                /           \ /
	       *-------------*
    adjacent_vertex      previous_vertex


  As you can see, the vertex has to be the triangle's vertex!
  Then we can find the previous vertex, and from these two vertices we can
  search for the adjacent triangle.
  Notice also that the previous vertex is NOT the adjacent vertex in the
  triangle! The adjacent vertex follows the triangle's orientation, while the
  previous vertex is adjacent to the adjacent vertex.
  It is confusing, but necessary to keep the consistent notation and 
  orientation of the triangles. 
  */

  struct _vertex_structure   *previous_vertex;
  struct _vertex_structure   *probable_previous_vertex;
  struct _triangle_structure *adjacent_triangle;
  struct _vertex_triangles   *vertex_triangle;

  /* find the adjacent point */
  adjacent_triangle = NULL;
  previous_vertex   = NULL;
  if      (vertex==triangle->v1) previous_vertex = triangle->v3;
  else if (vertex==triangle->v2) previous_vertex = triangle->v1;
  else if (vertex==triangle->v3) previous_vertex = triangle->v2;
  else adjacent_triangle = NULL;

  if (previous_vertex!=NULL) {
    vertex_triangle = vertex->triangles;
    while (vertex_triangle!=NULL) {
      probable_previous_vertex = _PERUNIKA_adjacent_vertex_in_triangle 
	(vertex,vertex_triangle->triangle);
      if ( probable_previous_vertex==previous_vertex) {
	adjacent_triangle = vertex_triangle->triangle;
	break;
      }
      vertex_triangle = vertex_triangle->next;
    }
  }

  return adjacent_triangle;

}
