/* LELUYA - PERUNIKA program block */

/* this function goes over the vertex list to remove the fake points and 
   create a sorted list of triangles at the non-fake vertices */

/* also, it creates the bounding box around triangles */

#include "PERUNIKA.h"

int _PERUNIKA_clean_the_vertex_list (void) {
  struct _triangle_structure *loop_triangle;
  struct _triangle_structure *next_loop_triangle;
  struct _triangle_structure *trian;
  struct _triangle_structure *next_trian;
  double                      nu;
  struct _vertex_triangles   *vertex_triangle;
  struct _vertex_triangles   *another_vertex_triangle;
  struct _vertex_structure   *loop_vertex;
  struct _vertex_structure   *next_loop_vertex;
  struct _vertex_structure   *sort_vertex;
  int                         ID_vertex_number;
  int                         dust_i;
  int                         error;
  struct _vertex_structure   *vertex1, *vertex2;
  struct _vertex_structure   *vertex_to_be_killed;
  struct _vertex_structure   *vertex_to_survive,*vertex_to_survive2;
  struct _polygon_structure  *polygon,*polygon2,*polygon_vertex,*kill_polygon;
  struct _vertex_structure   *vertex_down,*vertex_up,*edge_vertex;
  double side1, side2, side3;
  double adjacent1, adjacent2;
  int can_I_delete_a_vertex_please;

  if (DEBUG.SCREEN_MESSAGE >=1 ){
printf("process#%.3d:Cleaning fake vertices and sorting triangle lists \n",
my_MPI_rank);
  }

  /* before we start with anything, we should clean the vertex_triangle 
     list at each grid vertex, because it was created without control!!
     Don't ask me why I can not control it!! The problems started when
     I had to add creation of the vertex_triangles in _PERUNIKA_split_polygon
     because of the removal of vertices which are too close to each 
     other (see the last part of the current function). But since I use 
     _PERUNIKA_split_polygon in many other different occasions, the vertices
     pick up vertex_triangles without control! 
     Ehhh... Anyway, let's remove all vertex_triangles and start with
     the controlled process this time. */
  loop_vertex = start_vertex_list;
  do {
    vertex_triangle=loop_vertex->triangles;
    while (vertex_triangle!=NULL) {
      another_vertex_triangle=vertex_triangle->next;
      free(vertex_triangle);
      vertex_triangle=another_vertex_triangle;
    }
    loop_vertex->triangles=NULL; 
    loop_vertex = loop_vertex->next;
  } while (loop_vertex != start_vertex_list);

  /* first we go over the list of triangles and add them to the triangle
     list at the triangle's vertices */
  loop_triangle = start_triangle_list;
  do {

    /* check if some vertices are doubled */
    if (loop_triangle->v1->x==loop_triangle->v2->x && 
        loop_triangle->v1->z==loop_triangle->v2->z) {
      /* remove one vertex */
      printf ("WARNING!!! DOUBLE VERTEX DETECTED AT [%E,%E]\n",
               loop_triangle->v2->x,loop_triangle->v2->z);
      trian = start_triangle_list;
      do { /* remove triangles that contain this vertex */
	next_trian = trian->next;
	if (trian!=loop_triangle && (trian->v1==loop_triangle->v2 ||
	     trian->v2==loop_triangle->v2 || trian->v3==loop_triangle->v2))
          error=_PERUNIKA_destroy_triangle(trian);
	trian = next_trian;
      } while (trian!=start_triangle_list);
      error=_PERUNIKA_destroy_vertex(loop_triangle->v2);
      loop_triangle->v2 = NULL;
    }

    if (loop_triangle->v1->x==loop_triangle->v3->x && 
        loop_triangle->v1->z==loop_triangle->v3->z) {
      /* remove one vertex */
      printf ("WARNING!!! DOUBLE VERTEX DETECTED AT [%E,%E]\n",
               loop_triangle->v3->x,loop_triangle->v3->z);
      trian = start_triangle_list;
      do { /* remove triangles that contain this vertex */
	next_trian = trian->next;
	if (trian!=loop_triangle && (trian->v1==loop_triangle->v3 ||
	     trian->v2==loop_triangle->v3 || trian->v3==loop_triangle->v3))
          error=_PERUNIKA_destroy_triangle(trian);
	trian = next_trian;
      } while (trian!=start_triangle_list);
      error=_PERUNIKA_destroy_vertex(loop_triangle->v3);
      loop_triangle->v3 = NULL;
    }

    if (loop_triangle->v2!=NULL && loop_triangle->v3!=NULL)
      if (loop_triangle->v3->x==loop_triangle->v2->x && 
          loop_triangle->v3->z==loop_triangle->v2->z) {
        /* remove one vertex */
        printf ("WARNING!!! DOUBLE VERTEX DETECTED AT  [%E,%E]\n",
                 loop_triangle->v3->x,loop_triangle->v3->z);
        trian = start_triangle_list;
        do { /* remove triangles that contain this vertex */
	  next_trian = trian->next;
 	  if (trian!=loop_triangle && (trian->v1==loop_triangle->v2 ||
	       trian->v2==loop_triangle->v2 || trian->v3==loop_triangle->v2))
            error=_PERUNIKA_destroy_triangle(trian);
	  trian = next_trian;
        } while (trian!=start_triangle_list);
        error=_PERUNIKA_destroy_vertex(loop_triangle->v2);
        loop_triangle->v2 = NULL;
      }

    next_loop_triangle= loop_triangle->next;
    /* create vertex_triangle list now */
    if (loop_triangle->v1!=NULL && loop_triangle->v2!=NULL && loop_triangle->v3!=NULL) {
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle (loop_triangle,
                                                        loop_triangle->v1);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle (loop_triangle,
                                                        loop_triangle->v2);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle (loop_triangle,
                                                        loop_triangle->v3);
    }
    else {
      /* remove this triangle */
      error=_PERUNIKA_destroy_triangle(loop_triangle);
    }

    loop_triangle = next_loop_triangle;
  } while(loop_triangle!=start_triangle_list);

  /* go over the vertex list , delete the vertices without the adjacent 
     triangles, sort the triangles at the vertices which have them */
  ID_vertex_number = 0;
  loop_vertex = start_vertex_list;
  do {
    next_loop_vertex = loop_vertex->next;

    /* remove this vertex if it is not part of any triangle */
    if (loop_vertex->triangles == NULL) {
      error=_PERUNIKA_destroy_vertex(loop_vertex);
      if (ID_vertex_number == 0) next_loop_vertex = start_vertex_list;
    }
    else {
      /* in this case, the vertex is a part of some triangles, and we 
	 have to sort them in the counter-clockwise direction */
      loop_vertex->identification_number=ID_vertex_number;
      ID_vertex_number++;

      _PERUNIKA_sort_triangles_at_vertex(loop_vertex);
    }

    loop_vertex = next_loop_vertex;
  } while (loop_vertex != start_vertex_list  || ID_vertex_number==0);

  /* ************************************************************************/
  /* ... we are still not done with the vertex list. There are two more
     important things to do: a) locate the points at x=0.0 which are also
     the edge points, and b) remove the points which are too close to each
     other. */

  /* Detection of the edge points at x=0.0 : 

               |     
     X < 0     |         X > 0
               | 
   loop_vertex O----?---O edge_vertex 
               |   
     X < 0     |         X > 0
               |     

  */
  loop_vertex = start_vertex_list;
  do {

    if ( loop_vertex->x == 0.0 && loop_vertex->type!=2) {
      vertex_up=vertex_down=edge_vertex=NULL;
      /* find edge_vertex */
      vertex_triangle=loop_vertex->triangles;
      do {
	vertex1 = _PERUNIKA_adjacent_vertex_in_triangle(loop_vertex,
						 vertex_triangle->triangle);
	vertex2 = _PERUNIKA_adjacent_antivertex_in_triangle(loop_vertex,
						 vertex_triangle->triangle);
        if (vertex1->x == 0.0) vertex_down=vertex1;
        if (vertex2->x == 0.0) vertex_up=vertex2;
        if (vertex1->type==2 && vertex1->x!=0.0) edge_vertex=vertex1; 
        else if (vertex2->type==2 && vertex2->x!=0.0) edge_vertex=vertex2;

        vertex_triangle=vertex_triangle->next;
      } while (vertex_triangle!=NULL);

      /* if edge_vertex exists, then loop_vertex is maybe an edge vertex */ 
      if (edge_vertex!=NULL){
	if (vertex_up==NULL || vertex_down==NULL) {
	  loop_vertex->type = 2;
	  loop_vertex->level = edge_vertex->level;
	}
	else for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
	  if (loop_vertex->T_dust[dust_i]>0.0 && 
	      vertex_down->T_dust[dust_i]<=0.0) {
	    loop_vertex->type = 2;
	    if (Dust_Density(vertex_down->x,vertex_down->z,dust_i,NULL,&error)<=0.0)
	      loop_vertex->level = 666;
	    else loop_vertex->level = 1000 + dust_i; 
	    break;
	  }
	  else if (loop_vertex->T_dust[dust_i]>0.0 && 
		   vertex_up->T_dust[dust_i]<=0.0) {
	    if (Dust_Density(vertex_up->x,vertex_up->z,dust_i,NULL,&error)>0.0)
	      loop_vertex->level = 666;
	    else loop_vertex->level = 1000 + dust_i;
	    break; 
	  }
	} /* end of the loop over the dust components */
      }

    } /* end of ( loop_vertex->x == 0.0 && loop_vertex->type!=2) */
    loop_vertex = loop_vertex->next;
  } while (loop_vertex != start_vertex_list);

  /* ************************************************************************/
  /* removal of the points which are too close to each other */
  loop_triangle = start_triangle_list;
  do {
    if (loop_triangle==NULL) loop_triangle = start_triangle_list;

    side1 = _MATH_LIB_distance (loop_triangle->v1->x,loop_triangle->v1->z,
				loop_triangle->v2->x,loop_triangle->v2->z);
    side2 = _MATH_LIB_distance (loop_triangle->v2->x,loop_triangle->v2->z,
				loop_triangle->v3->x,loop_triangle->v3->z); 
    side3 = _MATH_LIB_distance (loop_triangle->v3->x,loop_triangle->v3->z,
				loop_triangle->v1->x,loop_triangle->v1->z); 

    if (side1<=side2 && side1<=side3) 
      {vertex1 = loop_triangle->v1;vertex2 = loop_triangle->v2;}
    else if (side2<=side1 && side2<=side3) 
      {vertex1 = loop_triangle->v2;vertex2 = loop_triangle->v3;side1=side2;}
    else if (side3<=side2 && side3<=side1) 
      {vertex1 = loop_triangle->v3;vertex2 = loop_triangle->v1;side1=side3;}

    adjacent1 = Rout;
    adjacent2 = Rout;

    if (_MATH_LIB_distance (0,0,vertex1->x,vertex1->z)<
        _MATH_LIB_distance (0,0,vertex2->x,vertex2->z)){
     if (side1 > _ZELENJAK_size_limit(vertex1->x,vertex1->z)*_MIN_VERTEX_DISTANCE_)
      {vertex1=NULL; vertex2=NULL;}
    }
    else 
     if (side1 > _ZELENJAK_size_limit(vertex2->x,vertex2->z)*_MIN_VERTEX_DISTANCE_)
      {vertex1=NULL; vertex2=NULL;}

    /* I have decided to keep all edge vertices!! Removing them is introducing 
       too many practical problems in the code and, for now, it is much
       easier just to keep them all. */
    if (vertex1!=NULL && vertex2!=NULL) 
      if (vertex1->type==2 && vertex2->type==2)
	{vertex1=NULL; vertex2=NULL;}

    /* If a vertex has level=667 then it was created close to the dust surface.
       Such vertices can be very close to each other because they HAVE TO be like
       that. Do not remove them! */
    if (vertex1!=NULL && vertex2!=NULL) 
      if (vertex1->level==667 || vertex2->level==667)
	{vertex1=NULL; vertex2=NULL;}

    /* find out which vertex has to be killed */
    if (vertex1!=NULL && vertex2!=NULL) 
      if ((vertex1->type!=2 && vertex2->type!=2)  ||
	  (vertex1->type==2 && vertex2->type==2) ) {

        adjacent1 = Rout;
        vertex_triangle=vertex1->triangles;
        while (vertex_triangle!=NULL) {
	  sort_vertex = _PERUNIKA_adjacent_vertex_in_triangle(vertex1,
						 vertex_triangle->triangle);
	  if (sort_vertex!=vertex2)
            if (_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
			  	vertex1->x,vertex1->z) < adjacent1)
	      adjacent1=_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
		                          vertex1->x,vertex1->z);
	  sort_vertex = _PERUNIKA_adjacent_antivertex_in_triangle(vertex1,
						 vertex_triangle->triangle);
	  if (sort_vertex!=vertex2)
            if (_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
				  vertex1->x,vertex1->z) < adjacent1)
	      adjacent1=_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
					    vertex1->x,vertex1->z);
	  vertex_triangle=vertex_triangle->next;
        } 
        adjacent2 = Rout;
        vertex_triangle=vertex2->triangles;
        while (vertex_triangle!=NULL)  {
	  sort_vertex = _PERUNIKA_adjacent_vertex_in_triangle(vertex2,
						 vertex_triangle->triangle);
	  if (sort_vertex!=vertex1)
            if (_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
				    vertex2->x,vertex2->z) < adjacent2)
	      adjacent2=_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
					  vertex2->x,vertex2->z);
	  sort_vertex = _PERUNIKA_adjacent_antivertex_in_triangle(vertex2,
						 vertex_triangle->triangle);
	  if (sort_vertex!=vertex1)
            if (_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
				  vertex2->x,vertex2->z) < adjacent2)
	      adjacent2=_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
					  vertex2->x,vertex2->z);
	  vertex_triangle=vertex_triangle->next;
        }
      }

    /* remove vertex1 or vertex2 */
    if (vertex1!=NULL && vertex2!=NULL) {

      /* a small correction in case of x=0 or z=0 edge  */
      if ((vertex1->x==0.0 && vertex2->x!=0.0) ||
          (vertex1->x!=0.0 && vertex2->x==0.0) ||
          (vertex1->z==0.0 && vertex2->z!=0.0) ||
          (vertex1->z!=0.0 && vertex2->z==0.0) )
	{adjacent1=Rout; adjacent2=Rout;}
      /* decide which vertex is going to be killed */
      if ( (vertex1->type!=2 && vertex2->type==2 && vertex1->x!=0.0 && 
            vertex1->z!=0.0) || 
           (vertex1->type!=2 && vertex2->type==2 && vertex1->x==0.0 && 
            vertex2->x==0.0) ||
           (vertex1->type!=2 && vertex2->type==2 && vertex1->z==0.0 && 
            vertex2->z==0.0) ||
           (adjacent1<adjacent2) ||
	   (vertex1->x!=0.0 && vertex2->x==0.0) ||
           (vertex1->z!=0.0 && vertex2->z==0.0) ){
	vertex_to_be_killed = vertex1;
	vertex_to_survive = vertex2;
      }
      else {
	vertex_to_be_killed = vertex2;
	vertex_to_survive = vertex1;
      }

      /* this 'if' takes care about a very special case */
      if (vertex_to_be_killed->x>0.0 && vertex_to_be_killed->type==2 && 
          vertex_to_survive->x==0.0 && vertex_to_survive->type!=2) 
	vertex_to_survive->type=2;

      /* sort the list of triangles at vertex_to_be_killed */
      if (vertex_to_be_killed->triangles!=NULL)
        _PERUNIKA_sort_triangles_at_vertex(vertex_to_be_killed);

      /* go over the triangles at vertex_to_be_killed, create a polygon 
         from the vertices around it and split the polygon */ 
      vertex_triangle=vertex_to_be_killed->triangles;
      if (vertex_triangle!=NULL) {
	loop_triangle=NULL; /* this is just a mark which shows that 
                               'loop_triangle' is going to die */
	if (vertex_triangle->next==NULL)
	  error=_PERUNIKA_destroy_triangle (vertex_triangle->triangle);
	else {
          polygon = NULL;
	  polygon = _PERUNIKA_create_polygon_point (polygon, &error);
	  polygon->vertex = _PERUNIKA_adjacent_vertex_in_triangle
	    (vertex_to_be_killed,vertex_triangle->triangle);
	  polygon_vertex = _PERUNIKA_create_polygon_point(polygon,&error);
	  polygon_vertex->vertex = _PERUNIKA_adjacent_antivertex_in_triangle
	    (vertex_to_be_killed,vertex_triangle->triangle);
          vertex_triangle=vertex_triangle->next; 
	  while (vertex_triangle!=NULL) {
	    if (_PERUNIKA_adjacent_vertex_in_triangle(vertex_to_be_killed,
		vertex_triangle->triangle)!=polygon->prev->vertex) {
	      polygon_vertex = _PERUNIKA_create_polygon_point(polygon,&error);
	      polygon_vertex->vertex = _PERUNIKA_adjacent_vertex_in_triangle
		(vertex_to_be_killed,vertex_triangle->triangle);
	    }
	    if (_PERUNIKA_adjacent_antivertex_in_triangle(vertex_to_be_killed,
	        vertex_triangle->triangle) != polygon->vertex){
	      polygon_vertex = _PERUNIKA_create_polygon_point(polygon,&error);
	      polygon_vertex->vertex = _PERUNIKA_adjacent_antivertex_in_triangle
	        (vertex_to_be_killed,vertex_triangle->triangle);
	    }
	    vertex_triangle=vertex_triangle->next;
	  }
	  /* destroy all triangles around vertex_to_be_killed */
	  vertex_triangle=vertex_to_be_killed->triangles;
	  while (vertex_triangle!=NULL) {
	    error = _PERUNIKA_destroy_triangle(vertex_triangle->triangle);
	    vertex_triangle=vertex_to_be_killed->triangles;
	  }
	  /* split the polygon (that is, create new triangles out of it) */
	  error = _PERUNIKA_split_polygon (polygon);
	  if (error) printf("POLYGON ERROR! _PERUNIKA_clean_the_vertex_list\n");

	}
      } /* end if (vertex_to_be_killed->triangles!=NULL) */

      /* kill this stupid vertex, finally! */ 
      error = _PERUNIKA_destroy_vertex (vertex_to_be_killed);

    } /* end of vertex1 or vertex2 removal */ 

    if (loop_triangle!=NULL) loop_triangle= loop_triangle->next;
  } while(loop_triangle!=start_triangle_list);

  /* ************************************************************************/
  /* Let's deal with the "too close type=2" vertices.
     What's so problematic about the type=2 vertices, you may ask.
     Well, after a type=2 vertex is removed, we would like to keep the adjacent
     type=2 vertices connected. Here is an example of situation where it can
     go wrong:
                              O                        O
                             /|\                      /|\
                 BEFORE:    / | \         AFTER:     / | \
                           /  |  \                  /  |  \
                        --2---2---2--            --2   |   2--
                           \  |  /                  \  |  /
                            \ | /                    \ | /
                             \|/                      \|/
                              O                        O
     From that example you can see that the type=2 vertices are NOT CONNECTED
     any more!
     Now you can say: what's a big deal?! Just force them to be connected!!
     You are right! That's exactly what we will do, but the life is, of course,
     not so easy. There are situations when we can not force the vertices to 
     be connected, like on this example:

                          2                              
              BEFORE:   _/|\_                                
                      _/  |  \_                 AFTER?       
                    _/    O    \_                       O      
                  _/   __/ \__   \_                  __/ \__     
                 /  __/       \__  \              __/       \__   
              --2__/             \__2--     --2__/___???????___\__2--
                    
     We can't connect the vertices on the sides because this would interfere
     with already existing triangle and/or make a triangle with 
     incorrect orientation.
     Even though I think that this situation is not possible, because we
     have cleared the "too close type!=2 vertices", I'm not ready to make risks
     and thus I'll test if this weird situation is possible or not.
  */

  loop_triangle = start_triangle_list;
  do {
    if (loop_triangle==NULL) loop_triangle = start_triangle_list;

    /* do something only if at least two vertices are type=2 */
    if ((loop_triangle->v1->type==2 && loop_triangle->v2->type==2 &&
         loop_triangle->v1->level!=667 && loop_triangle->v2->level!=667) ||
        (loop_triangle->v2->type==2 && loop_triangle->v3->type==2 &&
         loop_triangle->v2->level!=667 && loop_triangle->v3->level!=667) ||
        (loop_triangle->v3->type==2 && loop_triangle->v1->type==2 &&
         loop_triangle->v3->level!=667 && loop_triangle->v1->level!=667)) {

      if (loop_triangle->v1->type==2 && loop_triangle->v2->type==2 &&
	  loop_triangle->v3->type==2){
        /* check if all three vertices are type=2 and choose the shortest side */
        side1 = _MATH_LIB_distance (loop_triangle->v1->x,loop_triangle->v1->z,
	                            loop_triangle->v2->x,loop_triangle->v2->z);
        side2 = _MATH_LIB_distance (loop_triangle->v2->x,loop_triangle->v2->z,
				    loop_triangle->v3->x,loop_triangle->v3->z);
        side3 = _MATH_LIB_distance (loop_triangle->v3->x,loop_triangle->v3->z,
				    loop_triangle->v1->x,loop_triangle->v1->z);
	if (side1<side2 && side1<side3)
			{vertex1 = loop_triangle->v1;vertex2 = loop_triangle->v2;}
        else if (side2<side1 && side2<side3)
			{vertex1 = loop_triangle->v2;vertex2 = loop_triangle->v3; side1=side2;}
        else if (side3<side1 && side3<side2)
			{vertex1 = loop_triangle->v3;vertex2 = loop_triangle->v1; side1=side3;}
      }
      else if (loop_triangle->v1->type==2 && loop_triangle->v2->type==2){
        side1 = _MATH_LIB_distance (loop_triangle->v1->x,loop_triangle->v1->z,
	                            loop_triangle->v2->x,loop_triangle->v2->z);
	vertex1 = loop_triangle->v1;vertex2 = loop_triangle->v2;
      }
      else if (loop_triangle->v2->type==2 && loop_triangle->v3->type==2) {
        side1 = _MATH_LIB_distance (loop_triangle->v2->x,loop_triangle->v2->z,
				    loop_triangle->v3->x,loop_triangle->v3->z);
	vertex1 = loop_triangle->v2;vertex2 = loop_triangle->v3;
      }
      else {
        side1 = _MATH_LIB_distance (loop_triangle->v3->x,loop_triangle->v3->z,
				    loop_triangle->v1->x,loop_triangle->v1->z);
	vertex1 = loop_triangle->v3;vertex2 = loop_triangle->v1;
      }

      /* check if the distance between the vertices is smaller than required */
      if (_MATH_LIB_distance (0,0,vertex1->x,vertex1->z)<
          _MATH_LIB_distance (0,0,vertex2->x,vertex2->z)){
       if (side1>_ZELENJAK_size_limit(vertex1->x,vertex1->z)*_MIN_VERTEX_DISTANCE_)
          {vertex1=NULL; vertex2=NULL;}
      }
      else 
       if (side1>_ZELENJAK_size_limit(vertex2->x,vertex2->z)*_MIN_VERTEX_DISTANCE_)
          {vertex1=NULL; vertex2=NULL;}

      /* Ok, one of the vertices 'vertex1' and 'vertex2' should be removed.
         Decide which one by checking the distances toward adjacent vertices:*/
    if (vertex1!=NULL && vertex2!=NULL){

      /* we would like to build a configuration like this:
             \   /                \   /                   \   /
              \ /                  \ /                     \ /
           ----O--------------------O-----------------------O----
       vertex_to_survive2   vertex_to_be_killed     vertex_to_survive
      */

      /* let's see first if one of the vertices has x=0 or z=0 */
      if (vertex1->x==0.0 || vertex1->z==0.0) {
	vertex_to_be_killed = vertex2;
        vertex_to_survive   = NULL; /* unknown, yet */
	vertex_to_survive2  = vertex1;
      }
      else if (vertex2->x==0.0 || vertex2->z==0.0) {
	vertex_to_be_killed = vertex1;
	vertex_to_survive   = vertex2;
        vertex_to_survive2  = NULL; /* unknown, yet */
      }
      else {
        /* note: this part of the code is the same as before, when we were
           cleaning the 'type!=2' vertices. */ 
        adjacent1 = Rout;
        vertex_triangle=vertex1->triangles;
        while (vertex_triangle!=NULL) {
	  sort_vertex = _PERUNIKA_adjacent_vertex_in_triangle(vertex1,
						 vertex_triangle->triangle);
	  if (sort_vertex!=vertex2)
            if (_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
			  	    vertex1->x,vertex1->z) < adjacent1)
	      adjacent1=_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
		                            vertex1->x,vertex1->z);
	  sort_vertex = _PERUNIKA_adjacent_antivertex_in_triangle(vertex1,
						 vertex_triangle->triangle);
	  if (sort_vertex!=vertex2)
            if (_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
				  vertex1->x,vertex1->z) < adjacent1)
	      adjacent1=_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
					    vertex1->x,vertex1->z);
	  vertex_triangle=vertex_triangle->next;
        } 
        adjacent2 = Rout;
        vertex_triangle=vertex2->triangles;
        while (vertex_triangle!=NULL)  {
	  sort_vertex = _PERUNIKA_adjacent_vertex_in_triangle(vertex2,
						 vertex_triangle->triangle);
	  if (sort_vertex!=vertex1)
            if (_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
				    vertex2->x,vertex2->z) < adjacent2)
	      adjacent2=_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
					  vertex2->x,vertex2->z);
	  sort_vertex = _PERUNIKA_adjacent_antivertex_in_triangle(vertex2,
						 vertex_triangle->triangle);
	  if (sort_vertex!=vertex1)
            if (_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
				  vertex2->x,vertex2->z) < adjacent2)
	      adjacent2=_MATH_LIB_distance (sort_vertex->x,sort_vertex->z,
					  vertex2->x,vertex2->z);
	  vertex_triangle=vertex_triangle->next;
        }
	/* end of the part which is the same as for "type!=2" vertices */ 
        if (adjacent1 > adjacent2) {
	vertex_to_be_killed = vertex2;
        vertex_to_survive   = NULL; /* unknown, yet */
	vertex_to_survive2  = vertex1;
	}
        else {
	vertex_to_be_killed = vertex1;
	vertex_to_survive   = vertex2;
        vertex_to_survive2  = NULL; /* unknown, yet */
	}
      }

      /* find the unknown survival vertex */
      vertex_triangle=vertex_to_be_killed->triangles;
      while (vertex_triangle!=NULL) {
        sort_vertex = _PERUNIKA_adjacent_vertex_in_triangle(vertex_to_be_killed,
						 vertex_triangle->triangle);
	if (sort_vertex->type==2 && 
	    sort_vertex->level==vertex_to_be_killed->level) {
          if (vertex_to_survive==NULL && sort_vertex!=vertex_to_survive2) {
	    vertex_to_survive=sort_vertex; break; 
	  }
          else if (vertex_to_survive2==NULL && sort_vertex!=vertex_to_survive){
            vertex_to_survive2=sort_vertex; break; 
	  }
	}
        sort_vertex=_PERUNIKA_adjacent_antivertex_in_triangle(vertex_to_be_killed,
						 vertex_triangle->triangle);
	if (sort_vertex->type==2 && 
	    sort_vertex->level==vertex_to_be_killed->level) {
          if (vertex_to_survive==NULL && sort_vertex!=vertex_to_survive2) {
	    vertex_to_survive=sort_vertex; break; 
	  }
          else if (vertex_to_survive2==NULL && sort_vertex!=vertex_to_survive){
            vertex_to_survive2=sort_vertex; break; 
	  }
	}
        vertex_triangle=vertex_triangle->next;
      }

      /* if all three vertices exist (vertex_to_be_killed,vertex_to_survive,
         vertex_to_survive2), check if vertex_to_survive and vertex_to_survive2
         can be connected.
         We need two polygons for that, one on each side of 
         the survive-survive2 line */

      if (vertex_to_be_killed!=NULL && vertex_to_survive!=NULL &&
          vertex_to_survive2) {
      /* be sure that we have more than one triangle around vertex_to_be_killed*/
      if (vertex_to_be_killed->triangles->next!=NULL) {

	/* sort the list of triangles at vertex_to_be_killed */
        _PERUNIKA_sort_triangles_at_vertex(vertex_to_be_killed);

        /*              'polygon' ABOVE the line
                 /                \   /                   \    
                /                  \ /                     \  
               O--------------------O-----------------------O    
        vertex_to_survive2   vertex_to_be_killed     vertex_to_survive
        */
        polygon = NULL;
        polygon = _PERUNIKA_create_polygon_point (polygon, &error);
        polygon->vertex = vertex_to_survive;
        /* find vertex_to_survive in the list of 
           triangles around vertex_to_be_killed */
        vertex_triangle=vertex_to_be_killed->triangles;
        while (vertex_triangle!=NULL) {
          if (_PERUNIKA_adjacent_vertex_in_triangle(vertex_to_be_killed,
              vertex_triangle->triangle)==vertex_to_survive) {
	    polygon_vertex = _PERUNIKA_create_polygon_point(polygon,&error);
	    polygon_vertex->vertex = _PERUNIKA_adjacent_antivertex_in_triangle
	                       (vertex_to_be_killed,vertex_triangle->triangle);
	    break;
	  }
	  if (_PERUNIKA_adjacent_antivertex_in_triangle(vertex_to_be_killed,
              vertex_triangle->triangle)==vertex_to_survive) break;
          vertex_triangle=vertex_triangle->next;
        }
	if (vertex_triangle==NULL) 
	  printf("GRANDE ERROR!!: _PERUNIKA_clean_the_vertex_list, polygon\n");
	/* create the rest of the polygon */  
	vertex_triangle=vertex_triangle->next;
	if (vertex_triangle==NULL) 
                                vertex_triangle=vertex_to_be_killed->triangles;
	while (vertex_triangle!=NULL) {
	  sort_vertex=_PERUNIKA_adjacent_vertex_in_triangle(vertex_to_be_killed,
						    vertex_triangle->triangle);
	  if (sort_vertex!=polygon->prev->vertex) {
	    polygon_vertex = _PERUNIKA_create_polygon_point(polygon,&error);
	    polygon_vertex->vertex = sort_vertex;
	  }
	  if (sort_vertex==vertex_to_survive2) break;
	 sort_vertex=_PERUNIKA_adjacent_antivertex_in_triangle(vertex_to_be_killed,
						   vertex_triangle->triangle);
	  polygon_vertex = _PERUNIKA_create_polygon_point(polygon,&error);
	  polygon_vertex->vertex = sort_vertex;
	  if (sort_vertex==vertex_to_survive2) break;
	  vertex_triangle=vertex_triangle->next;
	  if (vertex_triangle==NULL) 
                                vertex_triangle=vertex_to_be_killed->triangles;
	}

        /* FIND polygon2:

                 /                \   /                   \    
                /                  \ /                     \  
               O--------------------O-----------------------O    
        vertex_to_survive2   vertex_to_be_killed     vertex_to_survive
                \                  / \                     /
                 \                /   \                   /
                          'polygon2' BELOW the line
        */
        polygon2 = NULL;
        polygon2 = _PERUNIKA_create_polygon_point (polygon2, &error);
        polygon2->vertex = vertex_to_survive2;
        /* find vertex_to_survive2 in the list of 
           triangles around vertex_to_be_killed */
        vertex_triangle=vertex_to_be_killed->triangles;
        while (vertex_triangle!=NULL) {
          if (_PERUNIKA_adjacent_vertex_in_triangle(vertex_to_be_killed,
              vertex_triangle->triangle)==vertex_to_survive2) {
	    polygon_vertex = _PERUNIKA_create_polygon_point(polygon2,&error);
	    polygon_vertex->vertex = _PERUNIKA_adjacent_antivertex_in_triangle
	                       (vertex_to_be_killed,vertex_triangle->triangle);
	    break;
	  }
	  if (_PERUNIKA_adjacent_antivertex_in_triangle(vertex_to_be_killed,
              vertex_triangle->triangle)==vertex_to_survive2) break;
          vertex_triangle=vertex_triangle->next;
        }
	if (vertex_triangle==NULL) 
	  printf("GRANDE ERROR!!: _PERUNIKA_clean_the_vertex_list, polygon2\n");
	/* create the rest of the polygon2 */  
	vertex_triangle=vertex_triangle->next;
	if (vertex_triangle==NULL) 
                                vertex_triangle=vertex_to_be_killed->triangles;
	while (vertex_triangle!=NULL) {
	  sort_vertex=_PERUNIKA_adjacent_vertex_in_triangle(vertex_to_be_killed,
						    vertex_triangle->triangle);
	  if (sort_vertex!=polygon2->prev->vertex) {
	    polygon_vertex = _PERUNIKA_create_polygon_point(polygon2,&error);
	    polygon_vertex->vertex = sort_vertex;
	  }
	  if (sort_vertex==vertex_to_survive) break;
	 sort_vertex=_PERUNIKA_adjacent_antivertex_in_triangle(vertex_to_be_killed,
						   vertex_triangle->triangle);
	  polygon_vertex = _PERUNIKA_create_polygon_point(polygon2,&error);
	  polygon_vertex->vertex = sort_vertex;
	  if (sort_vertex==vertex_to_survive) break;
	  vertex_triangle=vertex_triangle->next;
	  if (vertex_triangle==NULL) 
                                vertex_triangle=vertex_to_be_killed->triangles;
	}

	/* now we have two polygons (polygon and polygon2) and we can
           check if the line between the vertices 'vertex_to_survive' and
           'vertex_to_survive2' can exist. We will do that by checking an
           angle between the line and the other vertices in the polygon.*/

	can_I_delete_a_vertex_please=TRUE;
	if (polygon->next->vertex != polygon->vertex && 
	    polygon->next->next->vertex != polygon->vertex) {
	  polygon_vertex=polygon->next;
	  while (polygon_vertex!=polygon->prev) {
	    nu = _MATH_LIB_nu_value(polygon->vertex->x, polygon->vertex->z,
			 polygon_vertex->vertex->x, polygon_vertex->vertex->z,
			 polygon->prev->vertex->x, polygon->prev->vertex->z);
	    if (nu>=2.0 || nu<=0.0) {
	      can_I_delete_a_vertex_please=FALSE;
	      break;
	    }
	    polygon_vertex=polygon_vertex->next;
	  }
	}
	if(can_I_delete_a_vertex_please)
	  if (polygon2->next->vertex != polygon2->vertex && 
	      polygon2->next->next->vertex != polygon2->vertex) {
	  polygon_vertex=polygon2->next;
	  while (polygon_vertex!=polygon2->prev) {
	    nu = _MATH_LIB_nu_value(polygon2->vertex->x, polygon2->vertex->z,
			 polygon_vertex->vertex->x, polygon_vertex->vertex->z,
			 polygon2->prev->vertex->x, polygon2->prev->vertex->z);
	    if (nu>=2.0 || nu<=0.0) {
	      can_I_delete_a_vertex_please=FALSE;
	      break;
	    }
	    polygon_vertex=polygon_vertex->next;
	  }
	}

	/* destroy all triangles around vertex_to_be_killed */
	if(can_I_delete_a_vertex_please) {
	  loop_triangle=NULL; /* this is just a mark which shows that 
                                'loop_triangle' is going to die */
	  vertex_triangle=vertex_to_be_killed->triangles;
	  while (vertex_triangle!=NULL) {
	    error = _PERUNIKA_destroy_triangle(vertex_triangle->triangle);
	    vertex_triangle=vertex_to_be_killed->triangles;
	  }
	}
	/* split the polygons if everything is OK */
	if(can_I_delete_a_vertex_please && 
	   polygon->next->vertex != polygon->vertex && 
	   polygon->next->next->vertex != polygon->vertex) {
	  error = _PERUNIKA_split_polygon (polygon);
	}
	else {
	  polygon_vertex = polygon->next;
          while (polygon_vertex != polygon) {
            kill_polygon = polygon_vertex->next;
            free(polygon_vertex);
            polygon_vertex = kill_polygon;
          }
          free(polygon);
          polygon = NULL;
	}

	if(can_I_delete_a_vertex_please && 
	   polygon2->next->vertex != polygon2->vertex && 
	   polygon2->next->next->vertex != polygon2->vertex) {
	  error = _PERUNIKA_split_polygon (polygon2);
	}
	else {
	  polygon_vertex = polygon2->next;
          while (polygon_vertex != polygon2) {
            kill_polygon = polygon_vertex->next;
            free(polygon_vertex);
            polygon_vertex = kill_polygon;
          }
          free(polygon2);
          polygon2 = NULL;
	}

        /* kill the 'vertex_to_be_killed' */ 
	if(can_I_delete_a_vertex_please)
	  error = _PERUNIKA_destroy_vertex (vertex_to_be_killed);

	} /* end if (NOT single triangle around vertex_to_be_killed) */
	else {
          /* just kill the vertex */
	  error = _PERUNIKA_destroy_triangle(vertex_to_be_killed->triangles->triangle);
	  error = _PERUNIKA_destroy_vertex (vertex_to_be_killed);
	  loop_triangle=NULL; /* this is just a mark which shows that 
                                'loop_triangle' is going to die */
	}
      } /* end if (killed!=NULL && survive!=NULL && survive2!=NULL) */
    } /* end if (vertex1!=NULL && vertex2!=NULL) */

    }/* end of the situation when at least two vertices are type=2 */ 
    if (loop_triangle!=NULL) loop_triangle = loop_triangle->next;
  } while(loop_triangle!=start_triangle_list);

  /* ************************************************************************/
  /* calculate the nu values for the triangles at each vertex and sort 
     these triangles, AGAIN! */
  ID_vertex_number = 0;
  loop_vertex = start_vertex_list;
  do {
    next_loop_vertex = loop_vertex->next;

    /* remove this vertex if it is not part of any triangle */
    if (loop_vertex->triangles == NULL) {
      error=_PERUNIKA_destroy_vertex(loop_vertex);
      if (ID_vertex_number == 0) next_loop_vertex = start_vertex_list;
    }
    else {
      loop_vertex->identification_number=ID_vertex_number;
      ID_vertex_number++;

      _PERUNIKA_sort_triangles_at_vertex(loop_vertex);
    }

    loop_vertex = next_loop_vertex;
  } while (loop_vertex != start_vertex_list  || ID_vertex_number==0);

  /********************************************************************/
  /********************************************************************/
  /*** SPECIAL GRID FIX: CIT3_Finite_Cone03_Final ****
  loop_vertex = start_vertex_list;
  do {
    //if (loop_vertex->x < 0.2595 && loop_vertex->x > 0.2593 && 
    //    loop_vertex->z < 0.9682 && loop_vertex->z > 0.9680)
    //  {
    //    loop_vertex->x = 0.258819045; 
    //    loop_vertex->z = 0.965925826;
    //    loop_vertex->T_dust[1]=loop_vertex->T_dust[0];
    //    loop_vertex->level = 1000;
    //    break;
    //  }
    //if (loop_vertex->x < 0.2423 && loop_vertex->x > 0.2421 &&
    //    loop_vertex->z < 0.8927 && loop_vertex->z > 0.8925)
    //  {
    //    loop_vertex->x = 0.242;
    //    loop_vertex->z = 0.888;
    //    loop_vertex->type = 2;
    //    loop_vertex->level = 1000;
    //    break;
    //  }
    if (loop_vertex->x < 0.001 && loop_vertex->x > -0.001 &&
        loop_vertex->z < 0.8851 && loop_vertex->z > 0.8849)
      {
        loop_vertex->x = 0.0;
        loop_vertex->z = 0.87;
        break;
      }
    loop_vertex = loop_vertex->next;
  } while (loop_vertex != start_vertex_list );
  ********************************************************************/
  /********************************************************************/


  /********************************************************************/
  /********************************************************************/
  /*** SPECIAL GRID FIX: Ohnaka model ****
  loop_vertex = start_vertex_list;
  do {
    if (loop_vertex->x < 0.9853 && loop_vertex->x > 0.9851 && 
        loop_vertex->z < 0.1718 && loop_vertex->z > 0.1716)
      {
	loop_vertex->x = sin(80*RAD1); 
        loop_vertex->z = cos(80*RAD1);
	break;
      }
    loop_vertex = loop_vertex->next;
  } while (loop_vertex != start_vertex_list );

  loop_vertex = start_vertex_list;
  do {
    if (loop_vertex->x < 0.9824 && loop_vertex->x > 0.9822 &&
        loop_vertex->z < 0.1718 && loop_vertex->z > 0.1716)
      {
        loop_vertex->x = 0.9823;
        loop_vertex->z = 0.1747;
        break;
      }
    loop_vertex = loop_vertex->next;
  } while (loop_vertex != start_vertex_list );

  loop_vertex = start_vertex_list;
  do {
    if (loop_vertex->type ==2)
      if (loop_vertex->x/loop_vertex->z > 5.64 &&
          loop_vertex->x/loop_vertex->z < 5.72) {
        loop_vertex->z = loop_vertex->x / tan(80*RAD1);
      }
    if (loop_vertex->type !=2 && loop_vertex->T_dust[0]>0.0 && loop_vertex->x>2.0)
      if (loop_vertex->x/loop_vertex->z < 5.67128182) {
        loop_vertex->x = sqrt(loop_vertex->x*loop_vertex->x+loop_vertex->z*loop_vertex->z)*sin(80.01*RAD1);
        loop_vertex->z = loop_vertex->x / tan(80.01*RAD1);
      }
    loop_vertex = loop_vertex->next;
  } while (loop_vertex != start_vertex_list );
  ********************************************************************/
  /********************************************************************/

  /* we have, finally, cleaned the triangle list from unwanted triangles, and
     now we can build the bounding boxes */
  ID_vertex_number = 0; /* use ID_vertex_number to numerate the triangles */
  loop_triangle = start_triangle_list;
  do {
    loop_triangle->ID_number = ID_vertex_number;
    ID_vertex_number++;

    loop_triangle->bound_box_down_left_x=loop_triangle->v1->x;
    loop_triangle->bound_box_down_left_z=loop_triangle->v1->z;
    loop_triangle->bound_box_up_right_x=loop_triangle->v1->x;
    loop_triangle->bound_box_up_right_z=loop_triangle->v1->z;
    if (loop_triangle->bound_box_down_left_x>loop_triangle->v2->x)
      loop_triangle->bound_box_down_left_x=loop_triangle->v2->x;
    if (loop_triangle->bound_box_down_left_z>loop_triangle->v2->z)
      loop_triangle->bound_box_down_left_z=loop_triangle->v2->z;
    if (loop_triangle->bound_box_up_right_x<loop_triangle->v2->x)
      loop_triangle->bound_box_up_right_x=loop_triangle->v2->x;
    if (loop_triangle->bound_box_up_right_z<loop_triangle->v2->z)
      loop_triangle->bound_box_up_right_z=loop_triangle->v2->z;
    if (loop_triangle->bound_box_down_left_x>loop_triangle->v3->x)
      loop_triangle->bound_box_down_left_x=loop_triangle->v3->x;
    if (loop_triangle->bound_box_down_left_z>loop_triangle->v3->z)
      loop_triangle->bound_box_down_left_z=loop_triangle->v3->z;
    if (loop_triangle->bound_box_up_right_x<loop_triangle->v3->x)
      loop_triangle->bound_box_up_right_x=loop_triangle->v3->x;
    if (loop_triangle->bound_box_up_right_z<loop_triangle->v3->z)
      loop_triangle->bound_box_up_right_z=loop_triangle->v3->z;

    /* find the integral_step for this triangle */
    error = _PERUNIKA_triangle_integral_step(loop_triangle);
    if (error) return 1;

    loop_triangle= loop_triangle->next;
  } while(loop_triangle!=start_triangle_list);


  return 0;
}
