/* LELUYA - PERUNIKA program block */
/* This block of the LELUYA code is generating the computational grid */

/* This function is calculating the initial temperatures and energy densities 
   from the initial approximation of a point source and negligible diffuse
   radiation */


/************************************************************************/
/* subroutine: find next rho or theta */
double _PERUNIKA_create_initial_Find_next_radius (double x_start, 
       double z_start, double r_end, int radial_j, int angular_n, 
       double *Tau_back, double *Tau_front, double *D_back, 
       double *D_front, int lambda_k);

/************************************************************************/

#include "PERUNIKA.h"

int _PERUNIKA_create_initial_array (void ) {
  int dust_i;             /* counter over the dust components */
  int radial_j;           /* counter over the radial grid steps */
  int lambda_k;           /* counter over the lambda grid steps */
  int angular_n;          /* counter over the angular grid steps */
  double step_j, step_n;  /* radial (logarithmic) and angular (linear) 
                             grid steps */
  int err, err_location;  /* error flags */
  double *U_source;       /* central energy source */
  double *Radial_Optical_Depth;/* optical depth at some grid point. This
                              variable is NOT an array over the grid points,
                              it goes only over the wavelengths. It is 
                              accumulating the optical depth from one point
                              to another in a loop over the grid points */
  double x,z,x0,z0,r,angle;/*used for calculation of the point's cooridnates */

  /* variables in case of temperature from a file */
  double *Temp_r,   *Temp_theta, **Temp_Temp; /* radial grid, angular grid, and
						 temperature grid */
  int    Nr, Ntheta; /* number of radial and angular grid points */
  double Db,Df,Temp1, Temp2, Temp;/* interpolation variables */
  int inter_N_r, inter_N_theta; /* interpolation temperature cell */
  double Tau_back, Tau_front, D_back, D_front;
  double *Radial_Density;

  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    printf("search for the initial temperature and energy density\n");
  }

  if (DEBUG.SCREEN_MESSAGE >= 2 ) {
    printf("total number of the radial (logarithmic) points: %d\n",
           _INITIAL_GRID_ARRAY_r);
    printf("total number of the angular (linear) points: %d\n",
            _INITIAL_GRID_ARRAY_theta);
    printf("angular grid starts at %lf RAD and stops at %lf RAD\n",
           _INITIAL_GRID_ARRAY_theta_min, _INITIAL_GRID_ARRAY_theta_max);
    
  }

  if (DEBUG.CONTINUE!=100 && DEBUG.CONTINUE!=200) {
    /********************************************************/
    /* calculate temperature from the initial approximation */ 
    /********************************************************/

  U_source = dvector(0,lambda_Max-1);
  /* first we need Psi */
  /* find the dust at the sublimation radius */
  x0 = sin(Theta_Total_Value);
  z0 = cos(Theta_Total_Value);
  Temp = 1.0; /* dust sublimation temperature */
  dust_i = -1;
  for (Nr=0; Nr<Max_Dust_Type; Nr++) {
    if (Dust_Sublim_Temp[Nr] > Temp) 
      if (Dust_Density (x0,z0,Nr,NULL,&err)>0.0) {
        dust_i = Nr;
        Temp =  Dust_Sublim_Temp[Nr]; 
      }
  }
  if (dust_i==-1) {
    printf("ERROR: Can't find dust at point 1\n");
    return 1;
  }

  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
    U_source[lambda_k] = q_abs(x0,z0,lambda_k,dust_i,&err)*Source_Flux[lambda_k];
  Psi = _MATH_LIB_simpson(lambda_Max,0,lambda_Max-1,Lambda_Grid,U_source);
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
    U_source[lambda_k] = q_abs(x0,z0,lambda_k,dust_i,&err)*
                  _MATH_LIB_Planck(Dust_Sublim_Temp[dust_i],Lambda_Grid[lambda_k]);
  Psi = Psi / _MATH_LIB_simpson(lambda_Max,0,lambda_Max-1,Lambda_Grid,U_source);

  /* get the dust temperature at r=1 and use it to calculate more precise Psi */
  Temp_r=dvector(0,Max_Dust_Type-1);
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
    if (Dust_Density(x0,z0,dust_i,NULL,&err)<=0.0 &&
        INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
      Temp_r[dust_i] = 1E30;
    else {
      initial_temperature_array[dust_i][0][0]=_GOROSTAS_Find_Temperature (x0,z0,
                     0.9*Dust_Sublim_Temp[0],1.1*Dust_Sublim_Temp[0],
                     dust_i,Source_Flux,&err);
      for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
        U_source[lambda_k] = q_abs(x0,z0,lambda_k,dust_i,&err)*Source_Flux[lambda_k];
      Psi = _MATH_LIB_simpson(lambda_Max,0,lambda_Max-1,Lambda_Grid,U_source);
      for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
        U_source[lambda_k] = q_abs(x0,z0,lambda_k,dust_i,&err)*
                  _MATH_LIB_Planck(Dust_Sublim_Temp[dust_i],Lambda_Grid[lambda_k]);
      Psi = Psi / _MATH_LIB_simpson(lambda_Max,0,lambda_Max-1,Lambda_Grid,U_source);
      /* source radius */
      Temp_r[dust_i] = Dust_Sublim_Temp[dust_i]/Source_Teff;
      Temp_r[dust_i] *=Temp_r[dust_i];
      Temp_r[dust_i] = Temp_r[dust_i] * 2.0/sqrt(Psi);
    }
  /* maximum source radius */
  dust_i = 0;
  for (Nr=1; Nr<Max_Dust_Type; Nr++) 
    if (Temp_r[Nr] > Temp_r[dust_i]) dust_i = Nr;

  /* this is the correct Psi */
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
    U_source[lambda_k] = q_abs(x0,z0,lambda_k,dust_i,&err)*Source_Flux[lambda_k];
  Psi = _MATH_LIB_simpson(lambda_Max,0,lambda_Max-1,Lambda_Grid,U_source);
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
    U_source[lambda_k] = q_abs(x0,z0,lambda_k,dust_i,&err)*
                  _MATH_LIB_Planck(Dust_Sublim_Temp[dust_i],Lambda_Grid[lambda_k]);
  Psi = Psi / _MATH_LIB_simpson(lambda_Max,0,lambda_Max-1,Lambda_Grid,U_source);

  Source_Radius=Dust_Sublim_Temp[dust_i]/Source_Teff;
  Source_Radius *= Source_Radius;
  Source_Radius = Source_Radius*2.0/sqrt(Psi);
  free_dvector(Temp_r,0,Max_Dust_Type-1);
  printf("Initial estimate on Psi = %E and source radius R=%E from dust#%d\n",Psi,Source_Radius,dust_i);

  /* now we can create the initial temperature guess */
  
  Radial_Optical_Depth = dvector(0,lambda_Max-1);

  step_j = exp(log(Rout)/(_INITIAL_GRID_ARRAY_r-1));
  step_n = (_INITIAL_GRID_ARRAY_theta_max-_INITIAL_GRID_ARRAY_theta_min)
           /(_INITIAL_GRID_ARRAY_theta-1);

  initial_radial_array[0] = 1.0;
  /* set the default values of initial_radial_array */
  for (radial_j=1; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++) 
    initial_radial_array[radial_j] = initial_radial_array[radial_j-1]*step_j;
  initial_radial_array[_INITIAL_GRID_ARRAY_r-1] = Rout;

  /* initialize the temperature profiles (a simple 1/y^0.5 profile) */
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) 
    for (angular_n=0; angular_n<_INITIAL_GRID_ARRAY_theta; angular_n++)
      for (radial_j=0; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++) {
        if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
          initial_temperature_array[dust_i][radial_j][angular_n]= Dust_Sublim_Temp[0] /
                                              sqrt(initial_radial_array[radial_j]);
          if (initial_temperature_array[dust_i][radial_j][angular_n] > Dust_Sublim_Temp[dust_i])
            /* the dust component dust_i can not exist at this temperature */
            initial_temperature_array[dust_i][radial_j][angular_n]= -1.0;
        }
        else 
	  initial_temperature_array[dust_i][radial_j][angular_n] = _T99999_; 
      }

  /* get the initial guess for Eta_Denominator */
  Radial_Density = dvector(0,_INITIAL_GRID_ARRAY_r-1);
  Eta_Denominator = 0.0;
  Nr = 0; /* counts the total nubmber of non-empty dust */
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) 
  if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
    Nr++;
    for (radial_j=0; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++) {
      x0 = initial_radial_array[radial_j] * sin(Theta_Total_Value);
      z0 = initial_radial_array[radial_j] * cos(Theta_Total_Value);
      Radial_Density[radial_j]=Dust_Density(x0,z0,dust_i,NULL,&err);
    }
    Eta_Denominator = Eta_Denominator +
                        (Sigma_Dust_Abs[dust_i][Lambda_Total] +
                         Sigma_Dust_Sca[dust_i][Lambda_Total]) *
         _MATH_LIB_simpson(_INITIAL_GRID_ARRAY_r,0,_INITIAL_GRID_ARRAY_r-1,
                           initial_radial_array,Radial_Density);
  }
  free_dvector(Radial_Density,0,_INITIAL_GRID_ARRAY_r-1);
  printf("Initial estimate on Eta_Denominator = %E\n",Eta_Denominator);

  /* go over all angular directions */
  if (Nr>1) /* we need this only if more than one dust exists: */
  for (angular_n=_INITIAL_GRID_ARRAY_theta-1; angular_n>=0; angular_n--) {
    angle = angular_n*step_n+_INITIAL_GRID_ARRAY_theta_min;
    /* just to be sure that the last point 
       is exactly _INITIAL_GRID_ARRAY_theta_max */
    if (angular_n==_INITIAL_GRID_ARRAY_theta-1) 
       angle = _INITIAL_GRID_ARRAY_theta_max;

    if (DEBUG.SCREEN_MESSAGE >= 2 ) {
      printf("Grid angle #%d = %lf RAD ...\n",angular_n,angle);
    }

    x0 = sin(angle);
    z0 = cos(angle);
    r = 1.0;
    /* set all optical depth to zero (this is at the point (x0,z0)) */
    for (lambda_k=0; lambda_k<lambda_Max; lambda_k++) {
      Radial_Optical_Depth[lambda_k] = 0.0;
      U_source[lambda_k] = Source_Flux[lambda_k];
    }
    /* calculate the dust temperature at the point (x0,z0) for all dust
       components */
    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
    if (Dust_Density(x0,z0,dust_i,NULL,&err)<=0.0 &&
        INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
        initial_temperature_array[dust_i][0][angular_n]=-1.0;
      }
    else {
      initial_temperature_array[dust_i][0][angular_n]=
                     _GOROSTAS_Find_Temperature (x0,z0,
                     0.9*Dust_Sublim_Temp[0],1.1*Dust_Sublim_Temp[0],
                     dust_i,U_source,&err);
      if (err>0) {
        err_location=9;
        _PESJANEK__Error_Message (err_location,err);
        printf("(dust component #%d, angle(%d)=%lf RAD)\n",
               dust_i,angular_n,angle);
        printf("(radial distance(0): x=%lf,z=%lf)\n",x0,z0);
        return 1;
      }
      /* do not allow temperatures below a given minimum temperature */
      if (initial_temperature_array[dust_i][0][angular_n] <
                                  _INITIAL_GRID_ARRAY_min_temperature
          && INPUT.DUST_DENSITY_TYPE[dust_i]!=99999
          && initial_temperature_array[dust_i][0][angular_n]>0.0)
        initial_temperature_array[dust_i][0][angular_n] =
	   _INITIAL_GRID_ARRAY_min_temperature; 
      /* check if the dust temperatures are larger then sublimation */
      if (initial_temperature_array[dust_i][0][angular_n]
          /Dust_Sublim_Temp[dust_i]-1.0  > _SUBLIMATION_TOLERANCE_) 
                       initial_temperature_array[dust_i][0][angular_n] = -1.0;
    }


    if (DEBUG.SCREEN_MESSAGE >= 1) {
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
         printf("T[%d]=%lf ",dust_i,initial_temperature_array[dust_i][0][angular_n]);
      printf("\n");
    }

    /* set the energy density */
    for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
      initial_energy_array[0][angular_n][lambda_k]=U_source[lambda_k];

    /* prepare for the radil intergral */
    for (lambda_k=0; lambda_k<lambda_Max; lambda_k++) 
      Radial_Optical_Depth[lambda_k] = 0.0;
    if (angular_n==_INITIAL_GRID_ARRAY_theta-1) {
      Tau_back = 0.0;
      Tau_front = Tau_Total;
      D_back = 0.0;
      D_front = Rout-1;
    }

    /* go over all radial grid points and update the dust temperatures */
    for (radial_j=1; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++) {      

      if (angular_n==_INITIAL_GRID_ARRAY_theta-1 && radial_j<_INITIAL_GRID_ARRAY_r-1) {
	/* find the next radial point */
        Db = r*sin(Theta_Total_Value);
        Df = r*cos(Theta_Total_Value);
        r = _PERUNIKA_create_initial_Find_next_radius (Db,Df,-1.0,
             radial_j,_INITIAL_GRID_ARRAY_theta-1,&Tau_back,&Tau_front,&D_back,&D_front,Lambda_Total);
        initial_radial_array[radial_j] = r;
	if (DEBUG.SCREEN_MESSAGE >= 1) printf("NEXT R=%13.11lf  Tau=%lf\n",r,Tau_back);
      }

      r = initial_radial_array[radial_j];
      /* just to be sure that the last point is exactly Rout */
      if (radial_j==_INITIAL_GRID_ARRAY_r-1) r = Rout;
      x = r*x0;
      z = r*z0;

      /* find the optical depth */
      for (lambda_k=0; lambda_k<lambda_Max; lambda_k++) {
        Temp1 = Tau_Total; 
        Db = r - initial_radial_array[radial_j-1];
        Df = Rout - r;
        Temp = _PERUNIKA_create_initial_Find_next_radius (
          initial_radial_array[radial_j-1]*x0, initial_radial_array[radial_j-1]*z0,
          r, radial_j,angular_n,&(Radial_Optical_Depth[lambda_k]),&Temp1,&Db,&Df,lambda_k);
      }

      /* update the energy spectrum from the central source because the 
         optical depth has changed */
      for (lambda_k=0; lambda_k<lambda_Max; lambda_k++) 
        U_source[lambda_k] = Source_Flux[lambda_k] *
                             exp(-Radial_Optical_Depth[lambda_k]);

      /* find the new dust temperatures for all dust components */
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
        if (Dust_Density(x,z,dust_i,NULL,&err)<=0.0 &&
            INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
          initial_temperature_array[dust_i][radial_j][angular_n] = -1.0;
        }
        else if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999){
          initial_temperature_array[dust_i][radial_j][angular_n] = 
                  _GOROSTAS_Find_Temperature (x,z,
                  0.9*initial_temperature_array[dust_i][radial_j-1][angular_n],
                  1.1*initial_temperature_array[dust_i][radial_j-1][angular_n],
                  dust_i,U_source,&err);

          if (err>0) {
            err_location=9;
            _PESJANEK__Error_Message (err_location,err);
            printf("(dust component #%d, angle(%d)=%lf RAD)\n",
                 dust_i,angular_n,angle);
            printf("(radial distance(%d): x=%lf,z=%lf)\n",radial_j,x,z);
            return 1;
          }
          /* do not allow temperatures below a given minimum temperature */ 
          if (initial_temperature_array[dust_i][radial_j][angular_n] <
                                  _INITIAL_GRID_ARRAY_min_temperature
              && INPUT.DUST_DENSITY_TYPE[dust_i]!=99999
              && initial_temperature_array[dust_i][radial_j][angular_n]>0.0)
              initial_temperature_array[dust_i][radial_j][angular_n] =
	    _INITIAL_GRID_ARRAY_min_temperature;
          /* check if the new temperature is less then the sublimation 
             temperature + error tolerance */
          if (initial_temperature_array[dust_i][radial_j][angular_n]
              /Dust_Sublim_Temp[dust_i]-1.0 > _SUBLIMATION_TOLERANCE_) 
            initial_temperature_array[dust_i][radial_j][angular_n] = -1.0;
          /* check if the new temperature is acceptable, but higher then 
             the sublimation temperature */
          else 
            if (initial_temperature_array[dust_i][radial_j][angular_n] 
                                                    > Dust_Sublim_Temp[dust_i])
                initial_temperature_array[dust_i][radial_j][angular_n]
                                                    = Dust_Sublim_Temp[dust_i];
        } /* end of temperature */

      if (DEBUG.SCREEN_MESSAGE >= 1) {
        for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
          printf("T[%d]=%lf ",dust_i,initial_temperature_array[dust_i][radial_j][angular_n]);
        printf("\n");
      }

      /* memorize the energy density */
      for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
        initial_energy_array[radial_j][angular_n][lambda_k]=U_source[lambda_k];

      /* to speed up this thing use 1/r^0.5 temperature profile if all dust types
         have a non-zero temperature */
      Ntheta = 0; Nr = 0;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
        if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
	  Ntheta++;
          if (initial_temperature_array[dust_i][radial_j][angular_n]>0.0) Nr++;
	}
      if (Ntheta==Nr) {
	/* set the rest of radii to values larger than the current radius */
	/* set the 1/r^0.5 profile */
        Nr = radial_j+1;
        for (radial_j=Nr; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++) {
          if (angular_n==_INITIAL_GRID_ARRAY_theta-1 && radial_j<_INITIAL_GRID_ARRAY_r-1) {
	    step_j =  exp(log(Rout/initial_radial_array[Nr-1])/(_INITIAL_GRID_ARRAY_r-Nr));
            initial_radial_array[radial_j] =initial_radial_array[radial_j-1]*step_j;
	  } 
          for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
            if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
              initial_temperature_array[dust_i][radial_j][angular_n] = 
                initial_temperature_array[dust_i][Nr-1][angular_n]*
	        sqrt(initial_radial_array[Nr-1]/initial_radial_array[radial_j]);
	}
        radial_j=_INITIAL_GRID_ARRAY_r;
      }

    } /* end of the loop over radii */

    /* to speed up this thing we are copying now temperatures to all angles */
    /* In general, this is not a good thing, but it would slow down calculations
       quite a lot. */
    for (angular_n=_INITIAL_GRID_ARRAY_theta-2; angular_n>=0; angular_n--)
      for (radial_j=0; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++)
        for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
          if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
          initial_temperature_array[dust_i][radial_j][angular_n] = 
            initial_temperature_array[dust_i][radial_j][_INITIAL_GRID_ARRAY_theta-1];
    angular_n = -1;

  } /* end of the loop over angles */

  free_dvector(Radial_Optical_Depth,0,lambda_Max-1);
  free_dvector(U_source,0,lambda_Max-1);

  }
  else {
    /********************************************************/
    /* get temperature from the input file                  */ 
    /*      DEBUG.CONTINUE==100 || DEBUG.CONTINUE==200      */
    /********************************************************/

  /* first, read the temperature grid from a file */
  err = _PATULJAK__Read_Temperature(&Temp_r, &Nr, &Temp_theta, &Ntheta,
                                   &Temp_Temp);
  if (err>0) return 1;

/*
  Temp_Temp = dmatrix(0,Nr-1,0,Ntheta-1);
  Temp_r = dvector(0,Nr-1);
  Temp_theta = dvector(0,Ntheta-1);
*/

  if (DEBUG.SCREEN_MESSAGE >= 2 )
  for (radial_j=0; radial_j<Nr; radial_j++)
    for (angular_n=0; angular_n<Ntheta; angular_n++)
      printf("r[%d]=%E a[%d]=%E T=%E\n",radial_j,Temp_r[radial_j],
	     angular_n,Temp_theta[angular_n],Temp_Temp[radial_j][angular_n]);

  /* go over the initial grid (set the energy density to zero) */ 
  step_j = exp(log(Rout)/(_INITIAL_GRID_ARRAY_r-1));
  step_n = (_INITIAL_GRID_ARRAY_theta_max-_INITIAL_GRID_ARRAY_theta_min)
           /(_INITIAL_GRID_ARRAY_theta-1);
  /* set the default values of initial_radial_array */
  initial_radial_array[0] = 1.0;
  for (radial_j=1; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++) 
    initial_radial_array[radial_j] = initial_radial_array[radial_j-1]*step_j;
  initial_radial_array[_INITIAL_GRID_ARRAY_r-1] = Rout;
  /* go over all angular directions */
  for (angular_n=0; angular_n<_INITIAL_GRID_ARRAY_theta; angular_n++) {

     angle = angular_n*step_n+_INITIAL_GRID_ARRAY_theta_min;
    /* just to be sure that the last point 
       is exactly _INITIAL_GRID_ARRAY_theta_max */
    if (angular_n==_INITIAL_GRID_ARRAY_theta-1) 
       angle = _INITIAL_GRID_ARRAY_theta_max;

    if (DEBUG.SCREEN_MESSAGE >= 2 ) {
      printf("Grid angle #%d = %lf RAD ...\n",angular_n,angle);
    }

    x0 = sin(angle);
    z0 = cos(angle);
    r = 1.0;
 
    /* go over all radial grid points and update the dust temperatures */
    for (radial_j=0; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++) {      

      /* just to be sure that the last point is exactly Rout */
      if (radial_j==_INITIAL_GRID_ARRAY_r-1) r = Rout;
      x = r*x0;
      z = r*z0;

      /* find a temperature interpolation cell */
      inter_N_r = inter_N_theta = -1;

      if (r<=Temp_r[0]) inter_N_r=0;
      else if (r>=Temp_r[Nr-1]) inter_N_r=Nr-2;
      if (angle <= Temp_theta[0]) inter_N_theta = 0;
      else if (angle>=Temp_theta[Ntheta-1])inter_N_theta=Ntheta-2;

      if (inter_N_r<0) {
	inter_N_r = 0;
	while (r>Temp_r[inter_N_r+1]) inter_N_r++;
      }
      if (inter_N_theta<0){
	inter_N_theta=0;
	while (angle>Temp_theta[inter_N_theta+1]) inter_N_theta++;
      }

      /* find the new dust temperatures for all dust components */
      if (Temp_Temp[inter_N_r][inter_N_theta]<0.0 || 
          Temp_Temp[inter_N_r+1][inter_N_theta]<0.0 ||
          Temp_Temp[inter_N_r+1][inter_N_theta+1]<0.0 ||
          Temp_Temp[inter_N_r][inter_N_theta+1]<0.0)
        Temp = -1.0;
      else {
	if (Temp_r[inter_N_r]==Temp_r[inter_N_r+1]) {
	  Temp1 = Temp_Temp[inter_N_r][inter_N_theta];
	  Temp2 = Temp_Temp[inter_N_r][inter_N_theta+1];
	}
	else {
          err = _MATH_LIB_log_interpolation(
                 Temp_r[inter_N_r],Temp_Temp[inter_N_r][inter_N_theta],
                 Temp_r[inter_N_r+1],Temp_Temp[inter_N_r+1][inter_N_theta],
                 r,&Temp1);
          err = _MATH_LIB_log_interpolation(
                 Temp_r[inter_N_r],Temp_Temp[inter_N_r][inter_N_theta+1],
                 Temp_r[inter_N_r+1],Temp_Temp[inter_N_r+1][inter_N_theta+1],
                 r,&Temp2);
	}
	if (Temp_theta[inter_N_theta]==Temp_theta[inter_N_theta+1])
	  Temp = 0.5*(Temp1+Temp2);
	else err = _MATH_LIB_linear_interpolation(
                 Temp_theta[inter_N_theta],Temp1,
                 Temp_theta[inter_N_theta+1],Temp2,
                 angle,&Temp);
      }
      /* all dust components get the same temperature*/
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
	if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999){
	  /*if (Dust_Density(x,z,dust_i,NULL,&err)<=0.0) 
            initial_temperature_array[dust_i][radial_j][angular_n] = -1.0;
	    else */{
            initial_temperature_array[dust_i][radial_j][angular_n]=Temp;
            /* check if the new temperature is less then the sublimation 
               temperature + error tolerance */
            if (initial_temperature_array[dust_i][radial_j][angular_n]
               /Dust_Sublim_Temp[dust_i]-1.0 > _SUBLIMATION_TOLERANCE_) 
               initial_temperature_array[dust_i][radial_j][angular_n] = -1.0;
            /* check if the new temperature is acceptable, but higher then 
               the sublimation temperature */
            else 
              if (initial_temperature_array[dust_i][radial_j][angular_n] 
                                                    > Dust_Sublim_Temp[dust_i])
                initial_temperature_array[dust_i][radial_j][angular_n]
                                                    = Dust_Sublim_Temp[dust_i];
	  }
	}
	else 
	  /* empty space */ 
	  initial_temperature_array[dust_i][radial_j][angular_n]=_T99999_;
      } /* end of the dust-type loop */ 

      /* set the energy density */
      for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
        initial_energy_array[radial_j][angular_n][lambda_k]=0.0;

      /* increase the radius */ 
      r *= step_j;
    } /* end of the radial loop (counter: radial_j) */ 
  } /* end of theta-loop (counter: angular_n) */

  /* remove the input temperature arrays */
  free_dmatrix(Temp_Temp,0,Nr-1,0,Ntheta-1);
  free_dvector(Temp_r,0,Nr-1);
  free_dvector(Temp_theta,0,Ntheta-1);

  }
  return 0;
}
/************************************************************************/
/************************************************************************/
/* subroutine: find next rho or theta */
double _PERUNIKA_create_initial_Find_next_radius (double x_start, 
       double z_start, double r_end, int radial_j, int angular_n, 
       double *Tau_back, double *Tau_front, double *D_back, 
       double *D_front, int lambda_k){

  /* rho i theta of the current point in the search */
  double rho, theta, rho_new;
  /* points during vertex search */
  struct _3DVector r_old, r_new;
  /* increments */ 
  double tau_old, tau_new, L_old, L_new, dL; 
  int create_vertex;
  /* eta */
  double *temperature;
  double Eta_old, Eta_new;
  double temp_error;
  int dust_i;

  double ZETA = 2.0;      /* logarithmic scale in the optical depth criterium
                            for a new grid point */
  int dust_edge;
  int err;

  temperature = dvector(0,Max_Dust_Type-1);

  rho = sqrt(x_start*x_start + z_start*z_start);

  if (z_start==0.0) theta = PI/2.0;
  else theta = atan(x_start/z_start); 

  rho_new = rho;

  r_old.x = x_start; r_old.z = z_start;
  tau_old = 0.0; L_old = 0.0;
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
    temperature[dust_i] = 
      initial_temperature_array[dust_i][radial_j-1][angular_n];
  create_vertex = FALSE;
  Eta_old = Eta(x_start, z_start, lambda_k, temperature, &err);
  dL = 0.0001*_ZELENJAK_size_limit(r_old.x,r_old.z); /*??????*/

  /* start the search for the next opitmal vertex location */
  do {

    /* new point */
    r_new.x = r_old.x + dL*sin(theta);
    r_new.z = r_old.z + dL*cos(theta);
    rho_new = sqrt(r_new.x*r_new.x + r_new.z*r_new.z);
    L_new = L_old+dL;
    if (rho_new>Rout) {
      create_vertex = TRUE;
      rho_new=Rout;
      r_new.x = Rout*sin(theta);
      r_new.z = Rout*cos(theta);
      dL = Rout - rho; L_new = L_old + dL;
    }
    if (r_end>0.0 && rho_new>r_end) {
      create_vertex = TRUE;
      rho_new=r_end;
      r_new.x = r_end*sin(theta);
      r_new.z = r_end*cos(theta);
      dL = r_end - rho; L_new = L_old + dL;
    }

    /* new eta */
    Eta_new=Eta(r_new.x,r_new.z,lambda_k,temperature,&err);

    if (r_end>0.0) {
      /* THIS IS FOR THE CASE WHEN WE NEED JUST THE OPTICAL DEPTH */
      temp_error=0.5*dL*(Eta_old+Eta_new);
      if (tau_old == 0.0 && temp_error>0.1) 
        {create_vertex = FALSE; dL /= 4.0; continue;}
      else if (tau_old>0.0 && temp_error>4*tau_old)
        {create_vertex = FALSE; dL /= 2.2; continue;}
      tau_new = tau_old + temp_error;
      if (rho_new==r_end || rho_new==Rout) break;
    }
    else {
      /* THIS IS FOR THE CASE WHEN WE LOOK FOR THE NEXT POINT */
      if (rho_new>1.25*(rho_new-L_new)) {
        /* force a new point after 1.25*old_rho */
        create_vertex = TRUE;break;
      } else if (Eta_new==0.0 && (*Tau_front)==0.0) {
        /* continue with the search if empty space */
        create_vertex = FALSE;
        tau_new=tau_old;
      } else {
        /* check if the distance is optimal for a new vertex */
        temp_error=0.5*dL*(Eta_old+Eta_new);
        if (tau_old == 0.0 && temp_error>0.1) {
          /* decrease the initial step */
          dL /= 4.0;
          create_vertex = FALSE;
          continue;
        }
        else if (tau_old>0.0 && temp_error>4*tau_old){
          dL /= 2.2;/* step can not be larger than 4x the current tau */
          create_vertex = FALSE;
          continue;
        }
        tau_new = tau_old + temp_error;
        create_vertex = FALSE;
     if ((*Tau_back)<0.1) {
	  if (tau_new>0.1005) {
            dL/=1.8; /* locate the tau=0.1 surface precisely */
            create_vertex = FALSE;
            continue;
          }
          if (tau_new>0.1) {create_vertex = TRUE;break;}
        }
        else if (temp_error<0.01) dL *=2;
        else if (tau_new >= (*Tau_back)*
	         (ZETA*(*D_back)/((*D_back)+L_new)-1.0))
          {create_vertex = TRUE;break;}

        if ((*Tau_front)>0.1)
          if (tau_new>=((*Tau_front)-tau_new)*
	         (ZETA*((*D_front)-L_new)/(*D_front)-1.0))
            {create_vertex = TRUE;break;}
      }
    }

    /* prepare variables for the next step */
    rho = rho_new;
    r_old.x=r_new.x;
    r_old.z=r_new.z;
    L_old = L_new;
    Eta_old = Eta_new;
    tau_old = tau_new;
  } while (!create_vertex);

  (*Tau_back) += tau_new;
  (*Tau_front)-= tau_new;
  if ((*Tau_front)<0.0) (*Tau_front)=0.0;
  (*D_back) += L_new;
  (*D_front)-= L_new;
  if ((*D_front)<0.0) (*D_front)=0.0;

  free_dvector(temperature,0,Max_Dust_Type-1);

  return rho_new;
}

/************************************************************************/
