/* LELUYA - PERUNIKA program block */
/* This part of the LELUYA code is generating the computational grid */

/* This function calculates the optical depth and distance toward a given
   edge  point */

#include "PERUNIKA.h"
 
double _PERUNIKA_distance_to_edge(struct _vertex_structure *point, 
                              double edge_point_x,double edge_point_z,
                              int lambda_computational_grid, int curvature,
                              double *edge_point_distance,int status){
  /* point    = point from which we have to start the calculation
     edge_point_x, edge_point_y = coordinates of the point where we stop
                the calculation, if it has not been stopped before that
                if  (curvature==1) then (edge_point_x,edge_point_y = (x,z)
                if  (curvature==2) then (edge_point_x,edge_point_y = (r,theta)
                additionally: if curvature<0 then integrate over the edges 
     lambda_computational_grid = wavelength at which we need to create 
                                 the computational grid
     curvature = flag: 1=(x,z), 2=(r,theta)
     edge_point_distance = distance from the point to the detected dust edge
     status   = if status=0 or status=1, the temperature 
                values will be taken from the initial approximation arrays,
                if status = 2, then they will be taken from the grid
                built in the previous iteration */
  
  double *temperature, *temperature_old, temp_error;
  int dust_i,error,temp_step;
  int total_number_of_points = 50;
  short int grid_method;
  double optical_depth_distance;
  double x,z,dx,dz,dd,Dtotal,dtau;
  double eta_start,eta0, eta1;
  double theta;
  int direction, edge_detected, pass_edge;

  temperature = dvector(0,Max_Dust_Type-1);  
  temperature_old = dvector(0,Max_Dust_Type-1);  
  
  pass_edge = FALSE; /* this flag tells us if we need to ignore the edges and
                        go all the way to (edge_point_x,edge_point_z) */
  if (curvature<0) {curvature*= -1; pass_edge = TRUE;}

  *edge_point_distance = 0.0;
  optical_depth_distance = 0.0;
  edge_detected = FALSE;
  x = point->x;  z = point->z;
  if (curvature==1) {
    Dtotal = _MATH_LIB_distance(point->x,point->z,edge_point_x,edge_point_z);
  }
  if (curvature==2) {
    Dtotal = edge_point_z - atan(point->x/point->z);
    if (Dtotal>=0.0) direction=1;
    else {direction=-1;Dtotal=fabs(Dtotal);}
    Dtotal *= edge_point_x;
  }
  if (Dtotal==0.0) return 0.0;
  if (curvature==1) {
    dx = (edge_point_x - point->x)/Dtotal;
    dz = (edge_point_z - point->z)/Dtotal;
  }
  temp_error=-1.0; /* flag for not calculating T_error in _PERUNIKA_temperature */
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
      temperature_old[dust_i]=_PERUNIKA_temperature(x,z,dust_i,status,&temp_error);
    else temperature_old[dust_i]=-1.0;
  }
 
  eta_start = eta0 = Eta(x,z,lambda_computational_grid,temperature_old,&error);

  if (eta_start>0.0) {
    if (Dtotal < total_number_of_points*_ZELENJAK_size_limit(x,z)) 
      {grid_method = 0; dd = Dtotal/total_number_of_points;}
    else 
      {grid_method = 1; dd = _ZELENJAK_size_limit(x,z);}
    do {
      *edge_point_distance += dd;
      if (*edge_point_distance>Dtotal) {
        dd=Dtotal - (*edge_point_distance) + dd; 
        *edge_point_distance = Dtotal;
      }
      if (curvature==1) {

	if (z*(z+dd*dz)<0.0 && dz!=0.0) {
          *edge_point_distance -= dd;
          dd = - z/dz;
          z = 0.0;
          *edge_point_distance += dd;
        }
	else z += (dd*dz);
        x += (dd*dx);
      }
      if (curvature==2) {
        /* edge_point_x is equal to rho */ 
        if (z==0.0) {if(x>0.0) theta = PI/2; else theta=3*PI/2;}
        else {
          if (z>0.0 && x<0.0) theta = 2*PI+atan(x/z);
	  else if  (z<0.0) theta = PI+atan(x/z);
          else theta = atan(x/z);
	}
        dx=x;x = edge_point_x*sin(theta+direction*dd/edge_point_x);
        dz=z;z = edge_point_x*cos(theta+direction*dd/edge_point_x);
      }
      temp_error=-1.0; /* flag for not calculating T_error in _PERUNIKA_temperature */
      temp_step = FALSE;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
        if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
          temperature[dust_i]=_PERUNIKA_temperature(x,z,dust_i,status,&temp_error);
          if (temperature_old[dust_i]*temperature[dust_i]<0.0) temp_step = TRUE;
	}
        else temperature[dust_i]=-1.0;
      }
      eta1 = Eta(x,z,lambda_computational_grid,temperature,&error);
      dtau = (eta0+eta1)*dd/2.0;
      /* check if we cross the dust surface (while not having required resolution)
         or the inital tau step is too small */
      if ((temp_step || eta1<=0.0 || (optical_depth_distance==0.0 && dtau>0.1)) 
          && dd > 1.0E-10) {
        /* edge crossing */
        if (temp_step || eta1<=0.0) edge_detected = TRUE;
        (*edge_point_distance) -= dd;
        if (curvature==1) {
          x -= (dd*dx); 
          z -= (dd*dz);
        }
        if (curvature==2) {
          x = dx; 
          z = dz;
        }
        dd /= 4.0; /* grid refinement */
        eta1 = 1.0; /* to keep the global loop going */
      }
      else {
        /* no edge crossing */ 
        optical_depth_distance += dtau;
        eta0=eta1;
        /*if (grid_method==1) dd = _ZELENJAK_size_limit(x,z);*/
        for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
          temperature_old[dust_i]=temperature[dust_i];
        /* increse the step if it is OK for tau */
	if (edge_detected && dd <= 1.0E-10) edge_detected=FALSE; /* we passed the edge */
        if (!edge_detected) 
          if (dtau<0.01*optical_depth_distance && 
	      dd < 0.1*_ZELENJAK_size_limit(x,z)) {dd *= 2;}
      }

    } while ((*edge_point_distance) < Dtotal && eta1 > 0.0 && 
             (dd > 1.0E-10 || pass_edge));
  }
  else {
    *edge_point_distance = 0.0;
    optical_depth_distance = 0.0;
  }
 
  free_dvector(temperature,0,Max_Dust_Type-1);  
  free_dvector(temperature_old,0,Max_Dust_Type-1);  
  return optical_depth_distance;
}
