/* LELUYA - PERUNIKA program block */
/* This part of the LELUYA code is generating the computational grid */

/* The function is used for initialization of the energy density and 
   temperature arrays*/

#include "PERUNIKA.h"

/* functions used for plotting the initial temperature and energy density */
double _PERUNIKA_plot_initial_temperature(double x, double z, int dust_i,
                                      int dummy);
double _PERUNIKA_plot_initial_energy(double x, double z, int dummy, int lambda_k);
double _PERUNIKA_plot_dust_density(double x, double z, int dust_i, int dummy);
double _PERUNIKA_plot_Eta(double x, double z, int dummy, int lambda_k);
int    _PERUNIKA_initialize_initial_arrays(int status);
int    _PERUNIKA_image_html(char image_name[255], int image_plot_type, 
                        int image_type, double minX, 
                        double maxX, double minY, double maxY, int dust_i, 
                        int lambda_k);
/* some global variables used in the plotting process ***/
  double plot_initial_energy_log10_U_max = 1.;   /* plotting of the energy...*/
  double plot_initial_energy_log10_U_min = -10.; /* ..density in log scale   */
  double plot_initial_temperature_max;        /* the upper temperature limit */


/***** used for initialization of the energy density ******************/
/***************** and temperature arrays *****************************/
/***** (it is relevant only if status = 0 or status = 1 ) *************/
int  _PERUNIKA_initialize_initial_arrays(int status){
  int err_location, err;              /* error flags */
  int dust_i;                         /* counter over the dust components */
  int radial_j;                       /* counter over the radial grid steps */
  int lambda_k=0;                     /* counter over the lambda grid steps */
  int angular_n;                      /* counter over the angular grid steps */
  char image_name[255];  /* name of a debugger image */
  FILE *DUMP_FILE;       /* file with dumped initial data arrays */
  char *dumpfilename = "LELUYA.dump.STOP201.2dd";
  int  dummy_int;        /* dummy variable without special purpose */

  /*-------------------------------------------------------------------------*/
  /* calculate the initial arrays */
  if (status == 0) {
    err = _PERUNIKA_create_initial_array();
    if (err) return 1;

    if (DEBUG.STOP==201) {
      _PESJANEK_Dump_Data_STOP2 ();
      /* dump the initial arrays */
      DUMP_FILE = fopen (dumpfilename,"wb");
      if (DUMP_FILE==NULL) {
        printf("---- WARNING!! Can not opet the dump file:\n");
        printf("               %s\n",dumpfilename);
      }
      else {
        fprintf(DUMP_FILE,"%d %d %d %d",Max_Dust_Type,lambda_Max,
                _INITIAL_GRID_ARRAY_r,_INITIAL_GRID_ARRAY_theta);
        for (angular_n=0; angular_n<_INITIAL_GRID_ARRAY_theta; angular_n++) {
          for (radial_j=0; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++) {      
            for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) 
              fwrite(&initial_temperature_array[dust_i][radial_j][angular_n],
                     sizeof(double),1,DUMP_FILE);
            for (lambda_k=0;lambda_k<lambda_Max;lambda_k++) 
              fwrite(&initial_energy_array[radial_j][angular_n][lambda_k],
                     sizeof(double),1,DUMP_FILE);
	  }
	}
      }
      fclose(DUMP_FILE);
      printf("---- WARNING!! The program is stopped due to the user's \n");
      printf("               command line flag: ");
      printf("-s 201\n");
      printf("     location of the stop: after the initial approximative\n");
      printf("     temperature and energy density have been calculated\n");
      return 1;
    }

  }

  /*-------------------------------------------------------------------------*/
  /* read the initial arrays */
  if (status == 1 ) {
    DUMP_FILE = fopen (dumpfilename,"rb");
    if (DUMP_FILE==NULL) {
      err_location = 9; err = 3;
      _PESJANEK__Error_Message (err_location,err);
      printf("%s\n",dumpfilename);
      fclose(DUMP_FILE);
      return 1;
    }
    fscanf(DUMP_FILE,"%d",&dummy_int);
    if (dummy_int!=Max_Dust_Type) {
      err_location = 9; err = 4;
      _PESJANEK__Error_Message (err_location,err);
      printf("dump file: %s\n",dumpfilename);
      printf("dump file has %d dust components\n",dummy_int);
      printf("we need %d dust components\n",Max_Dust_Type);
      fclose(DUMP_FILE);
      return 1;
    }
    fscanf(DUMP_FILE,"%d",&dummy_int);
    if (dummy_int!=lambda_Max) {
      err_location = 9; err = 4;
      _PESJANEK__Error_Message (err_location,err);
      printf("dump file: %s\n",dumpfilename);
      printf("dump file has %d wavelengths\n",dummy_int);
      printf("we need %d wavelengths\n",lambda_Max);
      fclose(DUMP_FILE);
      return 1;
    }
    fscanf(DUMP_FILE,"%d",&dummy_int);
    if (dummy_int!=_INITIAL_GRID_ARRAY_r) {
      err_location = 9; err = 4;
      _PESJANEK__Error_Message (err_location,err);
      printf("dump file: %s\n",dumpfilename);
      printf("dump file has %d radial array points\n",dummy_int);
      printf("we need %d radial array points\n",_INITIAL_GRID_ARRAY_r);
      fclose(DUMP_FILE);
      return 1;
    }
    fscanf(DUMP_FILE,"%d",&dummy_int);
    if (dummy_int!=_INITIAL_GRID_ARRAY_theta) {
      err_location = 9; err = 4;
      _PESJANEK__Error_Message (err_location,err);
      printf("dump file: %s\n",dumpfilename);
      printf("dump file has %d angular array points\n",dummy_int);
      printf("we need %d angular array points\n",_INITIAL_GRID_ARRAY_theta);
      fclose(DUMP_FILE);
      return 1;
    }
    for (angular_n=0; angular_n<_INITIAL_GRID_ARRAY_theta; angular_n++) {
      for (radial_j=0; radial_j<_INITIAL_GRID_ARRAY_r; radial_j++) {      
        for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
          fread(&initial_temperature_array[dust_i][radial_j][angular_n],
                sizeof(double),1,DUMP_FILE);
	err = fread(initial_energy_array[radial_j][angular_n],sizeof(double),
              lambda_Max,DUMP_FILE);
        if (err != lambda_Max) {
          err_location = 9; err = 4;
          _PESJANEK__Error_Message (err_location,err);
          printf("dump file: %s\n",dumpfilename);
          printf("can not read the data!!\n");
          printf("(radial #%d, angular #%d)\n",radial_j,angular_n);
          fclose(DUMP_FILE);
          return 1;
	}
      }
    }
    fclose(DUMP_FILE);
  }

  /*-------------------------------------------------------------------------*/
  if ((status == 0 || status ==1) && (DEBUG.SCREEN_MESSAGE >= 2 )){
 
    /* find the upper scale for the temperature image */
    plot_initial_temperature_max = Dust_Sublim_Temp[0];
    if (Max_Dust_Type > 1) 
      for (dust_i=1; dust_i<Max_Dust_Type; dust_i++) 
        if (Dust_Sublim_Temp[dust_i] > plot_initial_temperature_max)
          plot_initial_temperature_max = Dust_Sublim_Temp[dust_i];

    for (dust_i=0;dust_i<Max_Dust_Type; dust_i++){ 
      sprintf(image_name,"2D_dusty.initial_temperature.dust%.3d.rect",dust_i);
      err = _PERUNIKA_image_html(image_name,1,0,
                             0,log10(Rout),0,log10(Rout),dust_i,0);
      err = _VODENJAK_plot_image(image_name,&_PERUNIKA_plot_initial_temperature,
                                 0,0,log10(Rout),0,log10(Rout),dust_i,0);
      sprintf(image_name,"2D_dusty.dust_density.dust%.3d.rect",dust_i);
      err = _PERUNIKA_image_html(image_name,0,0,
                             0,log10(Rout),0,log10(Rout),dust_i,0);
      err = _VODENJAK_plot_image(image_name,&_PERUNIKA_plot_dust_density,
                                 0,0,log10(Rout),0,log10(Rout),dust_i,0);
    }
    for (dust_i=0;dust_i<Max_Dust_Type; dust_i++){ 
      sprintf(image_name,"2D_dusty.initial_temperature.dust%.3d.polar",dust_i);
      err = _PERUNIKA_image_html(image_name,1,1,0,PI/2,0,log10(Rout),dust_i,0);
      err = _VODENJAK_plot_image(image_name,&_PERUNIKA_plot_initial_temperature,
                                 1,0,PI/2,0,log10(Rout),dust_i,0);
      sprintf(image_name,"2D_dusty.dust_density.dust%.3d.polar",dust_i);
      err = _PERUNIKA_image_html(image_name,0,1,0,PI/2,0,log10(Rout),dust_i,0);
      err = _VODENJAK_plot_image(image_name,&_PERUNIKA_plot_dust_density,
                                 1,0,PI/2,0,log10(Rout),dust_i,0);
    }
    /* plot Eta function at Lambda_Total */
    sprintf(image_name,"2D_dusty.Eta.%8lfmic.rect",Lambda_Total_Value);
    err = _VODENJAK_plot_image(image_name,&_PERUNIKA_plot_Eta,
                               0,0,log10(Rout),0,log10(Rout),0,Lambda_Total);

    if (DEBUG.SCREEN_MESSAGE >= 3)
      for (lambda_k=0;lambda_k<lambda_Max;lambda_k+=5) 
        if ((lambda_k>=15 && lambda_k<=20) || lambda_k==30 ||
            lambda_k==44 || lambda_k==60 || lambda_k==70 || lambda_k==82 ||
            lambda_k==95)   {
      sprintf(image_name,"2D_dusty.initial_energy.%8lfmic.rect",
              Lambda_Grid[lambda_k]);
        err = _VODENJAK_plot_image(image_name,&_PERUNIKA_plot_initial_energy,
                                   0,0,log10(Rout),0,log10(Rout),0,lambda_k);
      sprintf(image_name,"2D_dusty.initial_energy.%8lfmic.polar",
              Lambda_Grid[lambda_k]);
        err = _VODENJAK_plot_image(image_name,&_PERUNIKA_plot_initial_energy,
                                   1,0,PI/2,0,log10(Rout),0,lambda_k);
    }

  }

  return 0;
} /* end of _PERUNIKA_initialize_initial_arrays  */

/********* creates the HTML files which describe the image files ********/
int _PERUNIKA_image_html(char image_name[255], int image_plot_type, int image_type,
                     double minX, double maxX, double minY, double maxY, 
                     int dust_i, int lambda_k){
  /* input variables:
  image_name   = name of the file without the suffix ".html" 
  image_plot_type = plot type : 0 = dust density
                                1 = temperature
                                2 = energy density
  image_type   = image coordinates : 0 = rectangular plot (logarithmic radius)
                                     1 = polar plot
                 rectangular plot is an x-y image with the real axes x and z
                 polar plot is an image with the real axes theta and log10(r)
  minX, maxX   = if (image_type=0) then minX and maxX are minimal and maximal
                                        value of the log10(x) coordinate
                 if (image_type=1) then minX and maxX are minimal and maximal
                                        value of the theta angle
  minY, maxY   = if (image_type=0) then minY and maxY are minimal and maximal
                                        value of the log10(z) coordinate
                 if (image_type=1) then minY and maxY are minimal and maximal
                                        value of the log10(r) value
  dust_i       = dust component
  lambda_k     = wavelength's index
  */

  FILE *HTML_FILE;
  char filename[260];  /* name of the html file */


  /* check the image type */
  if (image_type<0 || image_type>1) {
    printf("WARNING !!! The HTML image type %d does not exist!!\n",image_type);
    return 1;
  }
  
  /* open the HTML file */
#ifdef _2DD_MPI_
  sprintf(filename,"%s/%s.html",_LELUYA_MPIOUT_PATH,image_name);
#else
  sprintf(filename,"%s/%s.html",_LELUYA_OUTPUT_PATH,image_name);
#endif
  HTML_FILE = fopen(filename,"wt");
  if (HTML_FILE == NULL) {
    printf("WARNING !!! Can not open the HTML file:\n");
    printf("%s    ....continue\n",filename);
    fclose(HTML_FILE);
    return 1;
  }

  if (DEBUG.SCREEN_MESSAGE >= 1 )
    printf("PERUNIKA block: HTML file: %s\n",filename); 

  fprintf(HTML_FILE,"<html><head><meta name=\"Author\"");
  fprintf(HTML_FILE,"content=\"PERUNIKA program block\">\n");
  fprintf(HTML_FILE,"<title>LELUYA: results</title></head>\n");
  fprintf(HTML_FILE,"<body text=\"#FFFFFF\" bgcolor=\"#000000\" ");
  fprintf(HTML_FILE,"link=\"#FFFF00\" vlink=\"#3366FF\" alink=\"#FF0000\">");

  fprintf(HTML_FILE,"<b><font size=+3>Image name: <font color=\"#FFCC33\">");
  fprintf(HTML_FILE,"%s </b></font></font><BR>\n",image_name);

  if (image_plot_type == 0 || image_plot_type == 1) {
    fprintf(HTML_FILE,"Temperature of the dust component #%d<BR>\n",dust_i);
    fprintf(HTML_FILE,"   The name of this dust component = %s<BR>\n",
            Dust_Component_Name[dust_i]);
    fprintf(HTML_FILE,"   The name of the density profile = %s<BR>\n",
            INPUT.DUST_DENSITY_TYPE_NAME[dust_i]);
  }

  fprintf(HTML_FILE,"<BR><BR><BR>\n");
  fprintf(HTML_FILE,"<table BORDER=0 CELLSPACING=0 CELLPADDING=0 COLS=3 ");
  fprintf(HTML_FILE,"WIDTH=\"%d\" NOSAVE >\n<tr NOSAVE>",
                     200+PLOT_IMAGE_SIZE_X);
  fprintf(HTML_FILE,"<td WIDTH=\"100\" NOSAVE>\n");
  fprintf(HTML_FILE,"<table BORDER=0 CELLSPACING=0 CELLPADDING=0 COLS=1 ");
  fprintf(HTML_FILE,"HEIGHT=\"%d\" NOSAVE >\n<tr NOSAVE>\n",PLOT_IMAGE_SIZE_Y);
  fprintf(HTML_FILE,"<td ALIGN=RIGHT VALIGN=TOP HEIGHT=\"50%%\" NOSAVE>\n");

  if (image_type==0) fprintf(HTML_FILE,"log10(Z)=%4.0lf",maxY);
  if (image_type==1) fprintf(HTML_FILE,"log(r)=%5.2lf",maxY);

  fprintf(HTML_FILE,"</td></tr><tr NOSAVE>");
  fprintf(HTML_FILE,"<td ALIGN=RIGHT VALIGN=BOTTOM HEIGHT=\"50%%\" NOSAVE>");
  
  if (image_type==0) fprintf(HTML_FILE,"log10(Z)=%5.2lf",minY);
  if (image_type==1) fprintf(HTML_FILE,"log(r)=%5.2lf",minY);

  fprintf(HTML_FILE,"</td></tr></table></td>\n");
  fprintf(HTML_FILE,"<td WIDTH=\"%d\" NOSAVE><img SRC=\"",PLOT_IMAGE_SIZE_X);
  fprintf(HTML_FILE,"%s.gif\" NOSAVE height=%d width=%d></td>\n",
          image_name,PLOT_IMAGE_SIZE_Y,PLOT_IMAGE_SIZE_X);
  fprintf(HTML_FILE,"<td WIDTH=\"100\" NOSAVE>\n");
  fprintf(HTML_FILE,"<table BORDER=0 CELLSPACING=0 CELLPADDING=0 "); 
  fprintf(HTML_FILE,"COLS=1 HEIGHT=\"%d\" NOSAVE >\n",PLOT_IMAGE_SIZE_Y);
  fprintf(HTML_FILE,"<tr NOSAVE><td ALIGN=LEFT VALIGN=TOP NOSAVE>\n");
  
  if (image_plot_type==0) fprintf(HTML_FILE,"density=1.0\n");
  if (image_plot_type==1) fprintf(HTML_FILE,"T=%5.0lfK\n",
                                              plot_initial_temperature_max);
  if (image_plot_type==2) fprintf(HTML_FILE,"log(energy\ndensity)=%3.0lf\n",
                                           plot_initial_energy_log10_U_max);
  
  fprintf(HTML_FILE,"</td></tr><tr NOSAVE>");
  fprintf(HTML_FILE,"<td ALIGN=LEFT VALIGN=BOTTOM NOSAVE>\n");

  if (image_plot_type==0) fprintf(HTML_FILE,"density=0.0\n");
  if (image_plot_type==1) fprintf(HTML_FILE,"T=0K\n");
  if (image_plot_type==2) fprintf(HTML_FILE,"log(energy\ndensity)=%3.0lf\n",
                                           plot_initial_energy_log10_U_min);

  fprintf(HTML_FILE,"</td></tr></table></td></tr>\n<tr NOSAVE><td></td>\n");
  fprintf(HTML_FILE,"<td ALIGN=CENTER VALIGN=TOP NOSAVE>\n");
  fprintf(HTML_FILE,"<table BORDER=0 CELLSPACING=0 CELLPADDING=0 COLS=2 ");
  fprintf(HTML_FILE,"WIDTH=\"100%%\" HEIGHT=\"100%%\" NOSAVE >\n");
  fprintf(HTML_FILE,"<tr NOSAVE><td ALIGN=LEFT VALIGN=TOP NOSAVE>\n");
  
  if (image_type==0) fprintf(HTML_FILE,"log10(X)=%5.2lf\n",minX);
  if (image_type==1) fprintf(HTML_FILE,"theta=%3.0lfdeg\n",180*minX/PI);

  fprintf(HTML_FILE,"</td><td ALIGN=RIGHT VALIGN=TOP NOSAVE>\n");
  
  if (image_type==0) fprintf(HTML_FILE,"log10(X)=%4.0lf\n",maxX);
  if (image_type==1) fprintf(HTML_FILE,"theta=%3.0lfdeg\n",180*maxX/PI);

  fprintf(HTML_FILE,"</td></tr></table></td><td></td></tr></table>\n");
  fprintf(HTML_FILE,"</body></html>");



  fclose(HTML_FILE);

  return 0;

}


/****************************************************************************/
/***** used for plotting the initial temperature ************************/
double _PERUNIKA_plot_initial_temperature(double x, double z, int dust_i,
                                      int dummy){
  /* x,z = coordinates of a point;
     dust_i = dust component
     dummy = it has no meaning. It is used just to be compatible with
             _PERUNIKA_plot_initial_energy
  */

  int i;
  double T;

  T = _PERUNIKA_initial_temperature (x, z, dust_i);
  if (T<=0.0) return -1.0;
  else if (T>=plot_initial_temperature_max) return 1.0;
  else T /= plot_initial_temperature_max;

  return T;
}


/***** used for plotting the initial energy density ************************/
double _PERUNIKA_plot_initial_energy(double x, double z, int dummy, int lambda_k){
  /* x,z = coordinates of a point
     dummy = it has no meaning. It is used just to be compatible with 
             _PERUNIKA_plot_initial_energy
     lambda_k = wavelength's index
  */

  int i;
  double U;

  U = _PERUNIKA_initial_energy_density(x,z, lambda_k);
  
  if (U <= 0.0) return -1.0;

  U = log10(U);

  U = (U - plot_initial_energy_log10_U_min) / 
      (plot_initial_energy_log10_U_max - plot_initial_energy_log10_U_min);
  if (U < 0.0) return 0.0;
  if (U > 1.0) return 1.0;
  return U;
}

/***** used for plotting the dust density ************************/
double _PERUNIKA_plot_dust_density(double x, double z, int dust_i, int dummy){
  /* x,z = coordinates of a point
     dummy = it has no meaning. It is used just to be compatible with 
             _PERUNIKA_plot_initial_energy
     dust_i = dust component
  */
  double density;
  double Temp;
  int err;

  density = Dust_Density(x,z,dust_i,NULL,&err);
  if (density <= 0.0) return -1.0;
  
  Temp = _PERUNIKA_initial_temperature (x, z, dust_i);
  if (Temp <= 0.0 || Temp > Dust_Sublim_Temp[dust_i]) return -1.0;

  return density;
}

/***** used for plotting the Eta function ****************************/
double _PERUNIKA_plot_Eta(double x, double z, int dummy, int lambda_k){
  /* x,z = coordinates of a point
     dummy = it has no meaning. It is used just to be compatible with 
             _PERUNIKA_plot_initial_energy
     lambda_k = wavelength's index
  */
  int dust_i, error;
  double *temperature;
  double eta_value;
  double temp_error;

  temperature = dvector(0,Max_Dust_Type-1);

  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) 
    temperature[dust_i]=_PERUNIKA_temperature(x,z,dust_i,0,&temp_error);
  eta_value=Eta_Denominator*Eta(x,z,lambda_k,temperature,&error);

  if (eta_value<= 0.0) eta_value = -1.0;
  else {eta_value = log10(eta_value)/1.8;if (eta_value<= 0.0) eta_value = 0.0;}

  free_dvector(temperature,0,Max_Dust_Type-1);

  return eta_value;

}
