/* LELUYA - PERUNIKA program block */
/* This part of the LELUYA code is generating the computational grid */

/* This function is searching through the old_triangle_list for a triangle
   which contains a given point (x,z) and dust component */

#include "PERUNIKA.h"

int _PERUNIKA_locate_point_in_old_grid(double x,double z,int dust_i,
                                struct _triangle_structure **old_triangle,
				struct _quadrangle_structure **old_quadrangle){
  /* search for 'triangle' or 'quadrangle' (if 'triangle' doesn't exist), 
     which contains the given point (x,z) where the dust component dust_i
     exists */

  int I_have_found_it;
  double nu1,nu2,nu3,nu4;
  struct _vertex_structure *v1,*v2,*v3,*v4;
  struct _triangle_structure *triangle;
  struct _quadrangle_structure *quadrangle;

  I_have_found_it = 0; 
  triangle=NULL; quadrangle=NULL;

  triangle=start_old_triangle_list;
  do {
   /* check if the point (x,z) is inside of the bounding box */
    if (x >= 0.9999*triangle->bound_box_down_left_x)
    if (x <= 1.0001*triangle->bound_box_up_right_x)
    if (z >= 0.9999*triangle->bound_box_down_left_z)
    if (z <= 1.0001*triangle->bound_box_up_right_z) {
      /* check if the point (x,z) is inside of the triangle */
      /* This is the case if the point (x,z) is on the left side relative
         to all triangle's sides:

                    (x,z)
                     /
                    /    LEFT
                   / 
                vertex(i)-------vertex(i+1)

                        RIGHT
      */

      nu1=_MATH_LIB_nu_value(triangle->v1->x,triangle->v1->z,
      		       triangle->v2->x,triangle->v2->z,x,z);
      nu2=_MATH_LIB_nu_value(triangle->v2->x,triangle->v2->z,
       		       triangle->v3->x,triangle->v3->z,x,z);
      nu3=_MATH_LIB_nu_value(triangle->v3->x,triangle->v3->z,
       		       triangle->v1->x,triangle->v1->z,x,z);
      if (0.0<=nu1 && 0.0<=nu2 && 0.0<=nu3 && 
          triangle->v1->T_dust[dust_i]>0.0 &&
          triangle->v2->T_dust[dust_i]>0.0 &&
          triangle->v3->T_dust[dust_i]>0.0 ) {
        /* bingo! That's the triangle! */
        I_have_found_it = 1; 
        break;
      }
    }
    triangle=triangle->next;
  } while (triangle!=start_old_triangle_list);

  if (I_have_found_it==0) {
    triangle=NULL; 
    /* check the quadrangles (extensions of the sublimation surfaces) */
    quadrangle=start_quadrangle_list;
    while (quadrangle!=NULL) {
      if (quadrangle->dust_component==dust_i     && 
	  x >= quadrangle->bound_box_down_left_x &&
          x <= quadrangle->bound_box_up_right_x  &&
          z >= quadrangle->bound_box_down_left_z &&
          z <= quadrangle->bound_box_up_right_z  ){

        v1=quadrangle->edge_vertex;
        v2=quadrangle->next_edge_vertex;
        v3=quadrangle->new_next_edge_vertex;
        v4=quadrangle->new_edge_vertex; 
        /*
        The general description of a quadrangle:

         (v3)                            (v2)***************************
   new_next_edge_vertex ----<------ next_edge_vertex********************
          |                                |****************************
          |                                |******** region where ******
          V                                ^***** the dust component ***
          |                                |*********** exists *********
          |                                |****************************
    new_edge_vertex ----->----------- edge_vertex***********************
         (v4)                            (v1)***************************
                                            ****************************
	*/
        /* check if the point (x,z) belongs to that quadrangle */
        nu1=_MATH_LIB_nu_value(v1->x,v1->z,v2->x,v2->z,x,z);
        if (v2==v3) nu2=1.0;
        else nu2=_MATH_LIB_nu_value(v2->x,v2->z,v3->x,v3->z,x,z);
        if (v3==v4) nu3=1.0;
        else nu3=_MATH_LIB_nu_value(v3->x,v3->z,v4->x,v4->z,x,z);
        if (v4==v1) nu4=1.0;
        else nu4=_MATH_LIB_nu_value(v4->x,v4->z,v1->x,v1->z,x,z);
        if (0.0<=nu1 && 0.0<=nu2 && 0.0<=nu3 &&0.0<=nu4){
	  /* yahoo! The sublimation extension saved me ! */ 
	  I_have_found_it = 1;
          break;
        } 
      }
      quadrangle=quadrangle->next; 
    }
  } /* end of the quadrangle check */

  if (I_have_found_it == 0) {triangle=NULL; quadrangle=NULL;}

  if(triangle!=NULL) *old_triangle=triangle;
  else *old_triangle=NULL;
  if(quadrangle!=NULL) *old_quadrangle=quadrangle;
  else *old_quadrangle=NULL;

  return 0;
}
