/* LELUYA - PERUNIKA program block */
/* This part of the LELUYA code is generating the computational grid */

/* This function is calculating the optical depth between two points in the
   dust cloud.*/

#include "PERUNIKA.h"

double _PERUNIKA_optical_depth_distance (double Ax,double Az,double Bx,double Bz,
                                     int lambda_computational_grid,
                                     int status, short int *split_flag){
  /* A(x,z), B(x,z)  = points between which we have to calculate the optical
                       depthcd 
     lambda_computational_grid = wavelength at which we need to create 
                                 the computational grid
     status   = if status=0 or status=1, the temperature 
                values will be taken from the initial approximation arrays,
                if status = 2, then they will be taken from the grid
                built in the previous iteration
     split_flag = flag which shows if the integration is going over a dust
                  boundary or the eta functions is extremely steep */

  double *distance,*temperature,*temperature_prev,temp_error;
  double eta_0, eta_1;
  int total_number_of_points = 20;
  double x,z;
  double dx,dz,dd,D,Dab;
  int counter_dd,dust_i,error;
  double optical_depth_distance;
  double eta_prev;

  distance = dvector(0,total_number_of_points);
  temperature = dvector(0,Max_Dust_Type-1);  
  temperature_prev = dvector(0,Max_Dust_Type-1);  

  Dab  = _MATH_LIB_distance(Ax,Az,Bx,Bz);
  temp_error=-1.0; /* flag for not calculating T_error in _PERUNIKA_temperature */
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
      temperature_prev[dust_i]=_PERUNIKA_temperature(Ax,Az,
                                               dust_i,status,&temp_error);
    else temperature_prev[dust_i]=-1.0;
  }
  eta_prev = Eta(Ax,Az,lambda_computational_grid,
                 temperature_prev,&error);

  /* NEW method  */
    optical_depth_distance = 0.0;
    x = Ax;  z = Az;
    dx = (Bx - Ax)/Dab;
    dz = (Bz - Az)/Dab;
    eta_0= eta_prev;
    dd = _ZELENJAK_size_limit(x,z)/10.;
    D = 0.0;
    while (D < Dab) {
      D += dd;
      if (D>Dab) {dd = Dab - D + dd;  D=Dab;}
      x += (dd*dx); 
      z += (dd*dz);
  temp_error=-1.0; /* flag for not calculating T_error in _PERUNIKA_temperature */
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
        if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
          temperature[dust_i]=_PERUNIKA_temperature(x,z,dust_i,status,&temp_error);
	else temperature[dust_i]=-1.0;
        /* locate a possible dust edge */ 
        if (temperature[dust_i]*temperature_prev[dust_i]<0.0) *split_flag=1;
      }
      eta_1 = Eta(x,z,lambda_computational_grid,temperature,&error);
      optical_depth_distance += ((eta_0+eta_1)*dd/2.0);
      /* locate a possible steep slope of eta */ 
      if (eta_0/eta_1 > ETA_RATIO_LIMIT ||
          eta_1/eta_0 > ETA_RATIO_LIMIT ) *split_flag=1;
      eta_0=eta_1;
      dd = _ZELENJAK_size_limit(x,z)/10.;
    }

  free_dvector(distance,0,total_number_of_points);
  free_dvector(temperature,0,Max_Dust_Type-1);  
  free_dvector(temperature_prev,0,Max_Dust_Type-1);  

  return optical_depth_distance;
}
