/* LELUYA - PERUNIKA program block */
/* This subroutine removes vertexes with too hot dust */


#include "PERUNIKA.h" 

void _PERUNIKA_remove_hot_dust (int status) {
  /* 
  status = 0 : calculate the grid from the initial approximation
                  (a point source, neglected diffuse radiation )
  status = 1 : read the initial approximation from a file 
  status = 2 : calculate the grid from a previous iteration
  */

  int err;              /* error flags */
  int dust_i;                         /* counter over the dust components */
  int lambda_k, lambda_kk;            /* counter over the lambda grid steps */
  short int split_flag;               /* used for calculation of the optical
                                         depth */
  struct _triangle_structure *current_triangle; /* triangle in the loop */
  struct _triangle_structure *next_triangle; /* next triangle in the loop */
  struct _vertex_structure *loop_vertex;
  int lambda_N_max; /* wavelength with the most grid points */
  int N_max;        /* number of vertices at lambda_N_max */
  double d_i,tau;   
  struct _3DVector startpoint, endpoint;
  double newAx, newAz,  newBx, newBz,  newCx, newCz, *newx, *newz;
  int new_total, new_n;

  /* find a grid with the largest number of points (let's call it grid_N_max)*/
  lambda_N_max = 0;
  N_max = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++) {
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > N_max) {
      N_max = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES;
      lambda_N_max = lambda_k;
    }
  }

  /**** new points memorized: ****************/
  start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle;
  start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
  TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
  TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
  new_total = TOTAL_NUMBER_OF_VERTICES;
  newx = dvector(0,new_total-1);
  newz = dvector(0,new_total-1);

  /***** add vertexes at the dust edges ******/
  current_triangle = grid_per_lambda[lambda_N_max].start_triangle;
  new_n = 0;
  do {
    next_triangle = current_triangle->prev;
    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
     /* check the v1-v2 side */
     newAx = newAz = -1.0;
     if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
         current_triangle->v1->T_dust[dust_i] > 0.0 &&
         current_triangle->v2->T_dust[dust_i] > 0.0 )
       if (current_triangle->v1->identification_number < 
           current_triangle->v2->identification_number)
       if (current_triangle->v1->T_dust[dust_i]!=Dust_Sublim_Temp[dust_i] &&
           current_triangle->v2->T_dust[dust_i]!=Dust_Sublim_Temp[dust_i])
       if ((current_triangle->v1->T_dust[dust_i]/Dust_Sublim_Temp[dust_i] - 1.0 
                                                    > _SUBLIMATION_TOLERANCE_ &&
            current_triangle->v1->T_dust[dust_i]>Max_Dust_Temp &&
            current_triangle->v2->T_dust[dust_i]<Dust_Sublim_Temp[dust_i]) 
                                  ||
           (current_triangle->v2->T_dust[dust_i]/Dust_Sublim_Temp[dust_i] - 1.0 
                                                    > _SUBLIMATION_TOLERANCE_ &&
            current_triangle->v2->T_dust[dust_i]>Max_Dust_Temp &&
            current_triangle->v1->T_dust[dust_i]<Dust_Sublim_Temp[dust_i])) {
	 d_i = (Dust_Sublim_Temp[dust_i]-current_triangle->v1->T_dust[dust_i])/
               (current_triangle->v2->T_dust[dust_i] - current_triangle->v1->T_dust[dust_i]);
	 newAx = current_triangle->v1->x +  
                	   (current_triangle->v2->x - current_triangle->v1->x)*d_i;
	 newAz = current_triangle->v1->z +  
                     	   (current_triangle->v2->z - current_triangle->v1->z)*d_i;
	 printf("12: d=%13.11lf x=%13.11lf z=%13.11lf ",d_i,newAx,newAz);
	 printf("T1=%lf T2=%lf\n",current_triangle->v1->T_dust[dust_i],current_triangle->v2->T_dust[dust_i]);
       }

     /* check the v2-v3 side */
     newBx = newBz = -1.0;
     if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
         current_triangle->v2->T_dust[dust_i] > 0.0 &&
         current_triangle->v3->T_dust[dust_i] > 0.0 )
       if (current_triangle->v2->identification_number < 
           current_triangle->v3->identification_number)
       if (current_triangle->v2->T_dust[dust_i]!=Dust_Sublim_Temp[dust_i] &&
           current_triangle->v3->T_dust[dust_i]!=Dust_Sublim_Temp[dust_i])
       if ((current_triangle->v2->T_dust[dust_i]/Dust_Sublim_Temp[dust_i] - 1.0 
                                                    > _SUBLIMATION_TOLERANCE_ &&
            current_triangle->v2->T_dust[dust_i]>Max_Dust_Temp &&
            current_triangle->v3->T_dust[dust_i]<Dust_Sublim_Temp[dust_i]) 
                                  ||
           (current_triangle->v3->T_dust[dust_i]/Dust_Sublim_Temp[dust_i] - 1.0 
                                                    > _SUBLIMATION_TOLERANCE_ &&
            current_triangle->v3->T_dust[dust_i]>Max_Dust_Temp &&
            current_triangle->v2->T_dust[dust_i]<Dust_Sublim_Temp[dust_i])) {
	 d_i = (Dust_Sublim_Temp[dust_i]-current_triangle->v2->T_dust[dust_i])/
               (current_triangle->v3->T_dust[dust_i] - current_triangle->v2->T_dust[dust_i]);
	 newBx = current_triangle->v2->x +  
                	   (current_triangle->v3->x - current_triangle->v2->x)*d_i;
	 newBz = current_triangle->v2->z +  
                     	   (current_triangle->v3->z - current_triangle->v2->z)*d_i;
	 printf("23: d=%13.11lf x=%13.11lf z=%13.11lf ",d_i,newBx,newBz);
	 printf("T2=%lf T3=%lf\n",current_triangle->v2->T_dust[dust_i],current_triangle->v3->T_dust[dust_i]);
       }

     /* check the v3-v1 side */
     newCx = newCz = -1.0;
     if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
         current_triangle->v3->T_dust[dust_i] > 0.0 &&
         current_triangle->v1->T_dust[dust_i] > 0.0 )
       if (current_triangle->v3->identification_number < 
          current_triangle->v1->identification_number)
       if (current_triangle->v3->T_dust[dust_i]!=Dust_Sublim_Temp[dust_i] &&
           current_triangle->v1->T_dust[dust_i]!=Dust_Sublim_Temp[dust_i])
      if ((current_triangle->v3->T_dust[dust_i]/Dust_Sublim_Temp[dust_i] - 1.0 
                                                    > _SUBLIMATION_TOLERANCE_ &&
            current_triangle->v3->T_dust[dust_i]>Max_Dust_Temp &&
            current_triangle->v1->T_dust[dust_i]<Dust_Sublim_Temp[dust_i]) 
                                  ||
           (current_triangle->v1->T_dust[dust_i]/Dust_Sublim_Temp[dust_i] - 1.0 
                                                    > _SUBLIMATION_TOLERANCE_ &&
            current_triangle->v1->T_dust[dust_i]>Max_Dust_Temp &&
            current_triangle->v3->T_dust[dust_i]<Dust_Sublim_Temp[dust_i])) {
	 d_i = (Dust_Sublim_Temp[dust_i]-current_triangle->v3->T_dust[dust_i])/
               (current_triangle->v1->T_dust[dust_i] - current_triangle->v3->T_dust[dust_i]);
	 newCx = current_triangle->v3->x +  
                	   (current_triangle->v1->x - current_triangle->v3->x)*d_i;
	 newCz = current_triangle->v3->z +  
                     	   (current_triangle->v1->z - current_triangle->v3->z)*d_i;
	 printf("31: d=%13.11lf x=%13.11lf z=%13.11lf ",d_i,newCx,newCz);
	 printf("T3=%lf T1=%lf\n",current_triangle->v3->T_dust[dust_i],current_triangle->v1->T_dust[dust_i]);
       }

     /* create new points */
     if (newAx>=0.0 && newAz>=0.0 && new_n<new_total) 
       {newx[new_n]=newAx;newz[new_n]=newAz;new_n++;}
     if (newBx>=0.0 && newBz>=0.0 && new_n<new_total) 
       {newx[new_n]=newBx;newz[new_n]=newBz;new_n++;}
     if (newCx>=0.0 && newCz>=0.0 && new_n<new_total) 
       {newx[new_n]=newCx;newz[new_n]=newCz;new_n++;}

     /* 
     //if (next_triangle==grid_per_lambda[lambda_N_max].start_triangle) split_flag=TRUE;
     //else split_flag=FALSE;
     //if (newAx>=0.0 && newAz>0.0)  err=_PERUNIKA_add_point_to_grids(newAx,newAz);
     //if (newBx>=0.0 && newBz>0.0)  err=_PERUNIKA_add_point_to_grids(newBx,newBz);
     //if (newCx>=0.0 && newCz>0.0)  err=_PERUNIKA_add_point_to_grids(newCx,newCz);
     ////if (split_flag) next_triangle=grid_per_lambda[lambda_N_max].start_triangle;
     //if ((newAx>=0.0 && newAz>0.0)||(newBx>=0.0 && newBz>0.0)||(newCx>=0.0 && newCz>0.0))
     //  next_triangle=grid_per_lambda[lambda_N_max].start_triangle->prev;

     //if ((newAx>=0.0 && newAz>=0.0)||(newBx>=0.0 && newBz>=0.0)||(newCx>=0.0 && newCz>=0.0))
     //  break;
     */
    }

    /* go to the next triangle */
    current_triangle = next_triangle;
  } while (current_triangle!=grid_per_lambda[lambda_N_max].start_triangle);

  /* create new points */
  /* 
  //for (lambda_k=0; lambda_k<new_n; lambda_k++)
  //  err=_PERUNIKA_add_point_to_grids(newx[lambda_k],newz[lambda_k]);
  */

  free_dvector(newx,0,new_total);
  free_dvector(newz,0,new_total);


  /**** remove vertexes *******/
  printf("Removing vertexes with hot dust \n");
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES>0) {
    TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES =
      grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;
    loop_vertex = grid_per_lambda[lambda_k].start_vertex;
    do {
     split_flag = FALSE;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) 
        if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
            loop_vertex->T_dust[dust_i]>0.0) {
   	  if (loop_vertex->T_dust[dust_i]/Dust_Sublim_Temp[dust_i] - 1.0 
                                                 > _SUBLIMATION_TOLERANCE_ &&
              loop_vertex->T_dust[dust_i]>Max_Dust_Temp) {
	    /* remove dust which is too hot */
            loop_vertex->T_dust[dust_i] = -1.0;
            loop_vertex->T_error[dust_i]=1E30;
            split_flag = TRUE;
          }
        }
      if (split_flag) {
        /* recognize if this is a new empty space point */ 
        split_flag = TRUE;
       for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
          if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
	      loop_vertex->T_dust[dust_i]>0.0) {split_flag=FALSE;break;}
        if (split_flag) TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES--;
      }
      loop_vertex = loop_vertex->next;
    } while (loop_vertex!=grid_per_lambda[lambda_k].start_vertex);
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES!=
        TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES) {
      printf("lambda_k=%d : new number of not empty grid points = %d\n",lambda_k,
      	 TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES);
      grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES =
         TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;
    }
    /* mark these grids */
    for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
      if (grid_per_lambda[lambda_k].start_vertex ==
          grid_per_lambda[lambda_kk].start_vertex)
        grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES *= -1;
  }
  /* remove markers */
  for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
    grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES *= -1;
  
  /* set the global grid (necessary for some subroutines) */
  TOTAL_NUMBER_OF_VERTICES =
    grid_per_lambda[0].TOTAL_NUMBER_OF_VERTICES;
  TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES =
    grid_per_lambda[0].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;
  start_vertex_list = grid_per_lambda[0].start_vertex;
  TOTAL_NUMBER_OF_TRIANGLES =
    grid_per_lambda[0].TOTAL_NUMBER_OF_TRIANGLES;
  start_triangle_list = grid_per_lambda[0].start_triangle;
 
  /* fix the eta denominator */
  startpoint.x = sin(Theta_Total_Value);
  startpoint.y = 0.0;
  startpoint.z = cos(Theta_Total_Value);
  endpoint.x = Rout*sin(Theta_Total_Value);
  endpoint.y = 0.0;
  endpoint.z = Rout*cos(Theta_Total_Value);
  tau = _ZELENJAK_tau_over_grid (startpoint,endpoint,Lambda_Total,FALSE);
  printf("(correction completed)\n");
  printf("old eta = %lf ",Eta_Denominator);
  Eta_Denominator *= (tau/Tau_Total); /* correction */
  printf("new eta=%lf\n",Eta_Denominator);
  /* 
  //err = _GOROSTAS_Find_Psi ();
  //printf("PSI value (AFTER the 'eta denominator' calculation)=%lf\n",Psi);
  */

  printf("(correction completed)\n");

  return;
}
