/* LELUYA - PERUNIKA program block */
/* This function splits triangles to smooth out their shape. */

/* The isotherm "smoothness" angle is provided with the flag -c 209 */

#include "PERUNIKA.h"

int _PERUNIKA_smooth_grid_cells_shape (void) {

  double cos_limit = -0.866025403;

  struct _triangle_structure *current_triangle; /* triangle in the loop */
  struct _triangle_structure *next_triangle; /* next triangle in the loop */
  int lambda_N_max; /* wavelength with the most grid points */
  int N_max;        /* number of vertices at lambda_N_max */
  int dust_i;       /* dust type used for isotherms */
  int lambda_k;
  double T1,T2,T3; /* angles at the vertices in current_triangle*/
  struct _3DVector n12, n23, n31; /* unit vectors pointing from one vertex to
                      another in current_triangle*/
  struct _triangle_structure *trian12, *trian23, *trian31; /* triangles
                      adjacent to current_triangle */
  struct _3DVector vertex12, vertex23, vertex31; /* new vertices 
			  created to split the triangle */
  int i; /* dummy counter */
  int err;
  struct _3DVector point1, point2;

  if (DEBUG.MAX_TRIANGLE_ANGLE<=90 || DEBUG.MAX_TRIANGLE_ANGLE>=180) return 1;
  cos_limit = cos (RAD1*DEBUG.MAX_TRIANGLE_ANGLE);
  printf("SMOOTH THE GRID BY SPLITTING TRIANGLES THAT HAVE A LARGE ANGLE\n");
  printf("LIMITING COS=%lf\n",cos_limit);

  /* find a grid with the largest number of points (let's call it grid_N_max)*/
  lambda_N_max = 0;
  N_max = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++) {
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > N_max) {
      N_max = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES;
      lambda_N_max = lambda_k;
    }
  }


  start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
  start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
  TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
  TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
  current_triangle = grid_per_lambda[lambda_N_max].start_triangle;
  do {
    next_triangle = current_triangle->prev;

    /*printf("...\nTrian#%d: [%E,%E][%E,%E][%E,%E]\n",current_triangle->ID_number, 
    	   current_triangle->v1->x,current_triangle->v1->z,
    	   current_triangle->v2->x,current_triangle->v2->z,
    	   current_triangle->v3->x,current_triangle->v3->z);*/

    dust_i = -1;
    for (i=0; i<Max_Dust_Type; i++) 
	  if (INPUT.DUST_DENSITY_TYPE[i]!=99999)
	    if (current_triangle->v1->T_dust[i]>0.0 &&
                current_triangle->v2->T_dust[i]>0.0 &&
                current_triangle->v3->T_dust[i]>0.0)
	      {dust_i=i; break;}

    if (dust_i==-1) {
      /* go to the next triangle */
      current_triangle = next_triangle;
      continue;
    }

    /* unit vectors pointing from one vertex to another */
    point1.y = point2.y = 0.0;

    point1.x = current_triangle->v1->x;
    point1.z = current_triangle->v1->z;
    point2.x = current_triangle->v2->x;
    point2.z = current_triangle->v2->z;
    n12 = _MATH_LIB_unit_vector(point1,point2);

    point1.x = current_triangle->v2->x;
    point1.z = current_triangle->v2->z;
    point2.x = current_triangle->v3->x;
    point2.z = current_triangle->v3->z;
    n23 = _MATH_LIB_unit_vector(point1,point2);

    point1.x = current_triangle->v3->x;
    point1.z = current_triangle->v3->z;
    point2.x = current_triangle->v1->x;
    point2.z = current_triangle->v1->z;
    n31 = _MATH_LIB_unit_vector(point1,point2);

    /* find adjacent triangles */
    trian12 = _PERUNIKA_adjacent_triangle(current_triangle->v2,current_triangle);
    trian23 = _PERUNIKA_adjacent_triangle(current_triangle->v3,current_triangle);
    trian31 = _PERUNIKA_adjacent_triangle(current_triangle->v1,current_triangle);

    /* check the triangle angles to see if new vertices have to be created */
    vertex12.x=vertex23.x=vertex31.x=-1.0; 
    vertex12.y=vertex23.y=vertex31.y=0.0; 
    vertex12.z=vertex23.z=vertex31.z=-1.0; 

    T2 = -(n31.x*n12.x+n31.z*n12.z);
    T3 = -(n12.x*n23.x+n12.z*n23.z);
    T1 = -(n23.x*n31.x+n23.z*n31.z);

    if (T1 < cos_limit) {
      /* create vertex 12 */
      vertex12.x =0.5*(current_triangle->v1->x+current_triangle->v2->x);
      vertex12.z =0.5*(current_triangle->v1->z+current_triangle->v2->z);
      printf("CREATE 12 in triangle#%d: [%E,%E] cos=%lf\n",
               current_triangle->ID_number,vertex12.x,vertex12.z,T1);
    } 
    else if (T2 < cos_limit) {
      /* create vertex 23 */
      vertex23.x =0.5*(current_triangle->v2->x+current_triangle->v3->x);
      vertex23.z =0.5*(current_triangle->v2->z+current_triangle->v3->z);
      printf("CREATE 23 in triangle#%d: [%E,%E] cos=%lf\n",
               current_triangle->ID_number,vertex23.x,vertex23.z,T2);
    }
    else if (T3 < cos_limit) {
      /* create vertex 31 */
      vertex31.x =0.5*(current_triangle->v3->x+current_triangle->v1->x);
      vertex31.z =0.5*(current_triangle->v3->z+current_triangle->v1->z);
      printf("CREATE 31 in triangle#%d: [%E,%E] cos=%lf\n",
               current_triangle->ID_number,vertex31.x,vertex31.z,T3);
    }

    /* create new triangles */

    if (vertex12.x>=0.0) {
      /*                     V3
                            / \
                           /   \
                          /     \
                         V1-V12-V2
                           \   /
                            \ /
                            P12  
      */
      if (trian12==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex12.x,vertex12.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;
    }
    else if (vertex23.x>=0.0) {
      /*                     V3----P23
                            / \    /
                           /  V23 /
                          /     \/
                         V1-----V2
      */
      if (trian23==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex23.x,vertex23.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;
    }
    else if (vertex31.x>=0.0) {
      /*              P31----V3
                       \    / \
                        \ V31  \
                         \/     \
                         V1-----V2
      */
      if (trian31==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex31.x,vertex31.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;
    }


   /* go to the next triangle */
    current_triangle = next_triangle;
  } while (current_triangle!=grid_per_lambda[lambda_N_max].start_triangle);

  return 0;
}
