/* LELUYA - PERUNIKA program block */
/* This function splits triangles to smooth out the isotherms */

/* The isotherm "smoothness" angle is provided with the flag ??? */

#include "PERUNIKA.h"

int _PERUNIKA_smooth_grid_temperature (void) {

  double cos_limit = -0.866025403;

  struct _triangle_structure *current_triangle; /* triangle in the loop */
  struct _triangle_structure *next_triangle; /* next triangle in the loop */
  int lambda_N_max; /* wavelength with the most grid points */
  int N_max;        /* number of vertices at lambda_N_max */
  int dust_i;       /* dust type used for isotherms */
  int lambda_k,lambda_kk;
  double T1,T2,T3; /* temperatures at the vertices in current_triangle*/
  double Tmin, Tmax; /* min and max vertex temperature in current_triangle*/
  double T0; /* = sqrt(Tmin*Tmax) */
  struct _3DVector n12, n23, n31; /* unit vectors pointing from one vertex to
                      another in current_triangle*/
  struct _3DVector v12, v23, v31; /* point between two 
		      current_triangle's verticies */
  struct _triangle_structure *trian12, *trian23, *trian31; /* triangles
                      adjacent to current_triangle */
  struct _vertex_structure *P12, *P23, *P31; /* side-vertices in adjacent 
						triangles */
  double TP12, TP23, TP31; /* temperature at P12, P23, P31 */
  struct _3DVector n1p,np12, n2p, np23, n3p, np31; /* unit vectors pointing 
                      to or from the side vertices */
  struct _3DVector PV12, PV23, PV31; /* point where T0 intersects adjacent
					triangles*/
  struct _3DVector a,b; /* unit vectors used for calculating the angle 
                           between isotherms */
  double cos_a_b; /* cosine of the angle between a and b */
  struct _3DVector vertex12, vertex23, vertex31; /* new vertices 
			  created to split the triangle */
  double *U; /* an array used for updateing the energy density at a new vertex */
  struct _triangle_structure *new_trianA, *new_trianB, *new_trianC, *new_trianD;
                                                    /* new triangles created */
  int i; /* dummy counter */
  int err;
  double dummy; 
  struct _3DVector point1, point2;

  if (DEBUG.ISOTHERM_BENDING<=0) return 1;
  cos_limit = cos ((180.0-DEBUG.ISOTHERM_BENDING)*RAD1);
  printf("LIMITING COS=%lf\n",cos_limit);

  /* find a grid with the largest number of points (let's call it grid_N_max)*/
  lambda_N_max = 0;
  N_max = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++) {
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > N_max) {
      N_max = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES;
      lambda_N_max = lambda_k;
    }
  }


  start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
  start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
  TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
  TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
  current_triangle = grid_per_lambda[lambda_N_max].start_triangle;
  do {
    next_triangle = current_triangle->prev;

    /*printf("...\nTrian#%d: [%E,%E][%E,%E][%E,%E]\n",current_triangle->ID_number, 
    	   current_triangle->v1->x,current_triangle->v1->z,
    	   current_triangle->v2->x,current_triangle->v2->z,
    	   current_triangle->v3->x,current_triangle->v3->z);*/

    dust_i = -1;
    for (i=0; i<Max_Dust_Type; i++) 
	  if (INPUT.DUST_DENSITY_TYPE[i]!=99999)
	    if (current_triangle->v1->T_dust[i]>0.0 &&
                current_triangle->v2->T_dust[i]>0.0 &&
                current_triangle->v3->T_dust[i]>0.0)
	      {dust_i=i; break;}

    if (dust_i==-1) {
      /* go to the next triangle */
      current_triangle = next_triangle;
      continue;
    }

    /* check this triangle for possible refinement */
    
    /* find the temperature range */
    T1 = current_triangle->v1->T_dust[dust_i];
    T2 = current_triangle->v2->T_dust[dust_i];
    T3 = current_triangle->v3->T_dust[dust_i];
    if (T1<T2) {Tmin=T1; Tmax=T2;}
    else {Tmin=T2; Tmax=T1;}
    if (Tmin>T3) Tmin=T3;
    else if (Tmax<T3) Tmax = T3;
    T0 = sqrt(Tmin*Tmax);

    if (Tmin==Tmax)  {
      /* go to the next triangle */
      current_triangle = next_triangle;
      continue;
    }

    /* unit vectors pointing from one vertex to another */
    point1.y = point2.y = 0.0;

    point1.x = current_triangle->v1->x;
    point1.z = current_triangle->v1->z;
    point2.x = current_triangle->v2->x;
    point2.z = current_triangle->v2->z;
    n12 = _MATH_LIB_unit_vector(point1,point2);

    point1.x = current_triangle->v2->x;
    point1.z = current_triangle->v2->z;
    point2.x = current_triangle->v3->x;
    point2.z = current_triangle->v3->z;
    n23 = _MATH_LIB_unit_vector(point1,point2);

    point1.x = current_triangle->v3->x;
    point1.z = current_triangle->v3->z;
    point2.x = current_triangle->v1->x;
    point2.z = current_triangle->v1->z;
    n31 = _MATH_LIB_unit_vector(point1,point2);

    /* find the intermediate vertices where T0 crosses the tirangle sides*/
    v12.x = v23.x = v31.x = -1;
    v12.y = v23.y = v31.y = 0.0;
    v12.z = v23.z = v31.z = -1;
    if ((T1<=T0 && T0<T2) || (T2<T0 && T0<=T1)) {
      dummy = 
	_MATH_LIB_distance(current_triangle->v1->x,current_triangle->v1->z,
			   current_triangle->v2->x,current_triangle->v2->z) *
	(T0-T1)/(T2-T1);
      v12.x = current_triangle->v1->x + n12.x*dummy;
      v12.z = current_triangle->v1->z + n12.z*dummy;
    }
    if ((T2<=T0 && T0<T3) || (T3<T0 && T0<=T2)) {
      dummy = 
	_MATH_LIB_distance(current_triangle->v2->x,current_triangle->v2->z,
			   current_triangle->v3->x,current_triangle->v3->z) *
	(T0-T2)/(T3-T2);
      v23.x = current_triangle->v2->x + n23.x*dummy;
      v23.z = current_triangle->v2->z + n23.z*dummy;
    }
    if ((T3<=T0 && T0<T1) || (T1<T0 && T0<=T3)) {
      dummy = 
	_MATH_LIB_distance(current_triangle->v3->x,current_triangle->v3->z,
			   current_triangle->v1->x,current_triangle->v1->z) *
	(T0-T3)/(T1-T3);
      v31.x = current_triangle->v3->x + n31.x*dummy;
      v31.z = current_triangle->v3->z + n31.z*dummy;
    }

    /*if (v12.x>-1.0) printf("12:[%E.%E] ",v12.x,v12.z);
    if (v23.x>-1.0) printf("23:[%E.%E] ",v23.x,v23.z);
    if (v31.x>-1.0) printf("31:[%E.%E] ",v31.x,v31.z);
    printf("\n");*/

    /* find adjacent triangles */
    trian12 = _PERUNIKA_adjacent_triangle(current_triangle->v2,current_triangle);
    trian23 = _PERUNIKA_adjacent_triangle(current_triangle->v3,current_triangle);
    trian31 = _PERUNIKA_adjacent_triangle(current_triangle->v1,current_triangle);

    /* find the side-vertices */
    P12 = P23 = P31 = NULL;
    TP12 = TP23 = TP31 = -1;
    if (trian12!=NULL) {
      if (current_triangle->v1==trian12->v1) P12 = trian12->v2;
      else if (current_triangle->v1==trian12->v2) P12 = trian12->v3;
      else if (current_triangle->v1==trian12->v3) P12 = trian12->v1;
      if (P12!=NULL) TP12 = P12->T_dust[dust_i];
      /*if (P12!=NULL) printf("sidepoint 12: [%E,%E] T=%lf\n",P12->x,P12->z,TP12);*/
   }
    if (trian23!=NULL) {
      if (current_triangle->v2==trian23->v1) P23 = trian23->v2;
      else if (current_triangle->v2==trian23->v2) P23 = trian23->v3;
      else if (current_triangle->v2==trian23->v3) P23 = trian23->v1;
      if (P23!=NULL) TP23 = P23->T_dust[dust_i];
      /*if (P23!=NULL) printf("sidepoint 23: [%E,%E] T=%lf\n",P23->x,P23->z,TP23);*/
   }
    if (trian31!=NULL) {
      if (current_triangle->v3==trian31->v1) P31 = trian31->v2;
      else if (current_triangle->v3==trian31->v2) P31 = trian31->v3;
      else if (current_triangle->v3==trian31->v3) P31 = trian31->v1;
      if (P31!=NULL) TP31 = P31->T_dust[dust_i];
      /*if (P23!=NULL) printf("sidepoint 31: [%E,%E] T=%lf\n",P31->x,P31->z,TP31);*/
   }

    /* find points where T0 intersects the adjacent triangles */
    PV12.x=PV12.z=PV23.x=PV23.z=PV31.x=PV31.z=-1;
    n1p.y = np12.y = n2p.y = np23.y = n3p.y = np31.y = 0.0;
    if (v12.x>-1 && P12!=NULL) {
      if ((T1<=T0 && T0<TP12) || (TP12<T0 && T0<=T1)) {
        point1.x = current_triangle->v1->x;
        point1.z = current_triangle->v1->z;
        point2.x = P12->x;
        point2.z = P12->z;
        n1p = _MATH_LIB_unit_vector(point1,point2);
        dummy = 
          _MATH_LIB_distance(current_triangle->v1->x,current_triangle->v1->z,
			   P12->x,P12->z) * (T0-T1)/(TP12-T1);
        PV12.x = current_triangle->v1->x + n1p.x*dummy;
        PV12.z = current_triangle->v1->z + n1p.z*dummy;
      }
      if ((TP12<=T0 && T0<T2) || (T2<T0 && T0<=TP12)) {
        point1.x = P12->x;
        point1.z = P12->z;
        point2.x = current_triangle->v2->x;
        point2.z = current_triangle->v2->z;
        np12 = _MATH_LIB_unit_vector(point1,point2);
        dummy = 
          _MATH_LIB_distance(current_triangle->v2->x,current_triangle->v2->z,
			   P12->x,P12->z) * (T0-TP12)/(T2-TP12);
        PV12.x = P12->x + np12.x*dummy;
        PV12.z = P12->z + np12.z*dummy;
      }
      /*printf("PV12.x=%E PV12.z=%E\n",PV12.x,PV12.z);*/
    }

    if (v23.x>-1 && P23!=NULL) {
      if ((T2<=T0 && T0<TP23) || (TP23<T0 && T0<=T2)) {
        point1.x = current_triangle->v2->x;
        point1.z = current_triangle->v2->z;
        point2.x = P23->x;
        point2.z = P23->z;
        n2p = _MATH_LIB_unit_vector(point1,point2);
        dummy = 
          _MATH_LIB_distance(current_triangle->v2->x,current_triangle->v2->z,
			   P23->x,P23->z) * (T0-T2)/(TP23-T2);
        PV23.x = current_triangle->v2->x + n2p.x*dummy;
        PV23.z = current_triangle->v2->z + n2p.z*dummy;
      }
      if ((TP23<=T0 && T0<T3) || (T3<T0 && T0<=TP23)) {
        point1.x = P23->x;
        point1.z = P23->z;
        point2.x = current_triangle->v3->x;
        point2.z = current_triangle->v3->z;
        np23 = _MATH_LIB_unit_vector(point1,point2);
        dummy = 
          _MATH_LIB_distance(current_triangle->v3->x,current_triangle->v3->z,
			   P23->x,P23->z) * (T0-TP23)/(T3-TP23);
        PV23.x = P23->x + np23.x*dummy;
        PV23.z = P23->z + np23.z*dummy;
      }
      /*printf("PV23.x=%E PV23.z=%E\n",PV23.x,PV23.z);*/
    }

    if (v31.x>-1 && P31!=NULL) {
      if ((T3<=T0 && T0<TP31) || (TP31<T0 && T0<=T3)) {
        point1.x = current_triangle->v3->x;
        point1.z = current_triangle->v3->z;
        point2.x = P31->x;
        point2.z = P31->z;
        n3p = _MATH_LIB_unit_vector(point1,point2);
        dummy = 
          _MATH_LIB_distance(current_triangle->v3->x,current_triangle->v3->z,
			   P31->x,P31->z) * (T0-T3)/(TP31-T3);
        PV31.x = current_triangle->v3->x + n3p.x*dummy;
        PV31.z = current_triangle->v3->z + n3p.z*dummy;
      }
      if ((TP31<=T0 && T0<T1) || (T1<T0 && T0<=TP31)) {
        point1.x = P31->x;
        point1.z = P31->z;
        point2.x = current_triangle->v1->x;
        point2.z = current_triangle->v1->z;
        np31 = _MATH_LIB_unit_vector(point1,point2);
        dummy = 
          _MATH_LIB_distance(current_triangle->v1->x,current_triangle->v1->z,
			   P31->x,P31->z) * (T0-TP31)/(T1-TP31);
        PV31.x = P31->x + np31.x*dummy;
        PV31.z = P31->z + np31.z*dummy;
      }
      /*printf("PV31.x=%E PV31.z=%E\n",PV31.x,PV31.z);*/
   }

    /* check the isotherms to see if new vertices have to be created */
    vertex12.x=vertex23.x=vertex31.x=-1.0; 
    vertex12.y=vertex23.y=vertex31.y=0.0; 
    vertex12.z=vertex23.z=vertex31.z=-1.0; 
    if (v12.x>-1 && PV12.x>-1) {
      point1.x = v12.x; point1.z = v12.z;
      if (v23.x>-1) {point2.x =v23.x; point2.z =v23.z;}
      else  {point2.x =v31.x; point2.z =v31.z;}
      a = _MATH_LIB_unit_vector(point1,point2);
      point2.x =PV12.x; point2.z =PV12.z;
      b = _MATH_LIB_unit_vector(point1,point2);
      cos_a_b = a.x*b.x+a.z*b.z;
      if (cos_a_b > cos_limit) {
	/* create vertex 12 */
	dummy = _MATH_LIB_distance(current_triangle->v1->x,
                                   current_triangle->v1->z,v12.x,v12.z)/
                _MATH_LIB_distance(current_triangle->v1->x,
                                   current_triangle->v1->z,
                                   current_triangle->v2->x,
		                   current_triangle->v2->z);
	if (0.2<dummy && dummy<0.8)
          {vertex12.x = v12.x;  vertex12.z = v12.z;}
        else {
          vertex12.x =0.5*(current_triangle->v1->x+current_triangle->v2->x);
          vertex12.z =0.5*(current_triangle->v1->z+current_triangle->v2->z);
	}
	printf("CREATE 12 in triangle#%d: [%E,%E] cos=%lf\n",
               current_triangle->ID_number,vertex12.x,vertex12.z,cos_a_b);
      } /* end of (cos_a_b > cos_limit) */
    } /* end of creating vertex12 */ 

    if (v23.x>-1 && PV23.x>-1) {
      point1.x = v23.x; point1.z = v23.z;
      if (v31.x>-1) {point2.x =v31.x; point2.z =v31.z;}
      else  {point2.x =v12.x; point2.z =v12.z;}
      a = _MATH_LIB_unit_vector(point1,point2);
      point2.x =PV23.x; point2.z =PV23.z;
      b = _MATH_LIB_unit_vector(point1,point2);
      cos_a_b = a.x*b.x+a.z*b.z;
      if (cos_a_b > cos_limit) {
	/* create vertex 23 */
	dummy = _MATH_LIB_distance(current_triangle->v2->x,
                                   current_triangle->v2->z,v23.x,v23.z)/
                _MATH_LIB_distance(current_triangle->v2->x,
                                   current_triangle->v2->z,
                                   current_triangle->v3->x,
		                   current_triangle->v3->z);
	if (0.2<dummy && dummy<0.8)
          {vertex23.x = v23.x;  vertex23.z = v23.z;}
        else {
          vertex23.x =0.5*(current_triangle->v2->x+current_triangle->v3->x);
          vertex23.z =0.5*(current_triangle->v2->z+current_triangle->v3->z);
	}
	printf("CREATE 23 in triangle#%d: [%E,%E] cos=%lf\n",
               current_triangle->ID_number,vertex23.x,vertex23.z,cos_a_b);
      }/* end of (cos_a_b > cos_limit) */
    } /* end of creating vertex23 */ 

    if (v31.x>-1 && PV31.x>-1) {
      point1.x = v31.x; point1.z = v31.z;
      if (v12.x>-1) {point2.x =v12.x; point2.z =v12.z;}
      else  {point2.x =v23.x; point2.z =v23.z;}
      a = _MATH_LIB_unit_vector(point1,point2);
      point2.x =PV31.x; point2.z =PV31.z;
      b = _MATH_LIB_unit_vector(point1,point2);
      cos_a_b = a.x*b.x+a.z*b.z;
      if (cos_a_b > cos_limit) {
	/* create vertex 31 */
	dummy = _MATH_LIB_distance(current_triangle->v3->x,
                                   current_triangle->v3->z,v31.x,v31.z)/
                _MATH_LIB_distance(current_triangle->v3->x,
                                   current_triangle->v3->z,
                                   current_triangle->v1->x,
		                   current_triangle->v1->z);
	if (0.2<dummy && dummy<0.8)
          {vertex31.x = v31.x;  vertex31.z = v31.z;}
        else {
          vertex31.x =0.5*(current_triangle->v3->x+current_triangle->v1->x);
          vertex31.z =0.5*(current_triangle->v3->z+current_triangle->v1->z);
	}
	printf("CREATE 31 in triangle#%d: [%E,%E] cos=%lf\n",
               current_triangle->ID_number,vertex31.x,vertex31.z,cos_a_b);
      }/* end of (cos_a_b > cos_limit) */
    } /* end of creating vertex31 */ 

    /*if (vertex12.x>=0.0 || vertex23.x>=0.0 || vertex31.x>=0.0)
    printf("Trian#%d: [%E,%E][%E,%E][%E,%E]\n....\n",current_triangle->ID_number, 
    	   current_triangle->v1->x,current_triangle->v1->z,
    	   current_triangle->v2->x,current_triangle->v2->z,
    	   current_triangle->v3->x,current_triangle->v3->z);*/

    /* create new triangles */

    new_trianA = new_trianB = new_trianC = new_trianD = NULL;
    if (vertex12.x>=0.0 && vertex23.x>=0.0) {
      /*                     V3----P23
                            / \    /
                           /  V23 /
                          /     \/
                         V1-V12-V2
                           \   /
                            \ /
                            P12  
      */
      if (trian12==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex12.x,vertex12.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;

      if (trian23==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex23.x,vertex23.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;

    }
    else if (vertex23.x>=0.0 && vertex31.x>=0.0) {
      /*              P31----V3----P23
                       \    / \    /
                        \ V31 V23 /
                         \/     \/
                         V1-----V2
      */
      if (trian23==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex23.x,vertex23.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;

      if (trian31==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex31.x,vertex31.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;

    }
    else if (vertex31.x>=0.0 && vertex12.x>=0.0) {
      /*              P31----V3
                       \    / \
                        \ V31  \
                         \/     \
                         V1-V12-V2
                           \   /
                            \ /
                            P12  
      */
      if (trian31==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex31.x,vertex31.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;

      if (trian12==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex12.x,vertex12.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;
    }
    else if (vertex12.x>=0.0) {
      /*                     V3
                            / \
                           /   \
                          /     \
                         V1-V12-V2
                           \   /
                            \ /
                            P12  
      */
      if (trian12==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex12.x,vertex12.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;
    }
    else if (vertex23.x>=0.0) {
      /*                     V3----P23
                            / \    /
                           /  V23 /
                          /     \/
                         V1-----V2
      */
      if (trian23==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex23.x,vertex23.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;
    }
    else if (vertex31.x>=0.0) {
      /*              P31----V3
                       \    / \
                        \ V31  \
                         \/     \
                         V1-----V2
      */
      if (trian31==next_triangle) next_triangle=
             (next_triangle!=start_triangle_list) ? next_triangle->prev : NULL;
      err=_PERUNIKA_add_point_to_grids(vertex31.x,vertex31.z);
      start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES=grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_TRIANGLES;
      if (next_triangle==NULL) next_triangle=start_triangle_list;
    }


   /* go to the next triangle */
    current_triangle = next_triangle;
  } while (current_triangle!=grid_per_lambda[lambda_N_max].start_triangle);

  return 0;
}
