/* LELUYA - PERUNIKA program block */
/* This part of the LELUYA code is generating the computational grid */

/* this part of the PERUNIKA block is dealing with the function for handling 
   a polygon's division into triangles */

#include "PERUNIKA.h"

/* split a polygon into triangles */
int _PERUNIKA_split_polygon (struct _polygon_structure *polygon){
  
  int error;               /* error flag */
  int total_vertex_number; /* total number of vertices in the polygon */
  struct _polygon_structure *polygon_vertex; /* a vertex in the polygon */
  struct _polygon_structure *split_vertex;   /* a vertex which will be part
			      of the split-line together with polygon_vertex */
  struct _polygon_structure *intersect_vertex; /* a vertex which is used
                              when we need to check if some polygon's side
                              intersects with the split-line */
  int intersection_exists = 0; /* a flag which shows if this intersection
				  exists or not */
  double alpha, beta; /* angles (actually, their nu values) used in the
			 search for the optimal split-line */
  double optimal_angle; /* optimal angle for a specific split-line during 
                           the search at a polygon vertex */
  struct _polygon_structure *polygon_Tom, *polygon_Jerry; /* two new polygons
       			     created after we find the optimal split-line */
  struct _triangle_structure *triangle; /* a new triangle if the polygon has
                              only three vertices */
  struct _vertex_triangles   *vertex_triangle;/* used for inserting a triangle
			      into the list of triangles at its vertices */

  /* first find how many vertices the polygon has */
  total_vertex_number = 0;
  polygon_vertex = polygon;
  do {
    total_vertex_number ++;
    polygon_vertex = polygon_vertex->next;
  } while (polygon_vertex != polygon);

  if (total_vertex_number<3) printf ("ERROR: kaj je ovo? nemre poly <3!\n");

  if (total_vertex_number==3) { /* create a triangle */
    /* this polygon is actually a triangle */
    triangle = _PERUNIKA_create_triangle(&error);
    triangle->v1 = polygon->vertex;
    triangle->v2 = polygon->next->vertex;
    triangle->v3 = polygon->next->next->vertex;
    /* the next three lines are here ONLY because of 
      _PERUNIKA_clean_the_vertex_list : when a vertex is removed, the triangles 
     around it have to be rearranged, but also placed into the list of 
     triangles at their vertices. */
    /* But we don't want the vertex_triangles before the cleaning process starts!
       The cleaning process starts in _PERUNIKA_clean_the_vertex_list, thus all
       vertices in the grid will loose their vertex_triangles in the beginnig of 
       that function, because they are maybe not correct!! */
    vertex_triangle=_PERUNIKA_create_new_vertex_triangle (triangle,triangle->v1);
    vertex_triangle=_PERUNIKA_create_new_vertex_triangle (triangle,triangle->v2);
    vertex_triangle=_PERUNIKA_create_new_vertex_triangle (triangle,triangle->v3);
  }
  else {
    /* the polygon has more than three vertices and it needs to be split */
    
    /* we will start with the calculation of an optimal split-line for 
       each vertex in the polygon. Later on, we will pick up the most optimal
       among these optimal lines and split the polygon with it. */
    /* So, make a loop over the polygon vertices : */
    polygon_vertex = polygon;
    do {
      /* and now make another loop, but this one should not include
         the vertex 'polygon_vertex' and its neighbors 'polygon_vertex->next' 
         and 'polygon_vertex->prev'. This loop will go only over the vertices 
         which are candidates for the optimal split-line at the vertex
         'polygon_vertex'. */
      split_vertex = polygon_vertex->next->next;
      while (split_vertex != polygon_vertex->prev) {
	/* notice that this loop would not start if total_vertex_number==3,
           because then polygon_vertex->next->next=polygon_vertex->prev */
        /* so, we are checking if the line between polygon_vertex and 
           split_vertex is the optimal one. But this line is maybe crossing 
           the space outside of the polygon and thus it is not allowed. 
           To check this possibility, we will first consider the following two
           special cases:

           SPECIAL CASE #1:

         polygon_vertex->prev       polygon_vertex->next
                           \ INSIDE /
                            \      /
                   OUTSIDE   a    b     OUTSIDE
                              \  /
                               \/           
                         polygon_vertex -----SPLIT-LINE----- split_vertex


           SPECIAL CASE #2:
                                  split_vertex->prev       split_vertex->next
                                                  \ INSIDE /
                                                   \      /
                                         OUTSIDE    c    d   OUTSIDE
                                                     \  /
                                                      \/
                 polygon_vertex -----SPLIT-LINE----- split_vertex

          In those two special cases, we can use the angles between the 
          split-line and the polygon's sides, (marked by a,b,c, and d on the
          sketch above). Let's define the angles (don't forget the right-hand
          rule):
  
          SPECIAL CASE #1:
           alpha = angle between the side b and the split-line
           beta  = angle between the side b and the side a
          SPECIAL CASE #2:
           alpha = angle between the side d and the split-line
           beta  = angle between the side d and the side c

          The split-line is inside of the polygon if alpha < beta.
         (do not forget that we actually work with the nu-values, not 
          with radians. See some additional code manual for the definition 
          of nu-values, but briefly: nu-values are the same like cosine,
          except that they span from 2 over 0 to -2 when the angle goes from 
           0 to Pi and 2Pi, while the cosine goes from 1 over -1 back to 1 )
          Let's write a little bit of the code , now.  */

        /* SPECIAL CASE #1 */
        alpha = _MATH_LIB_nu_value(
              polygon_vertex->vertex->x,      polygon_vertex->vertex->z,
              polygon_vertex->next->vertex->x,polygon_vertex->next->vertex->z,
              split_vertex->vertex->x,        split_vertex->vertex->z);
        beta  = _MATH_LIB_nu_value(
              polygon_vertex->vertex->x,      polygon_vertex->vertex->z,
              polygon_vertex->next->vertex->x,polygon_vertex->next->vertex->z,
              polygon_vertex->prev->vertex->x,polygon_vertex->prev->vertex->z);
        if (alpha <= beta) {/* we compare the nu values, which are smaller
                               for a larger angle! */ 
          split_vertex = split_vertex->next; 
          continue;
	}
        /* SPECIAL CASE #2 */
        alpha = _MATH_LIB_nu_value(
              split_vertex->vertex->x,      split_vertex->vertex->z,
              split_vertex->next->vertex->x,split_vertex->next->vertex->z,
              polygon_vertex->vertex->x,    polygon_vertex->vertex->z);
        beta  = _MATH_LIB_nu_value(
              split_vertex->vertex->x,      split_vertex->vertex->z,
              split_vertex->next->vertex->x,split_vertex->next->vertex->z,
              split_vertex->prev->vertex->x,split_vertex->prev->vertex->z);
        if (alpha <= beta) {/* we compare the nu values, which are smaller
                               for a larger angle! */
          split_vertex = split_vertex->next; 
          continue;
	}

	/* now check if any other polygon's side intersects with the 
           split-line  */
        intersection_exists = 0;
        intersect_vertex = polygon_vertex->next;
        while (intersect_vertex != polygon_vertex->prev && 
               intersection_exists == 0) {
          if (intersect_vertex!=split_vertex->prev &&
              intersect_vertex!=split_vertex)
            intersection_exists = _MATH_LIB_intersecting_lines (
	               polygon_vertex->vertex->x,  polygon_vertex->vertex->z,
                       split_vertex->vertex->x,    split_vertex->vertex->z,
                       intersect_vertex->vertex->x,intersect_vertex->vertex->z,
                       intersect_vertex->next->vertex->x,
                       intersect_vertex->next->vertex->z);
          intersect_vertex=intersect_vertex->next;
	}
	if (intersection_exists) {
          split_vertex = split_vertex->next; 
          continue;
	}

	/* OK, the split-line between polygon_vertex and split_vertex is
           allowed. Find if it is optimal. */
	/* angle at polygon_vertex: */
        alpha = _MATH_LIB_nu_value(
              polygon_vertex->vertex->x, polygon_vertex->vertex->z,
              polygon_vertex->next->vertex->x,polygon_vertex->next->vertex->z,
              split_vertex->vertex->x, split_vertex->vertex->z);
        beta  = _MATH_LIB_nu_value(
              polygon_vertex->vertex->x, polygon_vertex->vertex->z,
              split_vertex->vertex->x, split_vertex->vertex->z,
              polygon_vertex->prev->vertex->x,polygon_vertex->prev->vertex->z);
        optimal_angle = DMAX (alpha, beta );/* nu value is larger for 
                      a smaller angle, that's why we use DMAX and not DMIN */ 
	/* angle at split_vertex: */
        alpha = _MATH_LIB_nu_value(
              split_vertex->vertex->x, split_vertex->vertex->z,
              split_vertex->next->vertex->x,split_vertex->next->vertex->z,
              polygon_vertex->vertex->x, polygon_vertex->vertex->z);
        beta  = _MATH_LIB_nu_value(
              split_vertex->vertex->x, split_vertex->vertex->z,
              polygon_vertex->vertex->x, polygon_vertex->vertex->z,
              split_vertex->prev->vertex->x,split_vertex->prev->vertex->z);
        alpha = DMAX (alpha, beta );
        optimal_angle = DMAX (alpha,optimal_angle);/* nu value is larger for 
                      a smaller angle, that's why we use DMAX and not DMIN */
        
	/* ... and check if this split_vertex is better than any concidered
           before at polygon_vertex */
        if (alpha < polygon_vertex->optimal_angle) {/* the same here: we are
       					       comparing the nu values */ 
          polygon_vertex->optimal_angle = alpha;
          polygon_vertex->optimal_vertex = split_vertex;
	}

        split_vertex = split_vertex->next;

      } /* end of the loop over split_vertex */

      polygon_vertex = polygon_vertex->next;
    } while (polygon_vertex != polygon);


    /* let's find the most optimal split-line among the existing split-lines
       at each vertex in the polygon */
    split_vertex   = polygon;
    polygon_vertex = polygon->next;
    while (polygon_vertex != polygon) {
      if (polygon_vertex->optimal_angle < split_vertex->optimal_angle)
	/* we compare the nu values: an angle is larger if its nu value is 
           smaller! */
        split_vertex = polygon_vertex;
      polygon_vertex = polygon_vertex->next;
    }

    if (split_vertex->optimal_vertex == NULL) {
      printf("FATAL ERROR!!! Terrible error!! Please, send an e-mail to\n");
      printf("Dejan Vinkovic (dejan@pa.uky.edu)\n");
      printf("POLYGON VERTICES:\n");
      polygon_vertex = polygon;
      do {
        printf("[%15.13E,%15.13E] nu=%E level=%d\n",polygon_vertex->vertex->x,
               polygon_vertex->vertex->z, polygon_vertex->optimal_angle,
               polygon_vertex->vertex->level);
	if (polygon_vertex->vertex->x == polygon_vertex->next->next->vertex->x &&
            polygon_vertex->vertex->z == polygon_vertex->next->next->vertex->z)
	  printf("JESU!!\n");
        polygon_vertex = polygon_vertex->next;
      } while (polygon_vertex != polygon);
      return 1;
    }

    /* now we have a split-line which can be used to cut the polygon into
       two new polygons. Let's call them Tom and Jerry 

split_vertex->optimal_vertex->prev -----  ?????  ------- split_vertex->next
                               /                           \
                              /        POLYGON TOM          \
                             /                               \
                            /                                 \
   split_vertex->optimal_vertex ---------------------------- split_vertex
                            \                                  /
                             \                                /
                              \       POLYGON JERRY          /
                               \                            /  
split_vertex->optimal_vertex->next -----  ?????  -------- split_vertex->prev

    */
    polygon_Tom = NULL;
    polygon_Tom =_PERUNIKA_create_polygon_point (polygon_Tom, &error);
    polygon_Tom->vertex = split_vertex->vertex;
    polygon_vertex = split_vertex->next;
    while (polygon_vertex != split_vertex->optimal_vertex->next) {
      intersect_vertex=_PERUNIKA_create_polygon_point (polygon_Tom, &error);
      intersect_vertex->vertex = polygon_vertex->vertex;
      polygon_vertex = polygon_vertex->next;
    }
    
    polygon_Jerry = NULL;
    polygon_Jerry =_PERUNIKA_create_polygon_point (polygon_Jerry, &error);
    polygon_Jerry->vertex = split_vertex->optimal_vertex->vertex;
    polygon_vertex = split_vertex->optimal_vertex->next;
    while (polygon_vertex != split_vertex->next) {
      intersect_vertex=_PERUNIKA_create_polygon_point (polygon_Jerry, &error);
      intersect_vertex->vertex = polygon_vertex->vertex;
      polygon_vertex = polygon_vertex->next;
    }
    
    /* split the new polygons */
    error = _PERUNIKA_split_polygon (polygon_Tom);
    if (error) return 1;
    error = _PERUNIKA_split_polygon (polygon_Jerry);
    if (error) return 1;
    
  } /* end of a polygon with more than three vertices */

  /* destroy the polygon */
  polygon_vertex = polygon->next;
  while (polygon_vertex != polygon) {
    split_vertex = polygon_vertex->next;
    free(polygon_vertex);
    polygon_vertex = split_vertex;
  }
  free(polygon);
  polygon = NULL;

  return 0;

}

