/* LELUYA - PERUNIKA program block */

/* This function locates triangles that contain the given point and split 
   them */

#include "PERUNIKA.h"

/*************************************************************************/
/* subroutine for calculation of the 3D liner interpolation coefficients */
void _PERUNIKA_split_triangle_at_given_point_Interpolation_Coefficient 
       (double x1, double y1, double x2, double y2, double x3, double y3, 
        double x, double y, double *c1, double *c2, double *c3);
/*************************************************************************/
/*************************************************************************/
/* subroutine for calculation of the bounding box for a given triangle */
void _PERUNIKA_split_triangle_at_given_point_Bounding_Box
                                         (struct _triangle_structure *t);
/*************************************************************************/

/*************************************************************************/
int _PERUNIKA_split_triangle_at_given_point(double x, double z) {

  int err;/* error flags */
  int dust_i, lambda_k, lambda_kk; /* various counters */
  int lambda_N_max; /* wavelength with the most grid points */
  int N_max;        /* number of vertices at lambda_N_max */
  double r; /* sqrt(x^2 + z^2) */
  struct _triangle_structure *interpolation_triangle;
  struct _triangle_structure *T1, *T2, *T3; /* adjacent to interpolation_triangle */
  double c1,c2,c3;    /* interpolation coefficients */
  struct _vertex_structure *va,*vb,*vc; /* temporary vertexes */
  struct _vertex_structure *new_vertex12,*new_vertex23,*new_vertex31; 
                    /* new vertexes between the triangle vertexes */
  double *U; /* new array of energies */
  struct _triangle_structure *trianA,*trianB,*trianC,*trianD; /* new triangles */  
  struct _triangle_structure *trianA1,*trianA2,*trianB1,*trianB2,*trianC1,*trianC2; 
                             /* new adjacent triangles */
  struct _vertex_triangles   *vertex_triangle; /* vertex link to a triangle */


  printf("SPLIT THE TRIANGLE AT (%lf,%lf) :\n",x,z);

  r = sqrt (x*x+z*z);
  if (r>Rout || r<=Source_Radius) {
printf("ERROR: the required point (%lf,%lf) is out of the computational domain\n",x,z);
    return 1;
  }

  /* find a grid with the largest number of points (let's call it grid_N_max)*/
  lambda_N_max = 0;
  N_max = grid_per_lambda[lambda_N_max].TOTAL_NUMBER_OF_VERTICES;
  for (lambda_k=1; lambda_k<lambda_Max; lambda_k++) {
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > N_max) {
      N_max = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES;
      lambda_N_max = lambda_k;
    }
  }

  /* find which triangle it belongs to */
  start_triangle_list = grid_per_lambda[lambda_N_max].start_triangle; 
  start_vertex_list = grid_per_lambda[lambda_N_max].start_vertex;
  err = _ZELENJAK_locate_point_in_grid(x,z,&interpolation_triangle);

  /* is this out of te grid ? */
  if (interpolation_triangle==NULL) {
    printf("ERROR: the required point (%lf,%lf) is out of the grid\n",x,z);
    return 1;
  }
  /* is this in empty space ? */
  err = TRUE;
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
	 if ( interpolation_triangle->v1->T_dust[dust_i]>0.0 &&
	      interpolation_triangle->v2->T_dust[dust_i]>0.0 &&
              interpolation_triangle->v3->T_dust[dust_i]>0.0)
        {err=FALSE;break;}
  if (err) {
    printf("ERROR: the required point (%lf,%lf) is in empty space\n",x,z);
    return 1;
  }

  /* create temporary vertex */
  va = (struct _vertex_structure *) malloc (sizeof(struct _vertex_structure));
  va->T_dust = dvector(0, Max_Dust_Type-1);
  va->T_error = dvector(0, Max_Dust_Type-1);
  va->x = 0.5*(interpolation_triangle->v1->x+interpolation_triangle->v2->x);
  va->z = 0.5*(interpolation_triangle->v1->z+interpolation_triangle->v2->z);
  /* create temporary vertex */
  vb = (struct _vertex_structure *) malloc (sizeof(struct _vertex_structure));
  vb->T_dust = dvector(0, Max_Dust_Type-1);
  vb->T_error = dvector(0, Max_Dust_Type-1);
  vb->x = 0.5*(interpolation_triangle->v2->x+interpolation_triangle->v3->x);
  vb->z = 0.5*(interpolation_triangle->v2->z+interpolation_triangle->v3->z);
   /* create temporary vertex */
  vc = (struct _vertex_structure *) malloc (sizeof(struct _vertex_structure));
  vc->T_dust = dvector(0, Max_Dust_Type-1);
  vc->T_error = dvector(0, Max_Dust_Type-1);
  vc->x = 0.5*(interpolation_triangle->v3->x+interpolation_triangle->v1->x);
  vc->z = 0.5*(interpolation_triangle->v3->z+interpolation_triangle->v1->z);
 
  /* find temperature */
  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++){
    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
      /* interpolation at va */
      if (interpolation_triangle->v1->T_dust[dust_i]>0.0 &&
          interpolation_triangle->v2->T_dust[dust_i]>0.0){ 
          _PERUNIKA_split_triangle_at_given_point_Interpolation_Coefficient
            (interpolation_triangle->v1->x, interpolation_triangle->v1->z,
	     interpolation_triangle->v2->x, interpolation_triangle->v2->z,
	     interpolation_triangle->v3->x, interpolation_triangle->v3->z,
             va->x, va->z, &c1, &c2, &c3);
          va->T_dust[dust_i] =c1*interpolation_triangle->v1->T_dust[dust_i]+
                              c2*interpolation_triangle->v2->T_dust[dust_i]+
                              c3*interpolation_triangle->v3->T_dust[dust_i];
          /* va->T_error[dust_i]=c1*interpolation_triangle->v1->T_error[dust_i]+
                              c2*interpolation_triangle->v2->T_error[dust_i]+
                              c3*interpolation_triangle->v3->T_error[dust_i]; 
	  */
	  va->T_error[dust_i]=1E30;
      }
      /* interpolation at vb */
      if (interpolation_triangle->v2->T_dust[dust_i]>0.0 &&
          interpolation_triangle->v3->T_dust[dust_i]>0.0){ 
          _PERUNIKA_split_triangle_at_given_point_Interpolation_Coefficient
            (interpolation_triangle->v1->x, interpolation_triangle->v1->z,
	     interpolation_triangle->v2->x, interpolation_triangle->v2->z,
	     interpolation_triangle->v3->x, interpolation_triangle->v3->z,
             vb->x, vb->z, &c1, &c2, &c3);
          vb->T_dust[dust_i] =c1*interpolation_triangle->v1->T_dust[dust_i]+
                              c2*interpolation_triangle->v2->T_dust[dust_i]+
                              c3*interpolation_triangle->v3->T_dust[dust_i];
          /* vb->T_error[dust_i]=c1*interpolation_triangle->v1->T_error[dust_i]+
                              c2*interpolation_triangle->v2->T_error[dust_i]+
                              c3*interpolation_triangle->v3->T_error[dust_i];
	  */
	  vb->T_error[dust_i]=1E30;
      }
      /* interpolation at vc */
      if (interpolation_triangle->v3->T_dust[dust_i]>0.0 &&
          interpolation_triangle->v1->T_dust[dust_i]>0.0){ 
          _PERUNIKA_split_triangle_at_given_point_Interpolation_Coefficient
            (interpolation_triangle->v1->x, interpolation_triangle->v1->z,
	     interpolation_triangle->v2->x, interpolation_triangle->v2->z,
	     interpolation_triangle->v3->x, interpolation_triangle->v3->z,
             vc->x, vc->z, &c1, &c2, &c3);
          vc->T_dust[dust_i] =c1*interpolation_triangle->v1->T_dust[dust_i]+
                              c2*interpolation_triangle->v2->T_dust[dust_i]+
                              c3*interpolation_triangle->v3->T_dust[dust_i];
          /* vc->T_error[dust_i]=c1*interpolation_triangle->v1->T_error[dust_i]+
                              c2*interpolation_triangle->v2->T_error[dust_i]+
                              c3*interpolation_triangle->v3->T_error[dust_i];
	  */
	  vc->T_error[dust_i]=1E30;
      }

    }
    else {
      va->T_dust[dust_i]=_T99999_;va->T_error[dust_i]=1.0E+30;
      vb->T_dust[dust_i]=_T99999_;vb->T_error[dust_i]=1.0E+30;
      vc->T_dust[dust_i]=_T99999_;vc->T_error[dust_i]=1.0E+30;
    }
  }
  
  /* go over wavelength grids */
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
    if(grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > 0) {
     
      /* printf("--- lambda index = %d\n",lambda_k); */
      start_triangle_list = grid_per_lambda[lambda_k].start_triangle; 
      start_vertex_list = grid_per_lambda[lambda_k].start_vertex;
      TOTAL_NUMBER_OF_VERTICES = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES;
      TOTAL_NUMBER_OF_TRIANGLES = grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_TRIANGLES;
      /* find which triangle it belongs to */
      err = _ZELENJAK_locate_point_in_grid(x,z,&interpolation_triangle);
      /* printf("Parent triangle [%E,%E][%E,%E][%E,%E]\n",
             interpolation_triangle->v1->x,interpolation_triangle->v1->z,
             interpolation_triangle->v2->x,interpolation_triangle->v2->z,
             interpolation_triangle->v3->x,interpolation_triangle->v3->z); */
      /* adjacent triangles */
      T1 = _PERUNIKA_adjacent_triangle(interpolation_triangle->v2,interpolation_triangle);
      T2 = _PERUNIKA_adjacent_triangle(interpolation_triangle->v3,interpolation_triangle);
      T3 = _PERUNIKA_adjacent_triangle(interpolation_triangle->v1,interpolation_triangle);

      /* new coordinates */
      va->x = 0.5*(interpolation_triangle->v1->x+interpolation_triangle->v2->x);
      va->z = 0.5*(interpolation_triangle->v1->z+interpolation_triangle->v2->z);
      vb->x = 0.5*(interpolation_triangle->v2->x+interpolation_triangle->v3->x);
      vb->z = 0.5*(interpolation_triangle->v2->z+interpolation_triangle->v3->z);
      vc->x = 0.5*(interpolation_triangle->v3->x+interpolation_triangle->v1->x);
      vc->z = 0.5*(interpolation_triangle->v3->z+interpolation_triangle->v1->z);
      /* create vertex V12*/
      new_vertex12 = _PERUNIKA_create_vertex (&err);
      new_vertex12->v1=new_vertex12->v2=new_vertex12->v3=new_vertex12->v4=
	new_vertex12->v5=new_vertex12->v6=NULL;
      new_vertex12->identification_number = TOTAL_NUMBER_OF_VERTICES-1;
      new_vertex12->level = 667;
      if (interpolation_triangle->v1->type==2 && 
          interpolation_triangle->v2->type==2) new_vertex12->type  = 2;
      else  new_vertex12->type  = 1;
      new_vertex12->x = va->x;  new_vertex12->z = va->z;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
        new_vertex12->T_dust[dust_i] = va->T_dust[dust_i];
        new_vertex12->T_error[dust_i] = va->T_error[dust_i];
      }
      new_vertex12->Eta =  Eta(va->x,va->z,lambda_k,new_vertex12->T_dust,&err);
      /* create vertex V23*/
      new_vertex23 = _PERUNIKA_create_vertex (&err);
      new_vertex23->v1=new_vertex23->v2=new_vertex23->v3=new_vertex23->v4=
	new_vertex23->v5=new_vertex23->v6=NULL;
      new_vertex23->identification_number = TOTAL_NUMBER_OF_VERTICES-1;
      new_vertex23->level = 667;
      if (interpolation_triangle->v2->type==2 && 
          interpolation_triangle->v3->type==2) new_vertex23->type  = 2;
      else  new_vertex23->type  = 1;
      new_vertex23->x = vb->x;  new_vertex23->z = vb->z;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
        new_vertex23->T_dust[dust_i] = vb->T_dust[dust_i];
        new_vertex23->T_error[dust_i] = vb->T_error[dust_i];
      }
      new_vertex23->Eta =  Eta(vb->x,vb->z,lambda_k,new_vertex23->T_dust,&err);

      /* create vertex V31*/
      new_vertex31 = _PERUNIKA_create_vertex (&err);
      new_vertex31->v1=new_vertex31->v2=new_vertex31->v3=new_vertex31->v4=
	new_vertex31->v5=new_vertex31->v6=NULL;
      new_vertex31->identification_number = TOTAL_NUMBER_OF_VERTICES-1;
      new_vertex31->level = 667;
      if (interpolation_triangle->v3->type==2 && 
          interpolation_triangle->v1->type==2) new_vertex31->type  = 2;
      else new_vertex31->type  = 1;
      new_vertex31->x = vc->x;  new_vertex31->z = vc->z;
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
        new_vertex31->T_dust[dust_i] = vc->T_dust[dust_i];
        new_vertex31->T_error[dust_i] = vc->T_error[dust_i];
      }
      new_vertex31->Eta =  Eta(vc->x,vc->z,lambda_k,new_vertex31->T_dust,&err);

      /* update the energy with this new vertexes included */
      for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
	if (grid_per_lambda[lambda_kk].start_vertex==start_vertex_list) {
          grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES = TOTAL_NUMBER_OF_VERTICES;
          /* stellar */
	  U = dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
          for (err=0;err<TOTAL_NUMBER_OF_VERTICES-3;err++)
	    U[err]=grid_per_lambda[lambda_kk].Ustar[err];
          U[TOTAL_NUMBER_OF_VERTICES-1]=0.0;/* this is the new one */
          U[TOTAL_NUMBER_OF_VERTICES-2]=0.0;/* this is the new one */
          U[TOTAL_NUMBER_OF_VERTICES-3]=0.0;/* this is the new one */
          free_dvector(grid_per_lambda[lambda_kk].Ustar,
                       0,TOTAL_NUMBER_OF_VERTICES-4);
          grid_per_lambda[lambda_kk].Ustar = U;
          /* emission */
	  U = dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
          for (err=0;err<TOTAL_NUMBER_OF_VERTICES-3;err++)
	    U[err]=grid_per_lambda[lambda_kk].Uem[err];
          U[TOTAL_NUMBER_OF_VERTICES-1]=0.0;/* this is the new one */
          U[TOTAL_NUMBER_OF_VERTICES-2]=0.0;/* this is the new one */
          U[TOTAL_NUMBER_OF_VERTICES-3]=0.0;/* this is the new one */
          free_dvector(grid_per_lambda[lambda_kk].Uem,
                       0,TOTAL_NUMBER_OF_VERTICES-4);
          grid_per_lambda[lambda_kk].Uem = U;
          /* scattering */
	  U = dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
          for (err=0;err<TOTAL_NUMBER_OF_VERTICES-3;err++)
	    U[err]=grid_per_lambda[lambda_kk].Usca[err];
          U[TOTAL_NUMBER_OF_VERTICES-1]=0.0;/* this is the new one */
          U[TOTAL_NUMBER_OF_VERTICES-2]=0.0;/* this is the new one */
          U[TOTAL_NUMBER_OF_VERTICES-3]=0.0;/* this is the new one */
          free_dvector(grid_per_lambda[lambda_kk].Usca,
                       0,TOTAL_NUMBER_OF_VERTICES-4);
          grid_per_lambda[lambda_kk].Usca = U;

          /* total */
	  U = dvector(0,TOTAL_NUMBER_OF_VERTICES-1);
          for (err=0;err<TOTAL_NUMBER_OF_VERTICES-3;err++)
	    U[err]=grid_per_lambda[lambda_kk].Utot_new[err];

          /* find U_total for V12 */
          _PERUNIKA_split_triangle_at_given_point_Interpolation_Coefficient
             (interpolation_triangle->v1->x, interpolation_triangle->v1->z,
	      interpolation_triangle->v2->x, interpolation_triangle->v2->z,
	      interpolation_triangle->v3->x, interpolation_triangle->v3->z,
              new_vertex12->x, new_vertex12->z, &c1, &c2, &c3);
          U[new_vertex12->identification_number]= 
                 c1*grid_per_lambda[lambda_kk].Utot_new[
                            interpolation_triangle->v1->identification_number]
	       + c2*grid_per_lambda[lambda_kk].Utot_new[
                            interpolation_triangle->v2->identification_number]
	       + c3*grid_per_lambda[lambda_kk].Utot_new[
			    interpolation_triangle->v3->identification_number];
          /* find U_total for V23 */
          _PERUNIKA_split_triangle_at_given_point_Interpolation_Coefficient
             (interpolation_triangle->v1->x, interpolation_triangle->v1->z,
	      interpolation_triangle->v2->x, interpolation_triangle->v2->z,
	      interpolation_triangle->v3->x, interpolation_triangle->v3->z,
              new_vertex23->x, new_vertex23->z, &c1, &c2, &c3);
          U[new_vertex23->identification_number]= 
                 c1*grid_per_lambda[lambda_kk].Utot_new[
                            interpolation_triangle->v1->identification_number]
	       + c2*grid_per_lambda[lambda_kk].Utot_new[
                            interpolation_triangle->v2->identification_number]
	       + c3*grid_per_lambda[lambda_kk].Utot_new[
			    interpolation_triangle->v3->identification_number];
          /* find U_total for V31 */
          _PERUNIKA_split_triangle_at_given_point_Interpolation_Coefficient
             (interpolation_triangle->v1->x, interpolation_triangle->v1->z,
	      interpolation_triangle->v2->x, interpolation_triangle->v2->z,
	      interpolation_triangle->v3->x, interpolation_triangle->v3->z,
              new_vertex31->x, new_vertex31->z, &c1, &c2, &c3);
          U[new_vertex31->identification_number]= 
                 c1*grid_per_lambda[lambda_kk].Utot_new[
                            interpolation_triangle->v1->identification_number]
	       + c2*grid_per_lambda[lambda_kk].Utot_new[
                            interpolation_triangle->v2->identification_number]
	       + c3*grid_per_lambda[lambda_kk].Utot_new[
			    interpolation_triangle->v3->identification_number];

          free_dvector(grid_per_lambda[lambda_kk].Utot_new,
                       0,TOTAL_NUMBER_OF_VERTICES-4);
          grid_per_lambda[lambda_kk].Utot_new = U;
	}

      /* create new triangles: trianA, trianB, trianC , trianD
      
                       V3---V23---V2
                        \ C / \ B /
                         \ / D \ /
                         V31--V12
                           \ A /
                            \ /
                             V1  
      */
      trianA = _PERUNIKA_create_triangle(&err);
      trianA->level = interpolation_triangle->level+1;
      trianA->v1 = interpolation_triangle->v1;
      trianA->v2 = new_vertex12;
      trianA->v3 = new_vertex31;

      trianB = _PERUNIKA_create_triangle(&err);
      trianB->level = interpolation_triangle->level+1;
      trianB->v1 = new_vertex12;
      trianB->v2 = interpolation_triangle->v2;
      trianB->v3 = new_vertex23;

      trianC = _PERUNIKA_create_triangle(&err);
      trianC->level = interpolation_triangle->level+1;
      trianC->v1 = new_vertex31;
      trianC->v2 = new_vertex23;
      trianC->v3 = interpolation_triangle->v3;

      trianD = _PERUNIKA_create_triangle(&err);
      trianD->level = interpolation_triangle->level+1;
      trianD->v1 = new_vertex12;
      trianD->v2 = new_vertex23;
      trianD->v3 = new_vertex31;

      /* create new adjacent triangles: trianA1, trianB1, trianC1,
         trianA2, trianB2, trianC2
                         /   |   \
                        / B2 | B1 \
                       V3---V23---V2
                      / \         / \
                     / C1\       /A2 \
                     /   V31   V12   \
                    /   /  \   / \    \
                    / /  C2 \ /A1  \  \
                    O---- ---V1-------O
      */

      if (T1!=NULL) {
        trianA1 = _PERUNIKA_create_triangle(&err);
        trianA1->level = interpolation_triangle->level+1;
        trianA2 = _PERUNIKA_create_triangle(&err);
        trianA2->level = interpolation_triangle->level+1;
        if (T1->v1==interpolation_triangle->v2) {
           trianA1->v1 = T1->v3;
           trianA1->v2 = new_vertex12;
           trianA1->v3 = T1->v2;
           trianA2->v1 = T1->v3;
           trianA2->v2 = T1->v1;
           trianA2->v3 = new_vertex12;
	}
        else if (T1->v3==interpolation_triangle->v2) {
           trianA1->v1 = T1->v2;
           trianA1->v2 = new_vertex12;
           trianA1->v3 = T1->v1;
           trianA2->v1 = T1->v2;
           trianA2->v2 = T1->v3;
           trianA2->v3 = new_vertex12;
	}
        else  {
           trianA1->v1 = T1->v1;
           trianA1->v2 = new_vertex12;
           trianA1->v3 = T1->v3;
           trianA2->v1 = T1->v1;
           trianA2->v2 = T1->v2;
           trianA2->v3 = new_vertex12;
	}
      }
      else {trianA1=NULL;trianA2=NULL;}
      if (T2!=NULL) {
        trianB1 = _PERUNIKA_create_triangle(&err);
        trianB1->level = interpolation_triangle->level+1;
        trianB2 = _PERUNIKA_create_triangle(&err);
        trianB2->level = interpolation_triangle->level+1;
        if (T2->v1==interpolation_triangle->v3) {
           trianB1->v1 = T2->v3;
           trianB1->v2 = new_vertex23;
           trianB1->v3 = T2->v2;
           trianB2->v1 = T2->v3;
           trianB2->v2 = T2->v1;
           trianB2->v3 = new_vertex23;
	}
        else if (T2->v3==interpolation_triangle->v3) {
           trianB1->v1 = T2->v2;
           trianB1->v2 = new_vertex23;
           trianB1->v3 = T2->v1;
           trianB2->v1 = T2->v2;
           trianB2->v2 = T2->v3;
           trianB2->v3 = new_vertex23;
	}
        else  {
           trianB1->v1 = T2->v1;
           trianB1->v2 = new_vertex23;
           trianB1->v3 = T2->v3;
           trianB2->v1 = T2->v1;
           trianB2->v2 = T2->v2;
           trianB2->v3 = new_vertex23;
	}
      }
      else {trianB1=NULL;trianB2=NULL;}
      if (T3!=NULL) {
        trianC1 = _PERUNIKA_create_triangle(&err);
        trianC1->level = interpolation_triangle->level+1;
        trianC2 = _PERUNIKA_create_triangle(&err);
        trianC2->level = interpolation_triangle->level+1;
        if (T3->v1==interpolation_triangle->v1) {
           trianC1->v1 = T3->v3;
           trianC1->v2 = new_vertex31;
           trianC1->v3 = T3->v2;
           trianC2->v1 = T3->v3;
           trianC2->v2 = T3->v1;
           trianC2->v3 = new_vertex31;
	}
        else if (T3->v3==interpolation_triangle->v1) {
           trianC1->v1 = T3->v2;
           trianC1->v2 = new_vertex31;
           trianC1->v3 = T3->v1;
           trianC2->v1 = T3->v2;
           trianC2->v2 = T3->v3;
           trianC2->v3 = new_vertex31;
	}
        else  {
           trianC1->v1 = T3->v1;
           trianC1->v2 = new_vertex31;
           trianC1->v3 = T3->v3;
           trianC2->v1 = T3->v1;
           trianC2->v2 = T3->v2;
           trianC2->v3 = new_vertex31;
	}
      }
      else {trianC1=NULL;trianC2=NULL;}

       /* bounding box: */
      _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianA);
      _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianB);
      _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianC);
      _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianD);
      if (trianA1!=NULL) 
         _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianA1);
      if (trianA2!=NULL) 
         _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianA2);
      if (trianB1!=NULL) 
         _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianB1);
      if (trianB2!=NULL) 
         _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianB2);
      if (trianC1!=NULL) 
         _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianC1);
      if (trianC2!=NULL) 
         _PERUNIKA_split_triangle_at_given_point_Bounding_Box(trianC2);
      
      /* destroy the parent triangle */
      err=_PERUNIKA_destroy_triangle(interpolation_triangle);
      if (T1!=NULL) err=_PERUNIKA_destroy_triangle(T1);
      if (T2!=NULL) err=_PERUNIKA_destroy_triangle(T2);
      if (T3!=NULL) err=_PERUNIKA_destroy_triangle(T3);
      
      /* update the triangle list */
      for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
	if (grid_per_lambda[lambda_kk].start_vertex==start_vertex_list) {
          grid_per_lambda[lambda_kk].start_triangle = start_triangle_list; 
          grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_TRIANGLES = TOTAL_NUMBER_OF_TRIANGLES;
	}

      /* re-number the triangle list */
      lambda_kk=0;
      interpolation_triangle = start_triangle_list;
      do {
        interpolation_triangle->ID_number=lambda_kk;
        lambda_kk++;
	interpolation_triangle=interpolation_triangle->next;
      } while (interpolation_triangle != start_triangle_list);

      /* add these new triangles to the lists at each vertex */
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA,trianA->v1);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA,trianA->v2);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA,trianA->v3);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB,trianB->v1);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB,trianB->v2);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB,trianB->v3);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC,trianC->v1);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC,trianC->v2);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC,trianC->v3);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianD,trianD->v1);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianD,trianD->v2);
      vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianD,trianD->v3);

      if (trianA1!=NULL) {
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA1,trianA1->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA1,trianA1->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA1,trianA1->v3);
      }
      if (trianA2!=NULL) {
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA2,trianA2->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA2,trianA2->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianA2,trianA2->v3);
      }

      if (trianB1!=NULL) {
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB1,trianB1->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB1,trianB1->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB1,trianB1->v3);
      }
      if (trianB2!=NULL) {
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB2,trianB2->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB2,trianB2->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianB2,trianB2->v3);
      }

      if (trianC1!=NULL) {
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC1,trianC1->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC1,trianC1->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC1,trianC1->v3);
      }
      if (trianC2!=NULL) {
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC2,trianC2->v1);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC2,trianC2->v2);
        vertex_triangle=_PERUNIKA_create_new_vertex_triangle(trianC2,trianC2->v3);
      }
     
      /* sort the list of triangles at each vertex */
      _PERUNIKA_sort_triangles_at_vertex(trianA->v1);
      _PERUNIKA_sort_triangles_at_vertex(trianA->v2);
      _PERUNIKA_sort_triangles_at_vertex(trianA->v3);
      _PERUNIKA_sort_triangles_at_vertex(trianB->v2);
      _PERUNIKA_sort_triangles_at_vertex(trianB->v3);
      _PERUNIKA_sort_triangles_at_vertex(trianC->v3);
      if (trianA1!=NULL) _PERUNIKA_sort_triangles_at_vertex(trianA1->v1);
      if (trianB1!=NULL) _PERUNIKA_sort_triangles_at_vertex(trianB1->v1);
      if (trianC1!=NULL) _PERUNIKA_sort_triangles_at_vertex(trianC1->v1);
      /* find the triangle step for each new traingle */
      err = _PERUNIKA_triangle_integral_step(trianA);
      err = _PERUNIKA_triangle_integral_step(trianB);
      err = _PERUNIKA_triangle_integral_step(trianC);
      err = _PERUNIKA_triangle_integral_step(trianD);
      if (trianA1!=NULL) err = _PERUNIKA_triangle_integral_step(trianA1);
      if (trianA2!=NULL) err = _PERUNIKA_triangle_integral_step(trianA2);
      if (trianB1!=NULL) err = _PERUNIKA_triangle_integral_step(trianB1);
      if (trianB2!=NULL) err = _PERUNIKA_triangle_integral_step(trianB2);
      if (trianC1!=NULL) err = _PERUNIKA_triangle_integral_step(trianC1);
      if (trianC2!=NULL) err = _PERUNIKA_triangle_integral_step(trianC2);

      /* mark this grid as "checked" */
      for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
	if (grid_per_lambda[lambda_kk].start_vertex==start_vertex_list)
          grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES *= -1;
    }

  /* remove the mark from grids */
  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
    if(grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES < 0)
      grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES *= -1;

  /* free memory */
  free_dvector(va->T_dust, 0, Max_Dust_Type-1);
  free_dvector(va->T_error, 0, Max_Dust_Type-1);
  free (va);
  free_dvector(vb->T_dust, 0, Max_Dust_Type-1);
  free_dvector(vb->T_error, 0, Max_Dust_Type-1);
  free (vb);
  free_dvector(vc->T_dust, 0, Max_Dust_Type-1);
  free_dvector(vc->T_error, 0, Max_Dust_Type-1);
  free (vc);

  return 0;
}

/*************************************************************************/
/*************************************************************************/
/* subroutine for calculation of the 3D liner interpolation coefficients */
void _PERUNIKA_split_triangle_at_given_point_Interpolation_Coefficient 
       (double x1, double y1, double x2, double y2, double x3, double y3, 
        double x, double y, double *c1, double *c2, double *c3){

  double denominator;
  denominator = (x2-x1)*(y3-y1) - (x3-x1)*(y2-y1);
  if (denominator == 0.0) {
    *c1=0.0; *c2=0.0; *c3=0.0; return;
  }
  *c1 = 1 + ( (x-x1)*(y2-y3)+(y-y1)*(x3-x2) )/denominator;
  *c2 = ( (x-x1)*(y3-y1)-(y-y1)*(x3-x1) )/denominator;
  *c3 = ( (y-y1)*(x2-x1)-(x-x1)*(y2-y1) )/denominator;
  return;
}

/*************************************************************************/
/*************************************************************************/
/* subroutine for calculation of the bounding box for a given triangle */
void _PERUNIKA_split_triangle_at_given_point_Bounding_Box
                                        (struct _triangle_structure *t){

      t->bound_box_down_left_x=t->v1->x;
      t->bound_box_down_left_z=t->v1->z;
      t->bound_box_up_right_x=t->v1->x;
      t->bound_box_up_right_z=t->v1->z;
      if (t->bound_box_down_left_x>t->v2->x)
        t->bound_box_down_left_x=t->v2->x;
      if (t->bound_box_down_left_z>t->v2->z)
        t->bound_box_down_left_z=t->v2->z;
      if (t->bound_box_up_right_x<t->v2->x)
        t->bound_box_up_right_x=t->v2->x;
      if (t->bound_box_up_right_z<t->v2->z)
        t->bound_box_up_right_z=t->v2->z;
      if (t->bound_box_down_left_x>t->v3->x)
        t->bound_box_down_left_x=t->v3->x;
      if (t->bound_box_down_left_z>t->v3->z)
        t->bound_box_down_left_z=t->v3->z;
      if (t->bound_box_up_right_x<t->v3->x)
        t->bound_box_up_right_x=t->v3->x;
      if (t->bound_box_up_right_z<t->v3->z)
        t->bound_box_up_right_z=t->v3->z;

      return;
}
/*************************************************************************/
