/* LELUYA - PERUNIKA program block */
/* This part of the LELUYA code is generating the computational grid */

/* temperature at a given point (in the old grid or initial guess) 
   NOTE: the temperature error is calculated ONLY IF (*temp_error)>0 */

#include "PERUNIKA.h"

double _PERUNIKA_temperature(double x, double z, int dust_i,int status,
                         double *temp_error){

  double temperature;   
  struct _triangle_structure *old_triangle;
  double nu;
  struct _quadrangle_structure *quadrangle;
  struct _vertex_structure *v1,*v2,*v3,*v4;
  int error;
  double r;

  if (x<0.0) x*=-1.0;
  if (z<0.0) z*=-1.0;

  /* check if we are dealing with the empty space */
  if (INPUT.DUST_DENSITY_TYPE[dust_i]==99999) {
    if (Dust_Density(x,z,dust_i,NULL,&error)>0.0)
      temperature = _T99999_;
    else temperature = -1.0; 
    return temperature;
  }


  /* first check if (x,z) is out of the dust region */
  r = sqrt(x*x+z*z);
  if (r>Rout+0.0001) return -1.0;

  /***************************************************
  if (dust_i==1 && x<1.00805) {temperature=-1.; return temperature;}
  ****************************************************/
  /***************************************************
  if ((dust_i==1 || dust_i==0) && r>41.38 && z<0.32267074*x && z>0.1763*x) {temperature=244.; return temperature;}
  if (r>=1.0 && x<=z*tan(DEBUG.AZIMUTHAL_GRID1+0.017453293)) return 1.0;
  if (r>99.22 && x<=z*tan(DEBUG.AZIMUTHAL_GRID1+0.017453293)) return 179;
  ****************************************************/
  /***************************************************
  if (x<z*tan(DEBUG.AZIMUTHAL_GRID1)) return -1.0;
  ****************************************************
  if (fabs(x)>=1.0 && fabs(x)<1.999995 && fabs(z)<0.1) return 1500.0;
  ****************************************************/

  if (status==0 || status==1) 
    /* use the initial guess */ 
    temperature=_PERUNIKA_initial_temperature (x, z, dust_i);
  else {
    /* STATUS = 2 */
    /* interpolate from the old temperature grid */ 
    temperature = -1.0;
    if ((*temp_error)>0.0) *temp_error = 1.0E30;
    /* check if this dust type exists here */
    if (Dust_Density(x,z,dust_i,NULL,&error)<=0.0) return temperature;

    /* first find the triangle (or quadrangle) which contains the given 
       point (x,z) */
    old_triangle=NULL; quadrangle=NULL;
    error=_PERUNIKA_locate_point_in_old_grid(x,z,dust_i,&old_triangle,&quadrangle);

    if (old_triangle!=NULL) {
      temperature = _MATH_LIB_plane_interpolation_logZ
     (old_triangle->v1->x,old_triangle->v1->z,old_triangle->v1->T_dust[dust_i],
      old_triangle->v2->x,old_triangle->v2->z,old_triangle->v2->T_dust[dust_i],
      old_triangle->v3->x,old_triangle->v3->z,old_triangle->v3->T_dust[dust_i],
      x,z);
     if ((*temp_error)>0.0) 
       *temp_error = _MATH_LIB_plane_interpolation 
    (old_triangle->v1->x,old_triangle->v1->z,old_triangle->v1->T_error[dust_i],
     old_triangle->v2->x,old_triangle->v2->z,old_triangle->v2->T_error[dust_i],
     old_triangle->v3->x,old_triangle->v3->z,old_triangle->v3->T_error[dust_i],
      x,z);
    } 
    else if (quadrangle!=NULL) {
      if ((*temp_error)>0.0) *temp_error=1.0E30;
      v1=quadrangle->edge_vertex;
      v2=quadrangle->next_edge_vertex;
      v3=quadrangle->new_next_edge_vertex;
      v4=quadrangle->new_edge_vertex;
      /* quadrangle:   v3--v2
                       |    |
                       |    |
                       v4--v1
      
      interpolate from the quadrangle, but be careful: it is maybe a triangle*/
      if (v2==v3 || v3==v4) temperature = _MATH_LIB_plane_interpolation
			    (v1->x,v1->z,v1->T_dust[dust_i],
			     v2->x,v2->z,v2->T_dust[dust_i],
			     v4->x,v4->z,v4->T_dust[dust_i],  x,z); 
      else if (v1==v4) temperature = _MATH_LIB_plane_interpolation
			    (v1->x,v1->z,v1->T_dust[dust_i],
			     v2->x,v2->z,v2->T_dust[dust_i],
			     v3->x,v3->z,v3->T_dust[dust_i],  x,z); 
      else if (_MATH_LIB_distance(v1->x,v1->z,v3->x,v3->z) <
	       _MATH_LIB_distance(v2->x,v2->z,v4->x,v4->z) ) {
	/* interpolation triangles:     v3--v2
                                        | \  |
                                        |  \ |
                                        v4--v1  
	*/
	nu = _MATH_LIB_nu_value(v1->x,v1->z,v3->x,v3->z,x,z);
	if (0.0<=nu) temperature = _MATH_LIB_plane_interpolation
			    (v1->x,v1->z,v1->T_dust[dust_i],
			     v3->x,v3->z,v3->T_dust[dust_i],
			     v4->x,v4->z,v4->T_dust[dust_i],  x,z); 
        else temperature = _MATH_LIB_plane_interpolation
			    (v1->x,v1->z,v1->T_dust[dust_i],
			     v2->x,v2->z,v2->T_dust[dust_i],
			     v3->x,v3->z,v3->T_dust[dust_i],  x,z); 
      }
      else {
	/* interpolation triangles:     v3--v2
                                        |  / |
                                        | /  |
                                        v4--v1  
	*/
	nu = _MATH_LIB_nu_value(v4->x,v4->z,v2->x,v2->z,x,z);
	if (0.0<=nu) temperature = _MATH_LIB_plane_interpolation
			    (v2->x,v2->z,v2->T_dust[dust_i],
			     v3->x,v3->z,v3->T_dust[dust_i],
			     v4->x,v4->z,v4->T_dust[dust_i],  x,z);
 	else temperature = _MATH_LIB_plane_interpolation
			    (v1->x,v1->z,v1->T_dust[dust_i],
			     v2->x,v2->z,v2->T_dust[dust_i],
			     v4->x,v4->z,v4->T_dust[dust_i],  x,z); 
      }
    }

  } /* end of the status=2: interpolation from the old temperature grid */

  if (temperature<=0.0) temperature = -1.0;
  if (temperature/Dust_Sublim_Temp[dust_i] - 1.0 > 0.0) {
    if (Max_Dust_Temp<=0.0) {
      if (temperature/Dust_Sublim_Temp[dust_i] - 1.0 <= _SUBLIMATION_TOLERANCE_)
        temperature = Dust_Sublim_Temp[dust_i];
      else temperature = -1.0;
    }
    else if (temperature > Max_Dust_Temp) temperature = -1.0;
  }

  /*
  if (dust_i==1 && temperature>1511.) {temperature=-1.; return temperature;}
  if (dust_i==2 && temperature>1511.) {temperature=-1.; return temperature;}
  if (dust_i==3 && temperature>1511.) {temperature=-1.; return temperature;}
  */

/*
  if (temperature<0.0 && dust_i==0)
    if (x>z*tan(45.0*RAD1)) return 50;
  if (temperature<0.0 && dust_i==1 && x*x+z*z>8.9999 && x<97.5)
    if (x>z*tan(45.0*RAD1)) return 50;
*/

  return temperature;
}
