/* LELUYA - PESJANEK program block */
/* this function is saving the grid at each wavelength.
   That is, data stored in "grid_per_lambda" and pointer lists within. */
/* This function is activated with the flag '-s 202' */

/* the scheme of dumping the data:
   
   - the first 3 lines are comments
   for (lambda_k=0; lambda_k<lambda_Max; lambda_k++)
   | if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > 0)
   | | print grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES
   | | loop over vertices
   | | | print data stored at current vertex (except the list of triangles)
   | | endloop
   | | print  grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_TRIANGLES
   | | loop over triangles
   | | | print data at current triangle
   | | endloop
   | | loop over vertices
   | | | print the list of triangles at current vertex
   | | endloop
   | | for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
   | | | if (the same grid like the one at lambda_k)
   | | | | grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES *= -1
   | | | | print lambda_kk
   | | | | print grid_per_lambda[lambda_k].tau
   | | | | for (v=0; v<TOTAL_NUMBER_OF_VERTICES; v++)
   | | | | | print grid_per_lambda[lambda_k].Ustar[v]
   | | | | | print grid_per_lambda[lambda_k].Usca [v]
   | | | | | print grid_per_lambda[lambda_k].Uem  [v]
   | | | | | print grid_per_lambda[lambda_k].Utot_new[v]
   | | | | endfor
   | | | endif
   | | endfor
   | | print -1
   | endif
   endfor

*/

#include "PESJANEK.h"

void _PESJANEK_Dump_Data_STOP202 (FILE *DUMP_FILE) {

  int lambda_k, lambda_kk, dust_i, v; /* loop counters */
  struct _vertex_structure *loop_vertex;
  struct _vertex_triangles *loop_vertex_triangle;
  struct _triangle_structure *loop_triangle;

  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    printf("_PESJANEK_Dump_Data_STOP202: !!! DUMPING OF THE GRIDS !!!\n");
  }


  fprintf(DUMP_FILE,"#### LELUYA, dumped grids, author Dejan Vinkovic\n");
  fprintf(DUMP_FILE,"this file is created by  _PESJANEK_Dump_Data_STOP202\n");
  fprintf(DUMP_FILE,"DO NOT CHANGE ANYTHING IN THIS FILE! \n");

  for (lambda_k=0; lambda_k<lambda_Max; lambda_k++) {
    if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > 0) {

 /* save the vertices */ 
 fprintf(DUMP_FILE,"%d\n",grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES);
 loop_vertex = grid_per_lambda[lambda_k].start_vertex;
 do {
   fprintf(DUMP_FILE,"%d %d ",loop_vertex->level,loop_vertex->type);
   fprintf(DUMP_FILE,"%13.11E %13.11E ",loop_vertex->x,loop_vertex->z);
   for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
     fprintf(DUMP_FILE,"%E ",loop_vertex->T_dust[dust_i]);
     fprintf(DUMP_FILE,"%E ",loop_vertex->T_error[dust_i]);
   }
   fprintf(DUMP_FILE,"%E\n",loop_vertex->Eta);
   loop_vertex = loop_vertex->next;
 } while (loop_vertex !=grid_per_lambda[lambda_k].start_vertex); 

 /* save the triangles */
 fprintf(DUMP_FILE,"%d\n",grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_TRIANGLES);
 loop_triangle = grid_per_lambda[lambda_k].start_triangle;
 do {
   fprintf(DUMP_FILE,"%d %d %d ",loop_triangle->v1->identification_number,
                                 loop_triangle->v2->identification_number,
	                         loop_triangle->v3->identification_number);
   fprintf(DUMP_FILE,"%d ",loop_triangle->level);
   fprintf(DUMP_FILE,"%E %E %E %E ",loop_triangle->bound_box_down_left_x,
	                            loop_triangle->bound_box_down_left_z,
	                            loop_triangle->bound_box_up_right_x,
	                            loop_triangle->bound_box_up_right_z);
   fprintf(DUMP_FILE,"%E\n",loop_triangle->integral_step);
   loop_triangle = loop_triangle->next;
 } while (loop_triangle != grid_per_lambda[lambda_k].start_triangle);

 /* save the list of triangles at each vertex */ 
 loop_vertex = grid_per_lambda[lambda_k].start_vertex;
 do {
   loop_vertex_triangle = loop_vertex->triangles;
   while (loop_vertex_triangle!=NULL) {
     fprintf(DUMP_FILE,"%d ",loop_vertex_triangle->triangle->ID_number);
     fprintf(DUMP_FILE,"%E ",loop_vertex_triangle->nu);
     loop_vertex_triangle = loop_vertex_triangle->next;
   }
   fprintf(DUMP_FILE,"-1\n");
   loop_vertex = loop_vertex->next;
 } while (loop_vertex !=grid_per_lambda[lambda_k].start_vertex); 

 /* list of wavelengths with the grid above */
 for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++) {
   if (grid_per_lambda[lambda_kk].start_vertex == 
       grid_per_lambda[lambda_k].start_vertex) {
     fprintf(DUMP_FILE,"%d\n",lambda_kk);
     fprintf(DUMP_FILE,"%E\n",grid_per_lambda[lambda_kk].tau);
     /* save the energy densities */
     for (v=0; v<grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES; v++) {
       fprintf(DUMP_FILE,"%E ",grid_per_lambda[lambda_kk].Ustar[v]);
       fprintf(DUMP_FILE,"%E ",grid_per_lambda[lambda_kk].Usca[v]);
       fprintf(DUMP_FILE,"%E ",grid_per_lambda[lambda_kk].Uem[v]);
       fprintf(DUMP_FILE,"%E ",grid_per_lambda[lambda_kk].Utot_new[v]);
     }
     fprintf(DUMP_FILE,"\n");
     /* mark that grid */ 
     grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES *= -1;
   }
 } 
 fprintf(DUMP_FILE,"-1\n");

    } /* end if (grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES > 0) */
  } /* end of the loop over wavelengths (lambda_k) */

  for (lambda_kk=0; lambda_kk<lambda_Max; lambda_kk++)
    grid_per_lambda[lambda_kk].TOTAL_NUMBER_OF_VERTICES *= -1;

  /* save the flag showing the type of grid (azimuthal for DEBUG.AZIMUTHAL_GRID>=0) */
  fprintf(DUMP_FILE,"%d",DEBUG.AZIMUTHAL_GRID);
  if (DEBUG.AZIMUTHAL_GRID==1 || DEBUG.AZIMUTHAL_GRID==2 || DEBUG.AZIMUTHAL_GRID==3) 
     fprintf(DUMP_FILE," %lf",DEBUG.AZIMUTHAL_GRID1);
  fprintf(DUMP_FILE,"\n");

  /* save the flag for hydrostatic equilibrium of flared disks */
  fprintf(DUMP_FILE,"%d ",DEBUG.DENSITY_UPDATE);
  fprintf(DUMP_FILE,"%lf ",DEBUG.DENSITY_UPDATE_M);
  fprintf(DUMP_FILE,"%lf\n",DEBUG.DENSITY_UPDATE_R);

  if (DEBUG.DENSITY_UPDATE>=0)
    if (Density_2D_Grid.triangle!=NULL) {
      /* save the density grid */
      
      fprintf(DUMP_FILE,"%d %d %d\n",Density_2D_Grid.lambda,
                                     Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES,
                                     Density_2D_Grid.dust);
      for (lambda_k=0; lambda_k<Density_2D_Grid.dust; lambda_k++)
        fprintf(DUMP_FILE," %d",Density_2D_Grid.dust_types[lambda_k]);
      fprintf(DUMP_FILE,"\n");
      for (lambda_k=0; lambda_k<Density_2D_Grid.dust; lambda_k++) {
 for (lambda_kk=0; lambda_kk<Density_2D_Grid.TOTAL_NUMBER_OF_VERTICES; lambda_kk++) 
          fprintf(DUMP_FILE,"%E ",Density_2D_Grid.density[lambda_k][lambda_kk]);
        fprintf(DUMP_FILE,"\n");
      }

    }

} /* end of _PESJANEK_Dump_Data_STOP202 */
