/* LELUYA - VILOZMAJ program block */
/* This part of the LELUYA code is dealing with the radiative transfer
   calculation (matrix calculation and inversion) from a given 2D grid for
   several wavelengths at the same time */

/* It is using integration over e^-Tau, not over Tau */

/* it DOES NOT recognize the stellar surface */

/* this program contains:

   loop over the grid vertices
   |  calculate the stellar contribution to the vector "B"
   |  loop over the angular integration rays
   |  |  loop over the optical depth integral steps along the ray 
   |  |  |  loop over the dust types
   |  |  |  |  find the interpolation points
   |  |  |  |  loop over the interpolation points
   |  |  |  |  |  calculate the matrix "A" element and vector "B" element
   |  |  |  |  end
   |  |  |  end
   |  |  end
   |  end
   end
   solve A*u=B by matrix inversion
*/

#include "VILOZMAJ.h"

/* BRISI OVO:*/
FILE *dump;
char filename[255];
/*************/

/* subroutine for calculation of the 3D liner interpolation coefficients */
void _VILOZMAJ_Radiative_Transfer_Solver_Coefficient 
       (double x1, double y1, double x2, double y2, double x3, double y3, 
        double x, double y, double *c1, double *c2, double *c3);

void _VILOZMAJ_Radiative_Transfer_Solver(int Lambda_List_Total,int *Lambda_List){
  /* we are solving several wavelengths at the same time. The indexes of
     these wavelengths are in Lambda_List[]. The counter over these 
     wavelengths is going to be "p", so that the matrix A for a wavelength 
     "p" is A[p][][] */
  int p,m,n; /* indices in the matrix A[p][m][n] */

#if (LEL_RADTRANS_N && LEL_RADTRANS_L)
  /* the following is decalred in LELUYA.c:
  double Rad_Trans_Matrix[LEL_RADTRANS_L][LEL_RADTRANS_N][LEL_RADTRANS_N];
  double Rad_Trans_Emission[LEL_RADTRANS_L][LEL_RADTRANS_N]; 
  */
#else 
  double ***Rad_Trans_Matrix;
  double **Rad_Trans_Emission;
#endif

  /*
  double ***Rad_Trans_Matrix;  the radiative transfer matrix "A" which 
                              is used for updating the energy density 
                              at wavelength Lambda_List[p]:
                                               A * U = B
                              where "U" is stored in grid_per_lambda[].Ustar,
                              grid_per_lambda[].Usca, and grid_per_lambda[].Uem
                              "B" is the dust emission stored in
                              Rad_Trans_Emission.
                              NOTE: this matrix covers only vertices which 
                              have Vertex_Status>=0 
  double **Rad_Trans_Emission;  vector "B" in the radiative transfer
                                 equation: A * U = B  */        
  double **matrixA, **matrixA_copy, *vectorB, *vectorB_copy; /* these are
                              matrix A and vector B in the radiative transfer
                              equation A*U=B. We also need a copy of them
                              because of the 'mprove' subroutine. The size of
                              these matrices and vectors is N_matrix */
  int *ludcmp_indx;        /* index permutation vector used in 'ludcmp' subroutine
                              as a part of solving A*U=B */
  double ludcmp_d;            /* parity flag in 'ludcmp' */

  int N_matrix;               /* dimension of the radiative transfer matrix.
                               This is not equal to  TOTAL_NUMBER_OF_VERTICES
                               because some vertices may be just the empty 
                               space or with converged temperatures or fixed
                               temperature */
  int m_matrix,n_matrix;      /* counters over the matrixA (from 1 to N_matrix)*/
  int *Vertex_Status;         /* this list shows if a vertex is EMPTY_SPACE 
                               (Vertex_Status=-1), with  converged temperatures
                               (=-2), with a fixed temperature (=-3), or a normal
                               vertex which needs temperature update (>=0) */

#ifdef _2DD_MPI_
#if (LEL_RADTRANS_N && LEL_RADTRANS_L)
  double vectorA_MPI[LEL_RADTRANS_N*LEL_RADTRANS_N-1];
#else
  double *vectorA_MPI;
#endif
#endif

  struct _vertex_structure   *loop_vertex; /* a grid point */

  struct _3DVector startpoint;/* start point in tau-integrals */
  struct _3DVector endpoint;  /* end point in tau-integrals */
  double *tau;                /* optical depth - calculated by tau-integrals*/
  double *tau_step;           /* integration tau step */
  double *exp_tau;            /* exp(-tau) */
  double sqr_startpoint;      /* =startpoint*startpoint  */
  double distance_step_refinement; /* refinement of the integral step */
  int tau_edge_flag; /* flag which shows if tau_step is crossing a dust edge */
  int tau_limit_flag;         /* if INPUT.TAU_LIMIT>0.0 then this flag shows if 
                                 tau>INPUT.TAU_LIMIT */

  int err;                    /* error flag */
  int dust_i;                 /* dust types counter */
  int lambda_k;               /* counter over wavelengths */

  double angular_weight; /* weight in the angular integral */
  int ray_counter;       /* counter over the angular grid rays */

  struct _3DVector current_point; /* a point in 3D space during integration */
  struct _3DVector old_current_point; /*...current_point from previous step */
  struct _3DVector current_point_xz;/* ..... cylindrical coordinates */
  double sqr_current_point; /* distance to the origin */
  double sqr_current_point_old;/* distance of previous point in a loop 
                                  to the origin */
  struct _triangle_structure *current_triangle; /* location of current_point */
  struct _triangle_structure *old_current_triangle; /*...from previous step */

  struct _3DVector next_point; /* point after current_point */
  struct _triangle_structure *next_triangle; /* location of next_point */

  double *temperature;   /* dust temperature at current_point */
  double *old_temperature; /* dust temperature at previous current_point */
  double *Eta_old,*Eta_new;/* Eta values at current_point and one point back */
  double **albedo_old,**albedo_new;/* albedo at current_point and point back */
double **upsilon_old,**upsilon_new;/*upsilon at current_point and point back */

  double distance_total; /* distance between startpoint and endpoint */
  double current_distance; /* distance between startpoint and current_point */
  double distance_step; /* integration spatial step */

  int inter_counter; /* counter over the three interpolation points */

  /* interpolation coefficients and indexes*/
  /* scattering part which goes into the matrix 'A' */
  /* old: (previous step in the integral ) */ 
  int interpol_sca_old_index1,interpol_sca_old_index2,interpol_sca_old_index3;
  double interpol_sca_old1,  interpol_sca_old2,  interpol_sca_old3;
  /* new: (current step in the integral ) */ 
  int interpol_sca_new_index1,interpol_sca_new_index2,interpol_sca_new_index3;
  double interpol_sca_new1,  interpol_sca_new2,  interpol_sca_new3;

  /* intermediate values of A and B*/
  /* six values: 3 interpolation points for old point and 3 for new point:
           Rad_Trans_Matrix[m][interpol_sca_old_index1] = A_mn[0]
           Rad_Trans_Matrix[m][interpol_sca_old_index2] = A_mn[1]
           Rad_Trans_Matrix[m][interpol_sca_old_index3] = A_mn[2]
           Rad_Trans_Matrix[m][interpol_sca_new_index1] = A_mn[3]
           Rad_Trans_Matrix[m][interpol_sca_new_index2] = A_mn[4]
           Rad_Trans_Matrix[m][interpol_sca_new_index3] = A_mn[5]
  */
  double A_mn[6], B_m, *B_m_old, *B_m_new;
  /* where:
         B_m_new = SUM_i upsilon_new(i)*(1-albedo_new(i))*|start_point|^2*
	           *(temperature(i)/Dust_Sublim_Temp[0])^4 *
		   *_MATH_LIB_Planck(temperature(i),Lambda_Grid[])
  */

  /* e_{-Tau} integral elements: 
    We use N_tau_substep steps over the e_-Tau function between two points
    on the radiative transfer integral ray. The source function is represented 
    by linear interpolation between these two points. */
  double *Sum_point_old, *Sum_point_new; /* factors that multiply the source 
                                    function at two integral points */
  int N_tau_substep = 4; /* number of optical depth substeps */
  int substep_n;  /* substep counter */
  double substep; /* substep spatial length */ 
  struct _3DVector subpoint1, subpoint2; /* two substep points */ 
  double *exp_tau_substep; /* list of exp(-tau) elements in the sum */
  int interpolate; /* flag which shows if ETA needs to be interpolated or not */
  double Eta1,Eta2; /* ETA value at two adjacent subpoints */

  int Is_this_empty_space; /* true/false: is a vertex in empty space or not? */
  int Is_temperature_error_small; /* true/false: is the error small enough to
                                     skip this point? (comes with the -t/T flag)*/
  struct _vertex_triangles *vertex_triangle; /* used for checking if the first 
                                    neighbors have their temperature error over
                                    the limit (comes with the -t/T flag)*/

  int Is_vertex_fixed; /* true/flase: is this vertex fixed and does not 
			  participate in the rad. transfer? */
 

  double dummy,dummy1;

#ifdef _2DD_MPI_
  int tag = 0;
  int receive_flag;
  MPI_Status Receive_Status, Status;
  double *Copy_Data;  /* used for the transfer of data from various processes to
                   the process 0 (length: from 0 to TOTAL_NUMBER_OF_VERTICES-1 */
  int job_m;  /* shows the vertex ID sent to the zero-process during the data 
                 transfer */
  int job_ID; /* shows the process ID required by the zero-process to start 
                 receiving the data */
  int *lookup_table; /* a table that shows which process is calculating 
                 which point */
  struct _lambda_job_struct *current_job; /* a lambda-job with matrix inversion */
#endif

  temperature = dvector(0,Max_Dust_Type-1);  
  old_temperature = dvector(0,Max_Dust_Type-1);  
  albedo_old = dmatrix(0,Lambda_List_Total-1,0,Max_Dust_Type-1); 
  albedo_new = dmatrix(0,Lambda_List_Total-1,0,Max_Dust_Type-1); 
  upsilon_old = dmatrix(0,Lambda_List_Total-1,0,Max_Dust_Type-1);
  upsilon_new = dmatrix(0,Lambda_List_Total-1,0,Max_Dust_Type-1);

  tau      = dvector (0,Lambda_List_Total-1);
  tau_step = dvector (0,Lambda_List_Total-1);
  exp_tau =  dvector (0,Lambda_List_Total-1);
  Eta_old = dvector (0,Lambda_List_Total-1);
  Eta_new = dvector (0,Lambda_List_Total-1);
  B_m_old = dvector (0,Lambda_List_Total-1);
  B_m_new = dvector (0,Lambda_List_Total-1);

  /* set the global grid (necessary for some subroutines) */
  TOTAL_NUMBER_OF_VERTICES = 
    grid_per_lambda[Lambda_List[0]].TOTAL_NUMBER_OF_VERTICES;
  TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES =
    grid_per_lambda[Lambda_List[0]].TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;
  start_vertex_list = grid_per_lambda[Lambda_List[0]].start_vertex;
  TOTAL_NUMBER_OF_TRIANGLES = 
    grid_per_lambda[Lambda_List[0]].TOTAL_NUMBER_OF_TRIANGLES;
  start_triangle_list = grid_per_lambda[Lambda_List[0]].start_triangle;

  /* create 3D cloud of points (used for calculating integral directions)*/
  err = _GOROSTAS_create_3D_cloud ();

  /* create the list of vertex status flags */
  Vertex_Status=ivector(0,TOTAL_NUMBER_OF_VERTICES-1);

  /* find how many vertices need the temperature update */
  N_matrix=0;
  loop_vertex = start_vertex_list;
  for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++){
    Vertex_Status[m]=N_matrix;
    /* check of the empty space vartices */
    Is_this_empty_space = TRUE;
    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
          loop_vertex->T_dust[dust_i]>0.0)
        Is_this_empty_space = FALSE;
    if (Is_this_empty_space) Vertex_Status[m]=-1;
    else {
      /* check for small temperature errors */
      Is_temperature_error_small = FALSE;
      if (INPUT.SKIP_POINT!=0)
        if (INPUT.SKIP_POINT==2 ||
            (INPUT.SKIP_POINT==1 && GLOBAL_ITERATION_COUNTER>0)) {
        Is_temperature_error_small = TRUE;
        for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
          if (fabs(loop_vertex->T_error[dust_i]) > T_error_tolerance &&
              INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
              loop_vertex->T_dust[dust_i]>0.0)
            Is_temperature_error_small = FALSE;
	/* check neightbours */
	if (Is_temperature_error_small && T_error_neighbors==1) {
	  vertex_triangle = loop_vertex->triangles;
	  while (vertex_triangle!=NULL && Is_temperature_error_small) {
            for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
  if ( (fabs(vertex_triangle->triangle->v1->T_error[dust_i])>T_error_tolerance &&
             vertex_triangle->triangle->v1->T_dust[dust_i]>0.0) ||
       (fabs(vertex_triangle->triangle->v2->T_error[dust_i])>T_error_tolerance &&
             vertex_triangle->triangle->v2->T_dust[dust_i]>0.0) ||
       (fabs(vertex_triangle->triangle->v3->T_error[dust_i])>T_error_tolerance &&
             vertex_triangle->triangle->v3->T_dust[dust_i]>0.0) )
                Is_temperature_error_small = FALSE;
	    vertex_triangle = vertex_triangle->next;
	  }
	}
      } /* end of checking for small temperature errors */
      if ( Is_temperature_error_small ) Vertex_Status[m]=-2;
      else if (TOTAL_NUMBER_OF_GRID_BINS>1 && Bin_Type==1){
	/* check for temperature bins */ 
	Is_vertex_fixed = TRUE;
	for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	  if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
	      loop_vertex->T_dust[dust_i]>
                  Temperature_Bins[GLOBAL_BIN_COUNTER+1]
	      && loop_vertex->T_dust[dust_i]<=
                  Temperature_Bins[GLOBAL_BIN_COUNTER])
	    {Is_vertex_fixed=FALSE; break;}
	if (Is_vertex_fixed)  Vertex_Status[m]=-3;
      }
      else if (TOTAL_NUMBER_OF_GRID_BINS>1 && Bin_Type==2){
	/* check for radial bins */ 
	Is_vertex_fixed = TRUE;
        dummy = sqrt(loop_vertex->x*loop_vertex->x + 
                     loop_vertex->z*loop_vertex->z);
	for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	  if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999 &&
	      loop_vertex->T_dust[dust_i] > 0.0 &&
              dummy > Radial_Bins[GLOBAL_BIN_COUNTER] &&
              dummy <= Radial_Bins[GLOBAL_BIN_COUNTER+1])
	    {Is_vertex_fixed=FALSE; break;}
	if (Is_vertex_fixed)  Vertex_Status[m]=-3;
      }
      if (Vertex_Status[m]>=0 && 
          ((INPUT.SKIP_POINT_R==3 && GLOBAL_ITERATION_COUNTER>0) || 
           INPUT.SKIP_POINT_R==4)) {
	  /* check for radial boundaries */
          dummy = sqrt(loop_vertex->x*loop_vertex->x + 
                     loop_vertex->z*loop_vertex->z);
          if (dummy<R_allowed_range_low || dummy>R_allowed_range_high)
            Vertex_Status[m]=-4;
      }
      if (Vertex_Status[m]>=0 && 
           ((INPUT.SKIP_POINT_T==5 && GLOBAL_ITERATION_COUNTER>0) || 
           INPUT.SKIP_POINT_T==6)) {
	  /* check for temperature boundaries */
	  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
            if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) {
	      if (loop_vertex->T_dust[dust_i]<T_allowed_range_low ||
                  loop_vertex->T_dust[dust_i]>T_allowed_range_high)
                 Is_vertex_fixed = TRUE;
              else {Is_vertex_fixed = FALSE;break;}
            }
          if (Is_vertex_fixed) Vertex_Status[m]=-4;
      }
    }

    if (Vertex_Status[m]>=0) N_matrix++;
    loop_vertex=loop_vertex->next;
  }
  if (my_MPI_rank==0) printf("TOTAL NUMBER OF POINTS FOR RAD.TRANS.=%d\n",
                              N_matrix);

#if (LEL_RADTRANS_N && LEL_RADTRANS_L)
  if(N_matrix>LEL_RADTRANS_N)
    printf("%d: ERROR!!! N_matrix>LEL_RADTRANS_N: %d>%d\n",
           my_MPI_rank,N_matrix,LEL_RADTRANS_N);
  if(Lambda_List_Total>LEL_RADTRANS_L)
    printf("%d: ERROR!!! Lambda_List_Total>LEL_RADTRANS_L: %d>%d\n",
           my_MPI_rank,Lambda_List_Total,LEL_RADTRANS_L);
#else
  /* create the radiative transfer matrix and vector */
  Rad_Trans_Matrix=d3tensor(0,Lambda_List_Total-1,0,N_matrix-1,0,N_matrix-1);
  Rad_Trans_Emission = dmatrix(0,Lambda_List_Total-1,0,N_matrix-1);
#endif

  /* Initialize the matrix and emission vector */
  for (p=0;p<Lambda_List_Total;p++)
  for (m=0;m<N_matrix;m++){
    Rad_Trans_Emission[p][m]=0.0;
    for (n=0;n<N_matrix;n++) Rad_Trans_Matrix[p][m][n]=0.0;
  }

  /* list of optical depth substeps */
  exp_tau_substep = dvector (0,N_tau_substep);
  Sum_point_old = dvector (0,Lambda_List_Total-1);
  Sum_point_new = dvector (0,Lambda_List_Total-1);

  if (my_MPI_rank==0) {
    for (p=0;p<Lambda_List_Total;p++)
      printf("%lf mic., ",Lambda_Grid[Lambda_List[p]]);
    printf("\n");
  /* printf("\nprocess%.3d: number of grid points=%d\n____\n",my_MPI_rank,
     TOTAL_NUMBER_OF_VERTICES); */
  }

#ifdef _2DD_MPI_

  /*printf("_______process%.3d:Radiative Transfer Solver (job#%d)\n",my_MPI_rank,
            current_lambda_job);*/
  fflush(stdout);

  /* create the lookup table of vertices ready for the radiative transfer */
  if (my_MPI_rank == 0) {
    lookup_table = ivector(0,TOTAL_NUMBER_OF_VERTICES-1);
    for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++) {
      if (Vertex_Status[m]>=0) lookup_table[m]=-1;
      else lookup_table[m]=total_MPI_process; 
    }
  }

  /* send the inital jobs to the non-zero processes */
  if (my_MPI_rank == 0)
    for (n=1; n<total_MPI_process; n++) 
      if (_MPI_Process_Available[n]){
        for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++) 
	  if (lookup_table[m]<0) {
	    MPI_Send(&m,1,MPI_INT,n,tag,MPI_communicator);
	    lookup_table[m]=n;
	    break;
	  }
	/* in case of less matrix lines than processors: */ 
        if (m==TOTAL_NUMBER_OF_VERTICES) {
	  m=-1;
          MPI_Send(&m,1,MPI_INT,n,tag,MPI_communicator);
	}
      }

  /* if this is the zero-process  then receive the data from the other
     processes and send them a new job */
  if (my_MPI_rank == 0) {
    Copy_Data = dvector(0,N_matrix-1);
    m=0;
    while (m<N_matrix) {
      MPI_Recv(&job_m,1,MPI_INT,MPI_ANY_SOURCE,tag,MPI_communicator,
               &Receive_Status);
      job_ID = Receive_Status.MPI_SOURCE;
      if (job_m>=0) {
	/* this is a signal from a processor sending a matrix line */
        /* receive the data */
        for (p=0;p<Lambda_List_Total;p++) {
          MPI_Recv(Copy_Data,N_matrix,MPI_DOUBLE,job_ID,
                   tag,MPI_communicator,&Status);
          for (n=0;n<N_matrix;n++)
            Rad_Trans_Matrix[p][Vertex_Status[job_m]][n] = Copy_Data[n];
          MPI_Recv(&dummy,1,MPI_DOUBLE,job_ID,tag,MPI_communicator,&Status);
          Rad_Trans_Emission[p][Vertex_Status[job_m]] = dummy;
          MPI_Recv(&dummy,1,MPI_DOUBLE,job_ID,tag,MPI_communicator,&Status);
          grid_per_lambda[Lambda_List[p]].Uem[job_m] = dummy;
          MPI_Recv(&dummy,1,MPI_DOUBLE,job_ID,tag,MPI_communicator,&Status);
          grid_per_lambda[Lambda_List[p]].Usca[job_m] = dummy;
          MPI_Recv(&dummy,1,MPI_DOUBLE,job_ID,tag,MPI_communicator,&Status);
          grid_per_lambda[Lambda_List[p]].Ustar[job_m] = dummy;
        }
        /* one point more is solved: */
        m++;
      }
      else {
	/* this is a signal from a processor doing the matrix inversion */
	p = -job_m-1;
        /* we need the lambda_job ID number to locate the wavelengths */
        MPI_Recv(&receive_flag,1,MPI_INT,job_ID,tag,MPI_communicator,&Status);
	/* locate the lambda-job to receive the data */
	current_job = lambda_jobs;
	while(current_job!=NULL && current_job->ID!=receive_flag)
	  current_job = current_job->next;
	/* receive the data for the wavelength p*/ 
	lambda_k = current_job->run[p];
            MPI_Recv(grid_per_lambda[lambda_k].Utot_new,
		     grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES,
	             MPI_DOUBLE,job_ID,tag,MPI_communicator,&Status);
	/* mark this process as availabe */
	_MPI_Process_Available[job_ID]=TRUE;
	/* we are closer to solving this lambda-job */
	current_job->solved += 1;
	/* mark this lambda_job as solved */
	if (current_job->solved==0) current_job->solved=TRUE;
	/* send the current lambda-job */
	MPI_Send(&current_lambda_job,1,MPI_INT,job_ID,tag,MPI_communicator);
      }

      /* find the next job for the 'job_ID' process */
      for (n=0;n<TOTAL_NUMBER_OF_VERTICES;n++)
        if (lookup_table[n]<0) break;
      if (n<TOTAL_NUMBER_OF_VERTICES) {
        MPI_Send(&n,1,MPI_INT,job_ID,tag,MPI_communicator);
        lookup_table[n]=job_ID;
      }
      else {n=-1;MPI_Send(&n,1,MPI_INT,job_ID,tag,MPI_communicator);}

    } /* end of: while (m<N_matrix) */
    free_dvector(Copy_Data,0,N_matrix-1);
  } /* end of the zero-process */
  else {

    /* a non-zero process */
    /* receive a job to start with */
    MPI_Recv(&m,1,MPI_INT,0,tag,MPI_communicator,&Status);

    /* calculate the radiative transfer for this point 'm' */
    while (m>=0) {

      /* find this verex in the grid */
      loop_vertex = grid_per_lambda[Lambda_List[0]].start_vertex;
      for (n=0;n<m;n++) loop_vertex = loop_vertex->next;
#else
  /* loop over the grid points */
  loop_vertex = grid_per_lambda[Lambda_List[0]].start_vertex;
  for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++) {
#endif

    if (Vertex_Status[m]>=0) {
      
       printf("process%.3d:----- m=%d matrix_line=%d [%E,%E]\n",
	 my_MPI_rank,m,Vertex_Status[m],loop_vertex->x,loop_vertex->z);
       fflush(stdout);

    /* the point loop_vertex */
    startpoint.x=loop_vertex->x; startpoint.y=0.0; startpoint.z=loop_vertex->z;
    sqr_startpoint=startpoint.x*startpoint.x+startpoint.z*startpoint.z;

    /* set A[m][m]=1 */
    for (p=0;p<Lambda_List_Total;p++) 
      Rad_Trans_Matrix[p][Vertex_Status[m]][Vertex_Status[m]]= 1.0;

    /* set the energy densities to zero */
    for (p=0;p<Lambda_List_Total;p++) {
      grid_per_lambda[Lambda_List[p]].Ustar[m]=0.0;
      grid_per_lambda[Lambda_List[p]].Usca [m]=0.0;
      grid_per_lambda[Lambda_List[p]].Uem  [m]=0.0;
    }

    /* create the angular integral rays */
    _GOROSTAS_create_integral_directions(loop_vertex); 

    printf("....... # of rays: %d\n",total_number_of_spherical_triangles);

    /* loop over the angular integral */
    for (ray_counter=0;ray_counter<total_number_of_spherical_triangles;
         ray_counter++) {

      /* printf("ray %d of %d\n",ray_counter,total_number_of_spherical_triangles);*/

      /* factor 2 comes from the axial symmetry */
      angular_weight = integral_directions[ray_counter].weight * 2.0; 
      angular_weight = angular_weight / (4.0*PI);

      /* correction ofr numerical erros when the ray is exactly in the XY plane */
      if (integral_directions[ray_counter].direction.z==0.0)
        integral_directions[ray_counter].direction.z = 1E-12;

      /* find the end point for the tau integral (a point on the Rout sphere)*/
      /* (recall that startpoint.y=0) */
      distance_total=sqrt(sqr_startpoint);
      dummy = startpoint.x*integral_directions[ray_counter].direction.x+
	    startpoint.z*integral_directions[ray_counter].direction.z;
      dummy = - dummy/distance_total;
      dummy1 = distance_total* distance_total * (1.0 - dummy*dummy);
      if (dummy >= 0.0 && Source_Radius*Source_Radius > dummy1) {
        /******* the ray goes through the source *****/
	distance_total=distance_total*dummy - 
                                    sqrt(Source_Radius*Source_Radius - dummy1);
      }
      else {
        /******* the ray does not go through the source ********/
	distance_total=distance_total*dummy+sqrt(Rout*Rout-dummy1);
      }

      endpoint.x=startpoint.x +
                 distance_total*integral_directions[ray_counter].direction.x;
      endpoint.y=distance_total*integral_directions[ray_counter].direction.y;
      endpoint.z=startpoint.z +
                 distance_total*integral_directions[ray_counter].direction.z;

      /* set the initial values for integration variables */
      for (p=0;p<Lambda_List_Total;p++) {tau[p]=0.0; exp_tau[p]=1.0;}
      current_distance = 0.0;
      current_triangle=loop_vertex->triangles->triangle;

      current_point = startpoint;

      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++){
	temperature[dust_i]=loop_vertex->T_dust[dust_i];
	for (p=0;p<Lambda_List_Total;p++) {
	  if (temperature[dust_i]>0.0) 
            albedo_old[p][dust_i]=albedo(current_point.x,current_point.z,
				         Lambda_List[p],dust_i,&err);
	  else albedo_old[p][dust_i]=0.0; 
	}
      }
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	old_temperature[dust_i]=temperature[dust_i];

      for (p=0;p<Lambda_List_Total;p++)
	Eta_old[p]=Eta(current_point.x,current_point.z,Lambda_List[p],
		       temperature,&err);

      sqr_current_point = current_point.x*current_point.x +
	                  current_point.z*current_point.z;
      sqr_current_point_old = sqr_current_point;

      for (p=0;p<Lambda_List_Total;p++) {
	B_m_old[p] = 0.0;
        for (dust_i=0; dust_i<Max_Dust_Type; dust_i++){
	  upsilon_old[p][dust_i]=upsilon(current_point.x,current_point.z,
				       Lambda_List[p],dust_i,temperature,&err);
	  if (temperature[dust_i]>0.0) {
	    dummy = temperature[dust_i]/Dust_Sublim_Temp[0];
	    dummy *= dummy;dummy *= dummy;
	    B_m_old[p]=B_m_old[p] + dummy*upsilon_old[p][dust_i]*
                       (1.0-albedo_old[p][dust_i])*
	    _MATH_LIB_Planck(temperature[dust_i],Lambda_Grid[Lambda_List[p]]);
	  }
	}
      }

      interpol_sca_old_index1=loop_vertex->identification_number;
      interpol_sca_old_index2 = -1;
      interpol_sca_old_index3 = -1;
      interpol_sca_old1=1.0; interpol_sca_old2=0.0; interpol_sca_old3=0.0;
      distance_step_refinement=1.0;
      next_triangle = NULL;

      /* loop over optical depth distance from start_point */
      while (current_distance<distance_total){

	old_current_point = current_point;
	old_current_triangle = current_triangle;
        /* memorize the old point */
        subpoint1 = current_point;

	do { /* this do-loop will check the tau step over the dust edge */
 
	/* find the next tau-intergal point */ 
	distance_step = _ZELENJAK_3Dstep (current_point,current_triangle,
		  integral_directions[ray_counter].direction,
                  distance_step_refinement,
                  &next_point,&next_triangle);

	/* check if the step is too big */
        if (distance_step_refinement < 1.0 && 
            current_triangle!=next_triangle) {
           distance_step_refinement=1.0;
           /* search again for the distance step */
           distance_step = _ZELENJAK_3Dstep (current_point,current_triangle,
		  integral_directions[ray_counter].direction,
                  distance_step_refinement,
                  &next_point,&next_triangle);
	}

        /* check if we reached the end_point */
        if (current_distance + distance_step > distance_total) {
          distance_step = distance_total - current_distance;
          current_distance = distance_total;
          current_point = endpoint;
          current_point_xz.x=_MATH_LIB_distance(0.0,0.0,current_point.x,
		                               current_point.y);
          current_point_xz.y = 0.0;
          if (current_point.z >= 0.0) current_point_xz.z = current_point.z;
          else current_point_xz.z = - current_point.z;
          if (old_current_triangle==NULL && next_triangle==NULL)
            current_triangle=NULL; /* this prevents a dead loop */
          else
            err=_ZELENJAK_locate_point_in_grid(current_point_xz.x,current_point_xz.z,
					&current_triangle);
          
        }
	else {
 	  current_distance += distance_step;
	  current_point = next_point;
	  current_triangle = next_triangle;
	  current_point_xz.x=_MATH_LIB_distance(0.0,0.0,current_point.x,
					      current_point.y);
	  current_point_xz.y = 0.0;
	  if (current_point.z >= 0.0) current_point_xz.z = current_point.z;
	  else current_point_xz.z = - current_point.z;
	}

	sqr_current_point = current_point_xz.x*current_point_xz.x +
	                    current_point_xz.z*current_point_xz.z;

	/* find the temperature and albedo at current_point */
	if (current_triangle==NULL)
            for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
	      temperature[dust_i]=-1.0;
	      for (p=0;p<Lambda_List_Total;p++) albedo_new[p][dust_i]=0.0;
	    }
	else 
          if (Vertex_Status[current_triangle->v1->identification_number]==-1 ||
	      Vertex_Status[current_triangle->v2->identification_number]==-1 ||
	      Vertex_Status[current_triangle->v3->identification_number]==-1)
	    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
	      temperature[dust_i]=-1.0;
	      for (p=0;p<Lambda_List_Total;p++) albedo_new[p][dust_i]=0.0;
	    }
	else 
	  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
  	    if (current_triangle->v1->T_dust[dust_i]>0.0 && 
	        current_triangle->v2->T_dust[dust_i]>0.0 &&
	        current_triangle->v3->T_dust[dust_i]>0.0){
	      temperature[dust_i]= _MATH_LIB_plane_interpolation 
                            (current_triangle->v1->x,current_triangle->v1->z,
                             current_triangle->v1->T_dust[dust_i],
		             current_triangle->v2->x,current_triangle->v2->z,
                             current_triangle->v2->T_dust[dust_i],
			     current_triangle->v3->x,current_triangle->v3->z,
                             current_triangle->v3->T_dust[dust_i],
			     current_point_xz.x,current_point_xz.z);
	      if (temperature[dust_i]==0.0) temperature[dust_i]=-1.0;
	      for (p=0;p<Lambda_List_Total;p++){
		if (temperature[dust_i]>0.0) 
		  albedo_new[p][dust_i]=albedo(current_point_xz.x,
			        current_point_xz.z,Lambda_List[p],dust_i,&err);
		else albedo_new[p][dust_i]=0.0;
	      }
	    }
	    else {
	      temperature[dust_i]=-1.0;
	      for (p=0;p<Lambda_List_Total;p++) albedo_new[p][dust_i]=0.0;
	    }
	  }

	/* check if we are crossing the dust edge */
	tau_edge_flag=FALSE;
	for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
          if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
	  if (temperature[dust_i]*old_temperature[dust_i]<0.0)
	    tau_edge_flag=TRUE;

        /* find new Eta */
        for (p=0;p<Lambda_List_Total;p++) 
            Eta_new[p]=Eta(current_point_xz.x,current_point_xz.z,
                           Lambda_List[p],temperature,&err);

	/* check if the step has to be smaller */ 
	if(tau_edge_flag)
	  for (p=0;p<Lambda_List_Total;p++) {
	    if ((Eta_old[p]+Eta_new[p])*distance_step*0.5 > 0.01) {
	      current_distance -= distance_step;
	      current_point=old_current_point;
	      current_triangle=old_current_triangle;
	      distance_step_refinement *= 2.0;
	      tau_edge_flag=TRUE;
	      break;
	    }
	    else
              tau_edge_flag=FALSE;
	  }

	} while(tau_edge_flag); /* reduce the step size if needed */ 

/*if (m==0){
if (current_triangle!=NULL)
 printf("Tau=%lf taustep=%13.11lf step=%13.11lf (%13.11lf,%13.11lf) triangle[%d,%d,%d] \n",
  tau[0],tau_step[0],distance_step,
  current_point_xz.x,current_point_xz.z,current_triangle->v1->identification_number,
  current_triangle->v2->identification_number,current_triangle->v3->identification_number);
else printf("Tau=%lf taustep=%13.11lf step=%13.11lf (%13.11lf,%13.11lf) triangle[NULL] \n",
  tau[0],tau_step[0],distance_step,current_point_xz.x,current_point_xz.z);
fflush(stdout);
}*/

	/* find the upsilon factor at current_point */
	for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	  for (p=0;p<Lambda_List_Total;p++)
	    upsilon_new[p][dust_i]=upsilon(current_point_xz.x,
					   current_point_xz.z,
					   Lambda_List[p],dust_i,
					   temperature,&err);

	/* --- find the tau step in the integral --- */
	for (p=0;p<Lambda_List_Total;p++) {
          /* We are using N_tau_substep substeps to calculate tau */
	  if (N_tau_substep<=1) {
	    /* linear optical depth interpolation*/
	    tau_step[p] = (Eta_old[p]+Eta_new[p])*distance_step*0.5;
	    Sum_point_old[p] = 0.5*exp_tau[p]*(1.0-exp(-tau_step[p]));
            Sum_point_new[p] = Sum_point_old[p];
	  }
	  else {
	    /* First, we need to know if each substep will have its own eta, or,
               if not, we will interpolate it. */
            interpolate = FALSE;
	    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) 
	      if ((albedo_new[p][dust_i]>0.0 && albedo_old[p][dust_i]==0.0) ||
                  (albedo_new[p][dust_i]==0.0 && albedo_old[p][dust_i]>0.0))
	        {interpolate = TRUE; break;}
            /* size of a substep */
            substep = distance_step / N_tau_substep;
            exp_tau_substep[0] = exp_tau[p];
            Eta1 = Eta_old[p];
	    /* loop over the substeps */
            tau_step[p]=0.0;
	    for (substep_n=1; substep_n<=N_tau_substep; substep_n++) {
	      if (interpolate) {
                dummy = (Eta_new[p]-Eta_old[p])/distance_step;
                Eta2 = dummy*substep_n*substep + Eta_old[p];
	      }
	      else {
                dummy = substep_n*substep;
	        subpoint2.x = subpoint1.x + 
                              dummy*integral_directions[ray_counter].direction.x;
	        subpoint2.y = subpoint1.y + 
                              dummy*integral_directions[ray_counter].direction.y;
	        subpoint2.z = subpoint1.z + 
                              dummy*integral_directions[ray_counter].direction.z;
	        subpoint2.x=_MATH_LIB_distance(0.0,0.0,subpoint2.x,subpoint2.y);
	        if (subpoint2.z < 0.0) subpoint2.z = - subpoint2.z;
                Eta2=Eta(subpoint2.x,subpoint2.z,Lambda_List[p],temperature,&err);
	      }
              tau_step[p] += ((Eta1+Eta2)*0.5*substep);
	      exp_tau_substep[substep_n] = exp_tau_substep[0]*exp(-tau_step[p]);
	      Eta1 = Eta2; 
	    }
	    /* Now, we have exp(optical depth) at each subpoint.
               Calculate Sum_point_new and Sum_point_old */
	    Sum_point_old[p] = Sum_point_new[p] = 0.0;
	    for (substep_n=0; substep_n<N_tau_substep; substep_n++) {
	      Eta1 = exp_tau_substep[substep_n] + exp_tau_substep[substep_n+1];
	      Eta2 = exp_tau_substep[substep_n] - exp_tau_substep[substep_n+1];
	      Sum_point_old[p] = Sum_point_old[p] + 
		              (Eta1 - 2.0*exp_tau_substep[N_tau_substep])*Eta2;
	      Sum_point_new[p] = Sum_point_new[p] + 
		              (2.0*exp_tau_substep[0]-Eta1)*Eta2;
	    }
	    dummy = 2.0*(exp_tau_substep[0] - exp_tau_substep[N_tau_substep]);
	    if (fabs(dummy)>1.0E-30) {
	      Sum_point_old[p] /= dummy;
	      Sum_point_new[p] /= dummy;
	    }
	  } /* end of substep tau integration */ 
	}/* end of tau_step calculation */

	/* find the energy density interpolation coefficients */
	interpol_sca_new_index1 = -1;
	interpol_sca_new_index2 = -1;
	interpol_sca_new_index3 = -1;
	interpol_sca_new1=0.0; interpol_sca_new2=0.0; interpol_sca_new3=0.0;
	if (current_triangle!=NULL){
	  interpol_sca_new_index1=current_triangle->v1->identification_number;
	  interpol_sca_new_index2=current_triangle->v2->identification_number;
	  interpol_sca_new_index3=current_triangle->v3->identification_number;
	  if (Vertex_Status[interpol_sca_new_index1]==-1 ||
              Vertex_Status[interpol_sca_new_index2]==-1 ||
              Vertex_Status[interpol_sca_new_index3]==-1) {
	    interpol_sca_new_index1 = -1;
	    interpol_sca_new_index2 = -1;
	    interpol_sca_new_index3 = -1;
	  }
	  else 
	    _VILOZMAJ_Radiative_Transfer_Solver_Coefficient 
	           (current_triangle->v1->x, current_triangle->v1->z,
	            current_triangle->v2->x, current_triangle->v2->z,
	            current_triangle->v3->x, current_triangle->v3->z,
	            current_point_xz.x,current_point_xz.z,
	            &interpol_sca_new1,&interpol_sca_new2,&interpol_sca_new3);
	}


	for (p=0;p<Lambda_List_Total;p++) {
	  /* set the intermediate values to zero */
	  B_m = 0.0; 
	  for (inter_counter=0; inter_counter<6; inter_counter++)
	    A_mn[inter_counter]=0.0;

	  /* loops over the dust types */ 
	  if (interpol_sca_old_index1>=0 || interpol_sca_old_index2>=0 ||
	      interpol_sca_old_index3>=0) 
	    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) 
		A_mn[2] += (upsilon_old[p][dust_i]*albedo_old[p][dust_i]);
	   
 A_mn[0]=Sum_point_old[p]*interpol_sca_old1*A_mn[2]/sqr_current_point_old;
 A_mn[1]=Sum_point_old[p]*interpol_sca_old2*A_mn[2]/sqr_current_point_old;
 A_mn[2]=Sum_point_old[p]*interpol_sca_old3*A_mn[2]/sqr_current_point_old;

	  if (interpol_sca_new_index1>=0 || interpol_sca_new_index2>=0 ||
	      interpol_sca_new_index3>=0) 
	    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	      if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999) 
		A_mn[5] += (upsilon_new[p][dust_i]*albedo_new[p][dust_i]);
	    
 A_mn[3]=Sum_point_new[p]*interpol_sca_new1*A_mn[5]/sqr_current_point;
 A_mn[4]=Sum_point_new[p]*interpol_sca_new2*A_mn[5]/sqr_current_point;
 A_mn[5]=Sum_point_new[p]*interpol_sca_new3*A_mn[5]/sqr_current_point;

	  B_m_new[p]=0.0;
	  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	    if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
	    if (temperature[dust_i]>0.0) {
	      dummy = temperature[dust_i]/Dust_Sublim_Temp[0];
	      dummy *= dummy;dummy *= dummy;
	      B_m_new[p]=B_m_new[p] + dummy*upsilon_new[p][dust_i]*
		        (1.0-albedo_new[p][dust_i])*_MATH_LIB_Planck(
		         temperature[dust_i],Lambda_Grid[Lambda_List[p]]);
	    }
	  B_m=B_m_old[p]*Sum_point_old[p] + B_m_new[p]*Sum_point_new[p];
	  /* end of the loops over the dust types */

	  /* add the calculated values into the matrix A and vector B 
             and update Usca and Uem */ 
	  dummy = - sqr_startpoint*angular_weight;

	  if (interpol_sca_old_index1>=0) {
	    dummy1 = dummy*A_mn[0]*
              grid_per_lambda[Lambda_List[p]].Utot_new[interpol_sca_old_index1];
	    if (Vertex_Status[interpol_sca_old_index1]>=0) {
Rad_Trans_Matrix[p][Vertex_Status[m]][Vertex_Status[interpol_sca_old_index1]] 
 += (dummy*A_mn[0]);
	    }
	    else if (Vertex_Status[interpol_sca_old_index1]!=-1)
		Rad_Trans_Emission[p][Vertex_Status[m]] -= dummy1;
	    if ( Vertex_Status[interpol_sca_old_index1]!=-1)
	      grid_per_lambda[Lambda_List[p]].Usca[m] -= dummy1;
	  }

	  if (interpol_sca_old_index2>=0) {
	    dummy1 = dummy*A_mn[1]*
              grid_per_lambda[Lambda_List[p]].Utot_new[interpol_sca_old_index2];
	    if (Vertex_Status[interpol_sca_old_index2]>=0){
Rad_Trans_Matrix[p][Vertex_Status[m]][Vertex_Status[interpol_sca_old_index2]] 
 += (dummy*A_mn[1]);
	    }
	    else if (Vertex_Status[interpol_sca_old_index2]!=-1)
	      Rad_Trans_Emission[p][Vertex_Status[m]] -= dummy1; 
	    if ( Vertex_Status[interpol_sca_old_index2]!=-1)
	      grid_per_lambda[Lambda_List[p]].Usca[m] -= dummy1;
	  }

	  if (interpol_sca_old_index3>=0) {
	    dummy1 = dummy*A_mn[2]*
              grid_per_lambda[Lambda_List[p]].Utot_new[interpol_sca_old_index3];
	    if (Vertex_Status[interpol_sca_old_index3]>=0){
Rad_Trans_Matrix[p][Vertex_Status[m]][Vertex_Status[interpol_sca_old_index3]] 
 += (dummy*A_mn[2]);
	    }
	    else if (Vertex_Status[interpol_sca_old_index3]!=-1){
	      Rad_Trans_Emission[p][Vertex_Status[m]] -= dummy1;
}
	    if ( Vertex_Status[interpol_sca_old_index3]!=-1)
	      grid_per_lambda[Lambda_List[p]].Usca[m] -= dummy1;
	  }

	  if (interpol_sca_new_index1>=0) {
	    dummy1 = dummy*A_mn[3]*
              grid_per_lambda[Lambda_List[p]].Utot_new[interpol_sca_new_index1];
	    if (Vertex_Status[interpol_sca_new_index1]>=0){
Rad_Trans_Matrix[p][Vertex_Status[m]][Vertex_Status[interpol_sca_new_index1]] 
 += (dummy*A_mn[3]);
	    }
	    else if (Vertex_Status[interpol_sca_new_index1]!=-1)
	      Rad_Trans_Emission[p][Vertex_Status[m]] -= dummy1;
	    if ( Vertex_Status[interpol_sca_new_index1]!=-1)
	      grid_per_lambda[Lambda_List[p]].Usca[m] -= dummy1;
	  }

	  if (interpol_sca_new_index2>=0) {
	    dummy1 = dummy*A_mn[4]*
              grid_per_lambda[Lambda_List[p]].Utot_new[interpol_sca_new_index2];
	    if (Vertex_Status[interpol_sca_new_index2]>=0){
Rad_Trans_Matrix[p][Vertex_Status[m]][Vertex_Status[interpol_sca_new_index2]] 
 += (dummy*A_mn[4]);
	    }
	    else if (Vertex_Status[interpol_sca_new_index2]!=-1)
	      Rad_Trans_Emission[p][Vertex_Status[m]] -= dummy1;
	    if ( Vertex_Status[interpol_sca_new_index2]!=-1)
	      grid_per_lambda[Lambda_List[p]].Usca[m] -= dummy1;
	  }

	  if (interpol_sca_new_index3>=0) {
	    dummy1 = dummy*A_mn[5]*
              grid_per_lambda[Lambda_List[p]].Utot_new[interpol_sca_new_index3];
	    if (Vertex_Status[interpol_sca_new_index3]>=0){
Rad_Trans_Matrix[p][Vertex_Status[m]][Vertex_Status[interpol_sca_new_index3]] 
 += (dummy*A_mn[5]);
	    }
	    else if (Vertex_Status[interpol_sca_new_index3]!=-1)
	      Rad_Trans_Emission[p][Vertex_Status[m]] -= dummy1;
	    if ( Vertex_Status[interpol_sca_new_index3]!=-1)
	      grid_per_lambda[Lambda_List[p]].Usca[m] -= dummy1;
	  }

          dummy = - Psi * dummy;          
	  (grid_per_lambda[Lambda_List[p]].Uem[m])  += (dummy*B_m);
	  Rad_Trans_Emission[p][Vertex_Status[m]] += (dummy*B_m); 

	}/* end of the loop over wavelengths Lambda_List[p] */


	/* prepare the variables for the next integration step */ 
        if (old_current_triangle == current_triangle && 
            current_triangle!=NULL) {
           distance_step_refinement /= 2.0;
	}
	else distance_step_refinement=1.0;

	tau_limit_flag = TRUE;
	for (p=0;p<Lambda_List_Total;p++) {
  	  tau[p] += tau_step[p];
	  if (tau[p]<INPUT.TAU_LIMIT) tau_limit_flag = FALSE;
	  exp_tau[p]=exp(-tau[p]);
	  Eta_old[p] = Eta_new[p];
	  for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
	    albedo_old[p][dust_i]=albedo_new[p][dust_i]; 
	    upsilon_old[p][dust_i]=upsilon_new[p][dust_i];
            old_temperature[dust_i]=temperature[dust_i];
	  }
	  B_m_old[p]=B_m_new[p];
	}
	interpol_sca_old1 = interpol_sca_new1;
	interpol_sca_old2 = interpol_sca_new2;
	interpol_sca_old3 = interpol_sca_new3;
	interpol_sca_old_index1=interpol_sca_new_index1;
	interpol_sca_old_index2=interpol_sca_new_index2;
	interpol_sca_old_index3=interpol_sca_new_index3;
	sqr_current_point_old = sqr_current_point;

	/* stop the integral if the optical depth is large */
	if (tau_limit_flag && INPUT.TAU_LIMIT>0.0) 
	  current_distance=distance_total+1.0;

      } /* end of the loop over optical depth */

    } /* end of the loop over the angular integral */


    /* printf("process%.3d: -----[V#=%d-Vstatus=%d] # of intergal rays=%d\n",
       my_MPI_rank,m,Vertex_Status[m],total_number_of_spherical_triangles); */

    /* destroy the angular rays */
    total_number_of_spherical_triangles=0;

    /* add the stellar flux */
    if (INPUT.POINT_SOURCE_LIMIT>=0.0) {

      distance_total=sqrt(sqr_startpoint);

      /* here we assume that there is no dust at 0.1*startpoint/distance_total*/ 
      endpoint.x=0.1*startpoint.x/distance_total;
      endpoint.y=0.0;
      endpoint.z=0.1*startpoint.z/distance_total;
      for (p=0;p<Lambda_List_Total;p++) {
        tau[0] = _ZELENJAK_tau_over_grid (startpoint,endpoint,Lambda_List[p],FALSE);
        grid_per_lambda[Lambda_List[p]].Ustar[m]=
          Finite_Source_Flux_Correction(startpoint.x,startpoint.z,tau[0],
                        Lambda_List[p],start_vertex_list,start_triangle_list)
          * Total_Source_Flux(Lambda_List[p],startpoint.x,startpoint.z,&err);
      }

    } /* end of stellar update */

    /* add the stellar contribution to the emission vector */
    for (p=0;p<Lambda_List_Total;p++){ 
      Rad_Trans_Emission[p][Vertex_Status[m]] += 
          grid_per_lambda[Lambda_List[p]].Ustar[m];

    }

    } /* end of: if (Vertex_Status[m]>=0) */

#ifdef _2DD_MPI_

    /* send the data for the point 'm' to the zero-process */
    MPI_Send(&m,1,MPI_INT,0,tag,MPI_communicator);
    /* ....now, send the data */
    for (p=0;p<Lambda_List_Total;p++) {
      MPI_Send(Rad_Trans_Matrix[p][Vertex_Status[m]],N_matrix,MPI_DOUBLE,
                 0,tag,MPI_communicator);
      MPI_Send(&(Rad_Trans_Emission[p][Vertex_Status[m]]),1,MPI_DOUBLE,
                 0,tag,MPI_communicator);
      MPI_Send(&(grid_per_lambda[Lambda_List[p]].Uem[m]),1,MPI_DOUBLE,
               0,tag,MPI_communicator);
      MPI_Send(&(grid_per_lambda[Lambda_List[p]].Usca[m]),1,MPI_DOUBLE,
               0,tag,MPI_communicator);
      MPI_Send(&(grid_per_lambda[Lambda_List[p]].Ustar[m]),1,MPI_DOUBLE,
               0,tag,MPI_communicator);
    }

    /* receive a new job from the zero-process */
    MPI_Recv(&m,1,MPI_INT,0,tag,MPI_communicator,&Status);

  }  /* end of the m>=0 loop */
  }  /* end of the non-zero process */

  /* free the lookup table */
  if (my_MPI_rank==0) free_ivector(lookup_table,0,TOTAL_NUMBER_OF_VERTICES-1);

#else 
    /* next grid point */ 
    loop_vertex = loop_vertex->next;
  } /* loop over 'm' */
#endif

#ifdef _2DD_MPI_
  /* this is a parallel matrix inversion */

#if (!LEL_RADTRANS_N || !LEL_RADTRANS_L)
  vectorA_MPI = dvector(0,N_matrix*N_matrix-1);
#endif

  if (my_MPI_rank==0) {

    /* check if we need this at all */
    for (p=0;p<Lambda_List_Total;p++) {

      receive_flag=-1;
      /* check if this wavelength does not need the scattering */
      if (Lambda_Grid[Lambda_List[p]]>INPUT.LAMBDA_EMISSION_ONLY &&
          INPUT.LAMBDA_EMISSION_ONLY > 0.0) {
        for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++)
          if (Vertex_Status[m]>=0)
            grid_per_lambda[Lambda_List[p]].Utot_new[m]=
              Rad_Trans_Emission[p][Vertex_Status[m]];
      }
      else {
	receive_flag=1; /* this flag shows that we will do matrix inversion */
      }

      if (receive_flag==1) {

        /* process #0 has to send the data to an available process: find it */
        job_ID=1;
        while (_MPI_Process_Available[job_ID]==FALSE && job_ID<total_MPI_process)
           job_ID++;
        if (job_ID==total_MPI_process){
	  /* receive data from one process to get this process free */
	  /* receive a signal from any process */
	  MPI_Recv(&m,1,MPI_INT,MPI_ANY_SOURCE,tag,MPI_communicator,
                   &Receive_Status);
	  /* this m flag should be -(m+1) where m=wavelength*/
	  m=-m-1;
	  /* we need the lambda_job ID number to locate the wavelengths */
	  MPI_Recv(&job_ID,1,MPI_INT,Receive_Status.MPI_SOURCE,tag,
		   MPI_communicator,&Status);
	  /* locate the job to receive the data */
	  current_job = lambda_jobs;
	  while(current_job!=NULL && current_job->ID!=job_ID)
	    current_job = current_job->next;
	  /* receive the data */ 
	  lambda_k = current_job->run[m];
	  MPI_Recv(grid_per_lambda[lambda_k].Utot_new,
                       grid_per_lambda[lambda_k].TOTAL_NUMBER_OF_VERTICES,
	               MPI_DOUBLE,Receive_Status.MPI_SOURCE,tag,
		       MPI_communicator,&Status);
	  /* mark this process as availabe */
	  _MPI_Process_Available[Receive_Status.MPI_SOURCE]=TRUE;
	  /* mark this lambda_job as partially solved */
	  current_job->solved += 1;
	  /* mark this lambda_job as solved */
	  if (current_job->solved==0) current_job->solved==TRUE;
	  /* send a signal to continue with matrix inversion */
	  job_ID = -1; /* instead of current_lambda_job */ 
	  MPI_Send(&job_ID,1,MPI_INT,Receive_Status.MPI_SOURCE,tag,
		   MPI_communicator);
	  /* we will use this same process for further matrix inversion */ 
	  job_ID = Receive_Status.MPI_SOURCE; 
	}/* end of receiving data from one process */

        /* send a flag to that processes */
        MPI_Send(&receive_flag,1,MPI_INT,job_ID,tag,MPI_communicator);
        /* mark it as "not available" */ 
        _MPI_Process_Available[job_ID]=FALSE;

        /* send the wavelength to the chosen process */
        MPI_Send(&p,1,MPI_INT,job_ID,tag,MPI_communicator);
        /* send the matrix A and vector B */
        printf("process #0 has chosen p=%d and the process #%d for matrix inversion\n",p,job_ID);
        /* prepare the matrix A for transport */ 
        for (m=0;m<N_matrix;m++) for (n=0;n<N_matrix;n++)
	    vectorA_MPI[m*N_matrix+n]=Rad_Trans_Matrix[p][m][n];
        /* send the matrix A */
        inter_counter=N_matrix*N_matrix;
        MPI_Send(vectorA_MPI,inter_counter,MPI_DOUBLE,job_ID,tag,
                 MPI_communicator);
        /* prepare the vector B */
        for (n=0;n<N_matrix;n++)
	  vectorA_MPI[n]=Rad_Trans_Emission[p][n];
        /* send the vector B*/
        inter_counter=N_matrix;
        MPI_Send(vectorA_MPI,inter_counter,MPI_DOUBLE,job_ID,tag,
                 MPI_communicator);
      } /* end of sending the data in case of matrix inversion */ 
      else {
        /* mark this lambda-job as partially solved */ 
        /* (lambda-jobs that undergo matrix inversion are marked 
	   when the zero-process receives the updated Utot_new data) */
        current_job = lambda_jobs;
        while(current_job!=NULL && current_job->ID!=current_lambda_job)
            current_job = current_job->next;
        current_job->solved += 1;
        /* mark this lambda-job as solved */ 
        if ( current_job->solved==0) current_job->solved = TRUE;
      }
    } /* end of the loop over p */
    job_m=-1;
    for (p=1;p<total_MPI_process;p++) 
      if (_MPI_Process_Available[p])
        MPI_Send(&job_m,1,MPI_INT,p,tag,MPI_communicator);

  } /* end: if (my_MPI_rank==0) */ 
  else { /* non-zero processes */
    do { /* repeat this as long as the zero-process wants it */
    /* get a sign from the zero-process */
    MPI_Recv(&job_ID,1,MPI_INT,0,tag,MPI_communicator,&Receive_Status);
    if (job_ID>=0) {
      /* this process has to invert the matrix */
      /* get the wavelength */ 
      MPI_Recv(&p,1,MPI_INT,0,tag,MPI_communicator,&Receive_Status);
      /* receive the matrix A and vector B*/
      inter_counter=N_matrix*N_matrix;
      MPI_Recv(vectorA_MPI,inter_counter,MPI_DOUBLE,0,tag,
		 MPI_communicator,&Receive_Status);
      /* create the rad. trans. matrix out of the received vector */
      for (m=0;m<N_matrix;m++) for (n=0;n<N_matrix;n++)
	  Rad_Trans_Matrix[p][m][n]=vectorA_MPI[m*N_matrix+n];
      inter_counter=N_matrix;
      MPI_Recv(vectorA_MPI,inter_counter,MPI_DOUBLE,0,tag,
		 MPI_communicator,&Receive_Status);
      /* create the emission vector out of the received vector */
      for (n=0;n<N_matrix;n++)
	  Rad_Trans_Emission[p][n]=vectorA_MPI[n];

#endif

      /********** MATRIX INVERSION **************/
    /* solve the radiative transfer equation by inversion od Rad_Trans_Matrix
       and store the result into grid_per_lambda[Lambda_List[p]].Utot_new */

    printf("process#%d: matrix inversion = START\n",my_MPI_rank);

#ifndef _2DD_MPI_
    for (p=0;p<Lambda_List_Total;p++) {
#endif 

    /* check if we need this at all */
    if (Lambda_Grid[Lambda_List[p]]>INPUT.LAMBDA_EMISSION_ONLY &&
        INPUT.LAMBDA_EMISSION_ONLY > 0.0) {
      for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++)
        if (Vertex_Status[m]>=0)
          grid_per_lambda[Lambda_List[p]].Utot_new[m]=
            Rad_Trans_Emission[p][Vertex_Status[m]];
    }
    else {

      /* yes, we need the scattering */
#ifdef _2DD_MPI_
#if (!LEL_RADTRANS_N || !LEL_RADTRANS_L)
  free_dvector(vectorA_MPI,0,N_matrix*N_matrix-1);
#endif
#endif

      matrixA      = dmatrix(1,N_matrix,1,N_matrix);
      matrixA_copy = dmatrix(1,N_matrix,1,N_matrix);
      vectorB      = dvector(1,N_matrix);
      vectorB_copy = dvector(1,N_matrix);
      ludcmp_indx  = ivector(1,N_matrix);

      /* prepare variables for solving the rad. trans. equation */
      for (m=0;m<N_matrix;m++){
        vectorB[m+1]=Rad_Trans_Emission[p][m];
        vectorB_copy[m+1]=Rad_Trans_Emission[p][m];
        for (n=0;n<N_matrix;n++){
	  matrixA[m+1][n+1]=Rad_Trans_Matrix[p][m][n];
	  matrixA_copy[m+1][n+1]=Rad_Trans_Matrix[p][m][n];
        }
      }

      ludcmp(matrixA,N_matrix,ludcmp_indx,&ludcmp_d);
      lubksb(matrixA,N_matrix,ludcmp_indx,vectorB);
      mprove(matrixA_copy,matrixA,N_matrix,ludcmp_indx,
      	   vectorB_copy,vectorB);
      for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++)
        if (Vertex_Status[m]>=0)
          grid_per_lambda[Lambda_List[p]].Utot_new[m]=
                                            vectorB[Vertex_Status[m]+1];

      free_dmatrix(matrixA,1,N_matrix,1,N_matrix);
      free_dmatrix(matrixA_copy,1,N_matrix,1,N_matrix);
      free_dvector(vectorB,1,N_matrix);
      free_dvector(vectorB_copy,1,N_matrix);
      free_ivector(ludcmp_indx,1,N_matrix);

#ifdef _2DD_MPI_
#if (!LEL_RADTRANS_N || !LEL_RADTRANS_L)
  vectorA_MPI = dvector(0,N_matrix*N_matrix-1);
#endif
#endif

    } /* end of if (lambda<INPUT.LAMBDA_EMISSION_ONLY) */
#ifndef _2DD_MPI_
    }
#endif

    printf("process#%d: matrix inversion = FINISHED\n",my_MPI_rank);

#ifdef _2DD_MPI_

      /* send the result to the zero-process */ 
      /* send the signal first:*/
      receive_flag=-(p+1);
      MPI_Send(&receive_flag,1,MPI_INT,0,tag,MPI_communicator);
      /* send the lambda-job ID */ 
      receive_flag=current_lambda_job;
      MPI_Send(&receive_flag,1,MPI_INT,0,tag,MPI_communicator);
      MPI_Send(grid_per_lambda[Lambda_List[p]].Utot_new,
                 grid_per_lambda[Lambda_List[p]].TOTAL_NUMBER_OF_VERTICES,
                 MPI_DOUBLE,0,tag,MPI_communicator);
      /* receive the info which lambda-job is next */
      MPI_Recv(&job_ID,1,MPI_INT,0,tag,MPI_communicator,&Status);
      if (job_ID>=0) {
	/* if the flag (job_ID) is positive then stop with the matrix inversion*/ 
        /* make current_lambda_job negative because in _VILOZMAJ_main this will be
           recognized as a signal from the matrix inversion situtation */
        current_lambda_job = -job_ID;
        job_ID=-1;
      }
      else job_ID = 1; 
    } /* end of matrix inversion */ 
 
    } while (job_ID>=0);
 } /* end of non-zero process */

#if (!LEL_RADTRANS_N || !LEL_RADTRANS_L)
  free_dvector(vectorA_MPI,0,N_matrix*N_matrix-1);
#endif

#endif

  if (my_MPI_rank == 0){

    /* VISUALIZATION: VRML-plot of the ratio between the old and new values */

  /* DELETE ALL THIS OUTPUT!! 

    for (p=0;p<Lambda_List_Total;p++) {
      sprintf(filename,"NEW_over_OLD_energyi%.3d_at_%-8.3lfmicrons",my_MPI_rank,
              Lambda_Grid[Lambda_List[p]]);
      start_vertex_list = grid_per_lambda[Lambda_List[p]].start_vertex;
      start_triangle_list = grid_per_lambda[Lambda_List[p]].start_triangle;
      _VODENJAK_plot_vrml(filename,2, 0, Lambda_List[p]);
    }

    for (p=0;p<Lambda_List_Total;p++) {

      sprintf(filename,"matrix_A%.3d.%-8.3lf.dat",my_MPI_rank,
	      Lambda_Grid[Lambda_List[p]]);
      dump = fopen(filename,"w");
      fprintf(dump,"%d\n",TOTAL_NUMBER_OF_VERTICES);
      for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++){
        for (n=0;n<TOTAL_NUMBER_OF_VERTICES;n++){
          fprintf(dump,"%E ", Rad_Trans_Matrix[p][m][n]);
        }
        fprintf(dump,"\n");
      }
      fclose(dump);

      sprintf(filename,"vector_B%.3d.%-8.3lf.dat",my_MPI_rank,
	      Lambda_Grid[Lambda_List[p]]);
      dump = fopen(filename,"w");
      fprintf(dump,"%d\n",TOTAL_NUMBER_OF_VERTICES); 
      for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++){
        fprintf(dump,"%E ", Rad_Trans_Emission[p][m]);
      }
      fclose(dump);

      sprintf(filename,"solution_X%.3d.%-8.3lf.dat",my_MPI_rank,
  	    Lambda_Grid[Lambda_List[p]]);
      dump = fopen(filename,"w");
      fprintf(dump,"%d\n",TOTAL_NUMBER_OF_VERTICES); 
      for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++){
        fprintf(dump,"%E ", grid_per_lambda[Lambda_List[p]].Utot_new[m]);
      }
      fclose(dump);

      sprintf(filename,"star_energy%.3d.%-8.3lf.dat",my_MPI_rank,
	      Lambda_Grid[Lambda_List[p]]);
      dump = fopen(filename,"w");
      fprintf(dump,"%d\n",TOTAL_NUMBER_OF_VERTICES); 
      for (m=0;m<TOTAL_NUMBER_OF_VERTICES;m++){
      fprintf(dump,"%E ", grid_per_lambda[Lambda_List[p]].Ustar[m]);
      }
      fclose(dump);
    }
  */


  /* 
  _VODENJAK_plot_radiative_transfer_matrix(Rad_Trans_Matrix,Rad_Trans_Emission,
                                           lambda_count);
  */
  /*************/

} /* end of (my_MPI_Rank == 0) */

#if (!LEL_RADTRANS_N || !LEL_RADTRANS_L)
  free_d3tensor(Rad_Trans_Matrix,0,Lambda_List_Total-1,
		0,N_matrix-1,0,N_matrix-1);
  free_dmatrix(Rad_Trans_Emission,0,Lambda_List_Total-1,0,N_matrix-1);
#endif
  /* destry the 3D cloud of points (used for calculating integral directions) */
  err=_GOROSTAS_destroy_3D_cloud ();
  /* free variables */
  free_ivector(Vertex_Status,0,TOTAL_NUMBER_OF_VERTICES-1);
  free_dvector(exp_tau_substep,0,N_tau_substep);
  free_dvector(Sum_point_old,0,Lambda_List_Total-1);
  free_dvector(Sum_point_new,0,Lambda_List_Total-1);
  free_dvector(tau,0,Lambda_List_Total-1);
  free_dvector(tau_step,0,Lambda_List_Total-1);
  free_dvector(exp_tau,0,Lambda_List_Total-1);
  free_dvector(Eta_old,0,Lambda_List_Total-1);
  free_dvector(Eta_new,0,Lambda_List_Total-1);
  free_dvector(B_m_old,0,Lambda_List_Total-1);
  free_dvector(B_m_new,0,Lambda_List_Total-1);

  free_dmatrix(upsilon_old,0,Lambda_List_Total-1,0,Max_Dust_Type-1);
  free_dmatrix(upsilon_new,0,Lambda_List_Total-1,0,Max_Dust_Type-1);
  free_dmatrix(albedo_old,0,Lambda_List_Total-1,0,Max_Dust_Type-1);
  free_dmatrix(albedo_new,0,Lambda_List_Total-1,0,Max_Dust_Type-1);
  free_dvector(temperature,0,Max_Dust_Type-1);
  free_dvector(old_temperature,0,Max_Dust_Type-1);

} /* end of _VILOZMAJ_Radiative_Transfer_Solver() */


/* subroutine for calculation of the 3D linear interpolation coefficients */
void _VILOZMAJ_Radiative_Transfer_Solver_Coefficient 
       (double x1, double y1, double x2, double y2, double x3, double y3, 
        double x, double y, double *c1, double *c2, double *c3){

  double denominator;

  denominator = (x2-x1)*(y3-y1) - (x3-x1)*(y2-y1);
  if (denominator == 0.0) {
    *c1=0.0; *c2=0.0; *c3=0.0; return;
  }

  *c1 = 1 + ( (x-x1)*(y2-y3)+(y-y1)*(x3-x2) )/denominator;
  *c2 = ( (x-x1)*(y3-y1)-(y-y1)*(x3-x1) )/denominator;
  *c3 = ( (y-y1)*(x2-x1)-(x-x1)*(y2-y1) )/denominator;

  return;
}
