/* LELUYA - VODENJAK program block */
/* This function creates a VRML file for the image visualisation */

#include "VODENJAK.h"

int _VODENJAK_image_vrml (double theta_view,
			  struct _image_cell_structure *Image) {
 
  FILE *VRML_model;
  char filename[259];  /* name of the file */
  struct _image_cell_structure *Image_cell; /* a cell in the image */

  struct _3DVector observer_direction; /* direction of the observer */
  struct _3DVector point_image1; /* a point in the image */
  struct _3DVector point_image2; /* a point in the image */
  struct _3DVector point_image3; /* a point in the image */
  struct _3DVector point_image4; /* a point in the image */
  struct _3DVector point_3D_1; /* a point in the VRML space */
  struct _3DVector point_3D_2; /* a point in the VRML space */
  struct _3DVector point_3D_3; /* a point in the VRML space */
  struct _3DVector point_3D_4; /* a point in the VRML space */

  double color_tau, r,g,b;
  int cell_counter;
  double dummy;

  /* direction of the observer */
  /* observer_direction.x = sin(theta_view);
  observer_direction.y = 0.0;
  observer_direction.z = cos(theta_view);*/
  observer_direction.x = 0.0;
  observer_direction.y = 0.0;
  observer_direction.z = 1.0;
	

  if (DEBUG.SCREEN_MESSAGE >= 1 ) 
    printf("\n... plotting the VRML image grid ...\n");

  dummy = theta_view*180.0/PI;
#ifdef _2DD_MPI_
  sprintf(filename,"%s/2D_Image-%-4.1lf.wrl",_LELUYA_MPIOUT_PATH,dummy);
#else
  sprintf(filename,"%s/2D_Image-%-4.1lf.wrl",_LELUYA_OUTPUT_PATH,dummy);
#endif

  VRML_model=fopen(filename,"wt");

  /* introduction of the VRML file and coordinate axes */
  fprintf(VRML_model,"#VRML V2.0 utf8\n\n");
  fprintf(VRML_model,"###########################################\n");
  fprintf(VRML_model,"# coordinate lines\n");
  fprintf(VRML_model,"Transform {\n");
  fprintf(VRML_model,"  children [\n");
  fprintf(VRML_model,"    Viewpoint { position %lf 0 %lf }\n ",
                     2*Rout*observer_direction.x,2*Rout*observer_direction.z);
  fprintf(VRML_model,"    Shape {\n ");
  fprintf(VRML_model,"         geometry IndexedLineSet {\n ");
  fprintf(VRML_model,"                  coord Coordinate {\n ");
  fprintf(VRML_model,"                        point [ 0 0 0, ");
  fprintf(VRML_model,"%lf 0 0 , 0 0 %lf]\n ",Rout,Rout);
  fprintf(VRML_model,"                        }\n ");
  fprintf(VRML_model,"                  coordIndex [ 0, 1, -1, 0, 2, -1, ]\n");
  fprintf(VRML_model,"                  } \n         }\n]\n");
  fprintf(VRML_model,"} #coordinate lines\n\n");
  fprintf(VRML_model,"############################################\n");
  fprintf(VRML_model,"# names of the coordinate lines\n");
  fprintf(VRML_model,"Transform {\n");
  fprintf(VRML_model,"  children [\n");
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      translation %lf 0 0\n",1.1*Rout);
  fprintf(VRML_model,"      children [ \n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry Text { \n");
  fprintf(VRML_model,"            string \"X\" \n");
  fprintf(VRML_model,"            fontStyle FontStyle { size 20 ");
  fprintf(VRML_model,"family \"SERIF\" style \"BOLD\" }\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        } \n");
  fprintf(VRML_model,"      ]\n");
  fprintf(VRML_model,"    } # end of X-name\n");
  fprintf(VRML_model,"   Transform {\n");
  fprintf(VRML_model,"      translation 0 0 %lf \n",1.1*Rout);
  fprintf(VRML_model,"      children [ \n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry Text {\n"); 
  fprintf(VRML_model,"            string \"Z\" \n");
  fprintf(VRML_model,"            fontStyle FontStyle { size 20 ");
  fprintf(VRML_model,"family \"SERIF\" style \"BOLD\" }\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        } \n");
  fprintf(VRML_model,"      ]\n");
  fprintf(VRML_model,"    } # end of Z-name\n");
  fprintf(VRML_model,"  ]\n");
  fprintf(VRML_model,"}\n");
  fprintf(VRML_model,"############################################\n");

  /* GRID in form of lines */ 

    fprintf(VRML_model,"# Triangles\n");
    fprintf(VRML_model,"    Transform {\n");
    fprintf(VRML_model,"      children [\n");
    fprintf(VRML_model,"        Shape {\n");
    fprintf(VRML_model,"          appearance Appearance { material ");
    fprintf(VRML_model,"Material { }}\n");
    fprintf(VRML_model,"          geometry IndexedLineSet{\n");
    fprintf(VRML_model,"            coord Coordinate {\n");
    fprintf(VRML_model,"              point [ \n");

  Image_cell = Image;
  do {

    point_image1.x = Rout*Image_cell->r1*cos(Image_cell->phi1);
    point_image1.y = Rout*Image_cell->r1*sin(Image_cell->phi1);
    point_image1.z = 0.0;

    point_image2.x = Rout*Image_cell->r2*cos(Image_cell->phi1);
    point_image2.y = Rout*Image_cell->r2*sin(Image_cell->phi1);
    point_image2.z = 0.0;

    point_image3.x = Rout*Image_cell->r2*cos(Image_cell->phi2);
    point_image3.y = Rout*Image_cell->r2*sin(Image_cell->phi2);
    point_image3.z = 0.0;

    /* 
    point_image4.x = Rout*Image_cell->r1*cos(Image_cell->phi2);
    point_image4.y = Rout*Image_cell->r1*sin(Image_cell->phi2);
    point_image4.z = 0.0;
    */

    point_3D_1.x =  observer_direction.z*point_image1.x +
                    observer_direction.x*point_image1.z;
    point_3D_1.y =  point_image1.y;
    point_3D_1.z = -observer_direction.x*point_image1.x +
                    observer_direction.z*point_image1.z;

    if (Image_cell->r2 < 100.0)
      fprintf(VRML_model,"%4.2E %4.2E %4.2E,\n",
              point_3D_1.x,point_3D_1.y,point_3D_1.z);
    else 
      fprintf(VRML_model,"%6.4E %6.4E %6.4E,\n",
              point_3D_1.x,point_3D_1.y,point_3D_1.z);

    point_3D_2.x =  observer_direction.z*point_image2.x +
                    observer_direction.x*point_image2.z;
    point_3D_2.y =  point_image2.y;
    point_3D_2.z = -observer_direction.x*point_image2.x +
                    observer_direction.z*point_image2.z;
    if (Image_cell->r2 < 100.0)
      fprintf(VRML_model,"%4.2E %4.2E %4.2E,\n",
              point_3D_2.x,point_3D_2.y,point_3D_2.z);
    else 
      fprintf(VRML_model,"%6.4E %6.4E %6.4E,\n",
              point_3D_2.x,point_3D_2.y,point_3D_2.z);

    point_3D_3.x =  observer_direction.z*point_image3.x +
                    observer_direction.x*point_image3.z;
    point_3D_3.y =  point_image3.y;
    point_3D_3.z = -observer_direction.x*point_image3.x +
                    observer_direction.z*point_image3.z;
    if (Image_cell->r2 < 100.0)
      fprintf(VRML_model,"%4.2E %4.2E %4.2E,\n",
              point_3D_3.x,point_3D_3.y,point_3D_3.z);
    else 
      fprintf(VRML_model,"%6.4E %6.4E %6.4E,\n",
              point_3D_3.x,point_3D_3.y,point_3D_3.z);

    /* 
    point_3D_4.x =  observer_direction.z*point_image4.x +
                    observer_direction.x*point_image4.z;
    point_3D_4.y =  point_image4.y;
    point_3D_4.z = -observer_direction.x*point_image4.x +
                    observer_direction.z*point_image4.z;
    fprintf(VRML_model,"%4.2E %4.2E %4.2E,\n",
              point_3D_1.x,point_3D_1.y,point_3D_1.z);
    */

    Image_cell = Image_cell->next;
  } while(Image_cell!=Image);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [\n");

  cell_counter = 0;
  Image_cell = Image;
  do {
    fprintf(VRML_model,"%d, %d, %d, -1,\n",cell_counter,cell_counter+1,
	    cell_counter+2);
    /*    cell_counter += 4;*/
    cell_counter += 3;
    Image_cell = Image_cell->next;
  } while(Image_cell!=Image);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [\n");

  Image_cell = Image;
  do {
    fprintf(VRML_model,"0.0 1.0 0.0,\n");
    fprintf(VRML_model,"0.0 1.0 0.0,\n");
    fprintf(VRML_model,"0.0 1.0 0.0,\n");
    /* fprintf(VRML_model,"0.0 1.0 0.0,\n"); */ 
    Image_cell = Image_cell->next;
  } while(Image_cell!=Image);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            colorPerVertex TRUE\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        }\n");
  fprintf(VRML_model,"  ]\n}\n");

  /* GRID in form of surfaces */ 
#ifdef _VRML_traingle_surface

    fprintf(VRML_model,"# Triangles\n");
    fprintf(VRML_model,"    Transform {\n");
    fprintf(VRML_model,"      children [\n");
    fprintf(VRML_model,"        Shape {\n");
    fprintf(VRML_model,"          appearance Appearance { material ");
    fprintf(VRML_model,"Material { }}\n");
    fprintf(VRML_model,"          geometry IndexedFaceSet{\n");
    fprintf(VRML_model,"            solid FALSE\n"); 
    fprintf(VRML_model,"            coord Coordinate {\n");
    fprintf(VRML_model,"              point [ \n");

  Image_cell = Image;
  do {

    point_image1.x = Rout*Image_cell->r1*cos(Image_cell->phi1);
    point_image1.y = Rout*Image_cell->r1*sin(Image_cell->phi1);
    point_image1.z = -0.005;

    point_image2.x = Rout*Image_cell->r2*cos(Image_cell->phi1);
    point_image2.y = Rout*Image_cell->r2*sin(Image_cell->phi1);
    point_image2.z = -0.005;

    point_image3.x = Rout*Image_cell->r2*cos(Image_cell->phi2);
    point_image3.y = Rout*Image_cell->r2*sin(Image_cell->phi2);
    point_image3.z = -0.005;

    point_image4.x = Rout*Image_cell->r1*cos(Image_cell->phi2);
    point_image4.y = Rout*Image_cell->r1*sin(Image_cell->phi2);
    point_image4.z = -0.005;

    point_3D_1.x =  observer_direction.z*point_image1.x +
                    observer_direction.x*point_image1.z;
    point_3D_1.y =  point_image1.y;
    point_3D_1.z = -observer_direction.x*point_image1.x +
                    observer_direction.z*point_image1.z;
    fprintf(VRML_model,"%4.2E %4.2E %4.2E,\n",
              point_3D_1.x,point_3D_1.y,point_3D_1.z);

    point_3D_2.x =  observer_direction.z*point_image2.x +
                    observer_direction.x*point_image2.z;
    point_3D_2.y =  point_image2.y;
    point_3D_2.z = -observer_direction.x*point_image2.x +
                    observer_direction.z*point_image2.z;
    fprintf(VRML_model,"%4.2E %4.2E %4.2E,\n",
              point_3D_1.x,point_3D_1.y,point_3D_1.z);

    point_3D_3.x =  observer_direction.z*point_image3.x +
                    observer_direction.x*point_image3.z;
    point_3D_3.y =  point_image3.y;
    point_3D_3.z = -observer_direction.x*point_image3.x +
                    observer_direction.z*point_image3.z;
    fprintf(VRML_model,"%4.2E %4.2E %4.2E,\n",
              point_3D_2.x,point_3D_2.y,point_3D_2.z);

    point_3D_4.x =  observer_direction.z*point_image4.x +
                    observer_direction.x*point_image4.z;
    point_3D_4.y =  point_image4.y;
    point_3D_4.z = -observer_direction.x*point_image4.x +
                    observer_direction.z*point_image4.z;
    fprintf(VRML_model,"%4.2E %4.2E %4.2E,\n",
              point_3D_3.x,point_3D_3.y,point_3D_3.z);

    Image_cell = Image_cell->next;
  } while(Image_cell!=Image);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [\n");

  cell_counter = 0;
  Image_cell = Image;
  do {
    fprintf(VRML_model,"%d, %d, %d, -1,\n",cell_counter,cell_counter+1,
	    cell_counter+2);
    fprintf(VRML_model,"%d, %d, %d, -1,\n",cell_counter,cell_counter+2,
	    cell_counter+3);
    cell_counter += 4;
    Image_cell = Image_cell->next;
  } while(Image_cell!=Image);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [\n");

  Image_cell = Image;
  do {

    if (Image_cell->tau[Lambda_Total]==0.0) {r=0.0;g=0.0;b=0.0;}
    else {
      color_tau = log10(Tau_Total/Image_cell->tau[Lambda_Total]);
      if ( color_tau > 2.0) {r=0.0;g=0.0;b=1.0;}
      else if (color_tau<0.0) {r=1.0;g=1.0;b=1.0;}
      else {r=1.0-color_tau/2.0;g=r; b=1.0;}
    }

    fprintf(VRML_model,"%4.3lf %4.3lf %4.3lf,\n",r,g,b);
    fprintf(VRML_model,"%4.3lf %4.3lf %4.3lf,\n",r,g,b);
    fprintf(VRML_model,"%4.3lf %4.3lf %4.3lf,\n",r,g,b);
    fprintf(VRML_model,"%4.3lf %4.3lf %4.3lf,\n",r,g,b);

    Image_cell = Image_cell->next;
  } while(Image_cell!=Image);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            colorPerVertex FALSE\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        }\n");
  fprintf(VRML_model,"  ]\n}\n");

#endif

  fclose(VRML_model);

  return 0;
}
