/* LELUYA - VODENJAK program block */
/* This function creates a PovRay file for the grid 
   visualisation */

#include "VODENJAK.h"

int _VODENJAK_plot_PovRay(char image_name[255], 
                        int dust_i, int lambda_k) {
  /* input variables:
  image_name   = name of the file without the suffix ".inc" 
  dust_i       = dust component
  lambda_k     = wavelength's index
  */

  FILE *POVRAY_model;
  char POVRAY_PovRay_filename[259];  /* name of the POVRAY source file */

  struct _vertex_structure *current_vertex; /* vertex in the loop over all
                                               vertices */
  struct _triangle_structure *current_triangle; /* triangle in the loop over 
                                               all triangles */
  double color_basic_value, color_basic_value_normalization; /* values
                  which will be transformed into appropriate colors */
  double color1r,color1g,color1b; 

  double box_size,scale_lines=20, distance;
  struct _vertex_triangles   *vertex_triangle;
  struct _vertex_structure *adjacent_vertex,*adjacent_antivertex;
  int counter;

  struct _vertex_structure *v1, *v2;
  struct _POVRAY_lathe_surface {
    int vertexID;
    float x,y;
    float r,g,b;
  } PovRay_lathe_surface[LIMIT_ON_THE_TOTAL_NUMBER_OF_VERTICES];
  int lathe_counter, lathe_counter1,lathe_TOTAL;
  double distance1,distance2,distance3;

  if (my_MPI_rank!=0) return 0;

  /**************************************************************/
  
  if (DEBUG.SCREEN_MESSAGE >= 1 ) 
    printf("\n... plotting the PovRay grid ...\n");

  start_vertex_list = grid_per_lambda[lambda_k].start_vertex;
  start_triangle_list = grid_per_lambda[lambda_k].start_triangle;

  /**************************************************************/
  /**************************************************************/
  /* the POVRAY source file: PLOT 2D GRID ***********************/

#ifdef _2DD_MPI_
  sprintf(POVRAY_PovRay_filename,"%s/%s.PovRay.inc",_LELUYA_MPIOUT_PATH,image_name);
#else
  sprintf(POVRAY_PovRay_filename,"%s/%s.PovRay.inc",_LELUYA_OUTPUT_PATH,image_name);
#endif
  POVRAY_model=fopen(POVRAY_PovRay_filename,"wt");

  fprintf(POVRAY_model,"//Source Radius = %E\n", Source_Radius);
  fprintf(POVRAY_model,"\n// GRID\n");

  current_triangle = start_triangle_list;
  do {

    if (current_triangle->v1->T_dust[dust_i]>0.0 &&
        current_triangle->v2->T_dust[dust_i]>0.0 &&
        current_triangle->v3->T_dust[dust_i]>0.0) {

    /* plot the lines only if the first point has a larger ID-number, or
       if this line doesn't have the adjacent triangle */

    /******** V1 & V2 ********************************************/
    if ((current_triangle->v1->identification_number > 
         current_triangle->v2->identification_number) ||
        (current_triangle->v1->identification_number < 
         current_triangle->v2->identification_number  &&
         ((current_triangle->v1->type==2 && current_triangle->v2->type==2) ||
          _PERUNIKA_adjacent_triangle(current_triangle->v2,current_triangle)==NULL
	 )
        )
       ){

      /* we need the size of the bottom side of the cone */
    box_size = Rout;
     current_vertex = current_triangle->v1;
    vertex_triangle = current_triangle->v1->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      adjacent_vertex = _PERUNIKA_adjacent_antivertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      vertex_triangle = vertex_triangle->next;
    }
    box_size /= scale_lines;
      /* the bottom side: */
      fprintf(POVRAY_model,"cone{<%10.8E,%10.8E,0.0>,%6.4E ",
              current_triangle->v1->x,current_triangle->v1->z,box_size);

      /* we need the size of the upper side of the cone */
    box_size = Rout;
    current_vertex = current_triangle->v2;
    vertex_triangle = current_triangle->v2->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      adjacent_vertex = _PERUNIKA_adjacent_antivertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      vertex_triangle = vertex_triangle->next;
    }
    box_size /= scale_lines;
      /* the upper side: */
      fprintf(POVRAY_model,",<%10.8E,%10.8E,0.0>,%6.4E ",
              current_triangle->v2->x,current_triangle->v2->z,box_size);
      fprintf(POVRAY_model,"pigment{Green}}\n");
    }

    /******** V2 & V3 ********************************************/
    if ((current_triangle->v2->identification_number > 
         current_triangle->v3->identification_number) ||
        (current_triangle->v2->identification_number < 
         current_triangle->v3->identification_number  &&
         ((current_triangle->v2->type==2 && current_triangle->v3->type==2) ||
          _PERUNIKA_adjacent_triangle(current_triangle->v3,current_triangle)==NULL
	  )
	 )
	){
      /* we need the size of the bottom side of the cone */
    box_size = Rout;
    current_vertex = current_triangle->v2;
    vertex_triangle = current_triangle->v2->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      adjacent_vertex = _PERUNIKA_adjacent_antivertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      vertex_triangle = vertex_triangle->next;
    }
    box_size /= scale_lines;
      /* the bottom side: */
      fprintf(POVRAY_model,"cone{<%10.8E,%10.8E,0.0>,%6.4E ",
              current_triangle->v2->x,current_triangle->v2->z,box_size);

      /* we need the size of the upper side of the cone */
    box_size = Rout;
    current_vertex = current_triangle->v3;
    vertex_triangle = current_triangle->v3->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      adjacent_vertex = _PERUNIKA_adjacent_antivertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      vertex_triangle = vertex_triangle->next;
    }
    box_size /= scale_lines;
      /* the upper side: */
      fprintf(POVRAY_model,",<%10.8E,%10.8E,0.0>,%6.4E ",
              current_triangle->v3->x,current_triangle->v3->z,box_size);
      fprintf(POVRAY_model,"pigment{Green}}\n");

    }

    /******** V3 & V1 ********************************************/
    if ((current_triangle->v3->identification_number > 
         current_triangle->v1->identification_number) ||
         (current_triangle->v3->identification_number < 
          current_triangle->v1->identification_number  &&
          ((current_triangle->v1->type==2 && current_triangle->v3->type==2) ||
            _PERUNIKA_adjacent_triangle(current_triangle->v1,current_triangle)==NULL
	   )
	  )
	){

      /* we need the size of the bottom side of the cone */
    box_size = Rout;
    current_vertex=current_triangle->v3;
    vertex_triangle = current_triangle->v3->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      adjacent_vertex = _PERUNIKA_adjacent_antivertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      vertex_triangle = vertex_triangle->next;
    }
    box_size /= scale_lines;
      /* the bottom side: */
      fprintf(POVRAY_model,"cone{<%10.8E,%10.8E,0.0>,%6.4E ",
              current_triangle->v3->x,current_triangle->v3->z,box_size);

      /* we need the size of the upper side of the cone */
    box_size = Rout;
    current_vertex=current_triangle->v1;
    vertex_triangle = current_triangle->v1->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      adjacent_vertex = _PERUNIKA_adjacent_antivertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      vertex_triangle = vertex_triangle->next;
    }
    box_size /= scale_lines;
      /* the upper side: */
      fprintf(POVRAY_model,",<%10.8E,%10.8E,0.0>,%6.4E ",
              current_triangle->v1->x,current_triangle->v1->z,box_size);
      fprintf(POVRAY_model,"pigment{Green}}\n");

    }

    }
    current_triangle=current_triangle->next;
  } while (current_triangle!=start_triangle_list);


  /************************************************************************/
  /* plot the vertices */
  color_basic_value_normalization=log10(Dust_Sublim_Temp[0]);
  fprintf(POVRAY_model,"\n// VERTICES \n");
  current_vertex = start_vertex_list;
  do {
    if (current_vertex->T_dust[dust_i]>0.0) {

    fprintf(POVRAY_model,"sphere{<%10.8E,%10.8E,0.0>,",
            current_vertex->x, current_vertex->z);
    /* with need the sphere's radius */
    box_size = Rout;
    vertex_triangle = current_vertex->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      adjacent_vertex = _PERUNIKA_adjacent_antivertex_in_triangle
    (current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      vertex_triangle = vertex_triangle->next;
    }
    box_size /= 5.0;
    fprintf(POVRAY_model,"%10.8E ",box_size);
    fprintf(POVRAY_model,"pigment{rgb<");

    /* find the sphere's color */
    if (current_vertex->T_dust[dust_i] <= 0.0)
      fprintf(POVRAY_model,"0.6,0.6,0.6");
    else if (current_vertex->type == 3) 
      fprintf(POVRAY_model,"0.2,0.2,0.2");
    else if (current_vertex->type == 2 && current_vertex->level>666) 
      fprintf(POVRAY_model,"0,0,1.0");
    else {
      if (current_vertex->type == 2) fprintf(POVRAY_model,"0,0,1");
      else {
        color_basic_value = log10(current_vertex->T_dust[dust_i]) /
                           color_basic_value_normalization;
        if (color_basic_value<=0.5 ) {color1r=2*color_basic_value;color1g=0;color1b=0;}
        else {color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
        fprintf(POVRAY_model,"%6.4E,%6.4E,%6.4E",color1r,color1g,color1b);
      }
    }

    fprintf(POVRAY_model,">}}\n");
    fflush(POVRAY_model);

    }
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fclose(POVRAY_model);

  /**************************************************************/
  /**************************************************************/
  /* the POVRAY source file: PLOT 2D GRID ***********************/

#ifdef _2DD_MPI_
  sprintf(POVRAY_PovRay_filename,"%s/%s.Sublim_surface.PovRay.inc",_LELUYA_MPIOUT_PATH,image_name);
#else
  sprintf(POVRAY_PovRay_filename,"%s/%s.Sublim_surface.PovRay.inc",_LELUYA_OUTPUT_PATH,image_name);
#endif
  POVRAY_model=fopen(POVRAY_PovRay_filename,"wt");

  fprintf(POVRAY_model,"//Source Radius = %E\n", Source_Radius);
  fprintf(POVRAY_model,"//Outer Radius = %E\n", Rout);
  fprintf(POVRAY_model,"\n// SUBLIMATION SURFACE\n");
  fprintf(POVRAY_model,"union {\n");
    fflush(POVRAY_model);
  color_basic_value_normalization=log10(Dust_Sublim_Temp[0]);

  lathe_TOTAL = 0;
  current_triangle = start_triangle_list;
  do {
    if (current_triangle->v1->T_dust[dust_i]>0.0 &&
        current_triangle->v2->T_dust[dust_i]>0.0 &&
        current_triangle->v3->T_dust[dust_i]>0.0) {

    v1 = NULL; v2=NULL;

    if (current_triangle->v1->type==2 || current_triangle->v2->type==2 ||
        current_triangle->v3->type==2 ) {
      distance1 = _MATH_LIB_distance(current_triangle->v1->x,
                                     current_triangle->v1->z,0,0);
      distance2 = _MATH_LIB_distance(current_triangle->v2->x,
                                     current_triangle->v2->z,0,0);
      distance3 = _MATH_LIB_distance(current_triangle->v3->x,
                                     current_triangle->v3->z,0,0);

      if ( current_triangle->v1->type==2 && current_triangle->v2->type==2 &&
          distance1<0.998*Rout && distance1>Source_Radius)
	{v1 = current_triangle->v1;v2 = current_triangle->v2;}
      if ( current_triangle->v2->type==2 && current_triangle->v3->type==2 &&
          distance2<0.998*Rout && distance2>Source_Radius)
	{v1 = current_triangle->v2;v2 = current_triangle->v3;}
      if ( current_triangle->v3->type==2 && current_triangle->v1->type==2 &&
          distance3<0.998*Rout && distance3>Source_Radius)
	{v1 = current_triangle->v3;v2 = current_triangle->v1;}
    }
    if (v1!=NULL && v2!=NULL ){ 
      if (v2->z < v1->z) {current_vertex=v2; v2=v1; v1=current_vertex;}

      /* check if v1 already exists */
      for (lathe_counter=0;lathe_counter<lathe_TOTAL;lathe_counter++)
	if (PovRay_lathe_surface[lathe_counter].vertexID==v1->identification_number)
	  break;

      if (lathe_counter==lathe_TOTAL) {
	PovRay_lathe_surface[lathe_TOTAL].vertexID = v1->identification_number;
	PovRay_lathe_surface[lathe_TOTAL].x=v1->x/Rout;
	PovRay_lathe_surface[lathe_TOTAL].y=v1->z/Rout;
        color_basic_value = 
           log10(v1->T_dust[dust_i])/color_basic_value_normalization;
        if (color_basic_value<=0.5 )
          {color1r=2*color_basic_value; color1g=0; color1b=0;}
        else {color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
	PovRay_lathe_surface[lathe_TOTAL].r=color1r;
	PovRay_lathe_surface[lathe_TOTAL].g=color1g;
	PovRay_lathe_surface[lathe_TOTAL].b=color1b;
	lathe_TOTAL++;
      }

      /* check if v2 already exists */
      for (lathe_counter=0;lathe_counter<lathe_TOTAL;lathe_counter++)
	if (PovRay_lathe_surface[lathe_counter].vertexID==v2->identification_number)
	  break;

      if (lathe_counter==lathe_TOTAL) {
	PovRay_lathe_surface[lathe_TOTAL].vertexID = v2->identification_number;
	PovRay_lathe_surface[lathe_TOTAL].x=v2->x/Rout;
	PovRay_lathe_surface[lathe_TOTAL].y=v2->z/Rout;
        color_basic_value = 
           log10(v2->T_dust[dust_i])/color_basic_value_normalization;
        if (color_basic_value<=0.5 )
          {color1r=2*color_basic_value; color1g=0; color1b=0;}
        else {color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
	PovRay_lathe_surface[lathe_TOTAL].r=color1r;
	PovRay_lathe_surface[lathe_TOTAL].g=color1g;
	PovRay_lathe_surface[lathe_TOTAL].b=color1b;
	lathe_TOTAL++;
      }

      fprintf(POVRAY_model,"difference{ cone{<0,%9.7E,0>,%9.7E <0,%9.7E,0>,%9.7E} ",
	      v1->z/Rout,v1->x*1.001/Rout,v2->z/Rout,v2->x*1.001/Rout);
      if (v2->x == v1->x) 
         fprintf(POVRAY_model,"cone{<0,%9.7E,0>,%9.7E <0,%9.7E,0>,%9.7E}}\n ",
	      0.999*v1->z/Rout,v1->x/Rout,1.001*v2->z/Rout,v2->x/Rout);
      else {
	distance = (v2->z - v1->z)/(v2->x - v1->x);
        fprintf(POVRAY_model,"cone{<0,%9.7E,0>,%9.7E <0,%9.7E,0>,%9.7E}}\n ",
	        -0.001*distance*v1->x/Rout + v1->z/Rout, 0.999*v1->x/Rout,
                distance*(1.001*v2->x-v1->x)/Rout+v1->z/Rout,1.001*v2->x/Rout);
      }
    } /* end of if (v1!=NULL && v2!=NULL ) */

    }
    current_triangle=current_triangle->next;
  } while (current_triangle!=start_triangle_list);

  /* sort the list of vertices according to their y-coordinate */
  for (lathe_counter=0;lathe_counter<lathe_TOTAL-1;lathe_counter++)
    for (lathe_counter1=lathe_counter+1;lathe_counter1<lathe_TOTAL;lathe_counter1++)
      if (PovRay_lathe_surface[lathe_counter1].y<PovRay_lathe_surface[lathe_counter].y) {
	counter = PovRay_lathe_surface[lathe_counter1].vertexID;
	PovRay_lathe_surface[lathe_counter1].vertexID=
	  PovRay_lathe_surface[lathe_counter].vertexID;
	PovRay_lathe_surface[lathe_counter].vertexID=counter;
	distance = PovRay_lathe_surface[lathe_counter1].x;
	PovRay_lathe_surface[lathe_counter1].x = 
	  PovRay_lathe_surface[lathe_counter].x;
	PovRay_lathe_surface[lathe_counter].x=distance;
	distance = PovRay_lathe_surface[lathe_counter1].y;
	PovRay_lathe_surface[lathe_counter1].y = 
	  PovRay_lathe_surface[lathe_counter].y;
	PovRay_lathe_surface[lathe_counter].y=distance;
	distance = PovRay_lathe_surface[lathe_counter1].r;
	PovRay_lathe_surface[lathe_counter1].r = 
	  PovRay_lathe_surface[lathe_counter].r;
	PovRay_lathe_surface[lathe_counter].r=distance;
	distance = PovRay_lathe_surface[lathe_counter1].g;
	PovRay_lathe_surface[lathe_counter1].g = 
	  PovRay_lathe_surface[lathe_counter].g;
	PovRay_lathe_surface[lathe_counter].g=distance;
	distance = PovRay_lathe_surface[lathe_counter1].b;
	PovRay_lathe_surface[lathe_counter1].b = 
	  PovRay_lathe_surface[lathe_counter].b;
	PovRay_lathe_surface[lathe_counter].b=distance;
    }

  /* create the surface */
  /*fprintf(POVRAY_model,"lathe {linear_spline %d,\n",lathe_TOTAL+2);
  for (lathe_counter=0;lathe_counter<lathe_TOTAL;lathe_counter++) {
    fprintf(POVRAY_model,"<%9.7E,%9.7E>,\n",
	    PovRay_lathe_surface[lathe_counter].x,
	    PovRay_lathe_surface[lathe_counter].y);
  } */

  /*
  for (lathe_counter=lathe_TOTAL-2;lathe_counter>0;lathe_counter--) {
    fprintf(POVRAY_model,"<%9.7E,%9.7E>,\n",
	    PovRay_lathe_surface[lathe_counter].x,
	    PovRay_lathe_surface[lathe_counter].y);
	    } */

  /*fprintf(POVRAY_model,"<%9.7E,0.0>\n",
	    PovRay_lathe_surface[lathe_TOTAL-1].x);
  fprintf(POVRAY_model,"<%9.7E,%9.7E>\n",
	    PovRay_lathe_surface[0].x,
	    PovRay_lathe_surface[0].y);
	    fprintf(POVRAY_model,"}\n");*/

  fprintf(POVRAY_model,"  pigment{gradient y color_map{\n");
  color1r=PovRay_lathe_surface[0].r;
  color1g=PovRay_lathe_surface[0].g;
  color1b=PovRay_lathe_surface[0].b;
  fprintf(POVRAY_model,"[%6.4E  color rgb <%6.4E,%6.4E,%6.4E>]\n",
	    PovRay_lathe_surface[0].y,
	    PovRay_lathe_surface[0].r,
	    PovRay_lathe_surface[0].g,
	    PovRay_lathe_surface[0].b);
  for (lathe_counter=1;lathe_counter<lathe_TOTAL;lathe_counter++) 
    if (lathe_counter==lathe_TOTAL-1 || 
        fabs(1.0-color1r/PovRay_lathe_surface[lathe_counter].r) > 0.05 ||
        fabs(1.0-color1g/PovRay_lathe_surface[lathe_counter].g) > 0.05 ||
        fabs(1.0-color1b/PovRay_lathe_surface[lathe_counter].b) > 0.05) {
    fprintf(POVRAY_model,"[%6.4E  color rgb <%6.4E,%6.4E,%6.4E>]\n",
	    PovRay_lathe_surface[lathe_counter].y,
	    PovRay_lathe_surface[lathe_counter].r,
	    PovRay_lathe_surface[lathe_counter].g,
	    PovRay_lathe_surface[lathe_counter].b);
    color1r=PovRay_lathe_surface[lathe_counter].r;
    color1g=PovRay_lathe_surface[lathe_counter].g;
    color1b=PovRay_lathe_surface[lathe_counter].b;
  }
  fprintf(POVRAY_model,"}}\n");
  fflush(POVRAY_model);

  fprintf(POVRAY_model,"scale %lf\n}\n",Rout);
  fclose(POVRAY_model);
  /**************************************************************/

  return 0;
}
