/* LELUYA - VODENJAK program block */
/* This function creates a VRML model of the angular integration directions
   around a given grid vertex */

#include "VODENJAK.h"

int _VODENJAK_plot_angular_integral_rays(struct _vertex_structure *main_vertex,
                                         char image_name[255],int N_phi_grid) {
  /* input variables:
  image_name   = name of the file without the suffix ".wrl" 

  Rotation of the X-Z grid around the Z-axis:
  int N_phi_grid; */ 
  double phi_angle, local_phi_angle;
  int phi_grid_counter;

  int ray_counter;
  int empty_space_flag;
  int dust_i; 

  FILE *VRML_model;
  char filename[259];  /* name of the file */

  struct _vertex_structure *current_vertex; /* vertex in the loop over all
                                               vertices */
  struct _triangle_structure *current_triangle; /* triangle in the loop over 
                                               all triangles */
  double color_basic_value, color_basic_value_normalization; /* values
		          which will be transformed into appropriate colors */
  double color1r,color1g,color1b; 
  struct _3DVector end_point;
  struct _3DVector unit_center;
  double dummy,distance_total,angle1,angle2;

  if (my_MPI_rank!=0) return 0;

  /**************************************************************/
  
  if (DEBUG.SCREEN_MESSAGE >= 1 ) {
    printf("\n... VRML model of the angular integration rays around:\n");
    printf("      x=%9.4lf z=%9.4lf ................................\n",
	   main_vertex->x,main_vertex->z);
  }

  /**************************************************************/
#ifdef _2DD_MPI_
  sprintf(filename,"%s/%s_3Dsphere.wrl",_LELUYA_MPIOUT_PATH,image_name);
#else
  sprintf(filename,"%s/%s_3Dsphere.wrl",_LELUYA_OUTPUT_PATH,image_name);
#endif
  VRML_model=fopen(filename,"wt");

  fprintf(VRML_model,"#VRML V2.0 utf8\n\n");
  fprintf(VRML_model,"    Viewpoint { position 0 0 5 }\n ");
  fprintf(VRML_model,"# coordinate lines\n");
  fprintf(VRML_model,"Transform {\n");
  fprintf(VRML_model,"  children [\n");
  fprintf(VRML_model,"    Shape {\n ");
  fprintf(VRML_model,"         geometry IndexedLineSet {\n ");
  fprintf(VRML_model,"                  coord Coordinate {\n ");
  fprintf(VRML_model,"                        point [ 0 0 0, ");
  distance_total = sqrt(main_vertex->x*main_vertex->x+main_vertex->z*main_vertex->z);
  fprintf(VRML_model,"%lf 0 %lf , ",-2*main_vertex->x/distance_total,
                                    -2*main_vertex->z/distance_total);
  fprintf(VRML_model,"%lf 0 %lf , ",-2*main_vertex->z/distance_total,
                                     2*main_vertex->x/distance_total);
  fprintf(VRML_model,"0 2 0]\n ");
  fprintf(VRML_model,"                        }\n ");
  fprintf(VRML_model,"                  coordIndex [ 0, 1, -1, 0, 2, -1,  0, 3, -1,]\n");
  fprintf(VRML_model,"                  color Color{ color [\n");
  fprintf(VRML_model,"0 1 0, 0 1 1,0 1 0,0 1 0,]}");
  fprintf(VRML_model,"                             } \n         }\n]\n");
  fprintf(VRML_model,"} #coordinate lines\n\n");
  fprintf(VRML_model,"############################################\n");
  for (ray_counter=0;ray_counter<total_number_of_spherical_triangles;
	 ray_counter++) {
      fprintf(VRML_model,"Group{children Shape{\n");
      fprintf(VRML_model,"geometry IndexedLineSet { coord Coordinate { point [");
      fprintf(VRML_model,"%E %E %E,\n",integral_directions[ray_counter].v1.x,
          integral_directions[ray_counter].v1.y,integral_directions[ray_counter].v1.z);
      fprintf(VRML_model,"%E %E %E,\n",integral_directions[ray_counter].v2.x,
          integral_directions[ray_counter].v2.y,integral_directions[ray_counter].v2.z);
      fprintf(VRML_model,"%E %E %E,\n",integral_directions[ray_counter].v3.x,
          integral_directions[ray_counter].v3.y,integral_directions[ray_counter].v3.z);
      fprintf(VRML_model,"]}\n");
      fprintf(VRML_model,"coordIndex [0,1,2,0,-1, ]}\n");
      fprintf(VRML_model,"}}\n");
  }

  fclose(VRML_model);

  /**************************************************************/
#ifdef _2DD_MPI_
  sprintf(filename,"%s/%s.wrl",_LELUYA_MPIOUT_PATH,image_name);
#else
  sprintf(filename,"%s/%s.wrl",_LELUYA_OUTPUT_PATH,image_name);
#endif
  VRML_model=fopen(filename,"wt");

  /* introduction of the VRML file and coordinate axes */
  fprintf(VRML_model,"#VRML V2.0 utf8\n\n");
  fprintf(VRML_model,"Viewpoint { position 0 %lf %lf }\n ",
                     0.5*Rout,2*Rout);
  fprintf(VRML_model,"Group { children Shape{\n");
  fprintf(VRML_model,"appearance Appearance { material Material { }}\n");
  fprintf(VRML_model,"geometry IndexedFaceSet{ coord Coordinate { point [\n");
  fprintf(VRML_model,"%lf 0 0, %lf 0 0, %lf 0 %lf,%lf 0 %lf,]}\n",
                      -Rout,Rout,Rout,-Rout,-Rout,-Rout);
  fprintf(VRML_model,"coordIndex [ 0,1,2,3,-1,]\n");
  fprintf(VRML_model,"color Color{ color [\n");
  fprintf(VRML_model,"0.1 0.1 0.3, 0.1 0.1 0.3,0.1 0.1 0.3,0.1 0.1 0.3,]}");
  fprintf(VRML_model,"}}}\n");
  fprintf(VRML_model,"############################################\n");
  fprintf(VRML_model,"# Triangles\n");
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      children [\n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry IndexedLineSet{\n");
  fprintf(VRML_model,"            coord Coordinate {\n");
  fprintf(VRML_model,"              point [ \n");

  /* plot the verticess */
  current_vertex = start_vertex_list;
  do {
    /* put the vertex into the VRML file */
    fprintf(VRML_model,"%12.6lf %12.6lf 0.0,\n",
            current_vertex->x, current_vertex->z);
    fflush(VRML_model);
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [\n");

  /* connect the points into a 3D structure */
  current_triangle = start_triangle_list;
  do {
    /* plot the lines only if the first point has a larger ID-number, or
       if this line doesn't have the adjacent triangle */
    if ((current_triangle->v1->identification_number > 
	 current_triangle->v2->identification_number) ||
	(current_triangle->v1->identification_number < 
	 current_triangle->v2->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v2,current_triangle)==NULL))
      fprintf(VRML_model,"%d, %d, -1,\n",
              current_triangle->v1->identification_number,
              current_triangle->v2->identification_number);
    if ((current_triangle->v2->identification_number > 
	 current_triangle->v3->identification_number) ||
	(current_triangle->v2->identification_number < 
	 current_triangle->v3->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v3,current_triangle)==NULL))
      fprintf(VRML_model,"%d, %d, -1,\n",
              current_triangle->v2->identification_number,
              current_triangle->v3->identification_number);
    if ((current_triangle->v3->identification_number > 
	 current_triangle->v1->identification_number) ||
	(current_triangle->v3->identification_number < 
	 current_triangle->v1->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v1,current_triangle)==NULL))
      fprintf(VRML_model,"%d, %d, -1,\n",
              current_triangle->v3->identification_number,
              current_triangle->v1->identification_number);

    current_triangle=current_triangle->next;
  } while (current_triangle!=start_triangle_list);


  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [\n");

  current_vertex = start_vertex_list;
  do {
    if (current_vertex->type == 3) fprintf(VRML_model,"0.2 0.2 0.2,\n");
    else if (current_vertex->type == 2 && current_vertex->level>666)
      fprintf(VRML_model,"0.0 0.0 1.0,\n");
    else if (current_vertex->type == 2) fprintf(VRML_model,"0.0 0.0 1.0,\n");
    else fprintf(VRML_model,"0.0 1.0 0.0,\n");
    fflush(VRML_model);
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            colorPerVertex TRUE\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        }\n");
  fprintf(VRML_model,"  ]\n}\n");

  /* VRML points due to the grid rotaion around Z-axis: */
  for (phi_grid_counter=1; phi_grid_counter<=N_phi_grid; phi_grid_counter++) {
    phi_angle=PI*phi_grid_counter/N_phi_grid;

    fprintf(VRML_model,"Group { children Shape { \n");
    fprintf(VRML_model,"geometry PointSet {\n");
    fprintf(VRML_model,"  coord Coordinate {\n");
    fprintf(VRML_model,"    point [ \n");
    current_vertex = start_vertex_list;
    do {
      if (current_vertex->x != 0.0) {
        local_phi_angle = sqrt(current_vertex->x*current_vertex->x +
                               current_vertex->z*current_vertex->z);
        local_phi_angle = 
             phi_grid_counter*(local_phi_angle/current_vertex->x)*PI/N_phi_grid;
        if (local_phi_angle>PI) {
	  current_vertex = current_vertex->next;
          continue;
	}
	empty_space_flag = FALSE;
	for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	  if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
	    if (current_vertex->T_dust[dust_i]>0.0)
	      {empty_space_flag = TRUE; break;}
	if (empty_space_flag)
          fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n", 
	        current_vertex->x*cos(local_phi_angle), current_vertex->z,
                -current_vertex->x*sin(local_phi_angle));
      }
      current_vertex = current_vertex->next;
    } while (current_vertex!=start_vertex_list);

    fprintf(VRML_model,"    ]}\n");
    fprintf(VRML_model,"  color Color {\n");
    fprintf(VRML_model,"     color [\n");

    current_vertex = start_vertex_list;
    do {
      if (current_vertex->x != 0.0) {
        local_phi_angle = sqrt(current_vertex->x*current_vertex->x +
                               current_vertex->z*current_vertex->z);
        local_phi_angle = 
             phi_grid_counter*(local_phi_angle/current_vertex->x)*PI/N_phi_grid;
        if (local_phi_angle>PI) {
	  current_vertex = current_vertex->next;
          continue;
	}
	empty_space_flag = FALSE;
	for (dust_i=0; dust_i<Max_Dust_Type; dust_i++)
	  if (INPUT.DUST_DENSITY_TYPE[dust_i]!=99999)
	    if (current_vertex->T_dust[dust_i]>0.0)
	      {empty_space_flag = TRUE; break;}
	if (empty_space_flag) {
          if (current_vertex->type == 3) 
            fprintf(VRML_model,"0.2 0.2 0.2,\n");
          else if (current_vertex->type == 2 && current_vertex->level>666)
            fprintf(VRML_model,"0.0 0.0 1.0,\n");
          else if (current_vertex->type==2) 
            fprintf(VRML_model,"0.0 0.0 1.0,\n");
          else 
            fprintf(VRML_model,"0.0 1.0 0.0,\n");
	}
      }
      current_vertex = current_vertex->next;
    } while (current_vertex!=start_vertex_list);

    fprintf(VRML_model,"     ]\n");
    fprintf(VRML_model,"  }\n");
    fprintf(VRML_model,"}}}\n");

  }

  /* create a VRML model of the integration rays streaming 
    out from 'main_vertex' */
  fprintf(VRML_model,"Transform {\n");

  /* translation %12.6lf %12.6lf 0 \n",
	  main_vertex->x,main_vertex->z);
  fprintf(VRML_model,"            scale %12.6lf %12.6lf %12.6lf\n",
  2*Rout, 2*Rout, 2*Rout); */ 

  fprintf(VRML_model,"children Shape {\n");
  fprintf(VRML_model,"geometry IndexedLineSet {\n");
  fprintf(VRML_model,"  coord Coordinate {\n");
  fprintf(VRML_model,"    point [ \n");
  fprintf(VRML_model,"%12.6lf 0 %12.6lf,\n",main_vertex->x,main_vertex->z);

  unit_center.x = - main_vertex->x;
  unit_center.y = 0.0;
  unit_center.z = - main_vertex->z;
  distance_total=sqrt(unit_center.x*unit_center.x +
                      unit_center.z*unit_center.z);
  unit_center.x /= distance_total;
  unit_center.z /= distance_total;

  for (ray_counter=0;ray_counter<total_number_of_spherical_triangles;
         ray_counter++) {
      /* find the end point for the line*/
      dummy = unit_center.x*integral_directions[ray_counter].direction.x+
	      unit_center.z*integral_directions[ray_counter].direction.z;
      dummy = distance_total*dummy+sqrt(Rout*Rout-
                  distance_total* distance_total * (1.0 - dummy*dummy));

      end_point.x=main_vertex->x + 
                  dummy*integral_directions[ray_counter].direction.x;
      end_point.y=dummy*integral_directions[ray_counter].direction.y;
      end_point.z=main_vertex->z + 
                  dummy*integral_directions[ray_counter].direction.z;

    fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n",
	    end_point.x,end_point.z,-end_point.y);
  }

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [\n");

  for (ray_counter=0;ray_counter<total_number_of_spherical_triangles;
         ray_counter++) {
    fprintf(VRML_model,"0,%d,-1,\n",ray_counter+1); 
  }

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [\n");
  fprintf(VRML_model,"1 0 0,\n");

  for (ray_counter=0;ray_counter<total_number_of_spherical_triangles;
         ray_counter++) {
    fprintf(VRML_model,"1 0 0,"); 
  }

  fprintf(VRML_model,"      ]}}\n");
  fprintf(VRML_model,"}}\n");

  fclose(VRML_model);

  return 0;
}


