/* LELUYA - VODENJAK program block */
/* This function creates a VRML file for the grid visualisation */

#include "VODENJAK.h"

int _VODENJAK_plot_dust_edge_vrml(char image_name[255],int lambda_k) {
  /* input variables:
  image_name   = name of the file without the suffix ".wrl" 
  lambda_k     = wavelength's index
  */

  FILE *VRML_model;
  char filename[259];  /* name of the file */
  int dust_i; /* dust sublimation component */
  struct _vertex_structure *current_vertex; /* vertex in the loop over all
                                               vertices */
  struct _triangle_structure *current_triangle; /* triangle in the loop over 
                                               all triangles */
  double color_basic_value, color_basic_value_normalization; /* values
		          which will be transformed into appropriate colors */
  double color1r,color1g,color1b; 

  int sublim_point_counter, END_sublim_point_counter;
  int ID1, ID2, ID3, ID4;
  int N_phi, N_phi_total = 18;
  double step_phi = PI/N_phi_total, phi;
  struct _quadrangle_structure *quadrangle;

  if (my_MPI_rank!=0) return 0;

  /**************************************************************/
  
  if (DEBUG.SCREEN_MESSAGE >= 1 ) 
    printf("\n... plotting the VRML model of the dust edges ...\n");

  sprintf(filename,"%s.wrl",image_name);
  VRML_model=fopen(filename,"wt");

  /* introduction of the VRML file and coordinate axes */
  fprintf(VRML_model,"#VRML V2.0 utf8\n\n");
  fprintf(VRML_model,"###########################################\n");
  fprintf(VRML_model,"# coordinate lines\n");
  fprintf(VRML_model,"Transform {\n");
  fprintf(VRML_model,"  children [\n");
  fprintf(VRML_model,"    Viewpoint { position 0 0 20 }\n");
  fprintf(VRML_model,"    Shape {\n ");
  fprintf(VRML_model,"         geometry IndexedLineSet {\n ");
  fprintf(VRML_model,"                  coord Coordinate {\n ");
  fprintf(VRML_model,"                        point [ 0 0 -0.01, ");
  fprintf(VRML_model,"%lf 0 -0.01, 0 %lf -0.01]\n ",Rout,Rout);
  fprintf(VRML_model,"                        }\n ");
  fprintf(VRML_model,"                  coordIndex [ 0, 1, -1, 0, 2, -1, ]\n");
  fprintf(VRML_model,"                  } \n         }\n]\n");
  fprintf(VRML_model,"} #coordinate lines\n\n");
  fprintf(VRML_model,"############################################\n");
  fprintf(VRML_model,"# names of the coordinate lines\n");
  fprintf(VRML_model,"Transform {\n");
  fprintf(VRML_model,"  children [\n");
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      translation %lf 0 0\n",1.1*Rout);
  fprintf(VRML_model,"      children [ \n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry Text { \n");
  fprintf(VRML_model,"            string \"X\" \n");
  fprintf(VRML_model,"            fontStyle FontStyle { size 20 ");
  fprintf(VRML_model,"family \"SERIF\" style \"BOLD\" }\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        } \n");
  fprintf(VRML_model,"      ]\n");
  fprintf(VRML_model,"    } # end of X-name\n");
  fprintf(VRML_model,"   Transform {\n");
  fprintf(VRML_model,"      translation 0 %lf 0 \n",1.1*Rout);
  fprintf(VRML_model,"      children [ \n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry Text {\n"); 
  fprintf(VRML_model,"            string \"Z\" \n");
  fprintf(VRML_model,"            fontStyle FontStyle { size 20 ");
  fprintf(VRML_model,"family \"SERIF\" style \"BOLD\" }\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        } \n");
  fprintf(VRML_model,"      ]\n");
  fprintf(VRML_model,"    } # end of Z-name\n");
  fprintf(VRML_model,"  ]\n");
  fprintf(VRML_model,"}\n");
  fprintf(VRML_model,"############################################\n");
  fprintf(VRML_model,"# Triangles\n");
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      children [\n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          appearance Appearance { material ");
  fprintf(VRML_model,"Material { }}\n");
  fprintf(VRML_model,"          geometry IndexedLineSet{\n");
  fprintf(VRML_model,"            coord Coordinate {\n");
  fprintf(VRML_model,"              point [ \n");

  /* plot the verticess */
  current_vertex = start_vertex_list;
  do {
    /* put the vertex into the VRML file */
    fprintf(VRML_model,"%12.6lf %12.6lf 0.0,\n",
            current_vertex->x, current_vertex->z);
    fprintf(VRML_model,"%12.6lf %12.6lf 0.0,\n",
            -current_vertex->x, current_vertex->z);
    fflush(VRML_model);
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [\n");

  /* connect the points into a 3D structure */
  current_triangle = start_triangle_list;
  do {
    if ((current_triangle->v1->identification_number > 
	 current_triangle->v2->identification_number) ||
	(current_triangle->v1->identification_number < 
	 current_triangle->v2->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v2,current_triangle)==NULL)){
      fprintf(VRML_model,"%d, %d, -1,\n",
              2*current_triangle->v1->identification_number,
              2*current_triangle->v2->identification_number);
      fprintf(VRML_model,"%d, %d, -1,\n",
              2*current_triangle->v1->identification_number+1,
              2*current_triangle->v2->identification_number+1);
    }

    if ((current_triangle->v2->identification_number > 
	 current_triangle->v3->identification_number) ||
	(current_triangle->v2->identification_number < 
	 current_triangle->v3->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v3,current_triangle)==NULL)){
      fprintf(VRML_model,"%d, %d, -1,\n",
              2*current_triangle->v2->identification_number,
              2*current_triangle->v3->identification_number);
      fprintf(VRML_model,"%d, %d, -1,\n",
              2*current_triangle->v2->identification_number+1,
              2*current_triangle->v3->identification_number+1);
    }

    if ((current_triangle->v3->identification_number > 
	 current_triangle->v1->identification_number) ||
	(current_triangle->v3->identification_number < 
	 current_triangle->v1->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v1,current_triangle)==NULL)){
      fprintf(VRML_model,"%d, %d, -1,\n",
              2*current_triangle->v3->identification_number,
              2*current_triangle->v1->identification_number);
      fprintf(VRML_model,"%d, %d, -1,\n",
              2*current_triangle->v3->identification_number+1,
              2*current_triangle->v1->identification_number+1);
    }

    current_triangle=current_triangle->next;
  } while (current_triangle!=start_triangle_list);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [\n");

  /* plot the temperature (of the dust component dust_i) as colors */
  dust_i = 0;
  color_basic_value_normalization = log10(Dust_Sublim_Temp[dust_i]);
  current_vertex = start_vertex_list;
  while (current_vertex!=NULL) {
    /* calculate the point colors */
    if (current_vertex->T_dust[dust_i] <= 0.0) {
      fprintf(VRML_model,"0.6 0.6 0.6,\n");
      fprintf(VRML_model,"0.6 0.6 0.6,\n");
    }
    else if (current_vertex->type == 3) {
      fprintf(VRML_model,"0.2 0.2 0.2,\n");
      fprintf(VRML_model,"0.2 0.2 0.2,\n");
    }
    else if (current_vertex->type == 2 && current_vertex->level>666) {
      fprintf(VRML_model,"0.0 0.0 1.0,\n");
      fprintf(VRML_model,"0.0 0.0 1.0,\n");
    }
    else if (current_vertex->type == 1 || 
             (current_vertex->type == 2 && current_vertex->level==666)) {
      fprintf(VRML_model,"0.0 1.0 0.0,\n");
      fprintf(VRML_model,"0.0 1.0 0.0,\n");
    }
    else {
      /* this is inactive */ 
      color_basic_value = log10(current_vertex->T_dust[dust_i]) /
	                       color_basic_value_normalization;
      if (color_basic_value<=0.5 ) {
        color1r=2*color_basic_value; color1g=0; color1b=0;}
      else {
        color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
      if (current_vertex->type == 2) {
        fprintf(VRML_model,"                0.0 0.0 1.0,\n");
        fprintf(VRML_model,"                0.0 0.0 1.0,\n");
      }
      else {
        fprintf(VRML_model,"                %10.6f %10.6f %10.6f,\n",
              color1r,color1g,color1b);
        fprintf(VRML_model,"                %10.6f %10.6f %10.6f,\n",
              color1r,color1g,color1b);
      }
    }
    fflush(VRML_model);
    current_vertex = current_vertex->next;
    if (current_vertex==start_vertex_list) current_vertex=NULL;
  }

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            colorPerVertex TRUE\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        }\n");

  fprintf(VRML_model,"  ]\n}\n");


  /* quadrangles which form the sublimation shells */ 
  fprintf(VRML_model,"# Quadrangles: sublimation surface\n");
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      children [\n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          appearance Appearance { material ");
  fprintf(VRML_model,"Material { }}\n");
  fprintf(VRML_model,"          geometry IndexedFaceSet{\n");
  fprintf(VRML_model,"            solid FALSE\n");
  fprintf(VRML_model,"            coord Coordinate {\n");
  fprintf(VRML_model,"              point [ \n");

  /* plot the verticess */
  sublim_point_counter = 0;
  current_vertex = start_vertex_list;
  do {
    if (current_vertex->type==2 && current_vertex->identification_number>=0) {
      current_vertex->identification_number=sublim_point_counter;
      sublim_point_counter++; 
      for (N_phi=0; N_phi<=N_phi_total; N_phi++) {
        phi = N_phi*step_phi;
        fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n",
                current_vertex->x*cos(phi),current_vertex->z, 
                -(current_vertex->x*sin(phi)));
      } 
      fflush(VRML_model);
    }
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  END_sublim_point_counter = sublim_point_counter;
  current_vertex = start_vertex_list;
  do {
    if (current_vertex->identification_number<0) {
      current_vertex->identification_number= - sublim_point_counter;
      sublim_point_counter++; 
      fprintf(VRML_model,"%12.6lf %12.6lf 0.0,\n",
                current_vertex->x,current_vertex->z);
    }
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [\n");

  /* quadrangles */
  current_triangle = start_triangle_list;
  do {

    if ( current_triangle->v1->type==2 && current_triangle->v2->type==2 &&
	 current_triangle->v1->identification_number>=0 && 
         current_triangle->v2->identification_number>=0)
      if ( current_triangle->v1->identification_number > 
	   current_triangle->v2->identification_number ||
	  (current_triangle->v1->identification_number < 
	   current_triangle->v2->identification_number  &&
       _PERUNIKA_adjacent_triangle(current_triangle->v2,current_triangle)==NULL)){
	ID1=current_triangle->v1->identification_number*(N_phi_total+1);
	ID2=current_triangle->v2->identification_number*(N_phi_total+1);
	for (N_phi=1; N_phi<=N_phi_total; N_phi++) {
	  fprintf(VRML_model,"%d, %d, %d, %d,-1,\n",
		  ID1+N_phi-1, ID2+N_phi-1, ID2+N_phi, ID1+N_phi);
	}
	fflush(VRML_model);
      }

    if ( current_triangle->v2->type==2 && current_triangle->v3->type==2 &&
	 current_triangle->v2->identification_number>=0 && 
         current_triangle->v3->identification_number>=0)
      if ( current_triangle->v2->identification_number > 
	   current_triangle->v3->identification_number ||
	  (current_triangle->v2->identification_number < 
	   current_triangle->v3->identification_number  &&
       _PERUNIKA_adjacent_triangle(current_triangle->v3,current_triangle)==NULL)){
	ID1=current_triangle->v2->identification_number*(N_phi_total+1);
	ID2=current_triangle->v3->identification_number*(N_phi_total+1);
	for (N_phi=1; N_phi<=N_phi_total; N_phi++) {
	  fprintf(VRML_model,"%d, %d, %d, %d,-1,\n",
		  ID1+N_phi-1, ID2+N_phi-1, ID2+N_phi, ID1+N_phi);
	}
	fflush(VRML_model);
      }

    if ( current_triangle->v3->type==2 && current_triangle->v1->type==2 &&
	 current_triangle->v3->identification_number>=0 && 
         current_triangle->v1->identification_number>=0)
      if ( current_triangle->v3->identification_number > 
	   current_triangle->v1->identification_number ||
	  (current_triangle->v3->identification_number < 
	   current_triangle->v1->identification_number  &&
       _PERUNIKA_adjacent_triangle(current_triangle->v1,current_triangle)==NULL)){
	ID1=current_triangle->v3->identification_number*(N_phi_total+1);
	ID2=current_triangle->v1->identification_number*(N_phi_total+1);
	for (N_phi=1; N_phi<=N_phi_total; N_phi++) {
	  fprintf(VRML_model,"%d, %d, %d, %d,-1,\n",
		  ID1+N_phi-1, ID2+N_phi-1, ID2+N_phi, ID1+N_phi);
	}
	fflush(VRML_model);
      }

    current_triangle=current_triangle->next;
  } while (current_triangle!=start_triangle_list);

  /* plot the corrections of the sublimation surfaces */
  quadrangle = start_quadrangle_list;
  while (quadrangle!=NULL) {
    ID1 = quadrangle->edge_vertex->identification_number*(N_phi_total+1);
    ID2 = quadrangle->next_edge_vertex->identification_number*(N_phi_total+1); 
    if (quadrangle->new_next_edge_vertex->identification_number > 0)
      ID3 = quadrangle->new_next_edge_vertex->identification_number
             *(N_phi_total+1);
    else 
      ID3 = END_sublim_point_counter*(N_phi_total+1) +
          (-quadrangle->new_next_edge_vertex->identification_number) - 
	  END_sublim_point_counter;
    if (quadrangle->new_edge_vertex->identification_number > 0) 
      ID4 = quadrangle->new_edge_vertex->identification_number*(N_phi_total+1);
    else 
      ID4 = END_sublim_point_counter*(N_phi_total+1) +
           (-quadrangle->new_edge_vertex->identification_number) - 
	   END_sublim_point_counter;
    fprintf(VRML_model,"%d, %d, %d, %d, -1,\n",
      ID1,ID2,ID3,ID4);
    quadrangle = quadrangle->next;
  }

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [\n");

  /* plot the temperature (of the dust component dust_i) as colors */
  sublim_point_counter = 0; /* this counter will fix the ID numbers */
  current_vertex = start_vertex_list;
  do {
    if (current_vertex->identification_number>=0) {
      current_vertex->identification_number=sublim_point_counter;
      sublim_point_counter++; 
    }
    if (current_vertex->type==2 && current_vertex->identification_number>=0) {
      for (N_phi=0; N_phi<=N_phi_total; N_phi++) {
        dust_i = current_vertex->level - 1000;
	if (dust_i<0) 
	  fprintf(VRML_model,"0.2 0.2 0.2,\n");
	else if (current_vertex->T_dust[dust_i] <= 0.0) 
	  fprintf(VRML_model,"0.6 0.6 0.6,\n");
	else if (current_vertex->type == 3) 
	  fprintf(VRML_model,"0.0 0.0 1.0,\n");
	else {
	  color_basic_value = current_vertex->T_dust[dust_i] / 
                              Dust_Sublim_Temp[0];
	  if (color_basic_value<=0.5 ) {
	    color1r=2*color_basic_value; color1g=0; color1b=0;}
	  else {color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
	  fprintf(VRML_model,"%10.6f %10.6f %10.6f,\n",
		  color1r,color1g,color1b);
	}
      } 
      fflush(VRML_model);
    }
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  current_vertex = start_vertex_list;
  do {
    if (current_vertex->identification_number<0) {
      dust_i = current_vertex->level - 1000;
      color_basic_value = current_vertex->T_dust[dust_i] / 
	                  Dust_Sublim_Temp[0];
      if (color_basic_value<=0.5 ) {
	color1r=2*color_basic_value; color1g=0; color1b=0;}
      else {color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
      fprintf(VRML_model,"%10.6f %10.6f %10.6f,\n",
	      color1r,color1g,color1b);

    }
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);


  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            colorPerVertex TRUE\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        }\n");
  fprintf(VRML_model,"  ]\n}\n");



  fclose(VRML_model);



  return 0;
}


