/* LELUYA - VODENJAK program block */
/* This function creates an interactive HTML/VRML file for the grid 
   visualisation */

#include "VODENJAK.h"

int _VODENJAK_plot_interactive_vrml(char image_name[255], 
                        int dust_i, int lambda_k) {
  /* input variables:
  image_name   = name of the file without the suffix ".wrl" 
  dust_i       = dust component
  lambda_k     = wavelength's index
  */

  FILE *VRML_model;
  char VRML_main_filename[259]; /* name of the main HTML file */
  char VRML_vrml_filename[259]; /* name of the HTML file with the VRML model */
  char VRML_wrl_filename[259];  /* name of the VRML source file */
  char VRML_data_filename[259]; /* name of the HTML file with the info about vertices*/

  struct _vertex_structure *current_vertex; /* vertex in the loop over all
                                               vertices */
  struct _triangle_structure *current_triangle; /* triangle in the loop over 
                                               all triangles */
  double color_basic_value, color_basic_value_normalization; /* values
		          which will be transformed into appropriate colors */
  double color1r,color1g,color1b; 

  double box_size  , distance;
  struct _vertex_triangles   *vertex_triangle;
  struct _vertex_structure *adjacent_vertex,*adjacent_antivertex;
  int counter;

  if (my_MPI_rank!=0) return 0;

  /**************************************************************/
  
  if (DEBUG.SCREEN_MESSAGE >= 1 ) 
    printf("\n... plotting the interactive VRML grid ...\n");

  start_vertex_list = grid_per_lambda[lambda_k].start_vertex;
  start_triangle_list = grid_per_lambda[lambda_k].start_triangle;

  /**************************************************************/
  /* the main HTML file */
#ifdef _2DD_MPI_
  sprintf(VRML_main_filename,"%s/%s.html",_LELUYA_MPIOUT_PATH,image_name);
#else
  sprintf(VRML_main_filename,"%s/%s.html",_LELUYA_OUTPUT_PATH,image_name);
#endif
  VRML_model=fopen(VRML_main_filename,"wt");

  fprintf(VRML_model,"<FRAMESET ROWS=\"*,300\">\n");
  fprintf(VRML_model,"<FRAME SRC=\"%s.vrml.html\" ",image_name);
  fprintf(VRML_model,"NAME=\"VRML_window\"  SCROLLING=\"no\" MARGINWIDTH=\"2\" ");
  fprintf(VRML_model,"MARGINHEIGHT=\"2\" BORDER=\"0\" NORESIZE>\n");
  fprintf(VRML_model,"<FRAME SRC=\"%s.data.html\" ",image_name);
  fprintf(VRML_model,"NAME=\"vertex_data_window\" SCROLLING=\"auto\" ");
  fprintf(VRML_model,"MARGINWIDTH=\"2\" MARGINHEIGHT=\"2\" BORDER=\"0\" NORESIZE>\n");
  fprintf(VRML_model,"</FRAMESET>\n");

  fclose(VRML_model);

  /**************************************************************/
  /* the HTML file with the VRML model */

#ifdef _2DD_MPI_
 sprintf(VRML_vrml_filename,"%s/%s.vrml.html",_LELUYA_MPIOUT_PATH,image_name);
#else
 sprintf(VRML_vrml_filename,"%s/%s.vrml.html",_LELUYA_OUTPUT_PATH,image_name);
#endif
  VRML_model=fopen(VRML_vrml_filename,"wt");

  fprintf(VRML_model,"<HTML><BODY BGCOLOR=#000000>\n");
  fprintf(VRML_model,"<EMBED SRC=\"%s.vrml.wrl\" ",image_name);
  fprintf(VRML_model,"WIDTH=100%% HEIGHT=100%% BORDER=NONE></BODY>\n</HTML>");

  fclose (VRML_model);

  /**************************************************************/
  /* the VRML source file */

#ifdef _2DD_MPI_
  sprintf(VRML_wrl_filename,"%s/%s.vrml.wrl",_LELUYA_MPIOUT_PATH,image_name);
#else
  sprintf(VRML_wrl_filename,"%s/%s.vrml.wrl",_LELUYA_OUTPUT_PATH,image_name);
#endif
  VRML_model=fopen(VRML_wrl_filename,"wt");

  /* introduction of the VRML file and coordinate axes */
  fprintf(VRML_model,"#VRML V2.0 utf8\n\n");
  fprintf(VRML_model,"Viewpoint {\n");
  fprintf(VRML_model,"  position %lf %lf %lf  orientation 0 0 -1 0\n",0.5*Rout,0.5*Rout,2*Rout);
  fprintf(VRML_model,"  description \"Start\"\n}\n");
  fprintf(VRML_model,"Viewpoint {\n");
  fprintf(VRML_model,"  position 0 0 10 orientation 0 0 -1 0\n");
  fprintf(VRML_model,"  description \"Center - Top\"\n}\n");
  fprintf(VRML_model,"###########################################\n");
  fprintf(VRML_model,"DirectionalLight { on TRUE direction 1 1 -1 }\n\n");
  fprintf(VRML_model,"Shape {geometry Sphere { radius %lf }\n",Source_Radius);
  fprintf(VRML_model,"appearance Appearance {material Material {\n");
  fprintf(VRML_model,"diffuseColor .8 .51 .09 specularColor .92 .43 .01 ");
  fprintf(VRML_model,"ambientIntensity .117 shininess .4\n");
  fprintf(VRML_model,"} } }\n\n");
  fprintf(VRML_model,"# coordinate lines\n");
  fprintf(VRML_model,"Transform {\n");
  fprintf(VRML_model,"  children [\n");
  fprintf(VRML_model,"    Shape {\n ");
  fprintf(VRML_model,"         geometry IndexedLineSet {\n ");
  fprintf(VRML_model,"                  coord Coordinate {\n ");
  fprintf(VRML_model,"                        point [ 0 0 -0.01, ");
  fprintf(VRML_model,"%lf 0 -0.01 , 0 %lf -0.01]\n ",Rout,Rout);
  fprintf(VRML_model,"                        }\n ");
  fprintf(VRML_model,"                  coordIndex [ 0, 1, -1, 0, 2, -1, ]\n");
  fprintf(VRML_model,"                  } \n         }\n]\n");
  fprintf(VRML_model,"} #coordinate lines\n\n");
  fprintf(VRML_model,"############################################\n");
  fprintf(VRML_model,"# names of the coordinate lines\n");
  fprintf(VRML_model,"Transform {\n");
  fprintf(VRML_model,"  children [\n");
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      translation %lf 0 0\n",1.1*Rout);
  fprintf(VRML_model,"      children [ \n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry Text { \n");
  fprintf(VRML_model,"            string \"X\" \n");
  fprintf(VRML_model,"            fontStyle FontStyle { size 20 ");
  fprintf(VRML_model,"family \"SERIF\" style \"BOLD\" }\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        } \n");
  fprintf(VRML_model,"      ]\n");
  fprintf(VRML_model,"    } # end of X-name\n");
  fprintf(VRML_model,"   Transform {\n");
  fprintf(VRML_model,"      translation 0 %lf 0 \n",1.1*Rout);
  fprintf(VRML_model,"      children [ \n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry Text {\n"); 
  fprintf(VRML_model,"            string \"Z\" \n");
  fprintf(VRML_model,"            fontStyle FontStyle { size 20 ");
  fprintf(VRML_model,"family \"SERIF\" style \"BOLD\" }\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        } \n");
  fprintf(VRML_model,"      ]\n");
  fprintf(VRML_model,"    } # end of Z-name\n");
  fprintf(VRML_model,"  ]\n");
  fprintf(VRML_model,"}\n");
  fprintf(VRML_model,"############################################\n");
  fprintf(VRML_model,"DEF DefaultView Viewpoint { position %lf %lf %lf ",
                     0.5*Rout, 0.5*Rout,2*Rout);
  fprintf(VRML_model,"description \"Default View\"}\n");
  fprintf(VRML_model,"DEF VertexView Viewpoint { position %lf %lf %lf ",
                     0.5*Rout, 0.5*Rout,2*Rout);
  fprintf(VRML_model,"description \"Current View\"}\n");
  fprintf(VRML_model,"############################################\n");
  fprintf(VRML_model,"# Triangles\n");
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      children [\n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          appearance Appearance { material ");
  fprintf(VRML_model,"Material { }}\n");
  fprintf(VRML_model,"          geometry IndexedLineSet{\n");
  fprintf(VRML_model,"            coord Coordinate {\n");
  fprintf(VRML_model,"              point [ \n");

  /* plot the verticess */
  current_vertex = start_vertex_list;
  do {
    /* put the vertex into the VRML file */
    fprintf(VRML_model,"%12.6lf %12.6lf 0.0,\n",
            current_vertex->x, current_vertex->z);
    fflush(VRML_model);
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [\n");

  /* connect the points into a 3D structure */
  current_triangle = start_triangle_list;
  do {
    /* plot the lines only if the first point has a larger ID-number, or
       if this line doesn't have the adjacent triangle */
    if ((current_triangle->v1->identification_number > 
	 current_triangle->v2->identification_number) ||
	(current_triangle->v1->identification_number < 
	 current_triangle->v2->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v2,current_triangle)==NULL))
      fprintf(VRML_model,"%d, %d, -1,\n",
              current_triangle->v1->identification_number,
              current_triangle->v2->identification_number);
    if ((current_triangle->v2->identification_number > 
	 current_triangle->v3->identification_number) ||
	(current_triangle->v2->identification_number < 
	 current_triangle->v3->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v3,current_triangle)==NULL))
      fprintf(VRML_model,"%d, %d, -1,\n",
              current_triangle->v2->identification_number,
              current_triangle->v3->identification_number);
    if ((current_triangle->v3->identification_number > 
	 current_triangle->v1->identification_number) ||
	(current_triangle->v3->identification_number < 
	 current_triangle->v1->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v1,current_triangle)==NULL))
      fprintf(VRML_model,"%d, %d, -1,\n",
              current_triangle->v3->identification_number,
              current_triangle->v1->identification_number);

    current_triangle=current_triangle->next;
  } while (current_triangle!=start_triangle_list);


  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [\n");

  /* plot the temperature (of the dust component dust_i) as colors */
  color_basic_value_normalization = log10(Dust_Sublim_Temp[dust_i]);
  current_vertex = start_vertex_list;
  while (current_vertex!=NULL) {
    /* calculate the point colors */
    if (current_vertex->T_dust[dust_i] <= 0.0) {
      fprintf(VRML_model,"0.6 0.6 0.6,\n");
    }
    else if (current_vertex->type == 3) {
      fprintf(VRML_model,"0.2 0.2 0.2,\n");
    }
    else if (current_vertex->type == 2 && current_vertex->level>666) {
      fprintf(VRML_model,"0.0 0.0 1.0,\n");
    }
    else {
      if (current_vertex->type == 2)
        fprintf(VRML_model,"0.0 0.0 1.0,\n");
      else {
        color_basic_value = log10(current_vertex->T_dust[dust_i]) /
	                       color_basic_value_normalization;
        if (color_basic_value<=0.5 ) {
          color1r=2*color_basic_value; color1g=0; color1b=0;}
        else {
          color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
        fprintf(VRML_model,"%10.6f %10.6f %10.6f,\n",
                color1r,color1g,color1b);
		} 
    }
    fflush(VRML_model);
    current_vertex = current_vertex->next;
    if (current_vertex==start_vertex_list) current_vertex=NULL;
  }

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            colorPerVertex TRUE\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        }\n");
  fprintf(VRML_model,"  ]\n}\n");
  fprintf(VRML_model,"############################################\n");
  fprintf(VRML_model,"#boxs at vertices\n");
  fprintf(VRML_model,"PROTO VertexAnchor [\n");
  fprintf(VRML_model,"  exposedField SFVec3f  position 0 0 0\n");
  fprintf(VRML_model,"  exposedField SFVec3f  scale 1 1 1\n");
  fprintf(VRML_model,"  exposedField SFColor  color 1 1 1\n");
  fprintf(VRML_model,"  exposedField SFString name \"Default Name\"\n");
  fprintf(VRML_model,"  exposedField MFString data_URL \"%s\"]{\n",
          VRML_wrl_filename);
  fprintf(VRML_model,"  Anchor { description IS name \n");
  fprintf(VRML_model,"    parameter [ \"target=vertex_data_window\" ]\n");
  fprintf(VRML_model,"    url IS data_URL\n");
  fprintf(VRML_model,"    children[ DEF VertexAnchorBox Transform {\n");
  fprintf(VRML_model,"        translation IS position\n");
  fprintf(VRML_model,"        scale IS scale\n");
  fprintf(VRML_model,"        children [ Shape { \n");
  fprintf(VRML_model,"            appearance Appearance {\n");
  fprintf(VRML_model,"material Material {diffuseColor IS color}\n");
  fprintf(VRML_model,"            }geometry Box { }}]}]\n}\n}\n");

  fflush(VRML_model);
  /* plot the boxes */
  current_vertex = start_vertex_list;
  do {
    fprintf(VRML_model,"DEF Vertex%.4d VertexAnchor{",
            current_vertex->identification_number);    fflush(VRML_model);
    fprintf(VRML_model,"position %12.6lf %12.6lf 0.0 \n",
            current_vertex->x, current_vertex->z);
    fflush(VRML_model);
    /* with need the box's radius */
    box_size = Rout;
    vertex_triangle = current_vertex->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
	(current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      adjacent_vertex = _PERUNIKA_adjacent_antivertex_in_triangle
	(current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance < box_size) box_size = distance;
      vertex_triangle = vertex_triangle->next;
    }
    box_size /= 5.0;
    fprintf(VRML_model,"scale %12.6lf %12.6lf %12.6lf ",
            box_size,box_size,box_size);
    /* find the box's color */
    if (current_vertex->T_dust[dust_i] <= 0.0)
      fprintf(VRML_model,"color 0.6 0.6 0.6 ");
    else if (current_vertex->type == 3) 
      fprintf(VRML_model,"color 0.2 0.2 0.2 ");
    else if (current_vertex->type == 2 && current_vertex->level>666) 
      fprintf(VRML_model,"color 0 0 1.0 ");
    else {
      if (current_vertex->type == 2) fprintf(VRML_model,"color 0.0 0.0 1.0,\n");
      else {
        color_basic_value = log10(current_vertex->T_dust[dust_i]) /
	                       color_basic_value_normalization;
        if (color_basic_value<=0.5 ) {color1r=2*color_basic_value;color1g=0;color1b=0;}
        else {color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
        fprintf(VRML_model,"color %10.6f %10.6f %10.6f,\n",color1r,color1g,color1b);
      }
    }
    fflush(VRML_model);
    fprintf(VRML_model,"name \"Vertex%.4d\" ",current_vertex->identification_number);
    fflush(VRML_model);
    fprintf(VRML_model,"data_URL \"%s.data.html#Vertex%.4d\"}\n",image_name,
            current_vertex->identification_number);
    fflush(VRML_model);
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fclose(VRML_model);

  /**************************************************************/
  /* the HTML file with the info about vertices*/

#ifdef _2DD_MPI_
  sprintf(VRML_data_filename,"%s/%s.data.html",_LELUYA_MPIOUT_PATH,image_name);
#else
  sprintf(VRML_data_filename,"%s/%s.data.html",_LELUYA_OUTPUT_PATH,image_name);
#endif
  VRML_model=fopen(VRML_data_filename,"wt");

  fprintf(VRML_model,"<HTML><HEAD><SCRIPT LANGUAGE=\"JavaScript\">\n");
  fprintf(VRML_model,"<!-- START HIDE\n");
  fprintf(VRML_model,"function setVRML() {\n");
  fprintf(VRML_model,"  newBrowser = top.VRML_window.document.embeds[0];\n");
  fprintf(VRML_model,"  CV = \"Vertex0001\"; CVAnchor = newBrowser.getNode(CV);\n");
  fprintf(VRML_model,"  CVColor = CVAnchor.getEventOut(\"color_changed\");\n");
  fprintf(VRML_model,"  CVColorValue = CVColor.getValue();\n");
  fprintf(VRML_model,"  WhiteColor = CVColor.getValue();\n");
  fprintf(VRML_model,"  WhiteColor[0] = 1;WhiteColor[1] = 1;WhiteColor[2] = 1;\n");
  fprintf(VRML_model,"  CVSetColor = CVAnchor.getEventIn(\"set_color\");\n");
  fprintf(VRML_model,"  CVView = newBrowser.getNode(\"VertexView\");\n");
  fprintf(VRML_model,"  CVViewSetPosition = CVView.getEventIn(\"set_position\");\n");
  fprintf(VRML_model,"  CVViewActive = CVView.getEventIn(\"set_bind\");\n}\n");
  fprintf(VRML_model,"function setVC(ID) {\n");
  fprintf(VRML_model,"  CVSetColor.setValue(CVColorValue);\n");
  fprintf(VRML_model,"  if (ID<10) CV=\"Vertex000\"+ID;\n"); 
  fprintf(VRML_model,"  else if (ID<100) CV=\"Vertex00\"+ID;\n"); 
  fprintf(VRML_model,"  else if (ID<1000) CV=\"Vertex0\"+ID;\n"); 
  fprintf(VRML_model,"  else CV=\"Vertex\"+ID;\n"); 
  fprintf(VRML_model,"  CVAnchor = newBrowser.getNode(CV);\n");
  fprintf(VRML_model,"  CVColor = CVAnchor.getEventOut(\"color_changed\");\n");
  fprintf(VRML_model,"  CVColorValue = CVColor.getValue();\n");
  fprintf(VRML_model,"  CVSetColor = CVAnchor.getEventIn(\"set_color\");\n");
  fprintf(VRML_model,"  CVSetColor.setValue(WhiteColor);\n}\n");
  fprintf(VRML_model,"function setVV(ID,x,y,z) {\n");
  fprintf(VRML_model,"  CVViewPosition = CVView.getEventOut(\"position_changed\");\n");
  fprintf(VRML_model,"  CVViewPosVal = CVViewPosition.getValue();\n");
  fprintf(VRML_model,"  CVViewPosVal[0] += (x - CVViewPosVal[0]);\n");
  fprintf(VRML_model,"  CVViewPosVal[1] += (y - CVViewPosVal[1]);\n");
  fprintf(VRML_model,"  CVViewPosVal[2] += (z - CVViewPosVal[2]);\n");
  fprintf(VRML_model,"  CVViewSetPosition.setValue(CVViewPosVal);\n");
  fprintf(VRML_model,"  CVViewActiveValue = true;\n");
  fprintf(VRML_model,"  CVViewActive.setValue(CVViewActiveValue);\n");
  fprintf(VRML_model,"  setVC(ID);\n}\n");
  fprintf(VRML_model,"function RecoverVertex(){\n");
  fprintf(VRML_model,"  CVSetColor.setValue(CVColorValue);\n}\n");
  fprintf(VRML_model,"// END HIDE -->\n</SCRIPT></HEAD>\n");
  fprintf(VRML_model,"<BODY text=\"#FFFFFF\" bgcolor=\"#000000\" link=\"#FFFF00\" ");
  fprintf(VRML_model,"vlink=\"#3366FF\" alink=\"#FF0000\" onLoad=\"setVRML()\" ");
  fprintf(VRML_model,"onUnload=\"RecoverVertex()\">\n");
  fprintf(VRML_model,"<FORM NAME=\"Vertices\">\n");

  /* write the data at each grid vertex */ 
  current_vertex = start_vertex_list;
  do {
    fprintf(VRML_model,"<font color=\"#000000\"><A NAME=\"Vertex%.4d\">\n",
	    current_vertex->identification_number);
    fprintf(VRML_model,"<INPUT TYPE=\"button\" VALUE=\"Vertex%d\"",
	    current_vertex->identification_number);
    fprintf(VRML_model,"onClick=\"setVV(%d, %9.4lf, %0.4lf, ",
	    current_vertex->identification_number,current_vertex->x,current_vertex->z);
    /* find the distance from the x-z plane */
    box_size = 0.0;
    vertex_triangle = current_vertex->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
	(current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance > box_size) box_size = distance;
      adjacent_vertex = _PERUNIKA_adjacent_antivertex_in_triangle
	(current_vertex, vertex_triangle->triangle);
      distance = _MATH_LIB_distance(current_vertex->x, current_vertex->z,
                                    adjacent_vertex->x, adjacent_vertex->z);
      if (distance > box_size) box_size = distance;
      vertex_triangle = vertex_triangle->next;
    }
    box_size *= 4.0;
    fprintf(VRML_model,"%9.4lf)\"> </A></font>",box_size);
    for (counter=0; counter<Max_Dust_Type; counter++)
      fprintf(VRML_model,"<b>T[%.2d]</b>=%8.3lf ",counter,
              current_vertex->T_dust[counter]);
    fprintf(VRML_model,"x=%9.4lf y=%9.4lf Eta=%8.6E \n",
            current_vertex->x,current_vertex->z,current_vertex->Eta);

    /* adjacent vertices */
    /*    fprintf(VRML_model,"<font color=\"#000000\">");
    adjacent_antivertex=NULL; 
    vertex_triangle = current_vertex->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
	(current_vertex, vertex_triangle->triangle);
      if (adjacent_vertex!=adjacent_antivertex) {
      fprintf(VRML_model,"<INPUT TYPE=\"button\" VALUE=\"%.4d\" ",
	      adjacent_vertex->identification_number);
      fprintf(VRML_model,"onClick=\"setVC(%d)\">\n",
	      adjacent_vertex->identification_number);
      }
      adjacent_antivertex = _PERUNIKA_adjacent_antivertex_in_triangle
	(current_vertex, vertex_triangle->triangle);
      fprintf(VRML_model,"<INPUT TYPE=\"button\" VALUE=\"%.4d\" ",
              adjacent_antivertex->identification_number);
      fprintf(VRML_model,"onClick=\"setVC(%d)\">\n",
	      adjacent_antivertex->identification_number);
      vertex_triangle = vertex_triangle->next;
    }
    fprintf(VRML_model,"</font><br>\n");*/

    /* adjacent vertices */
    fprintf(VRML_model,"(");
    adjacent_antivertex=NULL; 
    vertex_triangle = current_vertex->triangles;
    while (vertex_triangle!=NULL) {
      adjacent_vertex = _PERUNIKA_adjacent_vertex_in_triangle
	(current_vertex, vertex_triangle->triangle);
      if (adjacent_vertex!=adjacent_antivertex)
        fprintf(VRML_model,"%.4d ",adjacent_vertex->identification_number);
      adjacent_antivertex = _PERUNIKA_adjacent_antivertex_in_triangle
	(current_vertex, vertex_triangle->triangle);
      fprintf(VRML_model,"%.4d ",adjacent_antivertex->identification_number);
      vertex_triangle = vertex_triangle->next;
    }
    fprintf(VRML_model,")<br>\n");

    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fprintf(VRML_model,"\n");
  fprintf(VRML_model,"\n");
  fprintf(VRML_model,"\n");
  fprintf(VRML_model,"\n");
  fprintf(VRML_model,"\n");

  fprintf(VRML_model,"</FORM>\n</BODY></HTML>\n");

  fclose(VRML_model);

  return 0;
}


