/* LELUYA - VODENJAK program block */
/* This function creates a VRML file for the grid visualisation */

#include "VODENJAK.h"

int _VODENJAK_plot_vrml(char image_name[255],int vrml_type, 
                        int dust_i, int lambda_k) {
  /* input variables:
  image_name   = name of the file without the suffix ".wrl" 
  vrml_type    = plotting type: 0 = just the X-Z grid
                                1 = 3D plot, 3rd dimension is Eta, that is 
                                    the points are (x,z,0) and (x,z,Eta),
                                    where x and z are the vertex coordinates, 
                                    color represents the temperature
                                2 = 3D plot, 3rd dimension is the
                                    ratio between grid_per_lambda[].Utot_new 
                                    and grid_per_lambda[].Ustar+Uem+Usca
                                    (color represents the radial optical depth)
                                3 = 3D plot, 3rd dimension is 200K/temperature
                                    and color is the temperature's error
                                    (T_error[dust_i] at each vertex)
  dust_i       = dust component
  lambda_k     = wavelength's index
  */

  FILE *VRML_model;
  char filename[259];  /* name of the file */

  struct _vertex_structure *current_vertex; /* vertex in the loop over all
                                               vertices */
  struct _triangle_structure *current_triangle; /* triangle in the loop over 
                                               all triangles */
  double color_basic_value, color_basic_value_normalization; /* values
		          which will be transformed into appropriate colors */
  double color1r,color1g,color1b; 
  double U_ratio;
  double r;

  struct _3DVector startpoint;/* start point in tau-integrals */
  struct _3DVector endpoint;  /* end point in tau-integrals */

  if (my_MPI_rank!=0) return 0;
  /**************************************************************/
  
  if (DEBUG.SCREEN_MESSAGE >= 1 ) 
    printf("\n... plotting the VRML grid models ...\n");

  sprintf(filename,"%s.wrl",image_name);
  VRML_model=fopen(filename,"wt");

  /* introduction of the VRML file and coordinate axes */
  fprintf(VRML_model,"#VRML V2.0 utf8\n\n");
  fprintf(VRML_model,"###########################################\n");
  fprintf(VRML_model,"# coordinate lines\n");
  fprintf(VRML_model,"Transform {\n");
  fprintf(VRML_model,"  children [\n");
  fprintf(VRML_model,"    Viewpoint { position %lf %lf %lf }\n ",
                     0.5*Rout, 0.5*Rout,2*Rout);
  fprintf(VRML_model,"    Shape {\n ");
  fprintf(VRML_model,"         geometry IndexedLineSet {\n ");
  fprintf(VRML_model,"                  coord Coordinate {\n ");
  fprintf(VRML_model,"                        point [ 0 0 -0.01, ");
  fprintf(VRML_model,"%lf 0 -0.01 , 0 %lf -0.01]\n ",Rout,Rout);
  fprintf(VRML_model,"                        }\n ");
  fprintf(VRML_model,"                  coordIndex [ 0, 1, -1, 0, 2, -1, ]\n");
  fprintf(VRML_model,"                  } \n         }\n]\n");
  fprintf(VRML_model,"} #coordinate lines\n\n");
  fprintf(VRML_model,"############################################\n");
  fprintf(VRML_model,"# names of the coordinate lines\n");
  fprintf(VRML_model,"Transform {\n");
  fprintf(VRML_model,"  children [\n");
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      translation %lf 0 0\n",1.1*Rout);
  fprintf(VRML_model,"      children [ \n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry Text { \n");
  fprintf(VRML_model,"            string \"X\" \n");
  fprintf(VRML_model,"            fontStyle FontStyle { size 20 ");
  fprintf(VRML_model,"family \"SERIF\" style \"BOLD\" }\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        } \n");
  fprintf(VRML_model,"      ]\n");
  fprintf(VRML_model,"    } # end of X-name\n");
  fprintf(VRML_model,"   Transform {\n");
  fprintf(VRML_model,"      translation 0 %lf 0 \n",1.1*Rout);
  fprintf(VRML_model,"      children [ \n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry Text {\n"); 
  fprintf(VRML_model,"            string \"Z\" \n");
  fprintf(VRML_model,"            fontStyle FontStyle { size 20 ");
  fprintf(VRML_model,"family \"SERIF\" style \"BOLD\" }\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        } \n");
  fprintf(VRML_model,"      ]\n");
  fprintf(VRML_model,"    } # end of Z-name\n");
  fprintf(VRML_model,"   Transform {\n");
  fprintf(VRML_model,"      rotation 1 0 0 1.57079633\n");
  fprintf(VRML_model,"      translation 0 0 -0.015\n");
  fprintf(VRML_model,"      children [ \n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"         appearance Appearance {");
  fprintf(VRML_model,"material Material { emissiveColor .3 .3 .3}}\n");
  fprintf(VRML_model,"         geometry Cylinder {");
  fprintf(VRML_model,"radius 1 height 0.01 top  TRUE}\n");
  fprintf(VRML_model,"        } \n");
  fprintf(VRML_model,"      ]\n");
  fprintf(VRML_model,"    } # end of the R=1 disk\n");
  fprintf(VRML_model,"  ]\n");
  fprintf(VRML_model,"}\n");
  fprintf(VRML_model,"############################################\n");


  if (vrml_type==0 || vrml_type==1 || vrml_type==2 || vrml_type==3) {
    fprintf(VRML_model,"# Triangles\n");
    fprintf(VRML_model,"    Transform {\n");
    fprintf(VRML_model,"      children [\n");
    fprintf(VRML_model,"        Shape {\n");
    fprintf(VRML_model,"          appearance Appearance { material ");
    fprintf(VRML_model,"Material { }}\n");
    fprintf(VRML_model,"          geometry IndexedLineSet{\n");
    /*
  if (vrml_type==0) fprintf(VRML_model,"          geometry IndexedLineSet{\n");
  else fprintf(VRML_model,"          geometry IndexedFaceSet{\n");*/
    fprintf(VRML_model,"            coord Coordinate {\n");
    fprintf(VRML_model,"              point [ \n");

    /* plot the verticess */
    current_vertex = start_vertex_list;
    do {
      /* put the vertex into the VRML file */
      fprintf(VRML_model,"%12.6lf %12.6lf 0.0,\n",
              current_vertex->x, current_vertex->z);
      fflush(VRML_model);
      current_vertex = current_vertex->next;
    } while (current_vertex!=start_vertex_list);

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            }\n");
    fprintf(VRML_model,"            coordIndex [\n");

    /* connect the points into a 3D structure */
    current_triangle = start_triangle_list;
    do {
      /* plot the lines only if the first point has a larger ID-number, or
         if this line doesn't have the adjacent triangle */
      if ((current_triangle->v1->identification_number > 
	   current_triangle->v2->identification_number) ||
	  (current_triangle->v1->identification_number < 
	   current_triangle->v2->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v2,current_triangle)==NULL))
        fprintf(VRML_model,"%d, %d, -1,\n",
                current_triangle->v1->identification_number,
                current_triangle->v2->identification_number);
      if ((current_triangle->v2->identification_number > 
	   current_triangle->v3->identification_number) ||
	  (current_triangle->v2->identification_number < 
	   current_triangle->v3->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v3,current_triangle)==NULL))
        fprintf(VRML_model,"%d, %d, -1,\n",
                current_triangle->v2->identification_number,
                current_triangle->v3->identification_number);
      if ((current_triangle->v3->identification_number > 
	   current_triangle->v1->identification_number) ||
	  (current_triangle->v3->identification_number < 
	   current_triangle->v1->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v1,current_triangle)==NULL))
        fprintf(VRML_model,"%d, %d, -1,\n",
                current_triangle->v3->identification_number,
                current_triangle->v1->identification_number);

      current_triangle=current_triangle->next;
    } while (current_triangle!=start_triangle_list);


    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            color Color {\n");
    fprintf(VRML_model,"              color [\n");

    /* plot the temperature (of the dust component dust_i) as colors */
    color_basic_value_normalization = log10(Dust_Sublim_Temp[dust_i]);
    current_vertex = start_vertex_list;
    while (current_vertex!=NULL) {
      /* calculate the point colors */
      if (current_vertex->T_dust[dust_i] <= 0.0) {
        fprintf(VRML_model,"0.6 0.6 0.6,\n");
      }
      else if (current_vertex->type == 3) {
        fprintf(VRML_model,"0.2 0.2 0.2,\n");
      }
      else if (current_vertex->type == 2 && current_vertex->level>666) {
        fprintf(VRML_model,"0.0 0.0 1.0,\n");
      }
      else {
        if (current_vertex->type == 2)
        fprintf(VRML_model,"0.0 0.0 1.0,\n");
        else fprintf(VRML_model,"0.0 1.0 0.0,\n");
        /* else {
          color_basic_value = log10(current_vertex->T_dust[dust_i]) /
	                         color_basic_value_normalization;
          if (color_basic_value<=0.5 ) {
            color1r=2*color_basic_value; color1g=0; color1b=0;}
          else {
            color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
          fprintf(VRML_model,"                %10.6f %10.6f %10.6f,\n",
                  color1r,color1g,color1b);
		  }*/ 
      }
      fflush(VRML_model);
      current_vertex = current_vertex->next;
      if (current_vertex==start_vertex_list) current_vertex=NULL;
    }

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            }\n");
    fprintf(VRML_model,"            colorPerVertex TRUE\n");
    fprintf(VRML_model,"          }\n");
    fprintf(VRML_model,"        }\n");
    fprintf(VRML_model,"  ]\n}\n");
  }

  if (vrml_type==1) {
    fprintf(VRML_model,"# Triangles: surface\n");
    fprintf(VRML_model,"    Transform {\n");
    fprintf(VRML_model,"      children [\n");
    fprintf(VRML_model,"        Shape {\n");
    fprintf(VRML_model,"          appearance Appearance { material ");
    fprintf(VRML_model,"Material { }}\n");
    fprintf(VRML_model,"          geometry IndexedFaceSet{\n");
    fprintf(VRML_model,"            solid FALSE\n"); 
    fprintf(VRML_model,"            coord Coordinate {\n");
    fprintf(VRML_model,"              point [ \n");

    /* plot the verticess */
    current_vertex = start_vertex_list;
    do {
      fprintf(VRML_model,"%12.6lf %12.6lf %E,\n",
              current_vertex->x,current_vertex->z,
              Eta_Denominator*current_vertex->Eta);
      fflush(VRML_model);
      current_vertex = current_vertex->next;
    } while (current_vertex!=start_vertex_list);

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            }\n");
    fprintf(VRML_model,"            coordIndex [\n");

    /* triangles */
    current_triangle = start_triangle_list;
    do {
      fprintf(VRML_model,"%d, %d, %d, -1,\n",
              current_triangle->v1->identification_number,
              current_triangle->v2->identification_number,
              current_triangle->v3->identification_number);
      fflush(VRML_model);
      current_triangle=current_triangle->next;
    } while(current_triangle!=start_triangle_list);

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            color Color {\n");
    fprintf(VRML_model,"              color [\n");

    /* plot the temperature (of the dust component dust_i) as colors */
    color_basic_value_normalization = log10(Dust_Sublim_Temp[dust_i]);
    current_vertex = start_vertex_list;
    do {
      /* calculate the point colors */
      if (current_vertex->T_dust[dust_i] <= 0.0) {
        fprintf(VRML_model,"0.6 0.6 0.6,\n");
      }
      else if (current_vertex->type == 3) {
        fprintf(VRML_model,"0.0 0.0 1.0,\n");
      }
      else {
        color_basic_value = log10(current_vertex->T_dust[dust_i]) /
	                         color_basic_value_normalization;
        if (color_basic_value<=0.5 ) {
          color1r=2*color_basic_value; color1g=0; color1b=0;}
        else {
          color1r=1; color1g=2*(color_basic_value-0.5); color1b=0;}
        fprintf(VRML_model,"%10.6f %10.6f %10.6f,\n",
                color1r,color1g,color1b);
      }
      fflush(VRML_model);
      current_vertex = current_vertex->next;
    } while (current_vertex!=start_vertex_list);

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            }\n");
    fprintf(VRML_model,"            colorPerVertex TRUE\n");
    fprintf(VRML_model,"          }\n");
    fprintf(VRML_model,"        }\n");
    fprintf(VRML_model,"  ]\n}\n");
  }


  if (vrml_type==2) {
    fprintf(VRML_model,"# Triangles: surface\n");
    fprintf(VRML_model,"    Transform {\n");
    fprintf(VRML_model,"      children [\n");
    fprintf(VRML_model,"        Shape {\n");
    fprintf(VRML_model,"          appearance Appearance { material ");
    fprintf(VRML_model,"Material { }}\n");
    fprintf(VRML_model,"          geometry IndexedFaceSet{\n");
    fprintf(VRML_model,"            solid FALSE\n"); 
    fprintf(VRML_model,"            coord Coordinate {\n");
    fprintf(VRML_model,"              point [ \n");

    /* plot the verticess */
    current_vertex = grid_per_lambda[lambda_k].start_vertex;
    do {

      r = sqrt(current_vertex->x*current_vertex->x
	       +current_vertex->z*current_vertex->z);
      U_ratio =
   grid_per_lambda[lambda_k].Ustar[current_vertex->identification_number] +
   grid_per_lambda[lambda_k].Usca [current_vertex->identification_number] +
   grid_per_lambda[lambda_k].Uem  [current_vertex->identification_number];

      if (U_ratio==0.0) U_ratio = 0.5*r;
      else {
	U_ratio = 
 grid_per_lambda[lambda_k].Utot_new[current_vertex->identification_number] /
 U_ratio;
        /*U_ratio = log10(U_ratio);*/
        if (U_ratio<1.0 && U_ratio!=0.0) U_ratio= -1.0/U_ratio;
        if (U_ratio>0.0) U_ratio = U_ratio/4.0 - 0.25;
        else if (U_ratio<0.0) U_ratio = U_ratio/4.0 + 0.25;
	if (fabs(U_ratio)>0.5) U_ratio = (U_ratio>=0) ? 0.5*r : -0.5*r;
        else U_ratio = U_ratio*r;
      }
      /* put the vertex into the VRML file */
      fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n",
              current_vertex->x, current_vertex->z, U_ratio);
      fflush(VRML_model);
      current_vertex = current_vertex->next;
    } while (current_vertex!=grid_per_lambda[lambda_k].start_vertex);

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            }\n");
    fprintf(VRML_model,"            coordIndex [\n");

    /* triangles */
    current_triangle = grid_per_lambda[lambda_k].start_triangle;
    do {
      fprintf(VRML_model,"%d, %d, %d, -1,\n",
              current_triangle->v1->identification_number,
              current_triangle->v2->identification_number,
              current_triangle->v3->identification_number);
      fflush(VRML_model);
      current_triangle=current_triangle->next;
    } while(current_triangle!=grid_per_lambda[lambda_k].start_triangle);

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            color Color {\n");
    fprintf(VRML_model,"              color [\n");

    /* plot colors */
    current_vertex = grid_per_lambda[lambda_k].start_vertex;
    while (current_vertex!=NULL) {

      U_ratio =
   grid_per_lambda[lambda_k].Ustar[current_vertex->identification_number] +
   grid_per_lambda[lambda_k].Usca [current_vertex->identification_number] +
   grid_per_lambda[lambda_k].Uem  [current_vertex->identification_number];
      if (U_ratio==0.0) U_ratio = 1.0;
      else {
	U_ratio = 
 grid_per_lambda[lambda_k].Utot_new[current_vertex->identification_number] /
 U_ratio;
        /*U_ratio = log10(U_ratio);*/
        if (U_ratio<1.0 && U_ratio!=0.0) U_ratio= 1.0/U_ratio;
        U_ratio /= 10.0;
	if (fabs(U_ratio)>1.0) U_ratio = 1.0;
      }

      /* hej, let's see the radial optical depth !*/
      r = sqrt(current_vertex->x*current_vertex->x
	       +current_vertex->z*current_vertex->z);
      startpoint.x=0.1*current_vertex->x/r; 
      startpoint.y=0.0; 
      startpoint.z=0.1*current_vertex->z/r;
      endpoint.x=current_vertex->x;endpoint.y=0.0;endpoint.z=current_vertex->z;
      U_ratio = _ZELENJAK_tau_over_grid (startpoint,endpoint,lambda_k,FALSE);
      U_ratio /= grid_per_lambda[lambda_k].tau;

      /* calculate the point colors */
      color_basic_value = fabs(U_ratio);
      if (color_basic_value>1.0) color_basic_value=1.0;

      if (color_basic_value<=0.3333 ) {
	color1r=3.0*color_basic_value; color1g=0; color1b=0;}
      else if (color_basic_value>0.3333 && color_basic_value<=0.6666) {
	color1r=1.0; color1g=3.0*(color_basic_value-0.3333); color1b=0;}
      else { color1r=1; color1g=1.0; color1b=3.0*(color_basic_value-0.6666);}
      fprintf(VRML_model,"                %10.6f %10.6f %10.6f,\n",
	      color1r,color1g,color1b);

      fflush(VRML_model);
      current_vertex = current_vertex->next;
      if (current_vertex==grid_per_lambda[lambda_k].start_vertex) 
	current_vertex=NULL;
    }

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            }\n");
    fprintf(VRML_model,"            colorPerVertex TRUE\n");
    fprintf(VRML_model,"          }\n");
    fprintf(VRML_model,"        }\n");
    fprintf(VRML_model,"  ]\n}\n");
  } /* end of vrml_type=2 */


  if (vrml_type==3) {
    fprintf(VRML_model,"# Triangles: surface\n");
    fprintf(VRML_model,"    Transform {\n");
    fprintf(VRML_model,"      children [\n");
    fprintf(VRML_model,"        Shape {\n");
    fprintf(VRML_model,"          appearance Appearance { material ");
    fprintf(VRML_model,"Material { }}\n");
    fprintf(VRML_model,"          geometry IndexedFaceSet{\n");
    fprintf(VRML_model,"            solid FALSE\n"); 
    fprintf(VRML_model,"            coord Coordinate {\n");
    fprintf(VRML_model,"              point [ \n");

    /* plot the verticess */
    current_vertex = grid_per_lambda[lambda_k].start_vertex;
    do {

      if (current_vertex->T_dust[dust_i] > 0.0)
        U_ratio = 2000.0 / current_vertex->T_dust[dust_i];
      else U_ratio = 0.0;

      /* put the vertex into the VRML file */
      fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n",
              current_vertex->x, current_vertex->z, U_ratio);
      fflush(VRML_model);
      current_vertex = current_vertex->next;
    } while (current_vertex!=grid_per_lambda[lambda_k].start_vertex);

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            }\n");
    fprintf(VRML_model,"            coordIndex [\n");

    /* triangles */
    current_triangle = grid_per_lambda[lambda_k].start_triangle;
    do {
      fprintf(VRML_model,"%d, %d, %d, -1,\n",
              current_triangle->v1->identification_number,
              current_triangle->v2->identification_number,
              current_triangle->v3->identification_number);
      fflush(VRML_model);
      current_triangle=current_triangle->next;
    } while(current_triangle!=grid_per_lambda[lambda_k].start_triangle);

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            color Color {\n");
    fprintf(VRML_model,"              color [\n");

    /* plot colors */
    current_vertex = grid_per_lambda[lambda_k].start_vertex;
    while (current_vertex!=NULL) {

      if (current_vertex->T_dust[dust_i] > 0.0)
        color_basic_value = fabs(current_vertex->T_error[dust_i]);
      else color_basic_value = -1.0;
      if (color_basic_value>1.0) color_basic_value=1.0;

      /* calculate the point colors */
      if (color_basic_value<0.0) {color1r=0.3; color1g=0.3; color1b=0.3;}
      else if (color_basic_value<=0.3333 ) {
	color1r=3.0*color_basic_value; color1g=0; color1b=1;}
      else if (color_basic_value>0.3333 && color_basic_value<=0.6666) {
	color1r=1.0; color1g=3.0*(color_basic_value-0.3333); 
        color1b=1.0-3.0*(color_basic_value-0.3333);}
      else { color1r=1.0; color1g=1.0; color1b=3.0*(color_basic_value-0.6666);}
      fprintf(VRML_model,"                %10.6f %10.6f %10.6f,\n",
	      color1r,color1g,color1b);

      fflush(VRML_model);
      current_vertex = current_vertex->next;
      if (current_vertex==grid_per_lambda[lambda_k].start_vertex) 
	current_vertex=NULL;
    }

    fprintf(VRML_model,"              ]\n");
    fprintf(VRML_model,"            }\n");
    fprintf(VRML_model,"            colorPerVertex TRUE\n");
    fprintf(VRML_model,"          }\n");
    fprintf(VRML_model,"        }\n");
    fprintf(VRML_model,"  ]\n}\n");
  } /* end of vrml_type=3 */


  fclose(VRML_model);


  return 0;
}

