/* LELUYA - VODENJAK program block */
/* This function calculates optical depth with high precision in 3D space
   with purpose of checking the precision of _ZELENJAK_tau_over_grid 
   function. After that it creates the VRML visualistion of the ray.*/
/* IMPORTANT: switch off this function when you do not debug the code!!*/
/* IMPORTANT: this function DOES NOT RECOGNIZE THE SOURCE SURFACE! */

#include "VODENJAK.h"

double _VODENJAK_tau_over_grid (struct _3DVector startpoint, 
			        struct _3DVector endpoint, int lambda_k){

  /* calculate the optical depth between startpoint and endpoint over the
     computational grid */
  struct _3DVector direction; /* direction of integration */
  struct _3DVector current_point; /* a point in 3D space during integration */
  struct _3DVector next_point; /* point after current_point */
  struct _3DVector current_point_xz; /* cylindrical coordinates */
  struct _triangle_structure *current_triangle; /* location of current_point */
  struct _triangle_structure *next_triangle; /* location of next_point */
  double distance_total; /* distance between startpoint and endpoint */
  double current_distance; /* distance between startpoint and current_point */
  double distance_step; /* integration spatial step */
  double tau; /* optical depth distance between startpoint and current_point */
  double tau_step; /* integration tau step */
  double Eta0, Eta1; /* Eta values at current_point and one point back */
  double *temperature; /* dust temperature at current_point */
  int dust_i; /* dust component counter */
  int err; /* error flag */
  double A,B,C,distance_old,Eta_old; /* quadratic optical depth interpolation */
  int quadratic; /* flag which shows if the quadratic interpolation is used */

  /* VRML */
  FILE *VRML_model;
  char filename[259];  /* name of the file */
  struct _vertex_structure *current_vertex; /* vertex in the loop over all
                                               vertices */
  double color_basic_value, color_basic_value_normalization; /* values
		          which will be transformed into appropriate colors */
  double color1r,color1g,color1b; 
  double cosphi, sinphi;

  double dummy;

  temperature = dvector(0,Max_Dust_Type-1);  
  /* initial values */
  distance_total=_MATH_LIB_3Ddistance(startpoint,endpoint);
  if (distance_total == 0.0) return 0.0;

  direction.x = (endpoint.x - startpoint.x)/distance_total;
  direction.y = (endpoint.y - startpoint.y)/distance_total;
  direction.z = (endpoint.z - startpoint.z)/distance_total;
  

  /* EXACT OPTICAL DEPTH CALCULATION */ 
  current_point = startpoint;

  current_point_xz.x = _MATH_LIB_distance(0.0,0.0,startpoint.x,startpoint.y);
  current_point_xz.y = 0.0;
  if (startpoint.z >= 0.0) current_point_xz.z = startpoint.z;
  else current_point_xz.z = - startpoint.z;

  err = _ZELENJAK_locate_point_in_grid(current_point_xz.x,current_point_xz.z,
				      &current_triangle);
  tau = 0.0;
  current_distance = 0.0;

  if (current_triangle==NULL) 
    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) temperature[dust_i]=-1.0;
  else for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    if (current_triangle->v1->T_dust[dust_i]>0.0 && 
        current_triangle->v2->T_dust[dust_i]>0.0 && 
        current_triangle->v3->T_dust[dust_i]>0.0){
      temperature[dust_i]= _MATH_LIB_plane_interpolation 
                          (current_triangle->v1->x,current_triangle->v1->z,
                           current_triangle->v1->T_dust[dust_i],
			   current_triangle->v2->x,current_triangle->v2->z,
                           current_triangle->v2->T_dust[dust_i],
			   current_triangle->v3->x,current_triangle->v3->z,
                           current_triangle->v3->T_dust[dust_i],
			   current_point_xz.x,current_point_xz.z);
      if (temperature[dust_i]==0.0) temperature[dust_i]=-1.0;
    }
    else temperature[dust_i]=-1.0;
  }

  Eta0 = Eta(current_point_xz.x,current_point_xz.z,lambda_k,temperature,&err);

  printf("EXACT CALCULATION!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
  printf("d: eta : tau : x y z\n");
  printf("%E %E %E %E %E %E\n",current_distance,Eta0,tau,
	 current_point.x,current_point.y,current_point.z);

  while (current_distance<distance_total){

    distance_step = _ZELENJAK_size_limit(current_point_xz.x,current_point_xz.z)*
      _MIN_VERTEX_DISTANCE_/5.0;

    next_point.x = current_point.x + distance_step*direction.x;
    next_point.y = current_point.y + distance_step*direction.y;
    next_point.z = current_point.z + distance_step*direction.z;

    current_point_xz.x=_MATH_LIB_distance(0.0,0.0,next_point.x,next_point.y);
    current_point_xz.y = 0.0;
    if (next_point.z >= 0.0) current_point_xz.z = next_point.z;
    else current_point_xz.z = - next_point.z;
    err = _ZELENJAK_locate_point_in_grid(current_point_xz.x,current_point_xz.z,
					&next_triangle);

    /* check if we reached the endpoint */
    if (current_distance + distance_step > distance_total) {
      distance_step = distance_total - current_distance;
      current_distance = distance_total;
      current_point = endpoint;
      current_point_xz.x=_MATH_LIB_distance(0.0,0.0,current_point.x,
					    current_point.y);
      current_point_xz.y = 0.0;
      if (current_point.z >= 0.0) current_point_xz.z = current_point.z;
      else current_point_xz.z = - current_point.z;
      err=_ZELENJAK_locate_point_in_grid(current_point_xz.x,current_point_xz.z,
					&current_triangle);
    }
    else { /* ...or we have not reached the endpoint */ 
      current_distance += distance_step;
      current_point = next_point;
      current_point_xz.x=_MATH_LIB_distance(0.0,0.0,current_point.x,
					    current_point.y);
      current_point_xz.y = 0.0;
      if (current_point.z >= 0.0) current_point_xz.z = current_point.z;
      else current_point_xz.z = - current_point.z;
      current_triangle = next_triangle;
    }

    if (current_triangle==NULL) 
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) temperature[dust_i]=-1.0;
    else for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
      if (current_triangle->v1->T_dust[dust_i]>0.0 && 
          current_triangle->v2->T_dust[dust_i]>0.0 && 
          current_triangle->v3->T_dust[dust_i]>0.0){
        temperature[dust_i]= _MATH_LIB_plane_interpolation 
                            (current_triangle->v1->x,current_triangle->v1->z,
                             current_triangle->v1->T_dust[dust_i],
		             current_triangle->v2->x,current_triangle->v2->z,
                             current_triangle->v2->T_dust[dust_i],
			     current_triangle->v3->x,current_triangle->v3->z,
                             current_triangle->v3->T_dust[dust_i],
			     current_point_xz.x,current_point_xz.z);
        if (temperature[dust_i]==0.0) temperature[dust_i]=-1.0;
      }
      else temperature[dust_i]=-1.0;
    }

    Eta1=Eta(current_point_xz.x,current_point_xz.z,lambda_k,temperature,&err);

    tau_step = (Eta0+Eta1)*distance_step*0.5;
    tau += tau_step;

  printf("%E %E %E %E %E %E\n",current_distance,Eta1,tau,
	 current_point.x,current_point.y,current_point.z);

    /* prepare Eta for the next step */
    Eta0 = Eta1;
  }

  /* END OF THE EXACT OPTICAL DEPTH CALCULATION */

  /*  VRML VISUALISATION */

  sprintf(filename,"_VODENJAK_tau_over_grid.wrl");
  VRML_model=fopen(filename,"wt");

  /* introduction of the VRML file and coordinate axes */
  fprintf(VRML_model,"#VRML V2.0 utf8\n\n");
  fprintf(VRML_model,"Viewpoint { position 0 %lf %lf }\n ",
                     0.5*Rout,2*Rout);
  fprintf(VRML_model,"Group { children Shape{\n");
  fprintf(VRML_model,"appearance Appearance { material Material { }}\n");
  fprintf(VRML_model,"geometry IndexedFaceSet{ coord Coordinate { point [\n");
  fprintf(VRML_model,"%lf 0 0, %lf 0 0, %lf 0 %lf,%lf 0 %lf,]}\n",
                      -Rout,Rout,Rout,-Rout,-Rout,-Rout);
  fprintf(VRML_model,"coordIndex [ 0,1,2,3,-1,]\n");
  fprintf(VRML_model,"color Color{ color [\n");
  fprintf(VRML_model,"0.1 0.1 0.3, 0.1 0.1 0.3,0.1 0.1 0.3,0.1 0.1 0.3,]}");
  fprintf(VRML_model,"}}}\n");
  fprintf(VRML_model,"############################################\n");

  /* PLOT THE X-Z GRID PLANE  */
  fprintf(VRML_model,"# Triangles\n");
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      children [\n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry IndexedLineSet{\n");
  fprintf(VRML_model,"            coord Coordinate {\n");
  fprintf(VRML_model,"              point [ \n");


  /* plot the vertices */
  current_vertex = start_vertex_list;
  do {
    /* put the vertex into the VRML file */
    fprintf(VRML_model,"%12.6lf %12.6lf 0.0,\n",
            current_vertex->x, current_vertex->z);
    fflush(VRML_model);
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [\n");

  /* connect the points into a 3D structure */
  current_triangle = start_triangle_list;
  do {
    /* plot the lines only if the first point has a larger ID-number, or
       if this line doesn't have the adjacent triangle */
    if ((current_triangle->v1->identification_number > 
	 current_triangle->v2->identification_number) ||
	(current_triangle->v1->identification_number < 
	 current_triangle->v2->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v2,current_triangle)==NULL))
      fprintf(VRML_model,"%d, %d, -1,\n",
              current_triangle->v1->identification_number,
              current_triangle->v2->identification_number);
    if ((current_triangle->v2->identification_number > 
	 current_triangle->v3->identification_number) ||
	(current_triangle->v2->identification_number < 
	 current_triangle->v3->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v3,current_triangle)==NULL))
      fprintf(VRML_model,"%d, %d, -1,\n",
              current_triangle->v2->identification_number,
              current_triangle->v3->identification_number);
    if ((current_triangle->v3->identification_number > 
	 current_triangle->v1->identification_number) ||
	(current_triangle->v3->identification_number < 
	 current_triangle->v1->identification_number  &&
         _PERUNIKA_adjacent_triangle(current_triangle->v1,current_triangle)==NULL))
      fprintf(VRML_model,"%d, %d, -1,\n",
              current_triangle->v3->identification_number,
              current_triangle->v1->identification_number);

    current_triangle=current_triangle->next;
  } while (current_triangle!=start_triangle_list);


  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [\n");

  current_vertex = start_vertex_list;
  do {
    if (current_vertex->type == 3) fprintf(VRML_model,"0.2 0.2 0.2,\n");
    else if (current_vertex->type == 2 && current_vertex->level>666)
      fprintf(VRML_model,"0.0 0.0 1.0,\n");
    else if (current_vertex->type == 2) fprintf(VRML_model,"0.0 0.0 1.0,\n");
    else fprintf(VRML_model,"0.0 1.0 0.0,\n");
    fflush(VRML_model);
    current_vertex = current_vertex->next;
  } while (current_vertex!=start_vertex_list);

  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            colorPerVertex TRUE\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        }\n");
  fprintf(VRML_model,"  ]\n}\n");
  


  /* PLOT THE INDIVIDUAL TRIANGLES USED IN THE TAU CALCULATION */
  fprintf(VRML_model,"\n# integration line\n");

  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      children [\n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry IndexedLineSet{\n");
  fprintf(VRML_model,"            coord Coordinate {\n");
  fprintf(VRML_model,"              point [ \n");
  fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n",
	  startpoint.x, startpoint.z, - startpoint.y);
  fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n",
	  endpoint.x, endpoint.z, - endpoint.y);
  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [0, 1, -1,]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [ 1 1 1, 1 1 1,]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            colorPerVertex TRUE\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        }\n");
  fprintf(VRML_model,"  ]\n}\n");

  current_point = startpoint;

  current_point_xz.x = _MATH_LIB_distance(0.0,0.0,startpoint.x,startpoint.y);
  current_point_xz.y = 0.0;
  if (startpoint.z >= 0.0) current_point_xz.z = startpoint.z;
  else current_point_xz.z = - startpoint.z;

  err = _ZELENJAK_locate_point_in_grid(current_point_xz.x,current_point_xz.z,
                                      &current_triangle);
  tau = 0.0;
  current_distance = 0.0;

  if (current_triangle==NULL) 
    for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) temperature[dust_i]=-1.0;
  else for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
    if (current_triangle->v1->T_dust[dust_i]>0.0 && 
        current_triangle->v2->T_dust[dust_i]>0.0 && 
        current_triangle->v3->T_dust[dust_i]>0.0){
      temperature[dust_i]= _MATH_LIB_plane_interpolation 
                          (current_triangle->v1->x,current_triangle->v1->z,
                           current_triangle->v1->T_dust[dust_i],
                           current_triangle->v2->x,current_triangle->v2->z,
                           current_triangle->v2->T_dust[dust_i],
                           current_triangle->v3->x,current_triangle->v3->z,
                           current_triangle->v3->T_dust[dust_i],
                           current_point_xz.x,current_point_xz.z);
      if (temperature[dust_i]==0.0) temperature[dust_i]=-1.0;
    }
    else temperature[dust_i]=-1.0;
  }

  Eta0 = Eta(current_point_xz.x,current_point_xz.z,lambda_k,temperature,&err);

  /* now we can start with the integral */

  /* quadratic interpolation */
  distance_old = 0.0;
  Eta_old = 0.0;
  C = 0.0;
  quadratic=0;
  while (current_distance<distance_total){

    distance_step = _ZELENJAK_3Dstep (current_point,current_triangle,direction,
                                     1.0,&next_point,&next_triangle);

    /* check if we reached the endpoint */
    if (current_distance + distance_step > distance_total) {
      distance_step = distance_total - current_distance;
      current_distance = distance_total;
      current_point = endpoint;
      current_point_xz.x=_MATH_LIB_distance(0.0,0.0,current_point.x,
                                            current_point.y);
      current_point_xz.y = 0.0;
      if (current_point.z >= 0.0) current_point_xz.z = current_point.z;
      else current_point_xz.z = - current_point.z;
      err=_ZELENJAK_locate_point_in_grid(current_point_xz.x,current_point_xz.z,
                                        &current_triangle);
    }
    else { /* ...or we have not reached the endpoint */ 
      current_distance += distance_step;
      current_point = next_point;
      current_point_xz.x=_MATH_LIB_distance(0.0,0.0,current_point.x,
                                            current_point.y);
      current_point_xz.y = 0.0;
      if (current_point.z >= 0.0) current_point_xz.z = current_point.z;
      else current_point_xz.z = - current_point.z;
      current_triangle = next_triangle;
    }

    quadratic = 1;
    if (current_triangle==NULL) {
      for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) temperature[dust_i]=-1.0;
      quadratic = 0;
    }
    else for (dust_i=0; dust_i<Max_Dust_Type; dust_i++) {
      if (current_triangle->v1->T_dust[dust_i]>0.0 && 
          current_triangle->v2->T_dust[dust_i]>0.0 && 
          current_triangle->v3->T_dust[dust_i]>0.0){
	dummy = temperature[dust_i]; /* used for the flag "quadratic" */
        temperature[dust_i]= _MATH_LIB_plane_interpolation 
                            (current_triangle->v1->x,current_triangle->v1->z,
                             current_triangle->v1->T_dust[dust_i],
                             current_triangle->v2->x,current_triangle->v2->z,
                             current_triangle->v2->T_dust[dust_i],
                             current_triangle->v3->x,current_triangle->v3->z,
                             current_triangle->v3->T_dust[dust_i],
                             current_point_xz.x,current_point_xz.z);
        if (temperature[dust_i]==0.0) temperature[dust_i]=-1.0;
      }
      else temperature[dust_i]=-1.0;
      /* check the interpolation flag */ 
      if (dummy*temperature[dust_i] < 0.0) quadratic = 0;
    }

    Eta1=Eta(current_point_xz.x,current_point_xz.z,lambda_k,temperature,&err);

    /* optical depth interpolation*/
    if (quadratic == 0 || C==0.0) {
      /* linear interpolation */ 
    tau_step = (Eta0+Eta1)*distance_step*0.5;
      if (quadratic == 1 && C==0.0) C=1.0; /* this is a special case when we
       use linear interpolation only because it was also used in previous step */
      else C=0.0;
    }
    else {
      /* quadratic interpolation */
      B = (Eta0 - Eta_old)/(current_distance - distance_step - distance_old);
      C = (Eta1 - Eta_old)/(current_distance - distance_old) - B;
      C /= distance_step;
      B = B - C*(current_distance - distance_step + distance_old);
      A = Eta_old - distance_old*(B + C*distance_old);
      tau_step = distance_step*(A + B*(current_distance - 0.5*distance_step)+
	         C*(current_distance*current_distance - 
                    current_distance*distance_step + 
                    distance_step*distance_step/3.0));
      /* failed quadrature */
      if (tau_step<0.0) tau_step = (Eta0+Eta1)*distance_step*0.5;
    }

    tau += tau_step;

    fprintf(VRML_model,"\n# distance = %E\n", current_distance);

    if (current_triangle!=NULL) {

  fprintf(VRML_model,"  Transform {\n");
  fprintf(VRML_model,"      translation %12.6lf %12.6lf %12.6lf\n",
          current_point.x, current_point.z , - current_point.y);
  fprintf(VRML_model,"      children [\n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          appearance Appearance {\n");
  fprintf(VRML_model,"            material Material {diffuseColor 1 1 0 }\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"          geometry Sphere {");
  fprintf(VRML_model,"radius %E }\n",distance_step/20.0);
  fprintf(VRML_model,"        }\n");
  fprintf(VRML_model,"  ]\n}\n");

  fprintf(VRML_model,"# triangle =%d %d %d\n",
	  current_triangle->v1->identification_number,
          current_triangle->v2->identification_number,
	  current_triangle->v3->identification_number);
  fprintf(VRML_model,"    Transform {\n");
  fprintf(VRML_model,"      children [\n");
  fprintf(VRML_model,"        Shape {\n");
  fprintf(VRML_model,"          geometry IndexedLineSet{\n");
  fprintf(VRML_model,"            coord Coordinate {\n");
  fprintf(VRML_model,"              point [ \n");

  cosphi = current_point.x / current_point_xz.x;
  sinphi = current_point.y / current_point_xz.x;
  fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n",
	  current_triangle->v1->x * cosphi, 
	  SIGN(current_triangle->v1->z,current_point.z),
	  - current_triangle->v1->x * sinphi);
  fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n",
	  current_triangle->v2->x * cosphi,
	  SIGN(current_triangle->v2->z,current_point.z),
	  - current_triangle->v2->x * sinphi);
  fprintf(VRML_model,"%12.6lf %12.6lf %12.6lf,\n",
	  current_triangle->v3->x * cosphi, 
	  SIGN(current_triangle->v3->z,current_point.z),
	  - current_triangle->v3->x * sinphi);
  fprintf(VRML_model,"              ]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            coordIndex [0,1,2,0,-1,]\n");
  fprintf(VRML_model,"            color Color {\n");
  fprintf(VRML_model,"              color [ 1 0 0, 1 0 0, 1 0 0,]\n");
  fprintf(VRML_model,"            }\n");
  fprintf(VRML_model,"            colorPerVertex TRUE\n");
  fprintf(VRML_model,"          }\n");
  fprintf(VRML_model,"        }\n");
  fprintf(VRML_model,"  ]\n}\n");
    }


    /* prepare Eta for the next step */
    Eta_old = Eta0;
    Eta0 = Eta1;
    distance_old = current_distance - distance_step;
  }

  fclose(VRML_model);

  /* END OF THE VRML VISUALISATION */
  
  free_dvector(temperature,0,Max_Dust_Type-1);

  return tau;

} 
