/* LELUYA - ZELENJAK program block */
/* prototypes of functions which are used in the radiative transfer
   calculation as a source of information about the input data. */

#include <math.h>
#include <stdio.h>
#include "../_install_parameters.h"
#include "../_global_extern_variables.h"
#include "../MATH_LIB/nrutil.h"
#include "../PESJANEK/PESJANEK.h"

#ifndef _ZELENJAK_prototypes_
#define _ZELENJAK_prototypes_

  /* the calculation of stellar correction is performed on a grid over 
     the stellar surface. The grid resolution is: */
  #define _FINITE_SOURCE_N_theta 6
  #define _FINITE_SOURCE_N_phi   14

#include <math.h>
#include <stdio.h>
#include "../_install_parameters.h"
#include "../_global_extern_variables.h"
#include "../MATH_LIB/nrutil.h"
#include "../MATH_LIB/MATH_LIB.h"
#include "../PESJANEK/PESJANEK.h"
#include "../VODENJAK/VODENJAK.h"

double Total_Source_Flux (int lambda_count, double x, double z, int *err);
double Find_Source_Flux (int lambda_count, double cos_theta, double cos_alpha,
                    double impact_b, int *err);
double Dust_Density (double x, double z, int dust_counter,
                     struct _triangle_structure *interpol_triangle, int *err);
double q_abs (double x, double z, int lambda_i, int dust_component, int *err);
double albedo(double x, double z, int lambda_i, int dust_component, int *err);
double upsilon (double x, double z, int lambda_i, int dust_component, 
                double *T, int *err);
double Eta (double x, double z, int lambda_i, double *T, int *err);
int _ZELENJAK_Scale_Cross_Sections (void);
int _ZELENJAK_exchange_components(int dust1, int dust2);
double _ZELENJAK_size_limit(double x, double z);
double _ZELENJAK_3Dstep (struct _3DVector           startpoint, 
                        struct _triangle_structure *starttriangle,
                        struct _3DVector           direction,
			double                     refinement,
			struct _3DVector           *finalpoint,
			struct _triangle_structure **finaltriangle);
int _ZELENJAK_point_in_triangle(double x,double z,
                                struct _triangle_structure *t);
int _ZELENJAK_locate_point_in_grid(double x,double z,
                                struct _triangle_structure **find_triangle);
int _ZELENJAK_locate_nearby_point_in_grid(double x,double z,
                                struct _vertex_structure **find_vertex);
double _ZELENJAK_tau_over_grid (struct _3DVector startpoint, 
			       struct _3DVector endpoint, int lambda_k,
			       int tau_limit_flag);

int _ZELENJAK_Eta_Denominator(void);

int _ZELENJAK_Find_Source_Radius(void);

struct _vertex_structure *_ZELENJAK_Find_Adjacent_Temperature
           (struct _vertex_structure *v, int dust_i);

double Finite_Source_Flux_Correction (double x, double z,double Tau_center, 
                                 int lambda_k,
                                 struct _vertex_structure *Start_Vertex,
				 struct _triangle_structure *Start_Triangle);

double Finite_Source_Flux_Correction_with_direction (double x, double z,
                                 double Tau_center, int lambda_k,
                                 struct _3DVector Flux_Direction1,
                                 struct _3DVector Flux_Direction2,
                                 struct _vertex_structure *Start_Vertex,
				 struct _triangle_structure *Start_Triangle,
                                 double *correction2);
#endif
