/* LELUYA - ZELENJAK program block */
/* calculation of the source intensity toward a given ray  */
/* general description of the central source can be very complicated, but
   for now we are going to use a point source */
/* ERROR FLAG CODES:
   err = 1  : 'lambda_count' is out of range
   err = 2  : 'theta' is out of range (this is a message, not an error!!)
   err = 3  : 'alpha' is out of range (this is a message, not an error!!)
   err = 4  : impact parameter 'impact_b' is out of range 
              (this is a message, not an error!!)
   err = 5  : the source model 'INPUT.SOURCE_TYPE' doesn't exist
*/

#include "ZELENJAK.h"

double Find_Source_Flux (int lambda_count, double cos_theta, double cos_alpha,
                    double impact_b, int *err){
  /* lambda_count = pointer to the wavelength in the lambda grid */
  /* cos_theta    = cosine of the azimuthal angle toward the observer */
  /* cos_alpha    = cosine of the angle between the line toward the observer
                    and the ray of interest*/
  /* impact_b     = impact parameter of the ray of interest */
  /* err          = error flag */
 
  /* check if the lambda counter is OK */
  if (lambda_count < 0 || lambda_count >= lambda_Max) {
    if (DEBUG.SCREEN_MESSAGE >= 1 )
      printf("SOURCE FLUX ERROR:a grid point %d doesn't exist\n",lambda_count);
    *err = 1;
    return 0.0;
  }

  /* we can have different options */
  /*************************************/
  /* OPTION 0: use simple black body, a point source  */
  if (INPUT.SOURCE_TYPE == 0 || INPUT.SOURCE_TYPE == 1) {
    if (cos_theta < 0.0 || cos_theta > 1.0) {
      if (DEBUG.SCREEN_MESSAGE >= 1 ){
        printf(" SOURCE FLUX ERROR: source model 0: black body: point source");
        printf("\n  theta angle is not between 0 and 90 deg\n");
      }
      *err = 2;
      return 0.0;
    }
    if (cos_alpha != 1.0 ) {
      if (DEBUG.SCREEN_MESSAGE >= 1 ){
        printf(" SOURCE FLUX ERROR: source model 0: black body: point source");
        printf("\n  alpha angle is not 0 deg\n");
      }
      *err = 3;
      return 0.0;
    }
    if (impact_b > Source_Radius ) {
      if (DEBUG.SCREEN_MESSAGE >= 1 ){
        printf(" SOURCE FLUX ERROR: source model 0: black body: point source");
        printf("\n the impact parameter is larger than the source radius\n");
      }
      *err = 4;
      return 0.0;
    }

    /* everything is OK */
    *err = 0;
    return Source_Flux[lambda_count];

  } /* end of the OPTION 0: simple black body, a point source  */


  /* if the INPUT.SOURCE_TYPE doesn't exist, return an error flag */
  *err = 5; 
  return 0.0;

} /* end of the 'Source_Flux' function */
