/* LELUYA - ZELENJAK program block */
/* scaling of the dust cross sections by the absorption-extinction 
   coefficient of the dust component 0 at point _1_ */

#include "ZELENJAK.h"


int _ZELENJAK_Scale_Cross_Sections (void) {

  double x_1,z_1,density_1;
  int err, err_location;
  int counter1, counter2;

  x_1 = 1.0 * sin(Theta_Total_Value);  /* X-cooridnate */
  z_1 = 1.0 * cos(Theta_Total_Value);  /* Z-cooridnate */
  density_1 = Dust_Density(x_1,z_1,0,NULL,&err);
  if (err==2 || err==3 || density_1 <= 0.0) {
    printf("*****************************************************\n");
    printf("ERROR in _ZELENJAK_Scale_Cross_Sections\n");
    printf("Can't find the dust density at the point _1_:\n");
    printf("x=%lf, z=%lf, dust component #0\n",x_1,z_1);
    err_location = 999;
    err = 1;
    _PESJANEK__Error_Message (err_location,err);
    return 1;
  }
  err = 0;

  density_1 *= Sigma_Dust_Abs[0][Lambda_Total];
  /* now go over all cross sections and wavelengths */
  for (counter1=0; counter1<Max_Dust_Type; counter1++){
    for (counter2=0; counter2<lambda_Max; counter2++){
      Sigma_Dust_Abs[counter1][counter2] /= density_1;
      Sigma_Dust_Sca[counter1][counter2] /= density_1;
    }
  }

  return 0;

}
