/* LELUYA - ZELENJAK program block */

/* This function is searching through the triangle_list for a triangle
   which contains the closest point to a given point (x,z) */
/* NOTE: THIS SUBROTINE IS USEFUL ONLY IF _ZELENJAK_locate_point_in_grid DID NOT
         FIND ANY TRIANGLE!! */
/* NOTE: BE CAREFUL IN USING THIS FUNCTION! YOU HAVE TO SET start_triangle_list
         TO AN APPROPRIATE TRIANGLE POINTER. */

#include "ZELENJAK.h"

int _ZELENJAK_locate_nearby_point_in_grid(double x,double z,
                                struct _vertex_structure **find_vertex){
  /* search for 'find_triangle' which contains the closest point to
     the given point (x,z) */

  int I_have_found_it;
  double nu1,nu2,nu3;
  struct _triangle_structure *triangle;
  struct _vertex_structure *loop_vertex;
  struct _vertex_structure *nearby_vertex;
  double Rmin = 1000.0;

  I_have_found_it = 0; 
  nearby_vertex=NULL;

  triangle=start_triangle_list;
  do {
   /* check if the point (x,z) is inside of the bounding box */
    if (x >= 0.9999*triangle->bound_box_down_left_x) 
    if (z >= 0.9999*triangle->bound_box_down_left_z)
    if (x <= 1.0001*triangle->bound_box_up_right_x)  
    if (z <= 1.0001*triangle->bound_box_up_right_z) {
      /* look for the closest point to the (x,z) point */
      nu1 = _MATH_LIB_distance(x,z,triangle->v1->x,triangle->v1->z);
      if (nu1 < Rmin) {
        Rmin=nu1; nearby_vertex=triangle->v1;
      }
      nu2 = _MATH_LIB_distance(x,z,triangle->v2->x,triangle->v2->z);
      if (nu2 < Rmin) {
        Rmin=nu2; nearby_vertex=triangle->v2;
      }
      nu3 = _MATH_LIB_distance(x,z,triangle->v3->x,triangle->v3->z);
      if (nu3 < Rmin) {
        Rmin=nu3; nearby_vertex=triangle->v3;
      }
    }
    triangle=triangle->next;
  } while (triangle!=start_triangle_list);

  /* check if we found anything: */
  if (nearby_vertex==NULL) {
    /* damn! OK, let's go over all vertices and find the closest one */
    loop_vertex = start_vertex_list;
    do {
      nu1 = _MATH_LIB_distance(x,z,loop_vertex->x,loop_vertex->z);
      if (nu1 < Rmin) {
        Rmin=nu1; nearby_vertex=loop_vertex;
      }
      loop_vertex = loop_vertex->next;
    } while (loop_vertex!=start_vertex_list);
  }

  *find_vertex = nearby_vertex;

  return 0;
}
