/* LELUYA - ZELENJAK program block */

/* This function is searching through the triangle_list for a triangle
   which contains a given point (x,z) */
/* NOTE: BE CAREFUL IN USING THIS FUNCTION! YOU HAVE TO SET start_triangle_list
         TO AN APPROPRIATE TRIANGLE POINTER. */

#include "ZELENJAK.h"

int _ZELENJAK_locate_point_in_grid(double x,double z,
                                struct _triangle_structure **find_triangle){
  /* search for 'find_triangle' which contains the given point (x,z) */

  int I_have_found_it;
  double nu1,nu2,nu3;
  struct _triangle_structure *triangle;

  I_have_found_it = 0; 

  triangle=start_triangle_list;
  do {
   /* check if the point (x,z) is inside of the bounding box */
    if (x >= 0.9999*triangle->bound_box_down_left_x) 
    if (z >= 0.9999*triangle->bound_box_down_left_z)
    if (x <= 1.0001*triangle->bound_box_up_right_x)  
    if (z <= 1.0001*triangle->bound_box_up_right_z) {
      /* check if the point (x,z) is inside of the triangle */
      /* This is the case if the point (x,z) is on the left side relative
         to all triangle's sides:

                    (x,z)
                     /
                    /    LEFT
                   / 
                vertex(i)-------vertex(i+1)

                        RIGHT
      */

      nu1=_MATH_LIB_nu_value(triangle->v1->x,triangle->v1->z,
      		       triangle->v2->x,triangle->v2->z,x,z);
      nu2=_MATH_LIB_nu_value(triangle->v2->x,triangle->v2->z,
       		       triangle->v3->x,triangle->v3->z,x,z);
      nu3=_MATH_LIB_nu_value(triangle->v3->x,triangle->v3->z,
       		       triangle->v1->x,triangle->v1->z,x,z);

      if (0.0<=nu1 && 0.0<=nu2 && 0.0<=nu3) {
        /* bingo! That's the triangle! */
        I_have_found_it = 1; 
        break;
      }
    }
    triangle=triangle->next;
  } while (triangle!=start_triangle_list);

  if (I_have_found_it == 0) triangle=NULL;

  if(triangle!=NULL) *find_triangle=triangle;
  else *find_triangle=NULL;

  return 0;
}
