/* LELUYA - ZELENJAK program block */
/* calculation of the upsilon: weight factor of a dust component's 
   dust extinction in the total dust extinction at a given point */

#include "ZELENJAK.h"

double upsilon (double x, double z, int lambda_i, int dust_component, 
                double *T, int *err){
  /* 
  x, z     = coordinates of the point where we need upsilon
  lambda_i = lambda grid point
  dust_component = dust component for which we need upsilon
  T        = the dust's temperature at the point (x,z)
  err      = error flag
  */

  double denominator; /* denominator in the calcultaion of upsilon */
  double density;  /* dust density at the point (x,z) */
  int err_density; /* error flag in search for the dust density */
  int i;           /* counter over the dust components */
  

  *err = 0;
  if (INPUT.DUST_DENSITY_TYPE[dust_component]==99999) return 0.0;
  if (T[dust_component] <= 0.0) return 0.0;
  if (Max_Dust_Type==1 || 
      (Max_Dust_Type==2 && INPUT.DUST_DENSITY_TYPE[1]==99999))
    return 1.0;
  /* if (T[dust_component] > Dust_Sublim_Temp[dust_component] ) return 0.0; 
     else */ {
    denominator = 0.0;
    for (i=0; i<Max_Dust_Type; i++){
      if (INPUT.DUST_DENSITY_TYPE[i]!=99999)
      /* if (T[i] <= Dust_Sublim_Temp[i]) */ 
      if (T[i]>0.0) {
        density=Dust_Density (x,z,i,NULL,&err_density);
        denominator += (density * (Sigma_Dust_Abs[i][lambda_i] +
                                   Sigma_Dust_Sca[i][lambda_i]));
      }
    }
    if (denominator==0.0) return 0.0;
    density=Dust_Density (x,z,dust_component,NULL,&err_density);
    return (density * (Sigma_Dust_Abs[dust_component][lambda_i] +
            Sigma_Dust_Sca[dust_component][lambda_i])/denominator);
  }

}
