/*********************** Algorithm flags *******************************/
/*************** INPUT DATA FLAGS: */
#ifndef _PATULJAK_Data_Flags

#define _PATULJAK_Data_Flags

struct _Input_Data_Flags {
  int  LAMBDA_TYPE;     /* flag in the global input file which gives us
                           information how the lambda grid is specified */
  int  SOURCE_TYPE;     /* flag in the global input which gives us
                           information how the source flux is specified
                           (see ??? for the list of all possible input
                           options) */
  int  EMPTY_SPACE;     /* TRUE/FALSE: it shows if the empty space dust type
                           exist in the model - this helps to speed up the 
                           calculation */ 
  int  SKIP_POINT;      /* FALSE : default
                           1,2: if the temperature error at a grid point is 
                                less than a given value, this point 
                                does not need radiative transfer calculation
                                (1 = this does not apply to the first iteration
                                 2 = this apply to all iterations) */
  int  SKIP_POINT_R;    /* FALSE : default
                           3,4: if the radius at a grid point is less 
                                than a given value then this point
                                does not need radiative transfer calculation
                                (3 = this does not apply to the first iteration
                                 4 = this apply to all iterations) */
  int  SKIP_POINT_T;    /* FALSE : default
                           5,6: if the temperature at a grid point is less
                                than a given value then this point
                                does not need radiative transfer calculation
                                (5 = this does not apply to the first iteration
                                 6 = this apply to all iterations) */
  double TAU_LIMIT;     /* if TAU_LIMIT>0 then any integral over tau stops
                           when tau=TAU_LIMIT. */ 
  double LAMBDA_EMISSION_ONLY; /* for wavelengths longer than this one we do
                           not invert the scattering matrix, thus only dust
                           thermal emission exists. See the flag: -O */
  double TAU_SURFACE_SCALE; /* when the surface recognition is activated 
                           (with -R), this number controls the grid density
                           toward the edge of a dust configuration.
                           It should be > 1.0. */ 
  int DEEPLY_EMBEDDED;  /* flag that activates skipping optically deeply 
                           embedded vertexes */
  double POINT_SOURCE_LIMIT; /* if the apparent angular radius of the source is
                less than that then we use the point source approximation */
  double TEMPERATURE_ACCELERATE_min_err;
  double TEMPERATURE_ACCELERATE_scale; /* flag -b: acceleration method where
                the new temperature is:
                      Tnew=Told(1 + TEMPERATURE_ACCELERATE_scale*DeltaT)
                for |DeltaT| >= TEMPERATURE_ACCELERATE_min_err
                where DeltaT is the temperature error. Limits:
                       0.0 < TEMPERATURE_ACCELERATE_min_err
                       1.0 < TEMPERATURE_ACCELERATE_scale  */
  int *DUST_TYPE;       /* flag in the global input file which gives us
                           information how the dust chemical and physical
                           properties are specified. It will be an array
                           from 0 to Max_Dust_Type-1.
                           This will be used for calculation of the dust
                           cross sections.
                           (see ??? for the list
                           of all possible input options)*/
  int *DUST_DENSITY_TYPE;/* flag in the global input file which gives us
                           information how the dust densities are specified*/
  char **DUST_DENSITY_TYPE_NAME; /* name of the models */
  /******** LIST OF THE DUST DENSITY MODELS *************
  DUST_DENSITY_TYPE=99999
  UST_DENSITY_TYPE_NAME='Empty Space'
       This is not a real dust. This represents the empty space in models
       where we need a grid in regions without dust.
       It has a constant temperature _T99999_ and density _Density99999_ 
       and dust cross section _Sigma99999_ 
  ----------------------
  DUST_DENSITY_TYPE=1  
  DUST_DENSITY_TYPE_NAME='Trinity Model'
       This model is used by Dejan Vinkovic for testing of the code
       and it doesn't have a fixed structure. DO NOT USE THIS MODEL!!!
  ----------------------
  DUST_DENSITY_TYPE=2 
  DUST_DENSITY_TYPE_NAME='Smooth Sphere Model'
       This model is 1D and it is used by Dejan Vinkovic for testing of 
       the code. The dust density is proportional to a*r^(-p).
  ----------------------
  DUST_DENSITY_TYPE=3
  DUST_DENSITY_TYPE_NAME='Ellipsoid Model'
       This model has the dust density = 1/(x^2+ey^2)^(n/2) and if e=1 then
       it is just a 1D sphere. By changing e, one can make ellipsoidal density
       profiles (introduced by Zeljko Ivezic & Dejan Vinkovic)
  ----------------------
  DUST_DENSITY_TYPE=4
  DUST_DENSITY_TYPE_NAME='Flared Disk with Sharp Surface'
       this is a flared disk with the density:
         Rho = Rho0 * x^(-p)*exp(-h0*z^2/x^(2m)
       and the surface z=1.5*h0*x^m
  ----------------------
  DUST_DENSITY_TYPE=5
  DUST_DENSITY_TYPE_NAME='Cone Model'
       A spherical shell without a cone. The dust density in the cone
       is = a*rho^(-p)
  ----------------------
  DUST_DENSITY_TYPE=6
  DUST_DENSITY_TYPE_NAME='Finite Cone Model'
       A spherical shell without a cone. The dust density in the cone
       is a*rho^(-p) up to a radius Rcone
  ----------------------
  DUST_DENSITY_TYPE=7
  DUST_DENSITY_TYPE_NAME='Simple Disk'
       A simple disk model defined with the angle of cutoff cone and
       the outer radius Rdisk, with the density profile: a*rho^(-p)
  ----------------------
  DUST_DENSITY_TYPE=8
  DUST_DENSITY_TYPE_NAME='Flared Disk'
       this is a flared disk with the density:
         Rho = Rho0 * x^(-p)*exp(-h0*z^2/x^(2m))
 ----------------------
  DUST_DENSITY_TYPE=9
  DUST_DENSITY_TYPE_NAME='Flat Disk'
       this is a flat disk:
       Rho = Rho0 * x^(-p) for Xmin<x<Xmax and z<Zmax
  ----------------------
  DUST_DENSITY_TYPE=10
  DUST_DENSITY_TYPE_NAME='Flared Disk + Spherical Halo'
       this is a flared disk combined with a spherical halo with the density:
            Halo = halo dust density profile = Rho1 * r^(-s)
            Disk = disk dust density profile = Rho0 * x^(-p)*exp(-h0*z^2/x^(2m))
            if (Halo>=Disk) denisty = Halo
            else density = Disk
  DUST_DENSITY_TYPE=11
  DUST_DENSITY_TYPE_NAME='Flared Disk + Flared Halo'
       this is a combination of flared disk and flared halo with the density:
            Halo = halo dust density profile = Rho1 * x^(-s)*exp(-h1*z^2/x^(2k))
            Disk = disk dust density profile = Rho0 * x^(-p)*exp(-h0*z^2/x^(2m))
            if (Halo>=Disk) Rho = Halo
            else Rho = Disk
 ----------------------
  DUST_DENSITY_TYPE=102
  DUST_DENSITY_TYPE_NAME='CIT3_Jan17_01'
       This model was used for modeling the AGB star CIT3. It has a cone
       with r^-0.5 density profile in the cone and r^-2 elsewhere.
       (see _PATULJAK__Read_Density_Type.c)
  ----------------------
  DUST_DENSITY_TYPE=103
  DUST_DENSITY_TYPE_NAME='Step-Flat Disk'
       A flat disk with a step in density along the x-axis:
         Rho = Rho1 * x^(-p1) for Xmin<x<Xmid and z<Zmax
         Rho = Rho2 * x^(-p2) for Xmid<x<Xmax and z<Zmax
  ******************************************************/
  
};

#endif

/*************** CALCULATION FLAGS: */
#ifndef _PATULJAK_Calculation_Flags

#define _PATULJAK_Calculation_Flags


#endif


/*************** OUTPUT FLAGS: */
#ifndef _PATULJAK_Output_Flags

#define _PATULJAK_Output_Flags


#endif
