/* LELUYA - global variables */

#ifndef _global_extern_variables_

#define _global_extern_variables_


#include "_global_constants.h"
#include "_global_structures.h"
#include "_debugger_flags.h"
#include "_data_calc_output_flags.h"
#include "_install_parameters.h"

extern struct _Debugger_Flags DEBUG;
extern struct _Input_Data_Flags INPUT;

/***** some constants **********/
extern double SQRT3; /* sqrt(3) */
extern double SQRT3_2; /* sqrt(3)/2 */ 
extern double PI_SQRT3; /* PI/SQRT3 */

/*** lambda grid variables ***/
extern double *Lambda_Grid;/* vector with the wavelenghts at the grid 
                              points */
extern int    lambda_Max;  /* total number of grid points: index of
                              the last wavelength in the grid */
/*** total optical depth variables ***/
extern double Tau_Total;   /* total optical depth toward the central
                              energy source at azimuthal angle 
                              Theta_Total_Value and wavelength 
                              Lambda_Total_Value. This optical depth is
                              used for scaling of all other optical
                              depths used in the program */
extern double Theta_Total_Value;  /* azimuthal angle which defines the line
                              of total optical depth Tau_Total*/
extern int    Theta_Total; /* integer representation of Theta_Total_Value
                              in the angular grid of the central energy
                              source */
extern double Tau_Min_Grid; /* optical depth below which we do not calculate
			       the radiative transfer matrix, but rather use
                               the approximate solution of optically thin
			       dust (used in combination with Lambda_Min_Grid)*/
extern double Lambda_Min_Grid;/* the shortest wavelength for which we can
                               use the approximate solution of optically thin
                               dust and avoid calculation of radiative transfer 
                               matrix (used in combination with Tau_Min_Grid)*/
extern double Grid_Tolerance_Tau_Factor; /* if we have already built 
			       a grid at an optical depth tau, then we will use
			       the same grid for the optical depths between
			       "tau*Grid_Tolerance_Tau_Factor" and "tau"
			       (this also means that Grid_Tolerance_Tau_Factor
			       has to be smaller than 1)*/
extern double Lambda_Total_Value; /* wavelength at which the total optical 
                              depth Tau_Total is specified */
extern int    Lambda_Total;/* integer representation of Lambda_Total_Value
                              in Lambda_Grid */
extern double *Source_Flux;/* the central 'point' source flux*/ 
extern double Rout;        /* Outer radius of the dust envelope */
extern double Source_Radius;/* The scaled radius of the source */
extern double Source_Teff;  /* The effective source temperature */
/*** dust properties *********/
extern char **Dust_Component_Name;/* names of the dust components */
extern double *Dust_Sublim_Temp;/* sublimation temperatures */
extern double Max_Dust_Temp;    /* maximum dust temperature (it can be higher than 
	                           the sublimation temperature) */
extern int    Max_Dust_Type;    /* Total number of different dust type in 
                                   the multigrain mixture */
extern double **Sigma_Dust_Abs; /* dust absorption cross sections */
extern double **Sigma_Dust_Sca; /* dust scattering cross sections */
extern struct density_parameters *Dust_Density_Param; 
                           /* list of lists to all necessary parameters
                              required for calculation of the dust density
                              profiles for each of the dust components.*/
extern double Eta_Denominator; /* denominator in the Eta function */
extern struct _density_grid_structure Density_2D_Grid; /* density grid for 
                              a non-analytic dust density */
/*** radiative transfer variables ******/
extern double Psi;         /* 'Psi' variable (see equations for definition) */
/* here is the list of integral rays. It has to be created for each grid point
   separately. The ray is integral_directions[].direction.  */
extern struct _spherical_triangle_structure
                   integral_directions[MAX_NUMBER_OF_SPHERICAL_TRIANGLES+4];
extern int total_number_of_spherical_triangles; /* total number of rays */

/*** triangular grid variables *********/
extern struct _vertex_structure *start_vertex_list; /* this is the grid !!
                              that is a pointer to the grid vertices
                              which form the grid. */
extern struct _triangle_structure *start_triangle_list; /* when the vertices
                              are connected into the mesh, they create 
                              triangles. This triangles are the mesh cells. */
extern struct _grid_per_lambda_structure *grid_per_lambda; /* list of grids,
			      one grid at each wavelength */
extern struct _grid_per_lambda_structure *old_grid_per_lambda; /* these are the
			   grids from previous iteration */
extern struct _vertex_structure *start_old_vertex_list; /* vertex list from 
                              a previous iteration */
extern struct _triangle_structure *start_old_triangle_list; /* triangle list form 
                              a previous iteration */
extern struct _quadrangle_structure *start_quadrangle_list;/*these quadrangles are
                              corrections of the sublimation surfaces. They
                              are created in _PERUNIKA_sublimation_extension
                              and destroyed in _PERUNIKA_main */
extern int _INITIAL_GRID_ARRAY_r;   /* total number of the radial grid steps */
extern int _INITIAL_GRID_ARRAY_theta;/* total number of the angular grid steps
                                        in the array of intital approximation.
                                        is used for generation of the initial
                                        grid. */
extern double *initial_radial_array;/* radial grid for initial_temperature_array */
extern double ***initial_energy_array; /* array of the initial energy density 
                              and ... */
extern double ***initial_temperature_array; /*...initial dust temperature. This 
                              array is used for generation of the initial
                              triangular grid */
extern int START_LEVEL; /* the grid cell with their level number smaller or
                           equal then START_LEVEL will be created for sure */
extern double CELL_SIZE_LIMIT;/* the smallest geometrical size od a grid cell
                                 (at r=1) */
extern double ETA_RATIO_LIMIT;/* ratio between eta values at two grid 
                                 points has to be bigger than ETA_RATIO_LIMIT
                                 or 1/ETA_RATIO_LIMIT  */
extern int TOTAL_NUMBER_OF_VERTICES; /* total number of the grid vrtices */
extern int TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES; /* total number of the grid
                                 vertices which are not EMPTY_SPACE */
extern int TOTAL_NUMBER_OF_TRIANGLES;/* total number of the grid triangles */
extern double epsilon_GRID_ERROR; /* tolerance in the optical depth criterium
                                 for a new grid point */
extern struct _3DVector basic_icosahedron[12]; /* used for angluar integral rays 
                                  (see: _GOROSTAS_create_integral_directions)*/
extern int T_error_neighbors;  /* 0=no adjacent vertexes are involved in checking 
                                  their error for skipping rad.transfer calculation */
extern double T_error_tolerance; /* if INPUT.SKIP_POINT is 1 or 2 then this 
                                   number shows the temperature error below 
                                   which the radiative transfer is not required*/

extern double R_allowed_range_low, R_allowed_range_high; /* if INPUT.SKIP_POINT is
                                   3 or 4 then this is the range of radii for 
                                   radiative transfer: low<=R<=high */
extern double T_allowed_range_low, T_allowed_range_high; /* if INPUT.SKIP_POINT is
                                   5 or 6 then this is the range of temperatures for
                                   radiative transfer: low<=T<=high */

extern int STEP_FUNCTION_HAZARD_AVOIDANCE; /* a sudden step in the dust density can
                                   produce a grid point during the grid generation
                                   but indicate this point as outside the dust later
                                   on due to small errors in the floating point
                                   arithmetics. This flag shows when the dust
                                   density has to take care of this in
                                   _ZELENJAK_Dust_Density.c */

/* see _GOROSTAS_create_integral_direction subroutinefor this: */
extern double _GOROSTAS_cos_phi[_GOROSTAS_N_phi_grid+1],
              _GOROSTAS_sin_phi[_GOROSTAS_N_phi_grid+1];

/***** Luminosity/Flux calculation ************/
extern double Output_Lambda_Image; /* the wavelength at which we need an image
                                 (comes with the flag -L ) */
extern double Output_Theta_Image; /* the inclination angle at which we need
                                 an image (comes with the flag -A) */
/*** Image plotting variables (output) ********/
extern int PLOT_IMAGE_SIZE_X;/*size of the plotted image (the GIF image will */
extern int PLOT_IMAGE_SIZE_Y;/*be lager. This is just the part with the data)*/

/*** iterations *******************************/
extern int TOTAL_NUMBER_OF_ITERATIONS; /* the total number of iterations:
				       default=1 or specified by -n flag */
extern int GLOBAL_ITERATION_COUNTER;
extern int TOTAL_NUMBER_OF_GRID_BINS; /* the total number of steps within
		     one iteration step based on the vertex temperature:
				       default=1 or specified by -k flag */
extern int GLOBAL_BIN_COUNTER;
extern int INITIAL_BIN_COUNTER; /* set by -K flag, shows from
                     which temperature bin to start the first iteration */
extern double *Temperature_Bins; /* if TOTAL_NUMBER_OF_GRID_BINS>1 then
                       _GOROSTAS_Find_Temperature_Bins (flag -k) creates these bins 
                       according to the distribution of vertex temperatures */
extern double *Radial_Bins; /* if TOTAL_NUMBER_OF_GRID_BINS>1 then
                       _GOROSTAS_Find_Radial_Bins (flag -j) creates these bins 
                       according to the distribution of vertex radii */
extern int Bin_Type; /* method of bin selection: 0=none, 1=temperature, 2=radii */
extern int MAX_LAMBDA_PER_PROCESS; /* maximum number of rad.trans. matrices 
                                   calculated simultaneously (flag -M)*/


/* variables for -403 flag: output temperature array */
extern int TEMPERATURE_ARRAY_TYPE; /* flag showing: 1=Cartesian 2=Polar coordinates
                                      3=Polar with log(r)*/
extern double X1_403, X2_403, Z1_403,Z2_403; /* the temperature box*/
extern int Nx_403, Nz_403; /* grid resolution */ 
/* variables for -404 flag: */
extern double X_404,Z_404;

/* random generator seed */
extern long seed;

/*********** MPI variables *************/
extern int my_MPI_rank;       /* rank of my process */
extern int total_MPI_process; /* total number of processes */
#ifdef _2DD_MPI_
#include <mpi.h>
extern MPI_Comm MPI_communicator;
#endif

#if (LEL_RADTRANS_N && LEL_RADTRANS_L)
/* variables for _VILOZMAJ_Radiative_Transfer_Solver  */
extern double Rad_Trans_Matrix[LEL_RADTRANS_L][LEL_RADTRANS_N][LEL_RADTRANS_N];
extern double Rad_Trans_Emission[LEL_RADTRANS_L][LEL_RADTRANS_N]; 
#endif



#endif
