/* LELUYA - variable structures used by global variables */

#ifndef _global_struct_

#define _global_struct_

struct _3DVector {
  double x,y,z;
};

/* dust density structure: it is defined by the list of parameters
   for each dust component */
struct density_parameters{
  struct density_parameters *next;
  double                    param;
};

/* structure of the grid vertices */
struct _vertex_structure {
  struct _vertex_structure   *next, *prev;
  int                        identification_number;
  struct _vertex_structure   *v1,*v2,*v3,*v4,*v5,*v6;
  int                        level;
  int                        type;
  double                     x, z;
  double                     *T_dust;
  double                     *T_error;
  double                     Eta;
  struct _vertex_triangles   *triangles;
};

struct _vertex_triangles {
  struct _vertex_triangles   *next;
  struct _triangle_structure *triangle;
  double                     nu;
};

/* structure of the grid triangles */
struct _triangle_structure {
  struct _triangle_structure *next, *prev;
  int                        ID_number; 
  struct _vertex_structure   *v1,*v2,*v3;
  int                        level;
  double bound_box_down_left_x, bound_box_down_left_z;
  double bound_box_up_right_x, bound_box_up_right_z;
  double integral_step;
};

/* structure of a polygon: they are used in a process of connecting the 
   grid vertices into a triangle structure */
struct _polygon_structure {
   struct _polygon_structure *next,*prev;
   struct _vertex_structure  *vertex;
   double                     optimal_angle;
   struct _polygon_structure *optimal_vertex;
};


/* structure of a quadrangle: they are used for corrections of the 
   sublimation surfaces */
struct _quadrangle_structure {
  struct _quadrangle_structure *next;
  struct _vertex_structure     *edge_vertex;
  struct _vertex_structure     *next_edge_vertex;
  struct _vertex_structure     *new_next_edge_vertex;
  struct _vertex_structure     *new_edge_vertex;
  int                          dust_component;
  double bound_box_down_left_x, bound_box_down_left_z;
  double bound_box_up_right_x, bound_box_up_right_z;
};

/* structure of an integration direction (as the cetroid of 
   a spherical triangle) (see _GOROSTAS_create_integral_directions for details): */
struct _spherical_triangle_structure {
  int                        parent_triangle_ID_number;
  int                        ID_number;
  int                        ray_counter;
  struct _3DVector           direction,v1,v2,v3;
  double                     weight;
};

/* structure of a list of grids: each wavelengths has its own grid, this
   list shows which grid belongs to which wavelength */
struct _grid_per_lambda_structure {
  double lambda;
  double tau;
  double *Ustar, *Usca, *Uem, *Utot_new;
  int TOTAL_NUMBER_OF_VERTICES;
  int TOTAL_NUMBER_OF_NOT_EMPTY_VERTICES;
  struct _vertex_structure *start_vertex;
  int TOTAL_NUMBER_OF_TRIANGLES;
  struct _triangle_structure *start_triangle; 
};

/* structure of an image: image has many cells, each cell contains info about
   its location (polar coordinates) and the intensity */
struct _image_cell_structure {
  struct _image_cell_structure *next, *prev;
  double r1, r2, phi1, phi2;
  double *Intensity_em, *Intensity_sca, *Intensity_star;
  double *tau;
  int    number_of_visible_points;
  int    old_number_of_visible_points;
};

/* structure of luminosity cells (on spherical shells) */
struct _luminosity_cell_structure {
  struct _luminosity_cell_structure *next, *prev;
  int    ID;
  double theta, theta_minus, theta_plus;
  double *Fstar;
  double *Fsca_plus, *Fsca_minus;
  double *Fem_plus, *Fem_minus;
};

/* structure of the density grid */
struct _density_grid_structure {
  int lambda; 
  int TOTAL_NUMBER_OF_VERTICES;
  int dust;
  int *dust_types;
  double **density;
  struct _triangle_structure *triangle;
};

#endif
